@@ -1,15 +1,13 @@
 from __future__ import unicode_literals
 
-import warnings
-
 from django import forms
 from django.contrib import admin
+from django.contrib.admin import AdminSite
 from django.contrib.contenttypes.admin import GenericStackedInline
 from django.core import checks
-from django.core.exceptions import ImproperlyConfigured
-from django.test import override_settings, TestCase
+from django.test import SimpleTestCase, override_settings
 
-from .models import Song, Book, Album, TwoAlbumFKAndAnE, City, State, Influence
+from .models import Album, Book, City, Influence, Song, State, TwoAlbumFKAndAnE
 
 
 class SongForm(forms.ModelForm):
@@ -35,12 +33,15 @@ class ValidFormFieldsets(admin.ModelAdmin):
 
 
 class MyAdmin(admin.ModelAdmin):
-    @classmethod
-    def check(cls, model, **kwargs):
+    def check(self, **kwargs):
         return ['error!']
 
 
-class SystemChecksTestCase(TestCase):
+@override_settings(
+    SILENCED_SYSTEM_CHECKS=['fields.W342'],  # ForeignKey(unique=True)
+    INSTALLED_APPS=['django.contrib.auth', 'django.contrib.contenttypes', 'admin_checks']
+)
+class SystemChecksTestCase(SimpleTestCase):
 
     @override_settings(DEBUG=True)
     def test_checks_are_performed(self):
@@ -53,6 +54,44 @@ class SystemChecksTestCase(TestCase):
             admin.site.unregister(Song)
             admin.sites.system_check_errors = []
 
+    @override_settings(INSTALLED_APPS=['django.contrib.admin'])
+    def test_contenttypes_dependency(self):
+        errors = admin.checks.check_dependencies()
+        expected = [
+            checks.Error(
+                "'django.contrib.contenttypes' must be in "
+                "INSTALLED_APPS in order to use the admin application.",
+                id="admin.E401",
+            )
+        ]
+        self.assertEqual(errors, expected)
+
+    @override_settings(
+        INSTALLED_APPS=[
+            'django.contrib.admin',
+            'django.contrib.auth',
+            'django.contrib.contenttypes',
+        ],
+        TEMPLATES=[{
+            'BACKEND': 'django.template.backends.django.DjangoTemplates',
+            'DIRS': [],
+            'APP_DIRS': True,
+            'OPTIONS': {
+                'context_processors': [],
+            },
+        }],
+    )
+    def test_auth_contextprocessor_dependency(self):
+        errors = admin.checks.check_dependencies()
+        expected = [
+            checks.Error(
+                "'django.contrib.auth.context_processors.auth' must be in "
+                "TEMPLATES in order to use the admin application.",
+                id="admin.E402",
+            )
+        ]
+        self.assertEqual(errors, expected)
+
     @override_settings(DEBUG=True)
     def test_custom_adminsite(self):
         class CustomAdminSite(admin.AdminSite):
@@ -68,6 +107,21 @@ class SystemChecksTestCase(TestCase):
             custom_site.unregister(Song)
             admin.sites.system_check_errors = []
 
+    def test_field_name_not_in_list_display(self):
+        class SongAdmin(admin.ModelAdmin):
+            list_editable = ["original_release"]
+
+        errors = SongAdmin(Song, AdminSite()).check()
+        expected = [
+            checks.Error(
+                "The value of 'list_editable[0]' refers to 'original_release', "
+                "which is not contained in 'list_display'.",
+                obj=SongAdmin,
+                id='admin.E122',
+            )
+        ]
+        self.assertEqual(errors, expected)
+
     def test_readonly_and_editable(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ["original_release"]
@@ -78,13 +132,11 @@ class SystemChecksTestCase(TestCase):
                     "fields": ["title", "original_release"],
                 }),
             ]
-
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'list_editable[0]' refers to 'original_release', "
-                 "which is not editable through the admin."),
-                hint=None,
+                "The value of 'list_editable[0]' refers to 'original_release', "
+                "which is not editable through the admin.",
                 obj=SongAdmin,
                 id='admin.E125',
             )
@@ -101,15 +153,14 @@ class SystemChecksTestCase(TestCase):
                 }),
             ]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_custom_modelforms_with_fields_fieldsets(self):
         """
         # Regression test for #8027: custom ModelForms with fields/fieldsets
         """
-
-        errors = ValidFields.check(model=Song)
+        errors = ValidFields(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_custom_get_form_with_fieldsets(self):
@@ -118,23 +169,67 @@ class SystemChecksTestCase(TestCase):
         is overridden.
         Refs #19445.
         """
-
-        errors = ValidFormFieldsets.check(model=Song)
+        errors = ValidFormFieldsets(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
+    def test_fieldsets_fields_non_tuple(self):
+        """
+        Tests for a tuple/list for the first fieldset's fields.
+        """
+        class NotATupleAdmin(admin.ModelAdmin):
+            list_display = ["pk", "title"]
+            list_editable = ["title"]
+            fieldsets = [
+                (None, {
+                    "fields": "title"  # not a tuple
+                }),
+            ]
+
+        errors = NotATupleAdmin(Song, AdminSite()).check()
+        expected = [
+            checks.Error(
+                "The value of 'fieldsets[0][1]['fields']' must be a list or tuple.",
+                obj=NotATupleAdmin,
+                id='admin.E008',
+            )
+        ]
+        self.assertEqual(errors, expected)
+
+    def test_nonfirst_fieldset(self):
+        """
+        Tests for a tuple/list for the second fieldset's fields.
+        """
+        class NotATupleAdmin(admin.ModelAdmin):
+            fieldsets = [
+                (None, {
+                    "fields": ("title",)
+                }),
+                ('foo', {
+                    "fields": "author"  # not a tuple
+                }),
+            ]
+
+        errors = NotATupleAdmin(Song, AdminSite()).check()
+        expected = [
+            checks.Error(
+                "The value of 'fieldsets[1][1]['fields']' must be a list or tuple.",
+                obj=NotATupleAdmin,
+                id='admin.E008',
+            )
+        ]
+        self.assertEqual(errors, expected)
+
     def test_exclude_values(self):
         """
         Tests for basic system checks of 'exclude' option values (#12689)
         """
-
         class ExcludedFields1(admin.ModelAdmin):
             exclude = 'foo'
 
-        errors = ExcludedFields1.check(model=Book)
+        errors = ExcludedFields1(Book, AdminSite()).check()
         expected = [
             checks.Error(
                 "The value of 'exclude' must be a list or tuple.",
-                hint=None,
                 obj=ExcludedFields1,
                 id='admin.E014',
             )
@@ -145,11 +240,10 @@ class SystemChecksTestCase(TestCase):
         class ExcludedFields2(admin.ModelAdmin):
             exclude = ('name', 'name')
 
-        errors = ExcludedFields2.check(model=Book)
+        errors = ExcludedFields2(Book, AdminSite()).check()
         expected = [
             checks.Error(
                 "The value of 'exclude' contains duplicate field(s).",
-                hint=None,
                 obj=ExcludedFields2,
                 id='admin.E015',
             )
@@ -165,11 +259,10 @@ class SystemChecksTestCase(TestCase):
             model = Album
             inlines = [ExcludedFieldsInline]
 
-        errors = ExcludedFieldsAlbumAdmin.check(model=Album)
+        errors = ExcludedFieldsAlbumAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
                 "The value of 'exclude' must be a list or tuple.",
-                hint=None,
                 obj=ExcludedFieldsInline,
                 id='admin.E014',
             )
@@ -181,7 +274,6 @@ class SystemChecksTestCase(TestCase):
         Regression test for #9932 - exclude in InlineModelAdmin should not
         contain the ForeignKey field used in ModelAdmin.model
         """
-
         class SongInline(admin.StackedInline):
             model = Song
             exclude = ['album']
@@ -190,12 +282,11 @@ class SystemChecksTestCase(TestCase):
             model = Album
             inlines = [SongInline]
 
-        errors = AlbumAdmin.check(model=Album)
+        errors = AlbumAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
-                ("Cannot exclude the field 'album', because it is the foreign key "
-                 "to the parent model 'admin_checks.Album'."),
-                hint=None,
+                "Cannot exclude the field 'album', because it is the foreign key "
+                "to the parent model 'admin_checks.Album'.",
                 obj=SongInline,
                 id='admin.E201',
             )
@@ -207,14 +298,13 @@ class SystemChecksTestCase(TestCase):
         Regression test for #22034 - check that generic inlines don't look for
         normal ForeignKey relations.
         """
-
         class InfluenceInline(GenericStackedInline):
             model = Influence
 
         class SongAdmin(admin.ModelAdmin):
             inlines = [InfluenceInline]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_generic_inline_model_admin_non_generic_model(self):
@@ -222,18 +312,16 @@ class SystemChecksTestCase(TestCase):
         Ensure that a model without a GenericForeignKey raises problems if it's included
         in an GenericInlineModelAdmin definition.
         """
-
         class BookInline(GenericStackedInline):
             model = Book
 
         class SongAdmin(admin.ModelAdmin):
             inlines = [BookInline]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
                 "'admin_checks.Book' has no GenericForeignKey.",
-                hint=None,
                 obj=BookInline,
                 id='admin.E301',
             )
@@ -242,7 +330,6 @@ class SystemChecksTestCase(TestCase):
 
     def test_generic_inline_model_admin_bad_ct_field(self):
         "A GenericInlineModelAdmin raises problems if the ct_field points to a non-existent field."
-
         class InfluenceInline(GenericStackedInline):
             model = Influence
             ct_field = 'nonexistent'
@@ -250,11 +337,10 @@ class SystemChecksTestCase(TestCase):
         class SongAdmin(admin.ModelAdmin):
             inlines = [InfluenceInline]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
                 "'ct_field' references 'nonexistent', which is not a field on 'admin_checks.Influence'.",
-                hint=None,
                 obj=InfluenceInline,
                 id='admin.E302',
             )
@@ -263,7 +349,6 @@ class SystemChecksTestCase(TestCase):
 
     def test_generic_inline_model_admin_bad_fk_field(self):
         "A GenericInlineModelAdmin raises problems if the ct_fk_field points to a non-existent field."
-
         class InfluenceInline(GenericStackedInline):
             model = Influence
             ct_fk_field = 'nonexistent'
@@ -271,11 +356,10 @@ class SystemChecksTestCase(TestCase):
         class SongAdmin(admin.ModelAdmin):
             inlines = [InfluenceInline]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
                 "'ct_fk_field' references 'nonexistent', which is not a field on 'admin_checks.Influence'.",
-                hint=None,
                 obj=InfluenceInline,
                 id='admin.E303',
             )
@@ -283,8 +367,10 @@ class SystemChecksTestCase(TestCase):
         self.assertEqual(errors, expected)
 
     def test_generic_inline_model_admin_non_gfk_ct_field(self):
-        "A GenericInlineModelAdmin raises problems if the ct_field points to a field that isn't part of a GenericForeignKey"
-
+        """
+        A GenericInlineModelAdmin raises problems if the ct_field points to a
+        field that isn't part of a GenericForeignKey.
+        """
         class InfluenceInline(GenericStackedInline):
             model = Influence
             ct_field = 'name'
@@ -292,11 +378,11 @@ class SystemChecksTestCase(TestCase):
         class SongAdmin(admin.ModelAdmin):
             inlines = [InfluenceInline]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
-                "'admin_checks.Influence' has no GenericForeignKey using content type field 'name' and object ID field 'object_id'.",
-                hint=None,
+                "'admin_checks.Influence' has no GenericForeignKey using "
+                "content type field 'name' and object ID field 'object_id'.",
                 obj=InfluenceInline,
                 id='admin.E304',
             )
@@ -304,8 +390,10 @@ class SystemChecksTestCase(TestCase):
         self.assertEqual(errors, expected)
 
     def test_generic_inline_model_admin_non_gfk_fk_field(self):
-        "A GenericInlineModelAdmin raises problems if the ct_fk_field points to a field that isn't part of a GenericForeignKey"
-
+        """
+        A GenericInlineModelAdmin raises problems if the ct_fk_field points to
+        a field that isn't part of a GenericForeignKey.
+        """
         class InfluenceInline(GenericStackedInline):
             model = Influence
             ct_fk_field = 'name'
@@ -313,11 +401,11 @@ class SystemChecksTestCase(TestCase):
         class SongAdmin(admin.ModelAdmin):
             inlines = [InfluenceInline]
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
-                "'admin_checks.Influence' has no GenericForeignKey using content type field 'content_type' and object ID field 'name'.",
-                hint=None,
+                "'admin_checks.Influence' has no GenericForeignKey using "
+                "content type field 'content_type' and object ID field 'name'.",
                 obj=InfluenceInline,
                 id='admin.E304',
             )
@@ -328,16 +416,14 @@ class SystemChecksTestCase(TestCase):
         """
         Regression test for #15669 - Include app label in admin system check messages
         """
-
         class RawIdNonexistingAdmin(admin.ModelAdmin):
             raw_id_fields = ('nonexisting',)
 
-        errors = RawIdNonexistingAdmin.check(model=Album)
+        errors = RawIdNonexistingAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'raw_id_fields[0]' refers to 'nonexisting', which is "
-                 "not an attribute of 'admin_checks.Album'."),
-                hint=None,
+                "The value of 'raw_id_fields[0]' refers to 'nonexisting', "
+                "which is not an attribute of 'admin_checks.Album'.",
                 obj=RawIdNonexistingAdmin,
                 id='admin.E002',
             )
@@ -350,7 +436,6 @@ class SystemChecksTestCase(TestCase):
         given) make sure fk_name is honored or things blow up when there is more
         than one fk to the parent model.
         """
-
         class TwoAlbumFKAndAnEInline(admin.TabularInline):
             model = TwoAlbumFKAndAnE
             exclude = ("e",)
@@ -359,7 +444,7 @@ class SystemChecksTestCase(TestCase):
         class MyAdmin(admin.ModelAdmin):
             inlines = [TwoAlbumFKAndAnEInline]
 
-        errors = MyAdmin.check(model=Album)
+        errors = MyAdmin(Album, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_inline_self_check(self):
@@ -369,11 +454,10 @@ class SystemChecksTestCase(TestCase):
         class MyAdmin(admin.ModelAdmin):
             inlines = [TwoAlbumFKAndAnEInline]
 
-        errors = MyAdmin.check(model=Album)
+        errors = MyAdmin(Album, AdminSite()).check()
         expected = [
             checks.Error(
                 "'admin_checks.TwoAlbumFKAndAnE' has more than one ForeignKey to 'admin_checks.Album'.",
-                hint=None,
                 obj=TwoAlbumFKAndAnEInline,
                 id='admin.E202',
             )
@@ -388,14 +472,14 @@ class SystemChecksTestCase(TestCase):
         class MyAdmin(admin.ModelAdmin):
             inlines = [TwoAlbumFKAndAnEInline]
 
-        errors = MyAdmin.check(model=Album)
+        errors = MyAdmin(Album, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_readonly(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ("title",)
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_readonly_on_method(self):
@@ -405,7 +489,7 @@ class SystemChecksTestCase(TestCase):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = (my_function,)
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_readonly_on_modeladmin(self):
@@ -415,26 +499,39 @@ class SystemChecksTestCase(TestCase):
             def readonly_method_on_modeladmin(self, obj):
                 pass
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
+        self.assertEqual(errors, [])
+
+    def test_readonly_dynamic_attribute_on_modeladmin(self):
+        class SongAdmin(admin.ModelAdmin):
+            readonly_fields = ("dynamic_method",)
+
+            def __getattr__(self, item):
+                if item == "dynamic_method":
+                    def method(obj):
+                        pass
+                    return method
+                raise AttributeError
+
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_readonly_method_on_model(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ("readonly_method_on_model",)
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_nonexistent_field(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = ("title", "nonexistent")
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'readonly_fields[1]' is not a callable, an attribute "
-                 "of 'SongAdmin', or an attribute of 'admin_checks.Song'."),
-                hint=None,
+                "The value of 'readonly_fields[1]' is not a callable, an attribute "
+                "of 'SongAdmin', or an attribute of 'admin_checks.Song'.",
                 obj=SongAdmin,
                 id='admin.E035',
             )
@@ -446,12 +543,11 @@ class SystemChecksTestCase(TestCase):
             model = City
             readonly_fields = ['i_dont_exist']  # Missing attribute
 
-        errors = CityInline.check(State)
+        errors = CityInline(State, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'readonly_fields[0]' is not a callable, an attribute "
-                 "of 'CityInline', or an attribute of 'admin_checks.City'."),
-                hint=None,
+                "The value of 'readonly_fields[0]' is not a callable, an attribute "
+                "of 'CityInline', or an attribute of 'admin_checks.City'.",
                 obj=CityInline,
                 id='admin.E035',
             )
@@ -465,14 +561,14 @@ class SystemChecksTestCase(TestCase):
                     return "Best Ever!"
                 return "Status unknown."
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_readonly_lambda(self):
         class SongAdmin(admin.ModelAdmin):
             readonly_fields = (lambda obj: "test",)
 
-        errors = SongAdmin.check(model=Song)
+        errors = SongAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_graceful_m2m_fail(self):
@@ -481,16 +577,14 @@ class SystemChecksTestCase(TestCase):
         specifies the 'through' option is included in the 'fields' or the 'fieldsets'
         ModelAdmin options.
         """
-
         class BookAdmin(admin.ModelAdmin):
             fields = ['authors']
 
-        errors = BookAdmin.check(model=Book)
+        errors = BookAdmin(Book, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'fields' cannot include the ManyToManyField 'authors', "
-                 "because that field manually specifies a relationship model."),
-                hint=None,
+                "The value of 'fields' cannot include the ManyToManyField 'authors', "
+                "because that field manually specifies a relationship model.",
                 obj=BookAdmin,
                 id='admin.E013',
             )
@@ -504,12 +598,11 @@ class SystemChecksTestCase(TestCase):
                 ('Header 2', {'fields': ('authors',)}),
             )
 
-        errors = FieldsetBookAdmin.check(model=Book)
+        errors = FieldsetBookAdmin(Book, AdminSite()).check()
         expected = [
             checks.Error(
-                ("The value of 'fieldsets[1][1][\"fields\"]' cannot include the ManyToManyField "
-                 "'authors', because that field manually specifies a relationship model."),
-                hint=None,
+                "The value of 'fieldsets[1][1][\"fields\"]' cannot include the ManyToManyField "
+                "'authors', because that field manually specifies a relationship model.",
                 obj=FieldsetBookAdmin,
                 id='admin.E013',
             )
@@ -520,7 +613,7 @@ class SystemChecksTestCase(TestCase):
         class NestedFieldsAdmin(admin.ModelAdmin):
             fields = ('price', ('name', 'subtitle'))
 
-        errors = NestedFieldsAdmin.check(model=Book)
+        errors = NestedFieldsAdmin(Book, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_nested_fieldsets(self):
@@ -529,7 +622,7 @@ class SystemChecksTestCase(TestCase):
                 ('Main', {'fields': ('price', ('name', 'subtitle'))}),
             )
 
-        errors = NestedFieldsetAdmin.check(model=Book)
+        errors = NestedFieldsetAdmin(Book, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_explicit_through_override(self):
@@ -538,14 +631,13 @@ class SystemChecksTestCase(TestCase):
         is specified as a string, the admin should still be able use
         Model.m2m_field.through
         """
-
         class AuthorsInline(admin.TabularInline):
             model = Book.authors.through
 
         class BookAdmin(admin.ModelAdmin):
             inlines = [AuthorsInline]
 
-        errors = BookAdmin.check(model=Book)
+        errors = BookAdmin(Book, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_non_model_fields(self):
@@ -553,7 +645,6 @@ class SystemChecksTestCase(TestCase):
         Regression for ensuring ModelAdmin.fields can contain non-model fields
         that broke with r11737
         """
-
         class SongForm(forms.ModelForm):
             extra_data = forms.CharField()
 
@@ -561,7 +652,7 @@ class SystemChecksTestCase(TestCase):
             form = SongForm
             fields = ['title', 'extra_data']
 
-        errors = FieldsOnFormOnlyAdmin.check(model=Song)
+        errors = FieldsOnFormOnlyAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
     def test_non_model_first_field(self):
@@ -569,7 +660,6 @@ class SystemChecksTestCase(TestCase):
         Regression for ensuring ModelAdmin.field can handle first elem being a
         non-model field (test fix for UnboundLocalError introduced with r16225).
         """
-
         class SongForm(forms.ModelForm):
             extra_data = forms.CharField()
 
@@ -581,39 +671,17 @@ class SystemChecksTestCase(TestCase):
             form = SongForm
             fields = ['extra_data', 'title']
 
-        errors = FieldsOnFormOnlyAdmin.check(model=Song)
+        errors = FieldsOnFormOnlyAdmin(Song, AdminSite()).check()
         self.assertEqual(errors, [])
 
-    def test_validator_compatibility(self):
-        class MyValidator(object):
-            def validate(self, cls, model):
-                raise ImproperlyConfigured("error!")
-
-        class MyModelAdmin(admin.ModelAdmin):
-            validator_class = MyValidator
-
-        with warnings.catch_warnings(record=True):
-            warnings.filterwarnings('ignore', module='django.contrib.admin.options')
-            errors = MyModelAdmin.check(model=Song)
-
-            expected = [
-                checks.Error(
-                    'error!',
-                    hint=None,
-                    obj=MyModelAdmin,
-                )
-            ]
-            self.assertEqual(errors, expected)
-
     def test_check_sublists_for_duplicates(self):
         class MyModelAdmin(admin.ModelAdmin):
             fields = ['state', ['state']]
 
-        errors = MyModelAdmin.check(model=Song)
+        errors = MyModelAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
                 "The value of 'fields' contains duplicate field(s).",
-                hint=None,
                 obj=MyModelAdmin,
                 id='admin.E006'
             )
@@ -628,13 +696,29 @@ class SystemChecksTestCase(TestCase):
                 }),
             ]
 
-        errors = MyModelAdmin.check(model=Song)
+        errors = MyModelAdmin(Song, AdminSite()).check()
         expected = [
             checks.Error(
                 "There are duplicate field(s) in 'fieldsets[0][1]'.",
-                hint=None,
                 obj=MyModelAdmin,
                 id='admin.E012'
             )
         ]
         self.assertEqual(errors, expected)
+
+    def test_list_filter_works_on_through_field_even_when_apps_not_ready(self):
+        """
+        Ensure list_filter can access reverse fields even when the app registry
+        is not ready; refs #24146.
+        """
+        class BookAdminWithListFilter(admin.ModelAdmin):
+            list_filter = ['authorsbooks__featured']
+
+        # Temporarily pretending apps are not ready yet. This issue can happen
+        # if the value of 'list_filter' refers to a 'through__field'.
+        Book._meta.apps.ready = False
+        try:
+            errors = BookAdminWithListFilter(Book, AdminSite()).check()
+            self.assertEqual(errors, [])
+        finally:
+            Book._meta.apps.ready = True
