@@ -31,9 +31,9 @@ class Teacher(models.Model):
 @python_2_unicode_compatible
 class Child(models.Model):
     name = models.CharField(max_length=50)
-    teacher = models.ForeignKey(Teacher)
+    teacher = models.ForeignKey(Teacher, models.CASCADE)
 
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
     parent = GenericForeignKey()
 
@@ -52,7 +52,7 @@ class Author(models.Model):
 
 class NonAutoPKBook(models.Model):
     rand_pk = models.IntegerField(primary_key=True, editable=False)
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=50)
 
     def save(self, *args, **kwargs):
@@ -65,7 +65,7 @@ class NonAutoPKBook(models.Model):
 
 class EditablePKBook(models.Model):
     manual_pk = models.IntegerField(primary_key=True)
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=50)
 
 
@@ -75,7 +75,7 @@ class Holder(models.Model):
 
 class Inner(models.Model):
     dummy = models.IntegerField()
-    holder = models.ForeignKey(Holder)
+    holder = models.ForeignKey(Holder, models.CASCADE)
     readonly = models.CharField("Inner readonly label", max_length=1)
 
     def get_absolute_url(self):
@@ -88,7 +88,7 @@ class Holder2(models.Model):
 
 class Inner2(models.Model):
     dummy = models.IntegerField()
-    holder = models.ForeignKey(Holder2)
+    holder = models.ForeignKey(Holder2, models.CASCADE)
 
 
 class Holder3(models.Model):
@@ -97,7 +97,7 @@ class Holder3(models.Model):
 
 class Inner3(models.Model):
     dummy = models.IntegerField()
-    holder = models.ForeignKey(Holder3)
+    holder = models.ForeignKey(Holder3, models.CASCADE)
 
 # Models for ticket #8190
 
@@ -108,12 +108,12 @@ class Holder4(models.Model):
 
 class Inner4Stacked(models.Model):
     dummy = models.IntegerField(help_text="Awesome stacked help text is awesome.")
-    holder = models.ForeignKey(Holder4)
+    holder = models.ForeignKey(Holder4, models.CASCADE)
 
 
 class Inner4Tabular(models.Model):
     dummy = models.IntegerField(help_text="Awesome tabular help text is awesome.")
-    holder = models.ForeignKey(Holder4)
+    holder = models.ForeignKey(Holder4, models.CASCADE)
 
 # Models for #12749
 
@@ -127,13 +127,13 @@ class OutfitItem(models.Model):
 
 
 class Fashionista(models.Model):
-    person = models.OneToOneField(Person, primary_key=True)
+    person = models.OneToOneField(Person, models.CASCADE, primary_key=True)
     weaknesses = models.ManyToManyField(OutfitItem, through='ShoppingWeakness', blank=True)
 
 
 class ShoppingWeakness(models.Model):
-    fashionista = models.ForeignKey(Fashionista)
-    item = models.ForeignKey(OutfitItem)
+    fashionista = models.ForeignKey(Fashionista, models.CASCADE)
+    item = models.ForeignKey(OutfitItem, models.CASCADE)
 
 # Models for #13510
 
@@ -143,7 +143,7 @@ class TitleCollection(models.Model):
 
 
 class Title(models.Model):
-    collection = models.ForeignKey(TitleCollection, blank=True, null=True)
+    collection = models.ForeignKey(TitleCollection, models.SET_NULL, blank=True, null=True)
     title1 = models.CharField(max_length=100)
     title2 = models.CharField(max_length=100)
 
@@ -155,7 +155,7 @@ class Poll(models.Model):
 
 
 class Question(models.Model):
-    poll = models.ForeignKey(Poll)
+    poll = models.ForeignKey(Poll, models.CASCADE)
 
 
 class Novel(models.Model):
@@ -164,14 +164,14 @@ class Novel(models.Model):
 
 class Chapter(models.Model):
     name = models.CharField(max_length=40)
-    novel = models.ForeignKey(Novel)
+    novel = models.ForeignKey(Novel, models.CASCADE)
 
 
 class FootNote(models.Model):
     """
     Model added for ticket 19838
     """
-    chapter = models.ForeignKey(Chapter, on_delete=models.PROTECT)
+    chapter = models.ForeignKey(Chapter, models.PROTECT)
     note = models.CharField(max_length=40)
 
 # Models for #16838
@@ -183,17 +183,17 @@ class CapoFamiglia(models.Model):
 
 class Consigliere(models.Model):
     name = models.CharField(max_length=100, help_text='Help text for Consigliere')
-    capo_famiglia = models.ForeignKey(CapoFamiglia, related_name='+')
+    capo_famiglia = models.ForeignKey(CapoFamiglia, models.CASCADE, related_name='+')
 
 
 class SottoCapo(models.Model):
     name = models.CharField(max_length=100)
-    capo_famiglia = models.ForeignKey(CapoFamiglia, related_name='+')
+    capo_famiglia = models.ForeignKey(CapoFamiglia, models.CASCADE, related_name='+')
 
 
 class ReadOnlyInline(models.Model):
     name = models.CharField(max_length=100, help_text='Help text for ReadOnlyInline')
-    capo_famiglia = models.ForeignKey(CapoFamiglia)
+    capo_famiglia = models.ForeignKey(CapoFamiglia, models.CASCADE)
 
 
 # Models for #18433
@@ -206,7 +206,7 @@ class ParentModelWithCustomPk(models.Model):
 class ChildModel1(models.Model):
     my_own_pk = models.CharField(max_length=100, primary_key=True)
     name = models.CharField(max_length=100)
-    parent = models.ForeignKey(ParentModelWithCustomPk)
+    parent = models.ForeignKey(ParentModelWithCustomPk, models.CASCADE)
 
     def get_absolute_url(self):
         return '/child_model1/'
@@ -215,7 +215,7 @@ class ChildModel1(models.Model):
 class ChildModel2(models.Model):
     my_own_pk = models.CharField(max_length=100, primary_key=True)
     name = models.CharField(max_length=100)
-    parent = models.ForeignKey(ParentModelWithCustomPk)
+    parent = models.ForeignKey(ParentModelWithCustomPk, models.CASCADE)
 
     def get_absolute_url(self):
         return '/child_model2/'
@@ -224,7 +224,7 @@ class ChildModel2(models.Model):
 # Models for #19425
 class BinaryTree(models.Model):
     name = models.CharField(max_length=100)
-    parent = models.ForeignKey('self', null=True, blank=True)
+    parent = models.ForeignKey('self', models.SET_NULL, null=True, blank=True)
 
 # Models for #19524
 
@@ -238,7 +238,7 @@ class ExtraTerrestrial(LifeForm):
 
 
 class Sighting(models.Model):
-    et = models.ForeignKey(ExtraTerrestrial)
+    et = models.ForeignKey(ExtraTerrestrial, models.CASCADE)
     place = models.CharField(max_length=100)
 
 
@@ -250,7 +250,7 @@ class SomeParentModel(models.Model):
 class SomeChildModel(models.Model):
     name = models.CharField(max_length=1)
     position = models.PositiveIntegerField()
-    parent = models.ForeignKey(SomeParentModel)
+    parent = models.ForeignKey(SomeParentModel, models.CASCADE)
 
 # Other models
 
@@ -260,6 +260,6 @@ class ProfileCollection(models.Model):
 
 
 class Profile(models.Model):
-    collection = models.ForeignKey(ProfileCollection, blank=True, null=True)
+    collection = models.ForeignKey(ProfileCollection, models.SET_NULL, blank=True, null=True)
     first_name = models.CharField(max_length=100)
     last_name = models.CharField(max_length=100)
