@@ -90,7 +90,7 @@ class ChapterXtra1Admin(admin.ModelAdmin):
 class ArticleAdmin(admin.ModelAdmin):
     list_display = (
         'content', 'date', callable_year, 'model_year', 'modeladmin_year',
-        'model_year_reversed', 'section',
+        'model_year_reversed', 'section', lambda obj: obj.title,
     )
     list_editable = ('section',)
     list_filter = ('date', 'section')
@@ -208,8 +208,7 @@ class PersonAdmin(admin.ModelAdmin):
     save_as = True
 
     def get_changelist_formset(self, request, **kwargs):
-        return super(PersonAdmin, self).get_changelist_formset(request,
-            formset=BasePersonModelFormSet, **kwargs)
+        return super(PersonAdmin, self).get_changelist_formset(request, formset=BasePersonModelFormSet, **kwargs)
 
     def get_queryset(self, request):
         # Order by a field that isn't in list display, to be able to test
@@ -512,8 +511,7 @@ class CoverLetterAdmin(admin.ModelAdmin):
     """
     A ModelAdmin with a custom get_queryset() method that uses defer(), to test
     verbose_name display in messages shown after adding/editing CoverLetter
-    instances.
-    Note that the CoverLetter model defines a __unicode__ method.
+    instances. Note that the CoverLetter model defines a __str__ method.
     For testing fix for ticket #14529.
     """
 
@@ -549,8 +547,7 @@ class TelegramAdmin(admin.ModelAdmin):
     """
     A ModelAdmin with a custom get_queryset() method that uses only(), to test
     verbose_name display in messages shown after adding/editing Telegram
-    instances.
-    Note that the Telegram model defines a __unicode__ method.
+    instances. Note that the Telegram model defines a __str__ method.
     For testing fix for ticket #14529.
     """
 
@@ -597,9 +594,10 @@ class PluggableSearchPersonAdmin(admin.ModelAdmin):
         )
         try:
             search_term_as_int = int(search_term)
-            queryset |= self.model.objects.filter(age=search_term_as_int)
-        except:
+        except ValueError:
             pass
+        else:
+            queryset |= self.model.objects.filter(age=search_term_as_int)
         return queryset, use_distinct
 
 
@@ -1015,6 +1013,7 @@ site2.register(
     list_editable=['parent'],
     raw_id_fields=['parent'],
 )
+site2.register(Person, save_as_continue=False)
 
 site7 = admin.AdminSite(name="admin7")
 site7.register(Article, ArticleAdmin2)
