@@ -11,6 +11,7 @@ from django.contrib.contenttypes.fields import (
     GenericForeignKey, GenericRelation,
 )
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ValidationError
 from django.core.files.storage import FileSystemStorage
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
@@ -43,8 +44,9 @@ class Article(models.Model):
     title = models.CharField(max_length=100)
     content = models.TextField()
     date = models.DateTimeField()
-    section = models.ForeignKey(Section, null=True, blank=True)
-    sub_section = models.ForeignKey(Section, null=True, blank=True, on_delete=models.SET_NULL, related_name='+')
+    section = models.ForeignKey(Section, models.CASCADE, null=True, blank=True)
+    another_section = models.ForeignKey(Section, models.CASCADE, null=True, blank=True, related_name='+')
+    sub_section = models.ForeignKey(Section, models.SET_NULL, null=True, blank=True, related_name='+')
 
     def __str__(self):
         return self.title
@@ -74,7 +76,7 @@ class Book(models.Model):
 @python_2_unicode_compatible
 class Promo(models.Model):
     name = models.CharField(max_length=100, verbose_name='¿Name?')
-    book = models.ForeignKey(Book)
+    book = models.ForeignKey(Book, models.CASCADE)
 
     def __str__(self):
         return self.name
@@ -84,7 +86,7 @@ class Promo(models.Model):
 class Chapter(models.Model):
     title = models.CharField(max_length=100, verbose_name='¿Title?')
     content = models.TextField()
-    book = models.ForeignKey(Book)
+    book = models.ForeignKey(Book, models.CASCADE)
 
     def __str__(self):
         return self.title
@@ -96,7 +98,7 @@ class Chapter(models.Model):
 
 @python_2_unicode_compatible
 class ChapterXtra1(models.Model):
-    chap = models.OneToOneField(Chapter, verbose_name='¿Chap?')
+    chap = models.OneToOneField(Chapter, models.CASCADE, verbose_name='¿Chap?')
     xtra = models.CharField(max_length=100, verbose_name='¿Xtra?')
 
     def __str__(self):
@@ -105,7 +107,7 @@ class ChapterXtra1(models.Model):
 
 @python_2_unicode_compatible
 class ChapterXtra2(models.Model):
-    chap = models.OneToOneField(Chapter, verbose_name='¿Chap?')
+    chap = models.OneToOneField(Chapter, models.CASCADE, verbose_name='¿Chap?')
     xtra = models.CharField(max_length=100, verbose_name='¿Xtra?')
 
     def __str__(self):
@@ -150,7 +152,7 @@ class Color2(Color):
 @python_2_unicode_compatible
 class Thing(models.Model):
     title = models.CharField(max_length=20)
-    color = models.ForeignKey(Color, limit_choices_to={'warm': True})
+    color = models.ForeignKey(Color, models.CASCADE, limit_choices_to={'warm': True})
     pub_date = models.DateField(blank=True, null=True)
 
     def __str__(self):
@@ -170,7 +172,7 @@ class Actor(models.Model):
 @python_2_unicode_compatible
 class Inquisition(models.Model):
     expected = models.BooleanField(default=False)
-    leader = models.ForeignKey(Actor)
+    leader = models.ForeignKey(Actor, models.CASCADE)
     country = models.CharField(max_length=20)
 
     def __str__(self):
@@ -180,12 +182,27 @@ class Inquisition(models.Model):
 @python_2_unicode_compatible
 class Sketch(models.Model):
     title = models.CharField(max_length=100)
-    inquisition = models.ForeignKey(Inquisition, limit_choices_to={'leader__name': 'Palin',
-                                                                   'leader__age': 27,
-                                                                   'expected': False,
-                                                                   })
-    defendant0 = models.ForeignKey(Actor, limit_choices_to={'title__isnull': False}, related_name='as_defendant0')
-    defendant1 = models.ForeignKey(Actor, limit_choices_to={'title__isnull': True}, related_name='as_defendant1')
+    inquisition = models.ForeignKey(
+        Inquisition,
+        models.CASCADE,
+        limit_choices_to={
+            'leader__name': 'Palin',
+            'leader__age': 27,
+            'expected': False,
+        },
+    )
+    defendant0 = models.ForeignKey(
+        Actor,
+        models.CASCADE,
+        limit_choices_to={'title__isnull': False},
+        related_name='as_defendant0',
+    )
+    defendant1 = models.ForeignKey(
+        Actor,
+        models.CASCADE,
+        limit_choices_to={'title__isnull': True},
+        related_name='as_defendant1',
+    )
 
     def __str__(self):
         return self.title
@@ -211,7 +228,12 @@ class Character(models.Model):
 @python_2_unicode_compatible
 class StumpJoke(models.Model):
     variation = models.CharField(max_length=100)
-    most_recently_fooled = models.ForeignKey(Character, limit_choices_to=today_callable_dict, related_name="+")
+    most_recently_fooled = models.ForeignKey(
+        Character,
+        models.CASCADE,
+        limit_choices_to=today_callable_dict,
+        related_name="+",
+    )
     has_fooled_today = models.ManyToManyField(Character, limit_choices_to=today_callable_q, related_name="+")
 
     def __str__(self):
@@ -263,7 +285,7 @@ class Account(models.Model):
     types of accounts.
     """
     username = models.CharField(blank=False, max_length=80)
-    persona = models.ForeignKey(Persona, related_name="accounts")
+    persona = models.ForeignKey(Persona, models.CASCADE, related_name="accounts")
     servicename = 'generic service'
 
     def __str__(self):
@@ -309,18 +331,26 @@ class Podcast(Media):
 
 
 class Vodcast(Media):
-    media = models.OneToOneField(Media, primary_key=True, parent_link=True)
+    media = models.OneToOneField(Media, models.CASCADE, primary_key=True, parent_link=True)
     released = models.BooleanField(default=False)
 
 
 class Parent(models.Model):
     name = models.CharField(max_length=128)
 
+    def clean(self):
+        if self.name == '_invalid':
+            raise ValidationError('invalid')
+
 
 class Child(models.Model):
-    parent = models.ForeignKey(Parent, editable=False)
+    parent = models.ForeignKey(Parent, models.CASCADE, editable=False)
     name = models.CharField(max_length=30, blank=True)
 
+    def clean(self):
+        if self.name == '_invalid':
+            raise ValidationError('invalid')
+
 
 @python_2_unicode_compatible
 class EmptyModel(models.Model):
@@ -339,7 +369,7 @@ class Gallery(models.Model):
 class Picture(models.Model):
     name = models.CharField(max_length=100)
     image = models.FileField(storage=temp_storage, upload_to='test_upload')
-    gallery = models.ForeignKey(Gallery, related_name="pictures")
+    gallery = models.ForeignKey(Gallery, models.CASCADE, related_name="pictures")
 
 
 class Language(models.Model):
@@ -358,7 +388,7 @@ class Title(models.Model):
 
 
 class TitleTranslation(models.Model):
-    title = models.ForeignKey(Title)
+    title = models.ForeignKey(Title, models.CASCADE)
     text = models.CharField(max_length=100)
 
 
@@ -367,7 +397,7 @@ class Recommender(Title):
 
 
 class Recommendation(Title):
-    recommender = models.ForeignKey(Recommender)
+    recommender = models.ForeignKey(Recommender, models.CASCADE)
 
 
 class Collector(models.Model):
@@ -375,25 +405,25 @@ class Collector(models.Model):
 
 
 class Widget(models.Model):
-    owner = models.ForeignKey(Collector)
+    owner = models.ForeignKey(Collector, models.CASCADE)
     name = models.CharField(max_length=100)
 
 
 class DooHickey(models.Model):
     code = models.CharField(max_length=10, primary_key=True)
-    owner = models.ForeignKey(Collector)
+    owner = models.ForeignKey(Collector, models.CASCADE)
     name = models.CharField(max_length=100)
 
 
 class Grommet(models.Model):
     code = models.AutoField(primary_key=True)
-    owner = models.ForeignKey(Collector)
+    owner = models.ForeignKey(Collector, models.CASCADE)
     name = models.CharField(max_length=100)
 
 
 class Whatsit(models.Model):
     index = models.IntegerField(primary_key=True)
-    owner = models.ForeignKey(Collector)
+    owner = models.ForeignKey(Collector, models.CASCADE)
     name = models.CharField(max_length=100)
 
 
@@ -402,13 +432,13 @@ class Doodad(models.Model):
 
 
 class FancyDoodad(Doodad):
-    owner = models.ForeignKey(Collector)
+    owner = models.ForeignKey(Collector, models.CASCADE)
     expensive = models.BooleanField(default=True)
 
 
 @python_2_unicode_compatible
 class Category(models.Model):
-    collector = models.ForeignKey(Collector)
+    collector = models.ForeignKey(Collector, models.CASCADE)
     order = models.PositiveIntegerField()
 
     class Meta:
@@ -418,12 +448,14 @@ class Category(models.Model):
         return '%s:o%s' % (self.id, self.order)
 
 
+def link_posted_default():
+    return datetime.date.today() - datetime.timedelta(days=7)
+
+
 class Link(models.Model):
-    posted = models.DateField(
-        default=lambda: datetime.date.today() - datetime.timedelta(days=7)
-    )
+    posted = models.DateField(default=link_posted_default)
     url = models.URLField()
-    post = models.ForeignKey("Post")
+    post = models.ForeignKey("Post", models.CASCADE)
     readonly_link_content = models.TextField()
 
 
@@ -434,7 +466,7 @@ class PrePopulatedPost(models.Model):
 
 
 class PrePopulatedSubPost(models.Model):
-    post = models.ForeignKey(PrePopulatedPost)
+    post = models.ForeignKey(PrePopulatedPost, models.CASCADE)
     subtitle = models.CharField(max_length=100)
     subslug = models.SlugField()
 
@@ -484,7 +516,7 @@ class SuperVillain(Villain):
 class FunkyTag(models.Model):
     "Because we all know there's only one real use case for GFKs."
     name = models.CharField(max_length=25)
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
     content_object = GenericForeignKey('content_type', 'object_id')
 
@@ -495,8 +527,8 @@ class FunkyTag(models.Model):
 @python_2_unicode_compatible
 class Plot(models.Model):
     name = models.CharField(max_length=100)
-    team_leader = models.ForeignKey(Villain, related_name='lead_plots')
-    contact = models.ForeignKey(Villain, related_name='contact_plots')
+    team_leader = models.ForeignKey(Villain, models.CASCADE, related_name='lead_plots')
+    contact = models.ForeignKey(Villain, models.CASCADE, related_name='contact_plots')
     tags = GenericRelation(FunkyTag)
 
     def __str__(self):
@@ -506,7 +538,7 @@ class Plot(models.Model):
 @python_2_unicode_compatible
 class PlotDetails(models.Model):
     details = models.CharField(max_length=100)
-    plot = models.OneToOneField(Plot, null=True, blank=True)
+    plot = models.OneToOneField(Plot, models.CASCADE, null=True, blank=True)
 
     def __str__(self):
         return self.details
@@ -521,7 +553,7 @@ class PlotProxy(Plot):
 class SecretHideout(models.Model):
     """ Secret! Not registered with the admin! """
     location = models.CharField(max_length=100)
-    villain = models.ForeignKey(Villain)
+    villain = models.ForeignKey(Villain, models.CASCADE)
 
     def __str__(self):
         return self.location
@@ -531,16 +563,25 @@ class SecretHideout(models.Model):
 class SuperSecretHideout(models.Model):
     """ Secret! Not registered with the admin! """
     location = models.CharField(max_length=100)
-    supervillain = models.ForeignKey(SuperVillain)
+    supervillain = models.ForeignKey(SuperVillain, models.CASCADE)
 
     def __str__(self):
         return self.location
 
 
+@python_2_unicode_compatible
+class Bookmark(models.Model):
+    name = models.CharField(max_length=60)
+    tag = GenericRelation(FunkyTag, related_query_name='bookmark')
+
+    def __str__(self):
+        return self.name
+
+
 @python_2_unicode_compatible
 class CyclicOne(models.Model):
     name = models.CharField(max_length=25)
-    two = models.ForeignKey('CyclicTwo')
+    two = models.ForeignKey('CyclicTwo', models.CASCADE)
 
     def __str__(self):
         return self.name
@@ -549,7 +590,7 @@ class CyclicOne(models.Model):
 @python_2_unicode_compatible
 class CyclicTwo(models.Model):
     name = models.CharField(max_length=25)
-    one = models.ForeignKey(CyclicOne)
+    one = models.ForeignKey(CyclicOne, models.CASCADE)
 
     def __str__(self):
         return self.name
@@ -565,7 +606,7 @@ class Pizza(models.Model):
 
 
 class Album(models.Model):
-    owner = models.ForeignKey(User, null=True, blank=True, on_delete=models.SET_NULL)
+    owner = models.ForeignKey(User, models.SET_NULL, null=True, blank=True)
     title = models.CharField(max_length=30)
 
 
@@ -575,7 +616,7 @@ class Employee(Person):
 
 class WorkHour(models.Model):
     datum = models.DateField()
-    employee = models.ForeignKey(Employee)
+    employee = models.ForeignKey(Employee, models.CASCADE)
 
 
 class Question(models.Model):
@@ -584,7 +625,7 @@ class Question(models.Model):
 
 @python_2_unicode_compatible
 class Answer(models.Model):
-    question = models.ForeignKey(Question, on_delete=models.PROTECT)
+    question = models.ForeignKey(Question, models.PROTECT)
     answer = models.CharField(max_length=20)
 
     def __str__(self):
@@ -719,10 +760,11 @@ class MainPrepopulated(models.Model):
                  ('option two', 'Option Two')))
     slug1 = models.SlugField(blank=True)
     slug2 = models.SlugField(blank=True)
+    slug3 = models.SlugField(blank=True, allow_unicode=True)
 
 
 class RelatedPrepopulated(models.Model):
-    parent = models.ForeignKey(MainPrepopulated)
+    parent = models.ForeignKey(MainPrepopulated, models.CASCADE)
     name = models.CharField(max_length=75)
     pubdate = models.DateField()
     status = models.CharField(
@@ -770,8 +812,10 @@ class Simple(models.Model):
 
 
 class Choice(models.Model):
-    choice = models.IntegerField(blank=True, null=True,
-        choices=((1, 'Yes'), (0, 'No'), (None, 'No opinion')))
+    choice = models.IntegerField(
+        blank=True, null=True,
+        choices=((1, 'Yes'), (0, 'No'), (None, 'No opinion')),
+    )
 
 
 class ParentWithDependentChildren(models.Model):
@@ -790,7 +834,7 @@ class DependentChild(models.Model):
     Model that depends on validation of the parent class for one of its
     fields to validate during clean
     """
-    parent = models.ForeignKey(ParentWithDependentChildren)
+    parent = models.ForeignKey(ParentWithDependentChildren, models.CASCADE)
     family_name = models.CharField(max_length=255)
 
 
@@ -824,7 +868,7 @@ class State(models.Model):
 
 
 class City(models.Model):
-    state = models.ForeignKey(State)
+    state = models.ForeignKey(State, models.CASCADE)
     name = models.CharField(max_length=100)
 
     def get_absolute_url(self):
@@ -832,7 +876,7 @@ class City(models.Model):
 
 
 class Restaurant(models.Model):
-    city = models.ForeignKey(City)
+    city = models.ForeignKey(City, models.CASCADE)
     name = models.CharField(max_length=100)
 
     def get_absolute_url(self):
@@ -840,7 +884,7 @@ class Restaurant(models.Model):
 
 
 class Worker(models.Model):
-    work_at = models.ForeignKey(Restaurant)
+    work_at = models.ForeignKey(Restaurant, models.CASCADE)
     name = models.CharField(max_length=50)
     surname = models.CharField(max_length=50)
 
@@ -852,7 +896,10 @@ class ReferencedByParent(models.Model):
 
 class ParentWithFK(models.Model):
     fk = models.ForeignKey(
-        ReferencedByParent, to_field='name', related_name='hidden+',
+        ReferencedByParent,
+        models.CASCADE,
+        to_field='name',
+        related_name='hidden+',
     )
 
 
@@ -867,7 +914,10 @@ class ReferencedByInline(models.Model):
 
 class InlineReference(models.Model):
     fk = models.ForeignKey(
-        ReferencedByInline, to_field='name', related_name='hidden+',
+        ReferencedByInline,
+        models.CASCADE,
+        to_field='name',
+        related_name='hidden+',
     )
 
 
@@ -886,8 +936,8 @@ class Ingredient(models.Model):
 
 
 class RecipeIngredient(models.Model):
-    ingredient = models.ForeignKey(Ingredient, to_field='iname')
-    recipe = models.ForeignKey(Recipe, to_field='rname')
+    ingredient = models.ForeignKey(Ingredient, models.CASCADE, to_field='iname')
+    recipe = models.ForeignKey(Recipe, models.CASCADE, to_field='rname')
 
 
 # Model for #23839
@@ -925,4 +975,4 @@ class ParentWithUUIDPK(models.Model):
 
 
 class RelatedWithUUIDPKModel(models.Model):
-    parent = models.ForeignKey(ParentWithUUIDPK, on_delete=models.CASCADE)
+    parent = models.ForeignKey(ParentWithUUIDPK, on_delete=models.SET_NULL, null=True, blank=True)
