@@ -2,7 +2,6 @@
 # Unit and doctests for specific database backends.
 from __future__ import unicode_literals
 
-import copy
 import datetime
 import re
 import threading
@@ -10,7 +9,6 @@ import unittest
 import warnings
 from decimal import Decimal, Rounded
 
-from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management.color import no_style
 from django.db import (
@@ -18,25 +16,23 @@ from django.db import (
     reset_queries, transaction,
 )
 from django.db.backends.base.base import BaseDatabaseWrapper
-from django.db.backends.postgresql_psycopg2 import version as pg_version
+from django.db.backends.postgresql import version as pg_version
 from django.db.backends.signals import connection_created
 from django.db.backends.utils import CursorWrapper, format_number
 from django.db.models import Avg, StdDev, Sum, Variance
 from django.db.models.sql.constants import CURSOR
 from django.db.utils import ConnectionHandler
 from django.test import (
-    TestCase, TransactionTestCase, mock, override_settings, skipIfDBFeature,
-    skipUnlessDBFeature,
+    SimpleTestCase, TestCase, TransactionTestCase, mock, override_settings,
+    skipIfDBFeature, skipUnlessDBFeature,
 )
-from django.test.utils import ignore_warnings, str_prefix
 from django.utils import six
-from django.utils.deprecation import RemovedInDjango19Warning
 from django.utils.six.moves import range
 
 from . import models
 
 
-class DummyBackendTest(TestCase):
+class DummyBackendTest(SimpleTestCase):
 
     def test_no_databases(self):
         """
@@ -44,8 +40,7 @@ class DummyBackendTest(TestCase):
         """
         DATABASES = {}
         conns = ConnectionHandler(DATABASES)
-        self.assertEqual(conns[DEFAULT_DB_ALIAS].settings_dict['ENGINE'],
-            'django.db.backends.dummy')
+        self.assertEqual(conns[DEFAULT_DB_ALIAS].settings_dict['ENGINE'], 'django.db.backends.dummy')
         with self.assertRaises(ImproperlyConfigured):
             conns[DEFAULT_DB_ALIAS].ensure_connection()
 
@@ -123,28 +118,27 @@ class SQLiteTests(TestCase):
             statements = editor.collected_sql
         match = re.search('"id" ([^,]+),', statements[0])
         self.assertIsNotNone(match)
-        self.assertEqual('integer NOT NULL PRIMARY KEY AUTOINCREMENT',
-            match.group(1), "Wrong SQL used to create an auto-increment "
-            "column on SQLite")
+        self.assertEqual(
+            'integer NOT NULL PRIMARY KEY AUTOINCREMENT',
+            match.group(1),
+            "Wrong SQL used to create an auto-increment column on SQLite"
+        )
 
     def test_aggregation(self):
         """
         #19360: Raise NotImplementedError when aggregating on date/time fields.
         """
         for aggregate in (Sum, Avg, Variance, StdDev):
-            self.assertRaises(
-                NotImplementedError,
-                models.Item.objects.all().aggregate, aggregate('time'))
-            self.assertRaises(
-                NotImplementedError,
-                models.Item.objects.all().aggregate, aggregate('date'))
-            self.assertRaises(
-                NotImplementedError,
-                models.Item.objects.all().aggregate, aggregate('last_modified'))
-            self.assertRaises(
-                NotImplementedError,
-                models.Item.objects.all().aggregate,
-                **{'complex': aggregate('last_modified') + aggregate('last_modified')})
+            with self.assertRaises(NotImplementedError):
+                models.Item.objects.all().aggregate(aggregate('time'))
+            with self.assertRaises(NotImplementedError):
+                models.Item.objects.all().aggregate(aggregate('date'))
+            with self.assertRaises(NotImplementedError):
+                models.Item.objects.all().aggregate(aggregate('last_modified'))
+            with self.assertRaises(NotImplementedError):
+                models.Item.objects.all().aggregate(
+                    **{'complex': aggregate('last_modified') + aggregate('last_modified')}
+                )
 
     def test_memory_db_test_name(self):
         """
@@ -184,7 +178,11 @@ class PostgreSQLTests(TestCase):
         self.assert_parses("EnterpriseDB 9.3", 90300)
         self.assert_parses("PostgreSQL 9.3.6", 90306)
         self.assert_parses("PostgreSQL 9.4beta1", 90400)
-        self.assert_parses("PostgreSQL 9.3.1 on i386-apple-darwin9.2.2, compiled by GCC i686-apple-darwin9-gcc-4.0.1 (GCC) 4.0.1 (Apple Inc. build 5478)", 90301)
+        self.assert_parses(
+            "PostgreSQL 9.3.1 on i386-apple-darwin9.2.2, compiled by GCC "
+            "i686-apple-darwin9-gcc-4.0.1 (GCC) 4.0.1 (Apple Inc. build 5478)",
+            90301
+        )
 
     def test_nodb_connection(self):
         """
@@ -200,16 +198,14 @@ class PostgreSQLTests(TestCase):
         self.assertIsNone(nodb_conn.settings_dict['NAME'])
 
         # Now assume the 'postgres' db isn't available
-        del connection._nodb_connection
         with warnings.catch_warnings(record=True) as w:
             with mock.patch('django.db.backends.base.base.BaseDatabaseWrapper.connect',
                             side_effect=mocked_connect, autospec=True):
                 warnings.simplefilter('always', RuntimeWarning)
                 nodb_conn = connection._nodb_connection
-        del connection._nodb_connection
         self.assertIsNotNone(nodb_conn.settings_dict['NAME'])
-        self.assertEqual(nodb_conn.settings_dict['NAME'], settings.DATABASES[DEFAULT_DB_ALIAS]['NAME'])
-        # Check a RuntimeWarning nas been emitted
+        self.assertEqual(nodb_conn.settings_dict['NAME'], connection.settings_dict['NAME'])
+        # Check a RuntimeWarning has been emitted
         self.assertEqual(len(w), 1)
         self.assertEqual(w[0].message.__class__, RuntimeWarning)
 
@@ -245,9 +241,8 @@ class PostgreSQLTests(TestCase):
         PostgreSQL shouldn't roll back SET TIME ZONE, even if the first
         transaction is rolled back (#17062).
         """
-        databases = copy.deepcopy(settings.DATABASES)
-        new_connections = ConnectionHandler(databases)
-        new_connection = new_connections[DEFAULT_DB_ALIAS]
+        new_connection = connection.copy()
+
         try:
             # Ensure the database default time zone is different than
             # the time zone in new_connection.settings_dict. We can
@@ -259,17 +254,22 @@ class PostgreSQLTests(TestCase):
             new_tz = 'Europe/Paris' if db_default_tz == 'UTC' else 'UTC'
             new_connection.close()
 
+            # Invalidate timezone name cache, because the setting_changed
+            # handler cannot know about new_connection.
+            del new_connection.timezone_name
+
             # Fetch a new connection with the new_tz as default
             # time zone, run a query and rollback.
-            new_connection.settings_dict['TIME_ZONE'] = new_tz
-            new_connection.set_autocommit(False)
-            cursor = new_connection.cursor()
-            new_connection.rollback()
+            with self.settings(TIME_ZONE=new_tz):
+                new_connection.set_autocommit(False)
+                cursor = new_connection.cursor()
+                new_connection.rollback()
+
+                # Now let's see if the rollback rolled back the SET TIME ZONE.
+                cursor.execute("SHOW TIMEZONE")
+                tz = cursor.fetchone()[0]
+                self.assertEqual(new_tz, tz)
 
-            # Now let's see if the rollback rolled back the SET TIME ZONE.
-            cursor.execute("SHOW TIMEZONE")
-            tz = cursor.fetchone()[0]
-            self.assertEqual(new_tz, tz)
         finally:
             new_connection.close()
 
@@ -278,10 +278,9 @@ class PostgreSQLTests(TestCase):
         The connection wrapper shouldn't believe that autocommit is enabled
         after setting the time zone when AUTOCOMMIT is False (#21452).
         """
-        databases = copy.deepcopy(settings.DATABASES)
-        databases[DEFAULT_DB_ALIAS]['AUTOCOMMIT'] = False
-        new_connections = ConnectionHandler(databases)
-        new_connection = new_connections[DEFAULT_DB_ALIAS]
+        new_connection = connection.copy()
+        new_connection.settings_dict['AUTOCOMMIT'] = False
+
         try:
             # Open a database connection.
             new_connection.cursor()
@@ -307,10 +306,8 @@ class PostgreSQLTests(TestCase):
         default_level = read_committed if psycopg2.__version__ < '2.7' else None
         self.assertEqual(connection.connection.isolation_level, default_level)
 
-        databases = copy.deepcopy(settings.DATABASES)
-        databases[DEFAULT_DB_ALIAS]['OPTIONS']['isolation_level'] = serializable
-        new_connections = ConnectionHandler(databases)
-        new_connection = new_connections[DEFAULT_DB_ALIAS]
+        new_connection = connection.copy()
+        new_connection.settings_dict['OPTIONS']['isolation_level'] = serializable
         try:
             # Start a transaction so the isolation level isn't reported as 0.
             new_connection.set_autocommit(False)
@@ -335,7 +332,7 @@ class PostgreSQLTests(TestCase):
         self.assertEqual(a[0], b[0])
 
     def test_lookup_cast(self):
-        from django.db.backends.postgresql_psycopg2.operations import DatabaseOperations
+        from django.db.backends.postgresql.operations import DatabaseOperations
 
         do = DatabaseOperations(connection=None)
         for lookup in ('iexact', 'contains', 'icontains', 'startswith',
@@ -343,8 +340,8 @@ class PostgreSQLTests(TestCase):
             self.assertIn('::text', do.lookup_cast(lookup))
 
     def test_correct_extraction_psycopg2_version(self):
-        from django.db.backends.postgresql_psycopg2.base import psycopg2_version
-        version_path = 'django.db.backends.postgresql_psycopg2.base.Database.__version__'
+        from django.db.backends.postgresql.base import psycopg2_version
+        version_path = 'django.db.backends.postgresql.base.Database.__version__'
 
         with mock.patch(version_path, '2.6.9'):
             self.assertEqual(psycopg2_version(), (2, 6, 9))
@@ -362,7 +359,6 @@ class DateQuotingTest(TestCase):
         #12818__.
 
         __: http://code.djangoproject.com/ticket/12818
-
         """
         updated = datetime.datetime(2010, 2, 20)
         models.SchoolClass.objects.create(year=2009, last_updated=updated)
@@ -375,7 +371,6 @@ class DateQuotingTest(TestCase):
         which clash with strings passed to it (e.g. 'day') - see #12818__.
 
         __: http://code.djangoproject.com/ticket/12818
-
         """
         updated = datetime.datetime(2010, 2, 20)
         models.SchoolClass.objects.create(year=2009, last_updated=updated)
@@ -392,10 +387,7 @@ class LastExecutedQueryTest(TestCase):
         query has been run.
         """
         cursor = connection.cursor()
-        try:
-            connection.ops.last_executed_query(cursor, '', ())
-        except Exception:
-            self.fail("'last_executed_query' should not raise an exception.")
+        connection.ops.last_executed_query(cursor, '', ())
 
     def test_debug_sql(self):
         list(models.Reporter.objects.filter(first_name="test"))
@@ -420,8 +412,31 @@ class LastExecutedQueryTest(TestCase):
         # This shouldn't raise an exception
         query = "SELECT strftime('%Y', 'now');"
         connection.cursor().execute(query)
-        self.assertEqual(connection.queries[-1]['sql'],
-            str_prefix("QUERY = %(_)s\"SELECT strftime('%%Y', 'now');\" - PARAMS = ()"))
+        self.assertEqual(connection.queries[-1]['sql'], query)
+
+    @unittest.skipUnless(connection.vendor == 'sqlite',
+                         "This test is specific to SQLite.")
+    def test_parameter_quoting_on_sqlite(self):
+        # The implementation of last_executed_queries isn't optimal. It's
+        # worth testing that parameters are quoted. See #14091.
+        query = "SELECT %s"
+        params = ["\"'\\"]
+        connection.cursor().execute(query, params)
+        # Note that the single quote is repeated
+        substituted = "SELECT '\"''\\'"
+        self.assertEqual(connection.queries[-1]['sql'], substituted)
+
+    @unittest.skipUnless(connection.vendor == 'sqlite',
+                         "This test is specific to SQLite.")
+    def test_large_number_of_parameters_on_sqlite(self):
+        # If SQLITE_MAX_VARIABLE_NUMBER (default = 999) has been changed to be
+        # greater than SQLITE_MAX_COLUMN (default = 2000), last_executed_query
+        # can hit the SQLITE_MAX_COLUMN limit. See #26063.
+        cursor = connection.cursor()
+        sql = "SELECT MAX(%s)" % ", ".join(["%s"] * 2001)
+        params = list(range(2001))
+        # This should not raise an exception.
+        cursor.db.ops.last_executed_query(cursor.cursor, sql, params)
 
 
 class ParameterHandlingTest(TestCase):
@@ -434,8 +449,10 @@ class ParameterHandlingTest(TestCase):
             connection.ops.quote_name('root'),
             connection.ops.quote_name('square')
         ))
-        self.assertRaises(Exception, cursor.executemany, query, [(1, 2, 3)])
-        self.assertRaises(Exception, cursor.executemany, query, [(1,)])
+        with self.assertRaises(Exception):
+            cursor.executemany(query, [(1, 2, 3)])
+        with self.assertRaises(Exception):
+            cursor.executemany(query, [(1,)])
 
 
 # Unfortunately, the following tests would be a good test to run on all
@@ -455,13 +472,19 @@ class LongNameTest(TransactionTestCase):
         models.VeryLongModelNameZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ.objects.create()
 
     def test_sequence_name_length_limits_m2m(self):
-        """Test an m2m save of a model with a long name and a long m2m field name doesn't error as on Django >=1.2 this now uses object saves. Ref #8901"""
+        """
+        An m2m save of a model with a long name and a long m2m field name
+        doesn't error (#8901).
+        """
         obj = models.VeryLongModelNameZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ.objects.create()
         rel_obj = models.Person.objects.create(first_name='Django', last_name='Reinhardt')
         obj.m2m_also_quite_long_zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz.add(rel_obj)
 
     def test_sequence_name_length_limits_flush(self):
-        """Test that sequence resetting as part of a flush with model with long name and long pk name doesn't error. Ref #8901"""
+        """
+        Sequence resetting as part of a flush with model with long name and
+        long pk name doesn't error (#8901).
+        """
         # A full flush is expensive to the full test, so we dig into the
         # internals to generate the likely offending SQL and run it manually
 
@@ -652,11 +675,15 @@ class BackendTestCase(TransactionTestCase):
         models.Person(first_name="Clark", last_name="Kent").save()
         opts2 = models.Person._meta
         f3, f4 = opts2.get_field('first_name'), opts2.get_field('last_name')
-        query2 = ('SELECT %s, %s FROM %s ORDER BY %s'
-          % (qn(f3.column), qn(f4.column), connection.introspection.table_name_converter(opts2.db_table),
-             qn(f3.column)))
         cursor = connection.cursor()
-        cursor.execute(query2)
+        cursor.execute(
+            'SELECT %s, %s FROM %s ORDER BY %s' % (
+                qn(f3.column),
+                qn(f4.column),
+                connection.introspection.table_name_converter(opts2.db_table),
+                qn(f3.column),
+            )
+        )
         self.assertEqual(cursor.fetchone(), ('Clark', 'Kent'))
         self.assertEqual(list(cursor.fetchmany(2)), [('Jane', 'Doe'), ('John', 'Doe')])
         self.assertEqual(list(cursor.fetchall()), [('Mary', 'Agnelline'), ('Peter', 'Parker')])
@@ -775,8 +802,7 @@ class BackendTestCase(TransactionTestCase):
         """
         old_queries_limit = BaseDatabaseWrapper.queries_limit
         BaseDatabaseWrapper.queries_limit = 3
-        new_connections = ConnectionHandler(settings.DATABASES)
-        new_connection = new_connections[DEFAULT_DB_ALIAS]
+        new_connection = connection.copy()
 
         # Initialize the connection and clear initialization statements.
         with new_connection.cursor():
@@ -799,9 +825,10 @@ class BackendTestCase(TransactionTestCase):
             with warnings.catch_warnings(record=True) as w:
                 self.assertEqual(3, len(new_connection.queries))
                 self.assertEqual(1, len(w))
-                self.assertEqual(str(w[0].message), "Limit for query logging "
-                    "exceeded, only the last 3 queries will be returned.")
-
+                self.assertEqual(
+                    str(w[0].message),
+                    "Limit for query logging exceeded, only the last 3 queries will be returned."
+                )
         finally:
             BaseDatabaseWrapper.queries_limit = old_queries_limit
             new_connection.close()
@@ -844,7 +871,8 @@ class FkConstraintsTests(TransactionTestCase):
         a2 = models.Article(headline='This is another test', reporter=self.r,
                             pub_date=datetime.datetime(2012, 8, 3),
                             reporter_proxy_id=30)
-        self.assertRaises(IntegrityError, a2.save)
+        with self.assertRaises(IntegrityError):
+            a2.save()
 
     def test_integrity_checks_on_update(self):
         """
@@ -872,15 +900,21 @@ class FkConstraintsTests(TransactionTestCase):
         # Retrieve the second article from the DB
         a2 = models.Article.objects.get(headline='Another article')
         a2.reporter_proxy_id = 30
-        self.assertRaises(IntegrityError, a2.save)
+        with self.assertRaises(IntegrityError):
+            a2.save()
 
     def test_disable_constraint_checks_manually(self):
         """
-        When constraint checks are disabled, should be able to write bad data without IntegrityErrors.
+        When constraint checks are disabled, should be able to write bad data
+        without IntegrityErrors.
         """
         with transaction.atomic():
             # Create an Article.
-            models.Article.objects.create(headline="Test article", pub_date=datetime.datetime(2010, 9, 4), reporter=self.r)
+            models.Article.objects.create(
+                headline="Test article",
+                pub_date=datetime.datetime(2010, 9, 4),
+                reporter=self.r,
+            )
             # Retrieve it from the DB
             a = models.Article.objects.get(headline="Test article")
             a.reporter_id = 30
@@ -894,11 +928,16 @@ class FkConstraintsTests(TransactionTestCase):
 
     def test_disable_constraint_checks_context_manager(self):
         """
-        When constraint checks are disabled (using context manager), should be able to write bad data without IntegrityErrors.
+        When constraint checks are disabled (using context manager), should be
+        able to write bad data without IntegrityErrors.
         """
         with transaction.atomic():
             # Create an Article.
-            models.Article.objects.create(headline="Test article", pub_date=datetime.datetime(2010, 9, 4), reporter=self.r)
+            models.Article.objects.create(
+                headline="Test article",
+                pub_date=datetime.datetime(2010, 9, 4),
+                reporter=self.r,
+            )
             # Retrieve it from the DB
             a = models.Article.objects.get(headline="Test article")
             a.reporter_id = 30
@@ -915,7 +954,11 @@ class FkConstraintsTests(TransactionTestCase):
         """
         with transaction.atomic():
             # Create an Article.
-            models.Article.objects.create(headline="Test article", pub_date=datetime.datetime(2010, 9, 4), reporter=self.r)
+            models.Article.objects.create(
+                headline="Test article",
+                pub_date=datetime.datetime(2010, 9, 4),
+                reporter=self.r,
+            )
             # Retrieve it from the DB
             a = models.Article.objects.get(headline="Test article")
             a.reporter_id = 30
@@ -1117,13 +1160,13 @@ class DBConstraintTestCase(TestCase):
         self.assertEqual(models.Object.objects.count(), 2)
         self.assertEqual(obj.related_objects.count(), 1)
 
-        intermediary_model = models.Object._meta.get_field("related_objects").rel.through
+        intermediary_model = models.Object._meta.get_field("related_objects").remote_field.through
         intermediary_model.objects.create(from_object_id=obj.id, to_object_id=12345)
         self.assertEqual(obj.related_objects.count(), 1)
         self.assertEqual(intermediary_model.objects.count(), 2)
 
 
-class BackendUtilTests(TestCase):
+class BackendUtilTests(SimpleTestCase):
 
     def test_format_number(self):
         """
@@ -1178,125 +1221,6 @@ class BackendUtilTests(TestCase):
                   '1234600000')
 
 
-@ignore_warnings(category=UserWarning,
-                 message="Overriding setting DATABASES can lead to unexpected behavior")
-class DBTestSettingsRenamedTests(TestCase):
-
-    mismatch_msg = ("Connection 'test-deprecation' has mismatched TEST "
-                    "and TEST_* database settings.")
-
-    def setUp(self):
-        super(DBTestSettingsRenamedTests, self).setUp()
-        self.handler = ConnectionHandler()
-        self.db_settings = {'default': {}}
-
-    def test_mismatched_database_test_settings_1(self):
-        # if the TEST setting is used, all TEST_* keys must appear in it.
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {},
-                'TEST_NAME': 'foo',
-            }
-        })
-        with override_settings(DATABASES=self.db_settings):
-            with self.assertRaisesMessage(ImproperlyConfigured, self.mismatch_msg):
-                self.handler.prepare_test_settings('test-deprecation')
-
-    def test_mismatched_database_test_settings_2(self):
-        # if the TEST setting is used, all TEST_* keys must match.
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {'NAME': 'foo'},
-                'TEST_NAME': 'bar',
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            with self.assertRaisesMessage(ImproperlyConfigured, self.mismatch_msg):
-                self.handler.prepare_test_settings('test-deprecation')
-
-    def test_mismatched_database_test_settings_3(self):
-        # Verifies the mapping of an aliased key.
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {'CREATE_DB': 'foo'},
-                'TEST_CREATE': 'bar',
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            with self.assertRaisesMessage(ImproperlyConfigured, self.mismatch_msg):
-                self.handler.prepare_test_settings('test-deprecation')
-
-    def test_mismatched_database_test_settings_4(self):
-        # Verifies the mapping of an aliased key when the aliased key is missing.
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {},
-                'TEST_CREATE': 'bar',
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            with self.assertRaisesMessage(ImproperlyConfigured, self.mismatch_msg):
-                self.handler.prepare_test_settings('test-deprecation')
-
-    def test_mismatched_settings_old_none(self):
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {'CREATE_DB': None},
-                'TEST_CREATE': '',
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            with self.assertRaisesMessage(ImproperlyConfigured, self.mismatch_msg):
-                self.handler.prepare_test_settings('test-deprecation')
-
-    def test_mismatched_settings_new_none(self):
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {},
-                'TEST_CREATE': None,
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            with self.assertRaisesMessage(ImproperlyConfigured, self.mismatch_msg):
-                self.handler.prepare_test_settings('test-deprecation')
-
-    def test_matched_test_settings(self):
-        # should be able to define new settings and the old, if they match
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {'NAME': 'foo'},
-                'TEST_NAME': 'foo',
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            self.handler.prepare_test_settings('test-deprecation')
-
-    def test_new_settings_only(self):
-        # should be able to define new settings without the old
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST': {'NAME': 'foo'},
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            self.handler.prepare_test_settings('test-deprecation')
-
-    @ignore_warnings(category=RemovedInDjango19Warning)
-    def test_old_settings_only(self):
-        # should be able to define old settings without the new
-        self.db_settings.update({
-            'test-deprecation': {
-                'TEST_NAME': 'foo',
-            },
-        })
-        with override_settings(DATABASES=self.db_settings):
-            self.handler.prepare_test_settings('test-deprecation')
-
-    def test_empty_settings(self):
-        with override_settings(DATABASES=self.db_settings):
-            self.handler.prepare_test_settings('default')
-
-
 @unittest.skipUnless(connection.vendor == 'sqlite', 'SQLite specific test.')
 @skipUnlessDBFeature('can_share_in_memory_db')
 class TestSqliteThreadSharing(TransactionTestCase):
