@@ -3,10 +3,16 @@ from __future__ import unicode_literals
 from operator import attrgetter
 
 from django.db import connection
-from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature
-from django.test import override_settings
+from django.db.models import Value
+from django.db.models.functions import Lower
+from django.test import (
+    TestCase, override_settings, skipIfDBFeature, skipUnlessDBFeature,
+)
 
-from .models import Country, Restaurant, Pizzeria, State, TwoFields
+from .models import (
+    Country, NoFields, Pizzeria, ProxyCountry, ProxyMultiCountry,
+    ProxyMultiProxyCountry, ProxyProxyCountry, Restaurant, State, TwoFields,
+)
 
 
 class BulkCreateTests(TestCase):
@@ -34,21 +40,36 @@ class BulkCreateTests(TestCase):
         with self.assertNumQueries(1):
             Country.objects.bulk_create(self.data)
 
-    def test_inheritance(self):
-        Restaurant.objects.bulk_create([
-            Restaurant(name="Nicholas's")
-        ])
-        self.assertQuerysetEqual(Restaurant.objects.all(), [
-            "Nicholas's",
-        ], attrgetter("name"))
-        with self.assertRaises(ValueError):
+    def test_multi_table_inheritance_unsupported(self):
+        expected_message = "Can't bulk create a multi-table inherited model"
+        with self.assertRaisesMessage(ValueError, expected_message):
             Pizzeria.objects.bulk_create([
-                Pizzeria(name="The Art of Pizza")
+                Pizzeria(name="The Art of Pizza"),
             ])
-        self.assertQuerysetEqual(Pizzeria.objects.all(), [])
-        self.assertQuerysetEqual(Restaurant.objects.all(), [
-            "Nicholas's",
-        ], attrgetter("name"))
+        with self.assertRaisesMessage(ValueError, expected_message):
+            ProxyMultiCountry.objects.bulk_create([
+                ProxyMultiCountry(name="Fillory", iso_two_letter="FL"),
+            ])
+        with self.assertRaisesMessage(ValueError, expected_message):
+            ProxyMultiProxyCountry.objects.bulk_create([
+                ProxyMultiProxyCountry(name="Fillory", iso_two_letter="FL"),
+            ])
+
+    def test_proxy_inheritance_supported(self):
+        ProxyCountry.objects.bulk_create([
+            ProxyCountry(name="Qwghlm", iso_two_letter="QW"),
+            Country(name="Tortall", iso_two_letter="TA"),
+        ])
+        self.assertQuerysetEqual(ProxyCountry.objects.all(), {
+            "Qwghlm", "Tortall"
+        }, attrgetter("name"), ordered=False)
+
+        ProxyProxyCountry.objects.bulk_create([
+            ProxyProxyCountry(name="Netherlands", iso_two_letter="NT"),
+        ])
+        self.assertQuerysetEqual(ProxyProxyCountry.objects.all(), {
+            "Qwghlm", "Tortall", "Netherlands",
+        }, attrgetter("name"), ordered=False)
 
     def test_non_auto_increment_pk(self):
         State.objects.bulk_create([
@@ -91,7 +112,7 @@ class BulkCreateTests(TestCase):
 
     def test_large_batch(self):
         with override_settings(DEBUG=True):
-            connection.queries = []
+            connection.queries_log.clear()
             TwoFields.objects.bulk_create([
                 TwoFields(f1=i, f2=i + 1) for i in range(0, 1001)
             ])
@@ -112,11 +133,11 @@ class BulkCreateTests(TestCase):
     @skipUnlessDBFeature('has_bulk_insert')
     def test_large_batch_efficiency(self):
         with override_settings(DEBUG=True):
-            connection.queries = []
+            connection.queries_log.clear()
             TwoFields.objects.bulk_create([
                 TwoFields(f1=i, f2=i + 1) for i in range(0, 1001)
             ])
-            self.assertTrue(len(connection.queries) < 10)
+            self.assertLess(len(connection.queries), 10)
 
     def test_large_batch_mixed(self):
         """
@@ -124,7 +145,7 @@ class BulkCreateTests(TestCase):
         mixed together with objects without PK set.
         """
         with override_settings(DEBUG=True):
-            connection.queries = []
+            connection.queries_log.clear()
             TwoFields.objects.bulk_create([
                 TwoFields(id=i if i % 2 == 0 else None, f1=i, f2=i + 1)
                 for i in range(100000, 101000)])
@@ -142,19 +163,30 @@ class BulkCreateTests(TestCase):
         mixed together with objects without PK set.
         """
         with override_settings(DEBUG=True):
-            connection.queries = []
+            connection.queries_log.clear()
             TwoFields.objects.bulk_create([
                 TwoFields(id=i if i % 2 == 0 else None, f1=i, f2=i + 1)
                 for i in range(100000, 101000)])
-            self.assertTrue(len(connection.queries) < 10)
+            self.assertLess(len(connection.queries), 10)
 
     def test_explicit_batch_size(self):
         objs = [TwoFields(f1=i, f2=i) for i in range(0, 4)]
-        TwoFields.objects.bulk_create(objs, 2)
-        self.assertEqual(TwoFields.objects.count(), len(objs))
+        num_objs = len(objs)
+        TwoFields.objects.bulk_create(objs, batch_size=1)
+        self.assertEqual(TwoFields.objects.count(), num_objs)
         TwoFields.objects.all().delete()
-        TwoFields.objects.bulk_create(objs, len(objs))
-        self.assertEqual(TwoFields.objects.count(), len(objs))
+        TwoFields.objects.bulk_create(objs, batch_size=2)
+        self.assertEqual(TwoFields.objects.count(), num_objs)
+        TwoFields.objects.all().delete()
+        TwoFields.objects.bulk_create(objs, batch_size=3)
+        self.assertEqual(TwoFields.objects.count(), num_objs)
+        TwoFields.objects.all().delete()
+        TwoFields.objects.bulk_create(objs, batch_size=num_objs)
+        self.assertEqual(TwoFields.objects.count(), num_objs)
+
+    def test_empty_model(self):
+        NoFields.objects.bulk_create([NoFields() for i in range(2)])
+        self.assertEqual(NoFields.objects.count(), 2)
 
     @skipUnlessDBFeature('has_bulk_insert')
     def test_explicit_batch_size_efficiency(self):
@@ -164,3 +196,41 @@ class BulkCreateTests(TestCase):
         TwoFields.objects.all().delete()
         with self.assertNumQueries(1):
             TwoFields.objects.bulk_create(objs, len(objs))
+
+    @skipUnlessDBFeature('has_bulk_insert')
+    def test_bulk_insert_expressions(self):
+        Restaurant.objects.bulk_create([
+            Restaurant(name="Sam's Shake Shack"),
+            Restaurant(name=Lower(Value("Betty's Beetroot Bar")))
+        ])
+        bbb = Restaurant.objects.filter(name="betty's beetroot bar")
+        self.assertEqual(bbb.count(), 1)
+
+    @skipUnlessDBFeature('can_return_ids_from_bulk_insert')
+    def test_set_pk_and_insert_single_item(self):
+        countries = []
+        with self.assertNumQueries(1):
+            countries = Country.objects.bulk_create([self.data[0]])
+        self.assertEqual(len(countries), 1)
+        self.assertEqual(Country.objects.get(pk=countries[0].pk), countries[0])
+
+    @skipUnlessDBFeature('can_return_ids_from_bulk_insert')
+    def test_set_pk_and_query_efficiency(self):
+        countries = []
+        with self.assertNumQueries(1):
+            countries = Country.objects.bulk_create(self.data)
+        self.assertEqual(len(countries), 4)
+        self.assertEqual(Country.objects.get(pk=countries[0].pk), countries[0])
+        self.assertEqual(Country.objects.get(pk=countries[1].pk), countries[1])
+        self.assertEqual(Country.objects.get(pk=countries[2].pk), countries[2])
+        self.assertEqual(Country.objects.get(pk=countries[3].pk), countries[3])
+
+    @skipUnlessDBFeature('can_return_ids_from_bulk_insert')
+    def test_set_state(self):
+        country_nl = Country(name='Netherlands', iso_two_letter='NL')
+        country_be = Country(name='Belgium', iso_two_letter='BE')
+        Country.objects.bulk_create([country_nl])
+        country_be.save()
+        # Objects save via bulk_create() and save() should have equal state.
+        self.assertEqual(country_nl._state.adding, country_be._state.adding)
+        self.assertEqual(country_nl._state.db, country_be._state.db)
