@@ -60,11 +60,16 @@ class C:
         return 24
 
 
-class Unpickable(object):
+class Unpicklable(object):
     def __getstate__(self):
         raise pickle.PickleError()
 
 
+class UnpicklableType(object):
+    # Unpicklable using the default pickling protocol on Python 2.
+    __slots__ = 'a',
+
+
 @override_settings(CACHES={
     'default': {
         'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
@@ -124,14 +129,18 @@ class DummyCacheTests(SimpleTestCase):
     def test_incr(self):
         "Dummy cache values can't be incremented"
         cache.set('answer', 42)
-        self.assertRaises(ValueError, cache.incr, 'answer')
-        self.assertRaises(ValueError, cache.incr, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.incr('answer')
+        with self.assertRaises(ValueError):
+            cache.incr('does_not_exist')
 
     def test_decr(self):
         "Dummy cache values can't be decremented"
         cache.set('answer', 42)
-        self.assertRaises(ValueError, cache.decr, 'answer')
-        self.assertRaises(ValueError, cache.decr, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.decr('answer')
+        with self.assertRaises(ValueError):
+            cache.decr('does_not_exist')
 
     def test_data_types(self):
         "All data types are ignored equally by the dummy cache"
@@ -188,14 +197,18 @@ class DummyCacheTests(SimpleTestCase):
     def test_incr_version(self):
         "Dummy cache versions can't be incremented"
         cache.set('answer', 42)
-        self.assertRaises(ValueError, cache.incr_version, 'answer')
-        self.assertRaises(ValueError, cache.incr_version, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.incr_version('answer')
+        with self.assertRaises(ValueError):
+            cache.incr_version('does_not_exist')
 
     def test_decr_version(self):
         "Dummy cache versions can't be decremented"
         cache.set('answer', 42)
-        self.assertRaises(ValueError, cache.decr_version, 'answer')
-        self.assertRaises(ValueError, cache.decr_version, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.decr_version('answer')
+        with self.assertRaises(ValueError):
+            cache.decr_version('does_not_exist')
 
     def test_get_or_set(self):
         self.assertEqual(cache.get_or_set('mykey', 'default'), 'default')
@@ -223,14 +236,17 @@ _caches_setting_base = {
 }
 
 
-def caches_setting_for_tests(base=None, **params):
+def caches_setting_for_tests(base=None, exclude=None, **params):
     # `base` is used to pull in the memcached config from the original settings,
+    # `exclude` is a set of cache names denoting which `_caches_setting_base` keys
+    # should be omitted.
     # `params` are test specific overrides and `_caches_settings_base` is the
     # base config for the tests.
     # This results in the following search order:
     # params -> _caches_setting_base -> base
     base = base or {}
-    setting = {k: base.copy() for k in _caches_setting_base.keys()}
+    exclude = exclude or set()
+    setting = {k: base.copy() for k in _caches_setting_base.keys() if k not in exclude}
     for key, cache_params in setting.items():
         cache_params.update(_caches_setting_base[key])
         cache_params.update(params)
@@ -316,7 +332,8 @@ class BaseCacheTests(object):
         self.assertEqual(cache.incr('answer', 10), 52)
         self.assertEqual(cache.get('answer'), 52)
         self.assertEqual(cache.incr('answer', -10), 42)
-        self.assertRaises(ValueError, cache.incr, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.incr('does_not_exist')
 
     def test_decr(self):
         # Cache values can be decremented
@@ -326,7 +343,8 @@ class BaseCacheTests(object):
         self.assertEqual(cache.decr('answer', 10), 32)
         self.assertEqual(cache.get('answer'), 32)
         self.assertEqual(cache.decr('answer', -10), 42)
-        self.assertRaises(ValueError, cache.decr, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.decr('does_not_exist')
 
     def test_close(self):
         self.assertTrue(hasattr(cache, 'close'))
@@ -507,7 +525,7 @@ class BaseCacheTests(object):
         cache.add('key2', 'ham', None)
         self.assertEqual(cache.get('key2'), 'ham')
         added = cache.add('key1', 'new eggs', None)
-        self.assertEqual(added, False)
+        self.assertIs(added, False)
         self.assertEqual(cache.get('key1'), 'eggs')
 
         cache.set_many({'key3': 'sausage', 'key4': 'lobster bisque'}, None)
@@ -551,7 +569,7 @@ class BaseCacheTests(object):
     def test_zero_cull(self):
         self._perform_cull_test(caches['zero_cull'], 50, 19)
 
-    def test_invalid_keys(self):
+    def _perform_invalid_key_test(self, key, expected_warning):
         """
         All the builtin backends (except memcached, see below) should warn on
         keys that would be refused by memcached. This encourages portable
@@ -569,19 +587,31 @@ class BaseCacheTests(object):
         try:
             with warnings.catch_warnings(record=True) as w:
                 warnings.simplefilter("always")
-                # memcached does not allow whitespace or control characters in keys
-                cache.set('key with spaces', 'value')
-                self.assertEqual(len(w), 2)
-                self.assertIsInstance(w[0].message, CacheKeyWarning)
-            with warnings.catch_warnings(record=True) as w:
-                warnings.simplefilter("always")
-                # memcached limits key length to 250
-                cache.set('a' * 251, 'value')
+                cache.set(key, 'value')
                 self.assertEqual(len(w), 1)
                 self.assertIsInstance(w[0].message, CacheKeyWarning)
+                self.assertEqual(str(w[0].message.args[0]), expected_warning)
         finally:
             cache.key_func = old_func
 
+    def test_invalid_key_characters(self):
+        # memcached doesn't allow whitespace or control characters in keys.
+        key = 'key with spaces and 清'
+        expected_warning = (
+            "Cache key contains characters that will cause errors if used "
+            "with memcached: %r" % key
+        )
+        self._perform_invalid_key_test(key, expected_warning)
+
+    def test_invalid_key_length(self):
+        # memcached limits key length to 250.
+        key = ('a' * 250) + '清'
+        expected_warning = (
+            'Cache key will cause errors if used with memcached: '
+            '%r (longer than %s)' % (key, 250)
+        )
+        self._perform_invalid_key_test(key, expected_warning)
+
     def test_cache_versioning_get_set(self):
         # set, using default version = 1
         cache.set('answer1', 42)
@@ -741,54 +771,42 @@ class BaseCacheTests(object):
     def test_cache_versioning_get_set_many(self):
         # set, using default version = 1
         cache.set_many({'ford1': 37, 'arthur1': 42})
-        self.assertDictEqual(cache.get_many(['ford1', 'arthur1']),
-                         {'ford1': 37, 'arthur1': 42})
-        self.assertDictEqual(cache.get_many(['ford1', 'arthur1'], version=1),
-                         {'ford1': 37, 'arthur1': 42})
+        self.assertDictEqual(cache.get_many(['ford1', 'arthur1']), {'ford1': 37, 'arthur1': 42})
+        self.assertDictEqual(cache.get_many(['ford1', 'arthur1'], version=1), {'ford1': 37, 'arthur1': 42})
         self.assertDictEqual(cache.get_many(['ford1', 'arthur1'], version=2), {})
 
         self.assertDictEqual(caches['v2'].get_many(['ford1', 'arthur1']), {})
-        self.assertDictEqual(caches['v2'].get_many(['ford1', 'arthur1'], version=1),
-                         {'ford1': 37, 'arthur1': 42})
+        self.assertDictEqual(caches['v2'].get_many(['ford1', 'arthur1'], version=1), {'ford1': 37, 'arthur1': 42})
         self.assertDictEqual(caches['v2'].get_many(['ford1', 'arthur1'], version=2), {})
 
         # set, default version = 1, but manually override version = 2
         cache.set_many({'ford2': 37, 'arthur2': 42}, version=2)
         self.assertDictEqual(cache.get_many(['ford2', 'arthur2']), {})
         self.assertDictEqual(cache.get_many(['ford2', 'arthur2'], version=1), {})
-        self.assertDictEqual(cache.get_many(['ford2', 'arthur2'], version=2),
-                         {'ford2': 37, 'arthur2': 42})
+        self.assertDictEqual(cache.get_many(['ford2', 'arthur2'], version=2), {'ford2': 37, 'arthur2': 42})
 
-        self.assertDictEqual(caches['v2'].get_many(['ford2', 'arthur2']),
-                         {'ford2': 37, 'arthur2': 42})
+        self.assertDictEqual(caches['v2'].get_many(['ford2', 'arthur2']), {'ford2': 37, 'arthur2': 42})
         self.assertDictEqual(caches['v2'].get_many(['ford2', 'arthur2'], version=1), {})
-        self.assertDictEqual(caches['v2'].get_many(['ford2', 'arthur2'], version=2),
-                         {'ford2': 37, 'arthur2': 42})
+        self.assertDictEqual(caches['v2'].get_many(['ford2', 'arthur2'], version=2), {'ford2': 37, 'arthur2': 42})
 
         # v2 set, using default version = 2
         caches['v2'].set_many({'ford3': 37, 'arthur3': 42})
         self.assertDictEqual(cache.get_many(['ford3', 'arthur3']), {})
         self.assertDictEqual(cache.get_many(['ford3', 'arthur3'], version=1), {})
-        self.assertDictEqual(cache.get_many(['ford3', 'arthur3'], version=2),
-                         {'ford3': 37, 'arthur3': 42})
+        self.assertDictEqual(cache.get_many(['ford3', 'arthur3'], version=2), {'ford3': 37, 'arthur3': 42})
 
-        self.assertDictEqual(caches['v2'].get_many(['ford3', 'arthur3']),
-                         {'ford3': 37, 'arthur3': 42})
+        self.assertDictEqual(caches['v2'].get_many(['ford3', 'arthur3']), {'ford3': 37, 'arthur3': 42})
         self.assertDictEqual(caches['v2'].get_many(['ford3', 'arthur3'], version=1), {})
-        self.assertDictEqual(caches['v2'].get_many(['ford3', 'arthur3'], version=2),
-                         {'ford3': 37, 'arthur3': 42})
+        self.assertDictEqual(caches['v2'].get_many(['ford3', 'arthur3'], version=2), {'ford3': 37, 'arthur3': 42})
 
         # v2 set, default version = 2, but manually override version = 1
         caches['v2'].set_many({'ford4': 37, 'arthur4': 42}, version=1)
-        self.assertDictEqual(cache.get_many(['ford4', 'arthur4']),
-                         {'ford4': 37, 'arthur4': 42})
-        self.assertDictEqual(cache.get_many(['ford4', 'arthur4'], version=1),
-                         {'ford4': 37, 'arthur4': 42})
+        self.assertDictEqual(cache.get_many(['ford4', 'arthur4']), {'ford4': 37, 'arthur4': 42})
+        self.assertDictEqual(cache.get_many(['ford4', 'arthur4'], version=1), {'ford4': 37, 'arthur4': 42})
         self.assertDictEqual(cache.get_many(['ford4', 'arthur4'], version=2), {})
 
         self.assertDictEqual(caches['v2'].get_many(['ford4', 'arthur4']), {})
-        self.assertDictEqual(caches['v2'].get_many(['ford4', 'arthur4'], version=1),
-                         {'ford4': 37, 'arthur4': 42})
+        self.assertDictEqual(caches['v2'].get_many(['ford4', 'arthur4'], version=1), {'ford4': 37, 'arthur4': 42})
         self.assertDictEqual(caches['v2'].get_many(['ford4', 'arthur4'], version=2), {})
 
     def test_incr_version(self):
@@ -816,7 +834,8 @@ class BaseCacheTests(object):
         self.assertIsNone(caches['v2'].get('answer2', version=2))
         self.assertEqual(caches['v2'].get('answer2', version=3), 42)
 
-        self.assertRaises(ValueError, cache.incr_version, 'does_not_exist')
+        with self.assertRaises(ValueError):
+            cache.incr_version('does_not_exist')
 
     def test_decr_version(self):
         cache.set('answer', 42, version=2)
@@ -839,7 +858,8 @@ class BaseCacheTests(object):
         self.assertEqual(caches['v2'].get('answer2', version=1), 42)
         self.assertIsNone(caches['v2'].get('answer2', version=2))
 
-        self.assertRaises(ValueError, cache.decr_version, 'does_not_exist', version=2)
+        with self.assertRaises(ValueError):
+            cache.decr_version('does_not_exist', version=2)
 
     def test_custom_key_func(self):
         # Two caches with different key functions aren't visible to each other
@@ -853,7 +873,7 @@ class BaseCacheTests(object):
         self.assertEqual(caches['custom_key'].get('answer2'), 42)
         self.assertEqual(caches['custom_key2'].get('answer2'), 42)
 
-    def test_cache_write_unpickable_object(self):
+    def test_cache_write_unpicklable_object(self):
         update_middleware = UpdateCacheMiddleware()
         update_middleware.cache = cache
 
@@ -884,14 +904,13 @@ class BaseCacheTests(object):
         self.assertEqual(get_cache_data.cookies, response.cookies)
 
     def test_add_fail_on_pickleerror(self):
-        "See https://code.djangoproject.com/ticket/21200"
+        # Shouldn't fail silently if trying to cache an unpicklable type.
         with self.assertRaises(pickle.PickleError):
-            cache.add('unpickable', Unpickable())
+            cache.add('unpicklable', Unpicklable())
 
     def test_set_fail_on_pickleerror(self):
-        "See https://code.djangoproject.com/ticket/21200"
         with self.assertRaises(pickle.PickleError):
-            cache.set('unpickable', Unpickable())
+            cache.set('unpicklable', Unpicklable())
 
     def test_get_or_set(self):
         self.assertIsNone(cache.get('projector'))
@@ -956,8 +975,7 @@ class DBCacheTests(BaseCacheTests, TransactionTestCase):
     def test_second_call_doesnt_crash(self):
         out = six.StringIO()
         management.call_command('createcachetable', stdout=out)
-        self.assertEqual(out.getvalue(),
-            "Cache table 'test cache table' already exists.\n" * len(settings.CACHES))
+        self.assertEqual(out.getvalue(), "Cache table 'test cache table' already exists.\n" * len(settings.CACHES))
 
     @override_settings(CACHES=caches_setting_for_tests(
         BACKEND='django.core.cache.backends.db.DatabaseCache',
@@ -983,8 +1001,7 @@ class DBCacheTests(BaseCacheTests, TransactionTestCase):
             verbosity=2,
             stdout=out,
         )
-        self.assertEqual(out.getvalue(),
-            "Cache table 'test cache table' created.\n")
+        self.assertEqual(out.getvalue(), "Cache table 'test cache table' created.\n")
 
 
 @override_settings(USE_TZ=True)
@@ -1115,23 +1132,20 @@ class LocMemCacheTests(BaseCacheTests, TestCase):
 # To check the memcached backend, the test settings file will
 # need to contain at least one cache backend setting that points at
 # your memcache server.
-memcached_params = {}
+configured_caches = {}
 for _cache_params in settings.CACHES.values():
-    if _cache_params['BACKEND'].startswith('django.core.cache.backends.memcached.'):
-        memcached_params = _cache_params
+    configured_caches[_cache_params['BACKEND']] = _cache_params
 
-memcached_never_expiring_params = memcached_params.copy()
-memcached_never_expiring_params['TIMEOUT'] = None
+MemcachedCache_params = configured_caches.get('django.core.cache.backends.memcached.MemcachedCache')
+PyLibMCCache_params = configured_caches.get('django.core.cache.backends.memcached.PyLibMCCache')
 
-memcached_far_future_params = memcached_params.copy()
-memcached_far_future_params['TIMEOUT'] = 31536000  # 60*60*24*365, 1 year
+# The memcached backends don't support cull-related options like `MAX_ENTRIES`.
+memcached_excluded_caches = {'cull', 'zero_cull'}
 
 
-@unittest.skipUnless(memcached_params, "memcached not available")
-@override_settings(CACHES=caches_setting_for_tests(base=memcached_params))
-class MemcachedCacheTests(BaseCacheTests, TestCase):
+class BaseMemcachedTests(BaseCacheTests):
 
-    def test_invalid_keys(self):
+    def test_invalid_key_characters(self):
         """
         On memcached, we don't introduce a duplicate key validation
         step (for speed reasons), we just let the memcached API
@@ -1141,32 +1155,33 @@ class MemcachedCacheTests(BaseCacheTests, TestCase):
         that a generic exception of some kind is raised.
         """
         # memcached does not allow whitespace or control characters in keys
-        self.assertRaises(Exception, cache.set, 'key with spaces', 'value')
+        # when using the ascii protocol.
+        with self.assertRaises(Exception):
+            cache.set('key with spaces', 'value')
+
+    def test_invalid_key_length(self):
         # memcached limits key length to 250
-        self.assertRaises(Exception, cache.set, 'a' * 251, 'value')
+        with self.assertRaises(Exception):
+            cache.set('a' * 251, 'value')
 
-    # Explicitly display a skipped test if no configured cache uses MemcachedCache
-    @unittest.skipUnless(
-        memcached_params.get('BACKEND') == 'django.core.cache.backends.memcached.MemcachedCache',
-        "cache with python-memcached library not available")
-    def test_memcached_uses_highest_pickle_version(self):
-        # Regression test for #19810
-        for cache_key, cache_config in settings.CACHES.items():
-            if cache_config['BACKEND'] == 'django.core.cache.backends.memcached.MemcachedCache':
-                self.assertEqual(caches[cache_key]._cache.pickleProtocol,
-                                 pickle.HIGHEST_PROTOCOL)
-
-    @override_settings(CACHES=caches_setting_for_tests(base=memcached_never_expiring_params))
     def test_default_never_expiring_timeout(self):
         # Regression test for #22845
-        cache.set('infinite_foo', 'bar')
-        self.assertEqual(cache.get('infinite_foo'), 'bar')
+        with self.settings(CACHES=caches_setting_for_tests(
+                base=self.base_params,
+                exclude=memcached_excluded_caches,
+                TIMEOUT=None)):
+            cache.set('infinite_foo', 'bar')
+            self.assertEqual(cache.get('infinite_foo'), 'bar')
 
-    @override_settings(CACHES=caches_setting_for_tests(base=memcached_far_future_params))
     def test_default_far_future_timeout(self):
         # Regression test for #22845
-        cache.set('future_foo', 'bar')
-        self.assertEqual(cache.get('future_foo'), 'bar')
+        with self.settings(CACHES=caches_setting_for_tests(
+                base=self.base_params,
+                exclude=memcached_excluded_caches,
+                # 60*60*24*365, 1 year
+                TIMEOUT=31536000)):
+            cache.set('future_foo', 'bar')
+            self.assertEqual(cache.get('future_foo'), 'bar')
 
     def test_cull(self):
         # culling isn't implemented, memcached deals with it.
@@ -1189,12 +1204,52 @@ class MemcachedCacheTests(BaseCacheTests, TestCase):
         self.assertEqual(cache.get('small_value'), 'a')
 
         large_value = 'a' * (max_value_length + 1)
-        cache.set('small_value', large_value)
+        try:
+            cache.set('small_value', large_value)
+        except Exception:
+            # Some clients (e.g. pylibmc) raise when the value is too large,
+            # while others (e.g. python-memcached) intentionally return True
+            # indicating success. This test is primarily checking that the key
+            # was deleted, so the return/exception behavior for the set()
+            # itself is not important.
+            pass
         # small_value should be deleted, or set if configured to accept larger values
         value = cache.get('small_value')
         self.assertTrue(value is None or value == large_value)
 
 
+@unittest.skipUnless(MemcachedCache_params, "MemcachedCache backend not configured")
+@override_settings(CACHES=caches_setting_for_tests(
+    base=MemcachedCache_params,
+    exclude=memcached_excluded_caches,
+))
+class MemcachedCacheTests(BaseMemcachedTests, TestCase):
+    base_params = MemcachedCache_params
+
+    def test_memcached_uses_highest_pickle_version(self):
+        # Regression test for #19810
+        for cache_key, cache_config in settings.CACHES.items():
+            self.assertEqual(caches[cache_key]._cache.pickleProtocol, pickle.HIGHEST_PROTOCOL)
+
+
+@unittest.skipUnless(PyLibMCCache_params, "PyLibMCCache backend not configured")
+@override_settings(CACHES=caches_setting_for_tests(
+    base=PyLibMCCache_params,
+    exclude=memcached_excluded_caches,
+))
+class PyLibMCCacheTests(BaseMemcachedTests, TestCase):
+    base_params = PyLibMCCache_params
+
+    # By default, pylibmc/libmemcached don't verify keys client-side and so
+    # this test triggers a server-side bug that causes later tests to fail
+    # (#19914). The `verify_keys` behavior option could be set to True (which
+    # would avoid triggering the server-side bug), however this test would
+    # still fail due to https://github.com/lericson/pylibmc/issues/219.
+    @unittest.skip("triggers a memcached-server bug, causing subsequent tests to fail")
+    def test_invalid_key_characters(self):
+        pass
+
+
 @override_settings(CACHES=caches_setting_for_tests(
     BACKEND='django.core.cache.backends.filebased.FileBasedCache',
 ))
@@ -1237,6 +1292,10 @@ class FileBasedCacheTests(BaseCacheTests, TestCase):
         cache.set('foo', 'bar')
         os.path.exists(self.dirname)
 
+    def test_cache_write_unpicklable_type(self):
+        # This fails if not using the highest pickling protocol on Python 2.
+        cache.set('unpicklable', UnpicklableType())
+
 
 @override_settings(CACHES={
     'default': {
@@ -1464,6 +1523,7 @@ class CacheUtils(SimpleTestCase):
         tests = (
             # Initial Cache-Control, kwargs to patch_cache_control, expected Cache-Control parts
             (None, {'private': True}, {'private'}),
+            ('', {'private': True}, {'private'}),
 
             # Test whether private/public attributes are mutually exclusive
             ('private', {'private': True}, {'private'}),
@@ -1693,13 +1753,17 @@ class CacheI18nTest(TestCase):
         with timezone.override(CustomTzName()):
             CustomTzName.name = 'Hora estándar de Argentina'.encode('UTF-8')  # UTF-8 string
             sanitized_name = 'Hora_estndar_de_Argentina'
-            self.assertIn(sanitized_name, learn_cache_key(request, response),
-                    "Cache keys should include the time zone name when time zones are active")
+            self.assertIn(
+                sanitized_name, learn_cache_key(request, response),
+                "Cache keys should include the time zone name when time zones are active"
+            )
 
             CustomTzName.name = 'Hora estándar de Argentina'    # unicode
             sanitized_name = 'Hora_estndar_de_Argentina'
-            self.assertIn(sanitized_name, learn_cache_key(request, response),
-                    "Cache keys should include the time zone name when time zones are active")
+            self.assertIn(
+                sanitized_name, learn_cache_key(request, response),
+                "Cache keys should include the time zone name when time zones are active"
+            )
 
     @override_settings(
         CACHE_MIDDLEWARE_KEY_PREFIX="test",
@@ -2014,7 +2078,7 @@ class TestWithTemplateResponse(SimpleTestCase):
     Tests various headers w/ TemplateResponse.
 
     Most are probably redundant since they manipulate the same object
-    anyway but the Etag header is 'special' because it relies on the
+    anyway but the ETag header is 'special' because it relies on the
     content being complete (which is not necessarily always the case
     with a TemplateResponse)
     """
@@ -2112,18 +2176,15 @@ class TestMakeTemplateFragmentKey(SimpleTestCase):
 
     def test_with_one_vary_on(self):
         key = make_template_fragment_key('foo', ['abc'])
-        self.assertEqual(key,
-            'template.cache.foo.900150983cd24fb0d6963f7d28e17f72')
+        self.assertEqual(key, 'template.cache.foo.900150983cd24fb0d6963f7d28e17f72')
 
     def test_with_many_vary_on(self):
         key = make_template_fragment_key('bar', ['abc', 'def'])
-        self.assertEqual(key,
-            'template.cache.bar.4b35f12ab03cec09beec4c21b2d2fa88')
+        self.assertEqual(key, 'template.cache.bar.4b35f12ab03cec09beec4c21b2d2fa88')
 
     def test_proper_escaping(self):
         key = make_template_fragment_key('spam', ['abc:def%'])
-        self.assertEqual(key,
-            'template.cache.spam.f27688177baec990cdf3fbd9d9c3f469')
+        self.assertEqual(key, 'template.cache.spam.f27688177baec990cdf3fbd9d9c3f469')
 
 
 class CacheHandlerTest(SimpleTestCase):
