@@ -3,7 +3,7 @@ from __future__ import unicode_literals
 
 from datetime import datetime
 
-from django.test import TestCase, override_settings
+from django.test import SimpleTestCase, override_settings
 
 FULL_RESPONSE = 'Test conditional get response'
 LAST_MODIFIED = datetime(2007, 10, 21, 23, 21, 47)
@@ -16,7 +16,7 @@ EXPIRED_ETAG = '7fae4cd4b0f81e7d2914700043aa8ed6'
 
 
 @override_settings(ROOT_URLCONF='conditional_processing.urls')
-class ConditionalGet(TestCase):
+class ConditionalGet(SimpleTestCase):
 
     def assertFullResponse(self, response, check_last_modified=True, check_etag=True):
         self.assertEqual(response.status_code, 200)
