@@ -1,27 +1,41 @@
 from __future__ import unicode_literals
 
-from datetime import date
+import contextlib
+import time
 import unittest
+from datetime import date, datetime
 
 from django.core.exceptions import FieldError
-from django.db import models
-from django.db import connection
-from django.test import TestCase
-from .models import Author
+from django.db import connection, models
+from django.test import TestCase, override_settings
+from django.utils import timezone
+
+from .models import Author, MySQLUnixTimestamp
+
+
+@contextlib.contextmanager
+def register_lookup(field, *lookups):
+    try:
+        for lookup in lookups:
+            field.register_lookup(lookup)
+        yield
+    finally:
+        for lookup in lookups:
+            field._unregister_lookup(lookup)
 
 
 class Div3Lookup(models.Lookup):
     lookup_name = 'div3'
 
-    def as_sql(self, qn, connection):
-        lhs, params = self.process_lhs(qn, connection)
-        rhs, rhs_params = self.process_rhs(qn, connection)
+    def as_sql(self, compiler, connection):
+        lhs, params = self.process_lhs(compiler, connection)
+        rhs, rhs_params = self.process_rhs(compiler, connection)
         params.extend(rhs_params)
-        return '%s %%%% 3 = %s' % (lhs, rhs), params
+        return '(%s) %%%% 3 = %s' % (lhs, rhs), params
 
-    def as_oracle(self, qn, connection):
-        lhs, params = self.process_lhs(qn, connection)
-        rhs, rhs_params = self.process_rhs(qn, connection)
+    def as_oracle(self, compiler, connection):
+        lhs, params = self.process_lhs(compiler, connection)
+        rhs, rhs_params = self.process_rhs(compiler, connection)
         params.extend(rhs_params)
         return 'mod(%s, 3) = %s' % (lhs, rhs), params
 
@@ -29,20 +43,43 @@ class Div3Lookup(models.Lookup):
 class Div3Transform(models.Transform):
     lookup_name = 'div3'
 
-    def as_sql(self, qn, connection):
-        lhs, lhs_params = qn.compile(self.lhs)
-        return '%s %%%% 3' % (lhs,), lhs_params
+    def as_sql(self, compiler, connection):
+        lhs, lhs_params = compiler.compile(self.lhs)
+        return '(%s) %%%% 3' % lhs, lhs_params
 
-    def as_oracle(self, qn, connection):
-        lhs, lhs_params = qn.compile(self.lhs)
+    def as_oracle(self, compiler, connection):
+        lhs, lhs_params = compiler.compile(self.lhs)
         return 'mod(%s, 3)' % lhs, lhs_params
 
 
-class YearTransform(models.Transform):
-    lookup_name = 'year'
+class Div3BilateralTransform(Div3Transform):
+    bilateral = True
 
-    def as_sql(self, qn, connection):
-        lhs_sql, params = qn.compile(self.lhs)
+
+class Mult3BilateralTransform(models.Transform):
+    bilateral = True
+    lookup_name = 'mult3'
+
+    def as_sql(self, compiler, connection):
+        lhs, lhs_params = compiler.compile(self.lhs)
+        return '3 * (%s)' % lhs, lhs_params
+
+
+class UpperBilateralTransform(models.Transform):
+    bilateral = True
+    lookup_name = 'upper'
+
+    def as_sql(self, compiler, connection):
+        lhs, lhs_params = compiler.compile(self.lhs)
+        return 'UPPER(%s)' % lhs, lhs_params
+
+
+class YearTransform(models.Transform):
+    # Use a name that avoids collision with the built-in year lookup.
+    lookup_name = 'testyear'
+
+    def as_sql(self, compiler, connection):
+        lhs_sql, params = compiler.compile(self.lhs)
         return connection.ops.date_extract_sql('year', lhs_sql), params
 
     @property
@@ -50,14 +87,15 @@ class YearTransform(models.Transform):
         return models.IntegerField()
 
 
+@YearTransform.register_lookup
 class YearExact(models.lookups.Lookup):
     lookup_name = 'exact'
 
-    def as_sql(self, qn, connection):
+    def as_sql(self, compiler, connection):
         # We will need to skip the extract part, and instead go
         # directly with the originating field, that is self.lhs.lhs
-        lhs_sql, lhs_params = self.process_lhs(qn, connection, self.lhs.lhs)
-        rhs_sql, rhs_params = self.process_rhs(qn, connection)
+        lhs_sql, lhs_params = self.process_lhs(compiler, connection, self.lhs.lhs)
+        rhs_sql, rhs_params = self.process_rhs(compiler, connection)
         # Note that we must be careful so that we have params in the
         # same order as we have the parts in the SQL.
         params = lhs_params + rhs_params + lhs_params + rhs_params
@@ -66,35 +104,40 @@ class YearExact(models.lookups.Lookup):
         return ("%(lhs)s >= (%(rhs)s || '-01-01')::date "
                 "AND %(lhs)s <= (%(rhs)s || '-12-31')::date" %
                 {'lhs': lhs_sql, 'rhs': rhs_sql}, params)
-YearTransform.register_lookup(YearExact)
 
 
+@YearTransform.register_lookup
 class YearLte(models.lookups.LessThanOrEqual):
     """
     The purpose of this lookup is to efficiently compare the year of the field.
     """
 
-    def as_sql(self, qn, connection):
+    def as_sql(self, compiler, connection):
         # Skip the YearTransform above us (no possibility for efficient
         # lookup otherwise).
         real_lhs = self.lhs.lhs
-        lhs_sql, params = self.process_lhs(qn, connection, real_lhs)
-        rhs_sql, rhs_params = self.process_rhs(qn, connection)
+        lhs_sql, params = self.process_lhs(compiler, connection, real_lhs)
+        rhs_sql, rhs_params = self.process_rhs(compiler, connection)
         params.extend(rhs_params)
         # Build SQL where the integer year is concatenated with last month
         # and day, then convert that to date. (We try to have SQL like:
         #     WHERE somecol <= '2013-12-31')
         # but also make it work if the rhs_sql is field reference.
         return "%s <= (%s || '-12-31')::date" % (lhs_sql, rhs_sql), params
-YearTransform.register_lookup(YearLte)
 
 
-class SQLFunc(models.Lookup):
-    def __init__(self, name, *args, **kwargs):
-        super(SQLFunc, self).__init__(*args, **kwargs)
-        self.name = name
+class Exactly(models.lookups.Exact):
+    """
+    This lookup is used to test lookup registration.
+    """
+    lookup_name = 'exactly'
 
-    def as_sql(self, qn, connection):
+    def get_rhs_op(self, connection, rhs):
+        return connection.operators['exact'] % rhs
+
+
+class SQLFuncMixin(object):
+    def as_sql(self, compiler, connection):
         return '%s()', [self.name]
 
     @property
@@ -102,13 +145,28 @@ class SQLFunc(models.Lookup):
         return CustomField()
 
 
+class SQLFuncLookup(SQLFuncMixin, models.Lookup):
+    def __init__(self, name, *args, **kwargs):
+        super(SQLFuncLookup, self).__init__(*args, **kwargs)
+        self.name = name
+
+
+class SQLFuncTransform(SQLFuncMixin, models.Transform):
+    def __init__(self, name, *args, **kwargs):
+        super(SQLFuncTransform, self).__init__(*args, **kwargs)
+        self.name = name
+
+
 class SQLFuncFactory(object):
 
-    def __init__(self, name):
+    def __init__(self, key, name):
+        self.key = key
         self.name = name
 
     def __call__(self, *args, **kwargs):
-        return SQLFunc(self.name, *args, **kwargs)
+        if self.key == 'lookupfunc':
+            return SQLFuncLookup(self.name, *args, **kwargs)
+        return SQLFuncTransform(self.name, *args, **kwargs)
 
 
 class CustomField(models.TextField):
@@ -116,13 +174,13 @@ class CustomField(models.TextField):
     def get_lookup(self, lookup_name):
         if lookup_name.startswith('lookupfunc_'):
             key, name = lookup_name.split('_', 1)
-            return SQLFuncFactory(name)
+            return SQLFuncFactory(key, name)
         return super(CustomField, self).get_lookup(lookup_name)
 
     def get_transform(self, lookup_name):
         if lookup_name.startswith('transformfunc_'):
             key, name = lookup_name.split('_', 1)
-            return SQLFuncFactory(name)
+            return SQLFuncFactory(key, name)
         return super(CustomField, self).get_transform(lookup_name)
 
 
@@ -139,9 +197,9 @@ class InMonth(models.lookups.Lookup):
     """
     lookup_name = 'inmonth'
 
-    def as_sql(self, qn, connection):
-        lhs, lhs_params = self.process_lhs(qn, connection)
-        rhs, rhs_params = self.process_rhs(qn, connection)
+    def as_sql(self, compiler, connection):
+        lhs, lhs_params = self.process_lhs(compiler, connection)
+        rhs, rhs_params = self.process_rhs(compiler, connection)
         # We need to be careful so that we get the params in right
         # places.
         params = lhs_params + rhs_params + lhs_params + rhs_params
@@ -150,14 +208,46 @@ class InMonth(models.lookups.Lookup):
                 (lhs, rhs, lhs, rhs), params)
 
 
+class DateTimeTransform(models.Transform):
+    lookup_name = 'as_datetime'
+
+    @property
+    def output_field(self):
+        return models.DateTimeField()
+
+    def as_sql(self, compiler, connection):
+        lhs, params = compiler.compile(self.lhs)
+        return 'from_unixtime({})'.format(lhs), params
+
+
 class LookupTests(TestCase):
+
+    def test_custom_name_lookup(self):
+        a1 = Author.objects.create(name='a1', birthdate=date(1981, 2, 16))
+        Author.objects.create(name='a2', birthdate=date(2012, 2, 29))
+        custom_lookup_name = 'isactually'
+        custom_transform_name = 'justtheyear'
+        try:
+            models.DateField.register_lookup(YearTransform)
+            models.DateField.register_lookup(YearTransform, custom_transform_name)
+            YearTransform.register_lookup(Exactly)
+            YearTransform.register_lookup(Exactly, custom_lookup_name)
+            qs1 = Author.objects.filter(birthdate__testyear__exactly=1981)
+            qs2 = Author.objects.filter(birthdate__justtheyear__isactually=1981)
+            self.assertQuerysetEqual(qs1, [a1], lambda x: x)
+            self.assertQuerysetEqual(qs2, [a1], lambda x: x)
+        finally:
+            YearTransform._unregister_lookup(Exactly)
+            YearTransform._unregister_lookup(Exactly, custom_lookup_name)
+            models.DateField._unregister_lookup(YearTransform)
+            models.DateField._unregister_lookup(YearTransform, custom_transform_name)
+
     def test_basic_lookup(self):
         a1 = Author.objects.create(name='a1', age=1)
         a2 = Author.objects.create(name='a2', age=2)
         a3 = Author.objects.create(name='a3', age=3)
         a4 = Author.objects.create(name='a4', age=4)
-        models.IntegerField.register_lookup(Div3Lookup)
-        try:
+        with register_lookup(models.IntegerField, Div3Lookup):
             self.assertQuerysetEqual(
                 Author.objects.filter(age__div3=0),
                 [a3], lambda x: x
@@ -174,8 +264,6 @@ class LookupTests(TestCase):
                 Author.objects.filter(age__div3=3),
                 [], lambda x: x
             )
-        finally:
-            models.IntegerField._unregister_lookup(Div3Lookup)
 
     @unittest.skipUnless(connection.vendor == 'postgresql', "PostgreSQL specific SQL used")
     def test_birthdate_month(self):
@@ -183,8 +271,7 @@ class LookupTests(TestCase):
         a2 = Author.objects.create(name='a2', birthdate=date(2012, 2, 29))
         a3 = Author.objects.create(name='a3', birthdate=date(2012, 1, 31))
         a4 = Author.objects.create(name='a4', birthdate=date(2012, 3, 1))
-        models.DateField.register_lookup(InMonth)
-        try:
+        with register_lookup(models.DateField, InMonth):
             self.assertQuerysetEqual(
                 Author.objects.filter(birthdate__inmonth=date(2012, 1, 15)),
                 [a3], lambda x: x
@@ -205,12 +292,9 @@ class LookupTests(TestCase):
                 Author.objects.filter(birthdate__inmonth=date(2012, 4, 1)),
                 [], lambda x: x
             )
-        finally:
-            models.DateField._unregister_lookup(InMonth)
 
     def test_div3_extract(self):
-        models.IntegerField.register_lookup(Div3Transform)
-        try:
+        with register_lookup(models.IntegerField, Div3Transform):
             a1 = Author.objects.create(name='a1', age=1)
             a2 = Author.objects.create(name='a2', age=2)
             a3 = Author.objects.create(name='a3', age=3)
@@ -225,8 +309,119 @@ class LookupTests(TestCase):
             self.assertQuerysetEqual(
                 baseqs.filter(age__div3__in=[0, 2]),
                 [a2, a3], lambda x: x)
-        finally:
-            models.IntegerField._unregister_lookup(Div3Transform)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__in=[2, 4]),
+                [a2], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__gte=3),
+                [], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__range=(1, 2)),
+                [a1, a2, a4], lambda x: x)
+
+
+class BilateralTransformTests(TestCase):
+
+    def test_bilateral_upper(self):
+        with register_lookup(models.CharField, UpperBilateralTransform):
+            Author.objects.bulk_create([
+                Author(name='Doe'),
+                Author(name='doe'),
+                Author(name='Foo'),
+            ])
+            self.assertQuerysetEqual(
+                Author.objects.filter(name__upper='doe'),
+                ["<Author: Doe>", "<Author: doe>"], ordered=False)
+            self.assertQuerysetEqual(
+                Author.objects.filter(name__upper__contains='f'),
+                ["<Author: Foo>"], ordered=False)
+
+    def test_bilateral_inner_qs(self):
+        with register_lookup(models.CharField, UpperBilateralTransform):
+            with self.assertRaises(NotImplementedError):
+                Author.objects.filter(name__upper__in=Author.objects.values_list('name'))
+
+    def test_bilateral_multi_value(self):
+        with register_lookup(models.CharField, UpperBilateralTransform):
+            Author.objects.bulk_create([
+                Author(name='Foo'),
+                Author(name='Bar'),
+                Author(name='Ray'),
+            ])
+            self.assertQuerysetEqual(
+                Author.objects.filter(name__upper__in=['foo', 'bar', 'doe']).order_by('name'),
+                ['Bar', 'Foo'],
+                lambda a: a.name
+            )
+
+    def test_div3_bilateral_extract(self):
+        with register_lookup(models.IntegerField, Div3BilateralTransform):
+            a1 = Author.objects.create(name='a1', age=1)
+            a2 = Author.objects.create(name='a2', age=2)
+            a3 = Author.objects.create(name='a3', age=3)
+            a4 = Author.objects.create(name='a4', age=4)
+            baseqs = Author.objects.order_by('name')
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3=2),
+                [a2], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__lte=3),
+                [a3], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__in=[0, 2]),
+                [a2, a3], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__in=[2, 4]),
+                [a1, a2, a4], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__gte=3),
+                [a1, a2, a3, a4], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__range=(1, 2)),
+                [a1, a2, a4], lambda x: x)
+
+    def test_bilateral_order(self):
+        with register_lookup(models.IntegerField, Mult3BilateralTransform, Div3BilateralTransform):
+            a1 = Author.objects.create(name='a1', age=1)
+            a2 = Author.objects.create(name='a2', age=2)
+            a3 = Author.objects.create(name='a3', age=3)
+            a4 = Author.objects.create(name='a4', age=4)
+            baseqs = Author.objects.order_by('name')
+
+            self.assertQuerysetEqual(
+                baseqs.filter(age__mult3__div3=42),
+                # mult3__div3 always leads to 0
+                [a1, a2, a3, a4], lambda x: x)
+            self.assertQuerysetEqual(
+                baseqs.filter(age__div3__mult3=42),
+                [a3], lambda x: x)
+
+    def test_bilateral_fexpr(self):
+        with register_lookup(models.IntegerField, Mult3BilateralTransform):
+            a1 = Author.objects.create(name='a1', age=1, average_rating=3.2)
+            a2 = Author.objects.create(name='a2', age=2, average_rating=0.5)
+            a3 = Author.objects.create(name='a3', age=3, average_rating=1.5)
+            a4 = Author.objects.create(name='a4', age=4)
+            baseqs = Author.objects.order_by('name')
+            self.assertQuerysetEqual(
+                baseqs.filter(age__mult3=models.F('age')),
+                [a1, a2, a3, a4], lambda x: x)
+            self.assertQuerysetEqual(
+                # Same as age >= average_rating
+                baseqs.filter(age__mult3__gte=models.F('average_rating')),
+                [a2, a3], lambda x: x)
+
+
+@override_settings(USE_TZ=True)
+class DateTimeLookupTests(TestCase):
+    @unittest.skipUnless(connection.vendor == 'mysql', "MySQL specific SQL used")
+    def test_datetime_output_field(self):
+        with register_lookup(models.PositiveIntegerField, DateTimeTransform):
+            ut = MySQLUnixTimestamp.objects.create(timestamp=time.time())
+            y2k = timezone.make_aware(datetime(2000, 1, 1))
+            self.assertQuerysetEqual(
+                MySQLUnixTimestamp.objects.filter(timestamp__as_datetime__gt=y2k),
+                [ut], lambda x: x)
 
 
 class YearLteTests(TestCase):
@@ -244,19 +439,19 @@ class YearLteTests(TestCase):
     def test_year_lte(self):
         baseqs = Author.objects.order_by('name')
         self.assertQuerysetEqual(
-            baseqs.filter(birthdate__year__lte=2012),
+            baseqs.filter(birthdate__testyear__lte=2012),
             [self.a1, self.a2, self.a3, self.a4], lambda x: x)
         self.assertQuerysetEqual(
-            baseqs.filter(birthdate__year=2012),
+            baseqs.filter(birthdate__testyear=2012),
             [self.a2, self.a3, self.a4], lambda x: x)
 
-        self.assertNotIn('BETWEEN', str(baseqs.filter(birthdate__year=2012).query))
+        self.assertNotIn('BETWEEN', str(baseqs.filter(birthdate__testyear=2012).query))
         self.assertQuerysetEqual(
-            baseqs.filter(birthdate__year__lte=2011),
+            baseqs.filter(birthdate__testyear__lte=2011),
             [self.a1], lambda x: x)
         # The non-optimized version works, too.
         self.assertQuerysetEqual(
-            baseqs.filter(birthdate__year__lt=2012),
+            baseqs.filter(birthdate__testyear__lt=2012),
             [self.a1], lambda x: x)
 
     @unittest.skipUnless(connection.vendor == 'postgresql', "PostgreSQL specific SQL used")
@@ -269,10 +464,10 @@ class YearLteTests(TestCase):
         self.a4.save()
         baseqs = Author.objects.order_by('name')
         self.assertQuerysetEqual(
-            baseqs.filter(birthdate__year__lte=models.F('age')),
+            baseqs.filter(birthdate__testyear__lte=models.F('age')),
             [self.a3, self.a4], lambda x: x)
         self.assertQuerysetEqual(
-            baseqs.filter(birthdate__year__lt=models.F('age')),
+            baseqs.filter(birthdate__testyear__lt=models.F('age')),
             [self.a4], lambda x: x)
 
     def test_year_lte_sql(self):
@@ -281,24 +476,24 @@ class YearLteTests(TestCase):
         # error - not running YearLte SQL at all.
         baseqs = Author.objects.order_by('name')
         self.assertIn(
-            '<= (2011 || ', str(baseqs.filter(birthdate__year__lte=2011).query))
+            '<= (2011 || ', str(baseqs.filter(birthdate__testyear__lte=2011).query))
         self.assertIn(
-            '-12-31', str(baseqs.filter(birthdate__year__lte=2011).query))
+            '-12-31', str(baseqs.filter(birthdate__testyear__lte=2011).query))
 
     def test_postgres_year_exact(self):
         baseqs = Author.objects.order_by('name')
         self.assertIn(
-            '= (2011 || ', str(baseqs.filter(birthdate__year=2011).query))
+            '= (2011 || ', str(baseqs.filter(birthdate__testyear=2011).query))
         self.assertIn(
-            '-12-31', str(baseqs.filter(birthdate__year=2011).query))
+            '-12-31', str(baseqs.filter(birthdate__testyear=2011).query))
 
     def test_custom_implementation_year_exact(self):
         try:
             # Two ways to add a customized implementation for different backends:
             # First is MonkeyPatch of the class.
-            def as_custom_sql(self, qn, connection):
-                lhs_sql, lhs_params = self.process_lhs(qn, connection, self.lhs.lhs)
-                rhs_sql, rhs_params = self.process_rhs(qn, connection)
+            def as_custom_sql(self, compiler, connection):
+                lhs_sql, lhs_params = self.process_lhs(compiler, connection, self.lhs.lhs)
+                rhs_sql, rhs_params = self.process_rhs(compiler, connection)
                 params = lhs_params + rhs_params + lhs_params + rhs_params
                 return ("%(lhs)s >= str_to_date(concat(%(rhs)s, '-01-01'), '%%%%Y-%%%%m-%%%%d') "
                         "AND %(lhs)s <= str_to_date(concat(%(rhs)s, '-12-31'), '%%%%Y-%%%%m-%%%%d')" %
@@ -306,7 +501,7 @@ class YearLteTests(TestCase):
             setattr(YearExact, 'as_' + connection.vendor, as_custom_sql)
             self.assertIn(
                 'concat(',
-                str(Author.objects.filter(birthdate__year=2012).query))
+                str(Author.objects.filter(birthdate__testyear=2012).query))
         finally:
             delattr(YearExact, 'as_' + connection.vendor)
         try:
@@ -316,9 +511,9 @@ class YearLteTests(TestCase):
                 # This method should be named "as_mysql" for MySQL, "as_postgresql" for postgres
                 # and so on, but as we don't know which DB we are running on, we need to use
                 # setattr.
-                def as_custom_sql(self, qn, connection):
-                    lhs_sql, lhs_params = self.process_lhs(qn, connection, self.lhs.lhs)
-                    rhs_sql, rhs_params = self.process_rhs(qn, connection)
+                def as_custom_sql(self, compiler, connection):
+                    lhs_sql, lhs_params = self.process_lhs(compiler, connection, self.lhs.lhs)
+                    rhs_sql, rhs_params = self.process_rhs(compiler, connection)
                     params = lhs_params + rhs_params + lhs_params + rhs_params
                     return ("%(lhs)s >= str_to_date(CONCAT(%(rhs)s, '-01-01'), '%%%%Y-%%%%m-%%%%d') "
                             "AND %(lhs)s <= str_to_date(CONCAT(%(rhs)s, '-12-31'), '%%%%Y-%%%%m-%%%%d')" %
@@ -327,18 +522,19 @@ class YearLteTests(TestCase):
             YearTransform.register_lookup(CustomYearExact)
             self.assertIn(
                 'CONCAT(',
-                str(Author.objects.filter(birthdate__year=2012).query))
+                str(Author.objects.filter(birthdate__testyear=2012).query))
         finally:
             YearTransform._unregister_lookup(CustomYearExact)
             YearTransform.register_lookup(YearExact)
 
 
 class TrackCallsYearTransform(YearTransform):
-    lookup_name = 'year'
+    # Use a name that avoids collision with the built-in year lookup.
+    lookup_name = 'testyear'
     call_order = []
 
-    def as_sql(self, qn, connection):
-        lhs_sql, params = qn.compile(self.lhs)
+    def as_sql(self, compiler, connection):
+        lhs_sql, params = compiler.compile(self.lhs)
         return connection.ops.date_extract_sql('year', lhs_sql), params
 
     @property
@@ -356,33 +552,29 @@ class TrackCallsYearTransform(YearTransform):
 
 class LookupTransformCallOrderTests(TestCase):
     def test_call_order(self):
-        models.DateField.register_lookup(TrackCallsYearTransform)
-        try:
+        with register_lookup(models.DateField, TrackCallsYearTransform):
             # junk lookup - tries lookup, then transform, then fails
             with self.assertRaises(FieldError):
-                Author.objects.filter(birthdate__year__junk=2012)
+                Author.objects.filter(birthdate__testyear__junk=2012)
             self.assertEqual(TrackCallsYearTransform.call_order,
                              ['lookup', 'transform'])
             TrackCallsYearTransform.call_order = []
             # junk transform - tries transform only, then fails
             with self.assertRaises(FieldError):
-                Author.objects.filter(birthdate__year__junk__more_junk=2012)
+                Author.objects.filter(birthdate__testyear__junk__more_junk=2012)
             self.assertEqual(TrackCallsYearTransform.call_order,
                              ['transform'])
             TrackCallsYearTransform.call_order = []
             # Just getting the year (implied __exact) - lookup only
-            Author.objects.filter(birthdate__year=2012)
+            Author.objects.filter(birthdate__testyear=2012)
             self.assertEqual(TrackCallsYearTransform.call_order,
                              ['lookup'])
             TrackCallsYearTransform.call_order = []
             # Just getting the year (explicit __exact) - lookup only
-            Author.objects.filter(birthdate__year__exact=2012)
+            Author.objects.filter(birthdate__testyear__exact=2012)
             self.assertEqual(TrackCallsYearTransform.call_order,
                              ['lookup'])
 
-        finally:
-            models.DateField._unregister_lookup(TrackCallsYearTransform)
-
 
 class CustomisedMethodsTests(TestCase):
 
@@ -401,3 +593,15 @@ class CustomisedMethodsTests(TestCase):
     def test_overridden_get_transform_chain(self):
         q = CustomModel.objects.filter(field__transformfunc_banana__transformfunc_pear=3)
         self.assertIn('pear()', str(q.query))
+
+
+class SubqueryTransformTests(TestCase):
+    def test_subquery_usage(self):
+        with register_lookup(models.IntegerField, Div3Transform):
+            Author.objects.create(name='a1', age=1)
+            a2 = Author.objects.create(name='a2', age=2)
+            Author.objects.create(name='a3', age=3)
+            Author.objects.create(name='a4', age=4)
+            self.assertQuerysetEqual(
+                Author.objects.order_by('name').filter(id__in=Author.objects.filter(age__div3=2)),
+                [a2], lambda x: x)
