@@ -6,8 +6,9 @@ from django.utils import six
 
 from .models import (
     Book, Car, CustomManager, CustomQuerySet, DeconstructibleCustomManager,
-    FunPerson, OneToOneRestrictedModel, Person, PersonManager,
-    PublishedBookManager, RelatedModel, RestrictedModel,
+    FastCarAsBase, FastCarAsDefault, FunPerson, OneToOneRestrictedModel,
+    Person, PersonFromAbstract, PersonManager, PublishedBookManager,
+    RelatedModel, RestrictedModel,
 )
 
 
@@ -69,12 +70,12 @@ class CustomManagerTests(TestCase):
             manager = getattr(Person, manager_name)
             queryset = manager.filter()
             self.assertQuerysetEqual(queryset, ["Bugs Bunny"], six.text_type)
-            self.assertEqual(queryset._filter_CustomQuerySet, True)
+            self.assertIs(queryset._filter_CustomQuerySet, True)
 
             # Test that specialized querysets inherit from our custom queryset.
             queryset = manager.values_list('first_name', flat=True).filter()
             self.assertEqual(list(queryset), [six.text_type("Bugs")])
-            self.assertEqual(queryset._filter_CustomQuerySet, True)
+            self.assertIs(queryset._filter_CustomQuerySet, True)
 
             self.assertIsInstance(queryset.values(), CustomQuerySet)
             self.assertIsInstance(queryset.values().values(), CustomQuerySet)
@@ -101,7 +102,7 @@ class CustomManagerTests(TestCase):
         """
         queryset = Person.custom_queryset_custom_manager.filter()
         self.assertQuerysetEqual(queryset, ["Bugs Bunny"], six.text_type)
-        self.assertEqual(queryset._filter_CustomManager, True)
+        self.assertIs(queryset._filter_CustomManager, True)
 
     def test_related_manager(self):
         """
@@ -115,7 +116,8 @@ class CustomManagerTests(TestCase):
         The default manager, "objects", doesn't exist, because a custom one
         was provided.
         """
-        self.assertRaises(AttributeError, lambda: Book.objects)
+        with self.assertRaises(AttributeError):
+            Book.objects
 
     def test_filtering(self):
         """
@@ -516,6 +518,17 @@ class CustomManagerTests(TestCase):
         with self.assertRaisesMessage(ValueError, msg):
             mgr.deconstruct()
 
+    def test_abstract_model_with_custom_manager_name(self):
+        """
+        A custom manager may be defined on an abstract model.
+        It will be inherited by the abstract model's children.
+        """
+        PersonFromAbstract.abstract_persons.create(objects='Test')
+        self.assertQuerysetEqual(
+            PersonFromAbstract.abstract_persons.all(), ["Test"],
+            lambda c: c.objects,
+        )
+
 
 class TestCars(TestCase):
 
@@ -546,6 +559,34 @@ class TestCars(TestCase):
             ],
             lambda c: c.name
         )
+        # explicit default manager
+        self.assertQuerysetEqual(
+            FastCarAsDefault.cars.order_by("name"), [
+                "Corvette",
+                "Neon",
+            ],
+            lambda c: c.name
+        )
+        self.assertQuerysetEqual(
+            FastCarAsDefault._default_manager.all(), [
+                "Corvette",
+            ],
+            lambda c: c.name
+        )
+        # explicit base manager
+        self.assertQuerysetEqual(
+            FastCarAsBase.cars.order_by("name"), [
+                "Corvette",
+                "Neon",
+            ],
+            lambda c: c.name
+        )
+        self.assertQuerysetEqual(
+            FastCarAsBase._base_manager.all(), [
+                "Corvette",
+            ],
+            lambda c: c.name
+        )
 
 
 class CustomManagersRegressTestCase(TestCase):
@@ -590,3 +631,12 @@ class CustomManagersRegressTestCase(TestCase):
         obj = RelatedModel.objects.get(name="xyzzy")
         obj.delete()
         self.assertEqual(len(OneToOneRestrictedModel.plain_manager.all()), 0)
+
+    def test_queryset_with_custom_init(self):
+        """
+        BaseManager.get_queryset() should use kwargs rather than args to allow
+        custom kwargs (#24911).
+        """
+        qs_custom = Person.custom_init_queryset_manager.all()
+        qs_default = Person.objects.all()
+        self.assertQuerysetEqual(qs_custom, qs_default)
