@@ -2,9 +2,7 @@ from __future__ import unicode_literals
 
 import datetime
 
-from django.conf import settings
-from django.db import DEFAULT_DB_ALIAS, models, transaction
-from django.db.utils import ConnectionHandler
+from django.db import connection, models, transaction
 from django.test import TestCase, TransactionTestCase, skipUnlessDBFeature
 
 from .models import (
@@ -24,8 +22,7 @@ class DeleteLockingTest(TransactionTestCase):
 
     def setUp(self):
         # Create a second connection to the default database
-        new_connections = ConnectionHandler(settings.DATABASES)
-        self.conn2 = new_connections[DEFAULT_DB_ALIAS]
+        self.conn2 = connection.copy()
         self.conn2.set_autocommit(False)
 
     def tearDown(self):
@@ -63,7 +60,6 @@ class DeleteCascadeTests(TestCase):
         """
         Django cascades deletes through generic-related objects to their
         reverse relations.
-
         """
         person = Person.objects.create(name='Nelson Mandela')
         award = Award.objects.create(name='Nobel', content_object=person)
@@ -81,7 +77,6 @@ class DeleteCascadeTests(TestCase):
         some other model has an FK to that through model, deletion is cascaded
         from one of the participants in the M2M, to the through model, to its
         related model.
-
         """
         juan = Child.objects.create(name='Juan')
         paints = Toy.objects.create(name='Paints')
@@ -126,7 +121,6 @@ class DeleteCascadeTransactionTests(TransactionTestCase):
     def test_to_field(self):
         """
         Cascade deletion works with ForeignKey.to_field set to non-PK.
-
         """
         apple = Food.objects.create(name="apple")
         Eaten.objects.create(food=apple, meal="lunch")
@@ -156,7 +150,6 @@ class ProxyDeleteTest(TestCase):
     Tests on_delete behavior for proxy models.
 
     See #16128.
-
     """
     def create_image(self):
         """Return an Image referenced by both a FooImage and a FooFile."""
@@ -177,7 +170,6 @@ class ProxyDeleteTest(TestCase):
         """
         Deleting the *proxy* instance bubbles through to its non-proxy and
         *all* referring objects are deleted.
-
         """
         self.create_image()
 
@@ -195,7 +187,6 @@ class ProxyDeleteTest(TestCase):
         """
         Deleting a proxy-of-proxy instance should bubble through to its proxy
         and non-proxy parents, deleting *all* referring objects.
-
         """
         test_image = self.create_image()
 
@@ -221,7 +212,6 @@ class ProxyDeleteTest(TestCase):
         """
         Deleting an instance of a concrete model should also delete objects
         referencing its proxy subclass.
-
         """
         self.create_image()
 
@@ -244,7 +234,6 @@ class ProxyDeleteTest(TestCase):
         IntegrityError on databases unable to defer integrity checks).
 
         Refs #17918.
-
         """
         # Create an Image (proxy of File) and FooFileProxy (proxy of FooFile,
         # which has an FK to File)
