@@ -1279,3 +1279,17 @@ class CaseDocumentationExamples(TestCase):
             ),
             {'regular': 2, 'gold': 1, 'platinum': 3}
         )
+
+    def test_filter_example(self):
+        a_month_ago = date.today() - timedelta(days=30)
+        a_year_ago = date.today() - timedelta(days=365)
+        self.assertQuerysetEqual(
+            Client.objects.filter(
+                registered_on__lte=Case(
+                    When(account_type=Client.GOLD, then=a_month_ago),
+                    When(account_type=Client.PLATINUM, then=a_year_ago),
+                ),
+            ),
+            [('Jack Black', 'P')],
+            transform=attrgetter('name', 'account_type')
+        )
