@@ -1,12 +1,12 @@
 from __future__ import unicode_literals
 
-from collections import OrderedDict
 import datetime
+from collections import OrderedDict
 
 from django.contrib.auth.models import User
 from django.test import TestCase
 
-from .models import TestObject, Order, RevisionableModel
+from .models import Order, RevisionableModel, TestObject
 
 
 class ExtraRegressTests(TestCase):
@@ -46,15 +46,15 @@ class ExtraRegressTests(TestCase):
             }]
         )
 
-        self.assertQuerysetEqual(qs,
-            [('Second Revision', 'First Revision')],
+        self.assertQuerysetEqual(
+            qs, [('Second Revision', 'First Revision')],
             transform=lambda r: (r.title, r.base.title)
         )
 
         # Queryset to search for string in title:
         qs2 = RevisionableModel.objects.filter(title__contains="Revision")
-        self.assertQuerysetEqual(qs2,
-            [
+        self.assertQuerysetEqual(
+            qs2, [
                 ('First Revision', 'First Revision'),
                 ('Second Revision', 'First Revision'),
             ],
@@ -63,7 +63,8 @@ class ExtraRegressTests(TestCase):
         )
 
         # Following queryset should return the most recent revision:
-        self.assertQuerysetEqual(qs & qs2,
+        self.assertQuerysetEqual(
+            qs & qs2,
             [('Second Revision', 'First Revision')],
             transform=lambda r: (r.title, r.base.title),
             ordered=False
@@ -111,10 +112,8 @@ class ExtraRegressTests(TestCase):
         query as well.
         """
         self.assertEqual(
-            list(User.objects
-                .extra(select={"alpha": "%s"}, select_params=(-6,))
-                .filter(id=self.u.id)
-                .values_list('id', flat=True)),
+            list(User.objects.extra(select={"alpha": "%s"}, select_params=(-6,))
+                 .filter(id=self.u.id).values_list('id', flat=True)),
             [self.u.id]
         )
 
@@ -185,100 +184,174 @@ class ExtraRegressTests(TestCase):
         obj.save()
 
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values()),
-            [{'bar': 'second', 'third': 'third', 'second': 'second', 'whiz': 'third', 'foo': 'first', 'id': obj.pk, 'first': 'first'}]
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values()
+            ),
+            [{
+                'bar': 'second', 'third': 'third', 'second': 'second', 'whiz': 'third', 'foo': 'first',
+                'id': obj.pk, 'first': 'first'
+            }]
         )
 
         # Extra clauses after an empty values clause are still included
         self.assertEqual(
-            list(TestObject.objects.values().extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))),
-            [{'bar': 'second', 'third': 'third', 'second': 'second', 'whiz': 'third', 'foo': 'first', 'id': obj.pk, 'first': 'first'}]
+            list(
+                TestObject.objects
+                .values()
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+            ),
+            [{
+                'bar': 'second', 'third': 'third', 'second': 'second', 'whiz': 'third', 'foo': 'first',
+                'id': obj.pk, 'first': 'first'
+            }]
         )
 
         # Extra columns are ignored if not mentioned in the values() clause
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values('first', 'second')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values('first', 'second')
+            ),
             [{'second': 'second', 'first': 'first'}]
         )
 
         # Extra columns after a non-empty values() clause are ignored
         self.assertEqual(
-            list(TestObject.objects.values('first', 'second').extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))),
+            list(
+                TestObject.objects
+                .values('first', 'second')
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+            ),
             [{'second': 'second', 'first': 'first'}]
         )
 
         # Extra columns can be partially returned
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values('first', 'second', 'foo')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values('first', 'second', 'foo')
+            ),
             [{'second': 'second', 'foo': 'first', 'first': 'first'}]
         )
 
         # Also works if only extra columns are included
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values('foo', 'whiz')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values('foo', 'whiz')
+            ),
             [{'foo': 'first', 'whiz': 'third'}]
         )
 
         # Values list works the same way
         # All columns are returned for an empty values_list()
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list()),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list()
+            ),
             [('first', 'second', 'third', obj.pk, 'first', 'second', 'third')]
         )
 
         # Extra columns after an empty values_list() are still included
         self.assertEqual(
-            list(TestObject.objects.values_list().extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))),
+            list(
+                TestObject.objects
+                .values_list()
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+            ),
             [('first', 'second', 'third', obj.pk, 'first', 'second', 'third')]
         )
 
         # Extra columns ignored completely if not mentioned in values_list()
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('first', 'second')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('first', 'second')
+            ),
             [('first', 'second')]
         )
 
         # Extra columns after a non-empty values_list() clause are ignored completely
         self.assertEqual(
-            list(TestObject.objects.values_list('first', 'second').extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))),
+            list(
+                TestObject.objects
+                .values_list('first', 'second')
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+            ),
             [('first', 'second')]
         )
 
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('second', flat=True)),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('second', flat=True)
+            ),
             ['second']
         )
 
         # Only the extra columns specified in the values_list() are returned
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('first', 'second', 'whiz')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('first', 'second', 'whiz')
+            ),
             [('first', 'second', 'third')]
         )
 
         # ...also works if only extra columns are included
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('foo', 'whiz')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('foo', 'whiz')
+            ),
             [('first', 'third')]
         )
 
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('whiz', flat=True)),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('whiz', flat=True)
+            ),
             ['third']
         )
 
         # ... and values are returned in the order they are specified
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('whiz', 'foo')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('whiz', 'foo')
+            ),
             [('third', 'first')]
         )
 
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('first', 'id')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('first', 'id')
+            ),
             [('first', obj.pk)]
         )
 
         self.assertEqual(
-            list(TestObject.objects.extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third')))).values_list('whiz', 'first', 'bar', 'id')),
+            list(
+                TestObject.objects
+                .extra(select=OrderedDict((('foo', 'first'), ('bar', 'second'), ('whiz', 'third'))))
+                .values_list('whiz', 'first', 'bar', 'id')
+            ),
             [('third', 'first', 'second', obj.pk)]
         )
 
