@@ -6,29 +6,37 @@ import os
 import shutil
 import sys
 import tempfile
+import threading
 import time
 import unittest
 from datetime import datetime, timedelta
 
-try:
-    import threading
-except ImportError:
-    import dummy_threading as threading
-
 from django.core.cache import cache
-from django.core.exceptions import SuspiciousOperation
-from django.core.files.base import File, ContentFile
+from django.core.exceptions import SuspiciousFileOperation, SuspiciousOperation
+from django.core.files.base import ContentFile, File
 from django.core.files.storage import FileSystemStorage, get_storage_class
-from django.core.files.uploadedfile import (InMemoryUploadedFile, SimpleUploadedFile,
-    TemporaryUploadedFile)
-from django.test import LiveServerTestCase, SimpleTestCase
-from django.test import override_settings
-from django.utils import six
-from django.utils.six.moves.urllib.request import urlopen
+from django.core.files.uploadedfile import (
+    InMemoryUploadedFile, SimpleUploadedFile, TemporaryUploadedFile,
+)
+from django.db.models.fields.files import FileDescriptor
+from django.test import (
+    LiveServerTestCase, SimpleTestCase, TestCase, ignore_warnings,
+    override_settings,
+)
+from django.test.utils import requires_tz_support
+from django.urls import NoReverseMatch, reverse_lazy
+from django.utils import six, timezone
 from django.utils._os import upath
+from django.utils.deprecation import RemovedInDjango20Warning
+from django.utils.six.moves.urllib.request import urlopen
 
 from .models import Storage, temp_storage, temp_storage_location
 
+try:
+    import pytz
+except ImportError:
+    pytz = None
+
 
 FILE_SUFFIX_REGEX = '[A-Za-z0-9]{7}'
 
@@ -54,21 +62,19 @@ class GetStorageClassTests(SimpleTestCase):
         """
         get_storage_class raises an error if the requested class don't exist.
         """
-        self.assertRaises(ImportError, get_storage_class,
-                          'django.core.files.storage.NonExistingStorage')
+        with self.assertRaises(ImportError):
+            get_storage_class('django.core.files.storage.NonExistingStorage')
 
     def test_get_nonexisting_storage_module(self):
         """
         get_storage_class raises an error if the requested module don't exist.
         """
         # Error message may or may not be the fully qualified path.
-        with six.assertRaisesRegex(self, ImportError,
-                "No module named '?(django.core.files.)?non_existing_storage'?"):
-            get_storage_class(
-                'django.core.files.non_existing_storage.NonExistingStorage')
+        with six.assertRaisesRegex(self, ImportError, "No module named '?(django.core.files.)?non_existing_storage'?"):
+            get_storage_class('django.core.files.non_existing_storage.NonExistingStorage')
 
 
-class FileStorageDeconstructionTests(unittest.TestCase):
+class FileSystemStorageTests(unittest.TestCase):
 
     def test_deconstruction(self):
         path, args, kwargs = temp_storage.deconstruct()
@@ -84,14 +90,27 @@ class FileStorageDeconstructionTests(unittest.TestCase):
         path, args, kwargs = storage.deconstruct()
         self.assertEqual(kwargs, kwargs_orig)
 
+    def test_lazy_base_url_init(self):
+        """
+        FileSystemStorage.__init__() shouldn't evaluate base_url.
+        """
+        storage = FileSystemStorage(base_url=reverse_lazy('app:url'))
+        with self.assertRaises(NoReverseMatch):
+            storage.url(storage.base_url)
 
-class FileStorageTests(unittest.TestCase):
+
+# Tests for TZ-aware time methods need pytz.
+requires_pytz = unittest.skipIf(pytz is None, "this test requires pytz")
+
+
+class FileStorageTests(TestCase):
+    # Changing TIME_ZONE may issue a query to set the database's timezone,
+    # hence TestCase.
     storage_class = FileSystemStorage
 
     def setUp(self):
         self.temp_dir = tempfile.mkdtemp()
-        self.storage = self.storage_class(location=self.temp_dir,
-            base_url='/test_media_url/')
+        self.storage = self.storage_class(location=self.temp_dir, base_url='/test_media_url/')
         # Set up a second temporary directory which is ensured to have a mixed
         # case name.
         self.temp_dir2 = tempfile.mkdtemp(suffix='aBc')
@@ -100,7 +119,7 @@ class FileStorageTests(unittest.TestCase):
         shutil.rmtree(self.temp_dir)
         shutil.rmtree(self.temp_dir2)
 
-    def test_emtpy_location(self):
+    def test_empty_location(self):
         """
         Makes sure an exception is raised if the location is empty
         """
@@ -125,7 +144,86 @@ class FileStorageTests(unittest.TestCase):
         self.storage.delete('storage_test')
         self.assertFalse(self.storage.exists('storage_test'))
 
-    def test_file_accessed_time(self):
+    def _test_file_time_getter(self, getter):
+        # Check for correct behavior under both USE_TZ=True and USE_TZ=False.
+        # The tests are similar since they both set up a situation where the
+        # system time zone, Django's TIME_ZONE, and UTC are distinct.
+        self._test_file_time_getter_tz_handling_on(getter)
+        self._test_file_time_getter_tz_handling_off(getter)
+
+    @override_settings(USE_TZ=True, TIME_ZONE='Africa/Algiers')
+    def _test_file_time_getter_tz_handling_on(self, getter):
+        # Django's TZ (and hence the system TZ) is set to Africa/Algiers which
+        # is UTC+1 and has no DST change. We can set the Django TZ to something
+        # else so that UTC, Django's TIME_ZONE, and the system timezone are all
+        # different.
+        now_in_algiers = timezone.make_aware(datetime.now())
+
+        # Use a fixed offset timezone so we don't need pytz.
+        with timezone.override(timezone.get_fixed_timezone(-300)):
+            # At this point the system TZ is +1 and the Django TZ
+            # is -5. The following will be aware in UTC.
+            now = timezone.now()
+            self.assertFalse(self.storage.exists('test.file.tz.on'))
+
+            f = ContentFile('custom contents')
+            f_name = self.storage.save('test.file.tz.on', f)
+            self.addCleanup(self.storage.delete, f_name)
+            dt = getter(f_name)
+            # dt should be aware, in UTC
+            self.assertTrue(timezone.is_aware(dt))
+            self.assertEqual(now.tzname(), dt.tzname())
+
+            # Check that the three timezones are indeed distinct.
+            naive_now = datetime.now()
+            algiers_offset = now_in_algiers.tzinfo.utcoffset(naive_now)
+            django_offset = timezone.get_current_timezone().utcoffset(naive_now)
+            utc_offset = timezone.utc.utcoffset(naive_now)
+            self.assertGreater(algiers_offset, utc_offset)
+            self.assertLess(django_offset, utc_offset)
+
+            # dt and now should be the same effective time.
+            self.assertLess(abs(dt - now), timedelta(seconds=2))
+
+    @override_settings(USE_TZ=False, TIME_ZONE='Africa/Algiers')
+    def _test_file_time_getter_tz_handling_off(self, getter):
+        # Django's TZ (and hence the system TZ) is set to Africa/Algiers which
+        # is UTC+1 and has no DST change. We can set the Django TZ to something
+        # else so that UTC, Django's TIME_ZONE, and the system timezone are all
+        # different.
+        now_in_algiers = timezone.make_aware(datetime.now())
+
+        # Use a fixed offset timezone so we don't need pytz.
+        with timezone.override(timezone.get_fixed_timezone(-300)):
+            # At this point the system TZ is +1 and the Django TZ
+            # is -5.
+            self.assertFalse(self.storage.exists('test.file.tz.off'))
+
+            f = ContentFile('custom contents')
+            f_name = self.storage.save('test.file.tz.off', f)
+            self.addCleanup(self.storage.delete, f_name)
+            dt = getter(f_name)
+            # dt should be naive, in system (+1) TZ
+            self.assertTrue(timezone.is_naive(dt))
+
+            # Check that the three timezones are indeed distinct.
+            naive_now = datetime.now()
+            algiers_offset = now_in_algiers.tzinfo.utcoffset(naive_now)
+            django_offset = timezone.get_current_timezone().utcoffset(naive_now)
+            utc_offset = timezone.utc.utcoffset(naive_now)
+            self.assertGreater(algiers_offset, utc_offset)
+            self.assertLess(django_offset, utc_offset)
+
+            # dt and naive_now should be the same effective time.
+            self.assertLess(abs(dt - naive_now), timedelta(seconds=2))
+            # If we convert dt to an aware object using the Algiers
+            # timezone then it should be the same effective time to
+            # now_in_algiers.
+            _dt = timezone.make_aware(dt, now_in_algiers.tzinfo)
+            self.assertLess(abs(_dt - now_in_algiers), timedelta(seconds=2))
+
+    @requires_pytz
+    def test_file_get_accessed_time(self):
         """
         File storage returns a Datetime object for the last accessed time of
         a file.
@@ -134,46 +232,101 @@ class FileStorageTests(unittest.TestCase):
 
         f = ContentFile('custom contents')
         f_name = self.storage.save('test.file', f)
+        self.addCleanup(self.storage.delete, f_name)
+        atime = self.storage.get_accessed_time(f_name)
+
+        self.assertEqual(atime, datetime.fromtimestamp(os.path.getatime(self.storage.path(f_name))))
+        self.assertLess(timezone.now() - self.storage.get_accessed_time(f_name), timedelta(seconds=2))
+
+    @requires_pytz
+    @requires_tz_support
+    def test_file_get_accessed_time_timezone(self):
+        self._test_file_time_getter(self.storage.get_accessed_time)
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    def test_file_accessed_time(self):
+        """
+        File storage returns a datetime for the last accessed time of a file.
+        """
+        self.assertFalse(self.storage.exists('test.file'))
+
+        f = ContentFile('custom contents')
+        f_name = self.storage.save('test.file', f)
+        self.addCleanup(self.storage.delete, f_name)
         atime = self.storage.accessed_time(f_name)
 
-        self.assertEqual(atime, datetime.fromtimestamp(
-            os.path.getatime(self.storage.path(f_name))))
-        self.assertTrue(datetime.now() - self.storage.accessed_time(f_name) < timedelta(seconds=2))
-        self.storage.delete(f_name)
+        self.assertEqual(atime, datetime.fromtimestamp(os.path.getatime(self.storage.path(f_name))))
+        self.assertLess(datetime.now() - self.storage.accessed_time(f_name), timedelta(seconds=2))
 
+    @requires_pytz
+    def test_file_get_created_time(self):
+        """
+        File storage returns a datetime for the creation time of a file.
+        """
+        self.assertFalse(self.storage.exists('test.file'))
+
+        f = ContentFile('custom contents')
+        f_name = self.storage.save('test.file', f)
+        self.addCleanup(self.storage.delete, f_name)
+        ctime = self.storage.get_created_time(f_name)
+
+        self.assertEqual(ctime, datetime.fromtimestamp(os.path.getctime(self.storage.path(f_name))))
+        self.assertLess(timezone.now() - self.storage.get_created_time(f_name), timedelta(seconds=2))
+
+    @requires_pytz
+    @requires_tz_support
+    def test_file_get_created_time_timezone(self):
+        self._test_file_time_getter(self.storage.get_created_time)
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
     def test_file_created_time(self):
         """
-        File storage returns a Datetime object for the creation time of
-        a file.
+        File storage returns a datetime for the creation time of a file.
         """
         self.assertFalse(self.storage.exists('test.file'))
 
         f = ContentFile('custom contents')
         f_name = self.storage.save('test.file', f)
         ctime = self.storage.created_time(f_name)
+        self.addCleanup(self.storage.delete, f_name)
 
-        self.assertEqual(ctime, datetime.fromtimestamp(
-            os.path.getctime(self.storage.path(f_name))))
-        self.assertTrue(datetime.now() - self.storage.created_time(f_name) < timedelta(seconds=2))
+        self.assertEqual(ctime, datetime.fromtimestamp(os.path.getctime(self.storage.path(f_name))))
+        self.assertLess(datetime.now() - self.storage.created_time(f_name), timedelta(seconds=2))
 
-        self.storage.delete(f_name)
-
-    def test_file_modified_time(self):
+    @requires_pytz
+    def test_file_get_modified_time(self):
         """
-        File storage returns a Datetime object for the last modified time of
-        a file.
+        File storage returns a datetime for the last modified time of a file.
         """
         self.assertFalse(self.storage.exists('test.file'))
 
         f = ContentFile('custom contents')
         f_name = self.storage.save('test.file', f)
+        self.addCleanup(self.storage.delete, f_name)
+        mtime = self.storage.get_modified_time(f_name)
+
+        self.assertEqual(mtime, datetime.fromtimestamp(os.path.getmtime(self.storage.path(f_name))))
+        self.assertLess(timezone.now() - self.storage.get_modified_time(f_name), timedelta(seconds=2))
+
+    @requires_pytz
+    @requires_tz_support
+    def test_file_get_modified_time_timezone(self):
+        self._test_file_time_getter(self.storage.get_modified_time)
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    def test_file_modified_time(self):
+        """
+        File storage returns a datetime for the last modified time of a file.
+        """
+        self.assertFalse(self.storage.exists('test.file'))
+
+        f = ContentFile('custom contents')
+        f_name = self.storage.save('test.file', f)
+        self.addCleanup(self.storage.delete, f_name)
         mtime = self.storage.modified_time(f_name)
 
-        self.assertEqual(mtime, datetime.fromtimestamp(
-            os.path.getmtime(self.storage.path(f_name))))
-        self.assertTrue(datetime.now() - self.storage.modified_time(f_name) < timedelta(seconds=2))
-
-        self.storage.delete(f_name)
+        self.assertEqual(mtime, datetime.fromtimestamp(os.path.getmtime(self.storage.path(f_name))))
+        self.assertLess(datetime.now() - self.storage.modified_time(f_name), timedelta(seconds=2))
 
     def test_file_save_without_name(self):
         """
@@ -198,8 +351,7 @@ class FileStorageTests(unittest.TestCase):
         Saving a pathname should create intermediate directories as necessary.
         """
         self.assertFalse(self.storage.exists('path/to'))
-        self.storage.save('path/to/test.file',
-            ContentFile('file saved with path'))
+        self.storage.save('path/to/test.file', ContentFile('file saved with path'))
 
         self.assertTrue(self.storage.exists('path/to'))
         with self.storage.open('path/to/test.file') as f:
@@ -219,8 +371,7 @@ class FileStorageTests(unittest.TestCase):
             self.assertFalse(file.closed)
             self.assertFalse(file.file.closed)
 
-        file = InMemoryUploadedFile(six.StringIO('1'), '', 'test',
-                                    'text/plain', 1, 'utf8')
+        file = InMemoryUploadedFile(six.StringIO('1'), '', 'test', 'text/plain', 1, 'utf8')
         with file:
             self.assertFalse(file.closed)
             self.storage.save('path/to/test.file', file)
@@ -236,8 +387,7 @@ class FileStorageTests(unittest.TestCase):
         f = ContentFile('custom contents')
         f_name = self.storage.save('test.file', f)
 
-        self.assertEqual(self.storage.path(f_name),
-            os.path.join(self.temp_dir, f_name))
+        self.assertEqual(self.storage.path(f_name), os.path.join(self.temp_dir, f_name))
 
         self.storage.delete(f_name)
 
@@ -245,20 +395,41 @@ class FileStorageTests(unittest.TestCase):
         """
         File storage returns a url to access a given file from the Web.
         """
-        self.assertEqual(self.storage.url('test.file'),
-            '%s%s' % (self.storage.base_url, 'test.file'))
+        self.assertEqual(self.storage.url('test.file'), self.storage.base_url + 'test.file')
 
         # should encode special chars except ~!*()'
         # like encodeURIComponent() JavaScript function do
-        self.assertEqual(self.storage.url(r"""~!*()'@#$%^&*abc`+ =.file"""),
-            """/test_media_url/~!*()'%40%23%24%25%5E%26*abc%60%2B%20%3D.file""")
+        self.assertEqual(
+            self.storage.url(r"~!*()'@#$%^&*abc`+ =.file"),
+            "/test_media_url/~!*()'%40%23%24%25%5E%26*abc%60%2B%20%3D.file"
+        )
+        self.assertEqual(self.storage.url("ab\0c"), "/test_media_url/ab%00c")
 
-        # should stanslate os path separator(s) to the url path separator
-        self.assertEqual(self.storage.url("""a/b\\c.file"""),
-            """/test_media_url/a/b/c.file""")
+        # should translate os path separator(s) to the url path separator
+        self.assertEqual(self.storage.url("""a/b\\c.file"""), "/test_media_url/a/b/c.file")
 
+        # #25905: remove leading slashes from file names to prevent unsafe url output
+        self.assertEqual(self.storage.url("/evil.com"), "/test_media_url/evil.com")
+        self.assertEqual(self.storage.url(r"\evil.com"), "/test_media_url/evil.com")
+        self.assertEqual(self.storage.url("///evil.com"), "/test_media_url/evil.com")
+        self.assertEqual(self.storage.url(r"\\\evil.com"), "/test_media_url/evil.com")
+
+        self.assertEqual(self.storage.url(None), "/test_media_url/")
+
+    def test_base_url(self):
+        """
+        File storage returns a url even when its base_url is unset or modified.
+        """
         self.storage.base_url = None
-        self.assertRaises(ValueError, self.storage.url, 'test.file')
+        with self.assertRaises(ValueError):
+            self.storage.url('test.file')
+
+        # #22717: missing ending slash in base_url should be auto-corrected
+        storage = self.storage_class(location=self.temp_dir, base_url='/no_ending_slash')
+        self.assertEqual(
+            storage.url('test.file'),
+            '%s%s' % (storage.base_url, 'test.file')
+        )
 
     def test_listdir(self):
         """
@@ -273,9 +444,8 @@ class FileStorageTests(unittest.TestCase):
         os.mkdir(os.path.join(self.temp_dir, 'storage_dir_1'))
 
         dirs, files = self.storage.listdir('')
-        self.assertEqual(set(dirs), set(['storage_dir_1']))
-        self.assertEqual(set(files),
-                         set(['storage_test_1', 'storage_test_2']))
+        self.assertEqual(set(dirs), {'storage_dir_1'})
+        self.assertEqual(set(files), {'storage_test_1', 'storage_test_2'})
 
         self.storage.delete('storage_test_1')
         self.storage.delete('storage_test_2')
@@ -286,8 +456,10 @@ class FileStorageTests(unittest.TestCase):
         File storage prevents directory traversal (files can only be accessed if
         they're below the storage location).
         """
-        self.assertRaises(SuspiciousOperation, self.storage.exists, '..')
-        self.assertRaises(SuspiciousOperation, self.storage.exists, '/etc/passwd')
+        with self.assertRaises(SuspiciousOperation):
+            self.storage.exists('..')
+        with self.assertRaises(SuspiciousOperation):
+            self.storage.exists('/etc/passwd')
 
     def test_file_storage_preserves_filename_case(self):
         """The storage backend should preserve case of filenames."""
@@ -299,8 +471,7 @@ class FileStorageTests(unittest.TestCase):
         file = other_temp_storage.open(mixed_case, 'w')
         file.write('storage contents')
         file.close()
-        self.assertEqual(os.path.join(self.temp_dir2, mixed_case),
-                         other_temp_storage.path(mixed_case))
+        self.assertEqual(os.path.join(self.temp_dir2, mixed_case), other_temp_storage.path(mixed_case))
         other_temp_storage.delete(mixed_case)
 
     def test_makedirs_race_handling(self):
@@ -325,19 +496,17 @@ class FileStorageTests(unittest.TestCase):
         try:
             os.makedirs = fake_makedirs
 
-            self.storage.save('normal/test.file',
-                ContentFile('saved normally'))
+            self.storage.save('normal/test.file', ContentFile('saved normally'))
             with self.storage.open('normal/test.file') as f:
                 self.assertEqual(f.read(), b'saved normally')
 
-            self.storage.save('raced/test.file',
-                ContentFile('saved with race'))
+            self.storage.save('raced/test.file', ContentFile('saved with race'))
             with self.storage.open('raced/test.file') as f:
                 self.assertEqual(f.read(), b'saved with race')
 
             # Check that OSErrors aside from EEXIST are still raised.
-            self.assertRaises(OSError,
-                self.storage.save, 'error/test.file', ContentFile('not saved'))
+            with self.assertRaises(OSError):
+                self.storage.save('error/test.file', ContentFile('not saved'))
         finally:
             os.makedirs = real_makedirs
 
@@ -373,7 +542,8 @@ class FileStorageTests(unittest.TestCase):
 
             # Check that OSErrors aside from ENOENT are still raised.
             self.storage.save('error.file', ContentFile('delete with error'))
-            self.assertRaises(OSError, self.storage.delete, 'error.file')
+            with self.assertRaises(OSError):
+                self.storage.delete('error.file')
         finally:
             os.remove = real_remove
 
@@ -397,9 +567,47 @@ class FileStorageTests(unittest.TestCase):
         with self.assertRaises(AssertionError):
             self.storage.delete('')
 
+    @override_settings(
+        MEDIA_ROOT='media_root',
+        MEDIA_URL='media_url/',
+        FILE_UPLOAD_PERMISSIONS=0o777,
+        FILE_UPLOAD_DIRECTORY_PERMISSIONS=0o777,
+    )
+    def test_setting_changed(self):
+        """
+        Properties using settings values as defaults should be updated on
+        referenced settings change while specified values should be unchanged.
+        """
+        storage = self.storage_class(
+            location='explicit_location',
+            base_url='explicit_base_url/',
+            file_permissions_mode=0o666,
+            directory_permissions_mode=0o666,
+        )
+        defaults_storage = self.storage_class()
+        settings = {
+            'MEDIA_ROOT': 'overriden_media_root',
+            'MEDIA_URL': 'overriden_media_url/',
+            'FILE_UPLOAD_PERMISSIONS': 0o333,
+            'FILE_UPLOAD_DIRECTORY_PERMISSIONS': 0o333,
+        }
+        with self.settings(**settings):
+            self.assertEqual(storage.base_location, 'explicit_location')
+            self.assertIn('explicit_location', storage.location)
+            self.assertEqual(storage.base_url, 'explicit_base_url/')
+            self.assertEqual(storage.file_permissions_mode, 0o666)
+            self.assertEqual(storage.directory_permissions_mode, 0o666)
+            self.assertEqual(defaults_storage.base_location, settings['MEDIA_ROOT'])
+            self.assertIn(settings['MEDIA_ROOT'], defaults_storage.location)
+            self.assertEqual(defaults_storage.base_url, settings['MEDIA_URL'])
+            self.assertEqual(defaults_storage.file_permissions_mode, settings['FILE_UPLOAD_PERMISSIONS'])
+            self.assertEqual(
+                defaults_storage.directory_permissions_mode, settings['FILE_UPLOAD_DIRECTORY_PERMISSIONS']
+            )
+
 
 class CustomStorage(FileSystemStorage):
-    def get_available_name(self, name):
+    def get_available_name(self, name, max_length=None):
         """
         Append numbers to duplicate files rather than underscores, like Trac.
         """
@@ -425,19 +633,73 @@ class CustomStorageTests(FileStorageTests):
         self.storage.delete(second)
 
 
-class FileFieldStorageTests(unittest.TestCase):
+class CustomStorageLegacyDatetimeHandling(FileSystemStorage):
+    # Use the legacy accessed_time() et al from FileSystemStorage and the
+    # shim get_accessed_time() et al from the Storage baseclass. Both of those
+    # raise warnings, so the testcase class ignores them all.
+
+    def get_accessed_time(self, name):
+        return super(FileSystemStorage, self).get_accessed_time(name)
+
+    def get_created_time(self, name):
+        return super(FileSystemStorage, self).get_created_time(name)
+
+    def get_modified_time(self, name):
+        return super(FileSystemStorage, self).get_modified_time(name)
+
+
+@ignore_warnings(category=RemovedInDjango20Warning)
+class CustomStorageLegacyDatetimeHandlingTests(FileStorageTests):
+    storage_class = CustomStorageLegacyDatetimeHandling
+
+
+class DiscardingFalseContentStorage(FileSystemStorage):
+    def _save(self, name, content):
+        if content:
+            return super(DiscardingFalseContentStorage, self)._save(name, content)
+        return ''
+
+
+class DiscardingFalseContentStorageTests(FileStorageTests):
+    storage_class = DiscardingFalseContentStorage
+
+    def test_custom_storage_discarding_empty_content(self):
+        """
+        When Storage.save() wraps a file-like object in File, it should include
+        the name argument so that bool(file) evaluates to True (#26495).
+        """
+        output = six.StringIO('content')
+        self.storage.save('tests/stringio', output)
+        self.assertTrue(self.storage.exists('tests/stringio'))
+
+        with self.storage.open('tests/stringio') as f:
+            self.assertEqual(f.read(), b'content')
+
+
+class FileFieldStorageTests(TestCase):
     def tearDown(self):
         shutil.rmtree(temp_storage_location)
 
+    def _storage_max_filename_length(self, storage):
+        """
+        Query filesystem for maximum filename length (e.g. AUFS has 242).
+        """
+        dir_to_test = storage.location
+        while not os.path.exists(dir_to_test):
+            dir_to_test = os.path.dirname(dir_to_test)
+        try:
+            return os.pathconf(dir_to_test, 'PC_NAME_MAX')
+        except Exception:
+            return 255  # Should be safe on most backends
+
     def test_files(self):
-        # Attempting to access a FileField from the class raises a descriptive
-        # error
-        self.assertRaises(AttributeError, lambda: Storage.normal)
+        self.assertIsInstance(Storage.normal, FileDescriptor)
 
         # An object without a file has limited functionality.
         obj1 = Storage()
         self.assertEqual(obj1.normal.name, "")
-        self.assertRaises(ValueError, lambda: obj1.normal.size)
+        with self.assertRaises(ValueError):
+            obj1.normal.size
 
         # Saving a file enables full functionality.
         obj1.normal.save("django_test.txt", ContentFile("content"))
@@ -451,7 +713,7 @@ class FileFieldStorageTests(unittest.TestCase):
         obj1.normal = SimpleUploadedFile("assignment.txt", b"content")
         dirs, files = temp_storage.listdir("tests")
         self.assertEqual(dirs, [])
-        self.assertFalse("assignment.txt" in files)
+        self.assertNotIn("assignment.txt", files)
 
         obj1.save()
         dirs, files = temp_storage.listdir("tests")
@@ -482,6 +744,14 @@ class FileFieldStorageTests(unittest.TestCase):
         self.assertEqual(list(obj.normal.chunks(chunk_size=2)), [b"co", b"nt", b"en", b"t"])
         obj.normal.close()
 
+    def test_filefield_reopen(self):
+        obj = Storage.objects.create(normal=SimpleUploadedFile('reopen.txt', b'content'))
+        with obj.normal as normal:
+            normal.open()
+        obj.normal.open()
+        obj.normal.file.seek(0)
+        obj.normal.close()
+
     def test_duplicate_filename(self):
         # Multiple files with the same name get _(7 random chars) appended to them.
         objs = [Storage() for i in range(2)]
@@ -495,6 +765,46 @@ class FileFieldStorageTests(unittest.TestCase):
             for o in objs:
                 o.delete()
 
+    def test_file_truncation(self):
+        # Given the max_length is limited, when multiple files get uploaded
+        # under the same name, then the filename get truncated in order to fit
+        # in _(7 random chars). When most of the max_length is taken by
+        # dirname + extension and there are not enough  characters in the
+        # filename to truncate, an exception should be raised.
+        objs = [Storage() for i in range(2)]
+        filename = 'filename.ext'
+
+        for o in objs:
+            o.limited_length.save(filename, ContentFile('Same Content'))
+        try:
+            # Testing truncation.
+            names = [o.limited_length.name for o in objs]
+            self.assertEqual(names[0], 'tests/%s' % filename)
+            six.assertRegex(self, names[1], 'tests/fi_%s.ext' % FILE_SUFFIX_REGEX)
+
+            # Testing exception is raised when filename is too short to truncate.
+            filename = 'short.longext'
+            objs[0].limited_length.save(filename, ContentFile('Same Content'))
+            with self.assertRaisesMessage(SuspiciousFileOperation, 'Storage can not find an available filename'):
+                objs[1].limited_length.save(*(filename, ContentFile('Same Content')))
+        finally:
+            for o in objs:
+                o.delete()
+
+    @unittest.skipIf(
+        sys.platform.startswith('win'),
+        "Windows supports at most 260 characters in a path.",
+    )
+    def test_extended_length_storage(self):
+        # Testing FileField with max_length > 255. Most systems have filename
+        # length limitation of 255. Path takes extra chars.
+        filename = (self._storage_max_filename_length(temp_storage) - 4) * 'a'  # 4 chars for extension.
+        obj = Storage()
+        obj.extended_length.save('%s.txt' % filename, ContentFile('Same Content'))
+        self.assertEqual(obj.extended_length.name, 'tests/%s.txt' % filename)
+        self.assertEqual(obj.extended_length.read(), b'Same Content')
+        obj.extended_length.close()
+
     def test_filefield_default(self):
         # Default values allow an object to access a single file.
         temp_storage.save('tests/default.txt', ContentFile('default content'))
@@ -514,7 +824,7 @@ class FileFieldStorageTests(unittest.TestCase):
         # upload_to can be empty, meaning it does not use subdirectory.
         obj = Storage()
         obj.empty.save('django_test.txt', ContentFile('more content'))
-        self.assertEqual(obj.empty.name, "./django_test.txt")
+        self.assertEqual(obj.empty.name, "django_test.txt")
         self.assertEqual(obj.empty.read(), b"more content")
         obj.empty.close()
 
@@ -526,6 +836,16 @@ class FileFieldStorageTests(unittest.TestCase):
         self.assertTrue(obj.random.name.endswith("/random_file"))
         obj.random.close()
 
+    def test_custom_valid_name_callable_upload_to(self):
+        """
+        Storage.get_valid_name() should be called when upload_to is a callable.
+        """
+        obj = Storage()
+        obj.custom_valid_name.save("random_file", ContentFile("random content"))
+        # CustomValidNameStorage.get_valid_name() appends '_valid' to the name
+        self.assertTrue(obj.custom_valid_name.name.endswith("/random_file_valid"))
+        obj.custom_valid_name.close()
+
     def test_filefield_pickling(self):
         # Push an object into the cache to make sure it pickles properly
         obj = Storage()
@@ -684,13 +1004,13 @@ class ContentFileStorageTestCase(unittest.TestCase):
         self.storage.save('unicode.txt', ContentFile("español"))
 
 
+@override_settings(ROOT_URLCONF='file_storage.urls')
 class FileLikeObjectTestCase(LiveServerTestCase):
     """
     Test file-like objects (#15644).
     """
 
     available_apps = []
-    urls = 'file_storage.urls'
 
     def setUp(self):
         self.temp_dir = tempfile.mkdtemp()
