@@ -10,15 +10,17 @@ from django.apps import apps
 from django.contrib.sites.models import Site
 from django.core import management
 from django.core.files.temp import NamedTemporaryFile
+from django.core.management import CommandError
+from django.core.management.commands.dumpdata import ProxyModelWarning
 from django.core.serializers.base import ProgressBar
 from django.db import IntegrityError, connection
 from django.test import (
-    TestCase, TransactionTestCase, ignore_warnings, mock, skipUnlessDBFeature,
+    TestCase, TransactionTestCase, mock, skipUnlessDBFeature,
 )
 from django.utils import six
 from django.utils.encoding import force_text
 
-from .models import Article, Spy, Tag, Visa
+from .models import Article, ProxySpy, Spy, Tag, Visa
 
 
 class TestCaseFixtureLoadingTests(TestCase):
@@ -364,13 +366,11 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
         )
 
         # Excluding a bogus app should throw an error
-        with six.assertRaisesRegex(self, management.CommandError,
-                "No installed app with label 'foo_app'."):
+        with self.assertRaisesMessage(management.CommandError, "No installed app with label 'foo_app'."):
             self._dumpdata_assert(['fixtures', 'sites'], '', exclude_list=['foo_app'])
 
         # Excluding a bogus model should throw an error
-        with six.assertRaisesRegex(self, management.CommandError,
-                "Unknown model in excludes: fixtures.FooModel"):
+        with self.assertRaisesMessage(management.CommandError, "Unknown model in excludes: fixtures.FooModel"):
             self._dumpdata_assert(['fixtures', 'sites'], '', exclude_list=['fixtures.FooModel'])
 
     @unittest.skipIf(sys.platform.startswith('win'), "Windows doesn't support '?' in filenames.")
@@ -415,8 +415,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             primary_keys='2'
         )
 
-        with six.assertRaisesRegex(self, management.CommandError,
-                "You can only use --pks option with one model"):
+        with self.assertRaisesMessage(management.CommandError, "You can only use --pks option with one model"):
             self._dumpdata_assert(
                 ['fixtures'],
                 '[{"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place on ESPN", '
@@ -425,8 +424,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
                 primary_keys='2,3'
             )
 
-        with six.assertRaisesRegex(self, management.CommandError,
-                "You can only use --pks option with one model"):
+        with self.assertRaisesMessage(management.CommandError, "You can only use --pks option with one model"):
             self._dumpdata_assert(
                 '',
                 '[{"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place on ESPN", '
@@ -435,8 +433,7 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
                 primary_keys='2,3'
             )
 
-        with six.assertRaisesRegex(self, management.CommandError,
-                "You can only use --pks option with one model"):
+        with self.assertRaisesMessage(management.CommandError, "You can only use --pks option with one model"):
             self._dumpdata_assert(
                 ['fixtures.Article', 'fixtures.category'],
                 '[{"pk": 2, "model": "fixtures.article", "fields": {"headline": "Poker has no place on ESPN", '
@@ -482,6 +479,38 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             management.call_command('dumpdata', 'fixtures', **options)
             self.assertEqual(new_io.getvalue(), '')
 
+    def test_dumpdata_proxy_without_concrete(self):
+        """
+        A warning is displayed if a proxy model is dumped without its concrete
+        parent.
+        """
+        ProxySpy.objects.create(name='Paul')
+
+        with warnings.catch_warnings(record=True) as warning_list:
+            warnings.simplefilter('always')
+            self._dumpdata_assert(['fixtures.ProxySpy'], '[]')
+        warning = warning_list.pop()
+        self.assertEqual(warning.category, ProxyModelWarning)
+        self.assertEqual(
+            str(warning.message),
+            "fixtures.ProxySpy is a proxy model and won't be serialized."
+        )
+
+    def test_dumpdata_proxy_with_concrete(self):
+        """
+        A warning isn't displayed if a proxy model is dumped with its concrete
+        parent.
+        """
+        spy = ProxySpy.objects.create(name='Paul')
+
+        with warnings.catch_warnings(record=True) as warning_list:
+            warnings.simplefilter('always')
+            self._dumpdata_assert(
+                ['fixtures.ProxySpy', 'fixtures.Spy'],
+                '[{"pk": %d, "model": "fixtures.spy", "fields": {"cover_blown": false}}]' % spy.pk
+            )
+        self.assertEqual(len(warning_list), 0)
+
     def test_compress_format_loading(self):
         # Load fixture 4 (compressed), using format specification
         management.call_command('loaddata', 'fixture4.json', verbosity=0)
@@ -533,12 +562,12 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             management.call_command('loaddata', 'invalid.json', verbosity=0)
             self.assertIn("Could not load fixtures.Article(pk=1):", cm.exception.args[0])
 
-    @ignore_warnings(category=UserWarning, message="No fixture named")
     def test_loaddata_app_option(self):
         """
         Verifies that the --app option works.
         """
-        management.call_command('loaddata', 'db_fixture_1', verbosity=0, app_label="someotherapp")
+        with self.assertRaisesMessage(CommandError, "No fixture named 'db_fixture_1' found."):
+            management.call_command('loaddata', 'db_fixture_1', verbosity=0, app_label="someotherapp")
         self.assertQuerysetEqual(Article.objects.all(), [])
         management.call_command('loaddata', 'db_fixture_1', verbosity=0, app_label="fixtures")
         self.assertQuerysetEqual(Article.objects.all(), [
@@ -564,11 +593,12 @@ class FixtureLoadingTests(DumpDataAssertMixin, TestCase):
             '<Article: Who needs to use compressed data?>',
         ])
 
-    @ignore_warnings(category=UserWarning, message="No fixture named")
     def test_unmatched_identifier_loading(self):
         # Try to load db fixture 3. This won't load because the database identifier doesn't match
-        management.call_command('loaddata', 'db_fixture_3', verbosity=0)
-        management.call_command('loaddata', 'db_fixture_3', verbosity=0, using='default')
+        with self.assertRaisesMessage(CommandError, "No fixture named 'db_fixture_3' found."):
+            management.call_command('loaddata', 'db_fixture_3', verbosity=0)
+        with self.assertRaisesMessage(CommandError, "No fixture named 'db_fixture_3' found."):
+            management.call_command('loaddata', 'db_fixture_3', verbosity=0, using='default')
         self.assertQuerysetEqual(Article.objects.all(), [])
 
     def test_output_formats(self):
@@ -629,30 +659,19 @@ class NonExistentFixtureTests(TestCase):
 
     def test_loaddata_not_existent_fixture_file(self):
         stdout_output = six.StringIO()
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            # With verbosity=2, we get both stdout output and a warning
-            management.call_command(
-                'loaddata',
-                'this_fixture_doesnt_exist',
-                verbosity=2,
-                stdout=stdout_output,
-            )
-        self.assertIn("No fixture 'this_fixture_doesnt_exist' in",
-            force_text(stdout_output.getvalue()))
-        self.assertEqual(len(w), 1)
-        self.assertEqual(force_text(w[0].message),
-            "No fixture named 'this_fixture_doesnt_exist' found.")
+        with self.assertRaisesMessage(CommandError, "No fixture named 'this_fixture_doesnt_exist' found."):
+            management.call_command('loaddata', 'this_fixture_doesnt_exist', stdout=stdout_output)
 
     @mock.patch('django.db.connection.enable_constraint_checking')
     @mock.patch('django.db.connection.disable_constraint_checking')
-    def test_nonexistent_fixture_no_constraint_checking(self,
-            disable_constraint_checking, enable_constraint_checking):
+    def test_nonexistent_fixture_no_constraint_checking(
+            self, disable_constraint_checking, enable_constraint_checking):
         """
         If no fixtures match the loaddata command, constraints checks on the
         database shouldn't be disabled. This is performance critical on MSSQL.
         """
-        management.call_command('loaddata', 'this_fixture_doesnt_exist', verbosity=0)
+        with self.assertRaisesMessage(CommandError, "No fixture named 'this_fixture_doesnt_exist' found."):
+            management.call_command('loaddata', 'this_fixture_doesnt_exist', verbosity=0)
         disable_constraint_checking.assert_not_called()
         enable_constraint_checking.assert_not_called()
 
