@@ -1,31 +1,27 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-import warnings
-
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.forms import (
     BooleanField, CharField, ChoiceField, DateField, DateTimeField,
     DecimalField, EmailField, FileField, FloatField, Form,
-    GenericIPAddressField, IntegerField, IPAddressField, ModelChoiceField,
+    GenericIPAddressField, IntegerField, ModelChoiceField,
     ModelMultipleChoiceField, MultipleChoiceField, RegexField,
-    SplitDateTimeField, TimeField, URLField, utils, ValidationError,
+    SplitDateTimeField, TimeField, URLField, ValidationError, utils,
 )
-from django.test import TestCase
-from django.utils.safestring import mark_safe
+from django.test import SimpleTestCase, TestCase
 from django.utils.encoding import python_2_unicode_compatible
+from django.utils.safestring import mark_safe
 
 
 class AssertFormErrorsMixin(object):
     def assertFormErrors(self, expected, the_callable, *args, **kwargs):
-        try:
+        with self.assertRaises(ValidationError) as cm:
             the_callable(*args, **kwargs)
-            self.fail("Testing the 'clean' method on %s failed to raise a ValidationError.")
-        except ValidationError as e:
-            self.assertEqual(e.messages, expected)
+        self.assertEqual(cm.exception.messages, expected)
 
 
-class FormsErrorMessagesTestCase(TestCase, AssertFormErrorsMixin):
+class FormsErrorMessagesTestCase(SimpleTestCase, AssertFormErrorsMixin):
     def test_charfield(self):
         e = {
             'required': 'REQUIRED',
@@ -118,7 +114,7 @@ class FormsErrorMessagesTestCase(TestCase, AssertFormErrorsMixin):
             'min_length': 'LENGTH %(show_value)s, MIN LENGTH %(limit_value)s',
             'max_length': 'LENGTH %(show_value)s, MAX LENGTH %(limit_value)s',
         }
-        f = RegexField(r'^\d+$', min_length=5, max_length=10, error_messages=e)
+        f = RegexField(r'^[0-9]+$', min_length=5, max_length=10, error_messages=e)
         self.assertFormErrors(['REQUIRED'], f.clean, '')
         self.assertFormErrors(['INVALID'], f.clean, 'abcde')
         self.assertFormErrors(['LENGTH 4, MIN LENGTH 5'], f.clean, '1234')
@@ -154,10 +150,16 @@ class FormsErrorMessagesTestCase(TestCase, AssertFormErrorsMixin):
         e = {
             'required': 'REQUIRED',
             'invalid': 'INVALID',
+            'max_length': '"%(value)s" has more than %(limit_value)d characters.',
         }
-        f = URLField(error_messages=e)
+        f = URLField(error_messages=e, max_length=17)
         self.assertFormErrors(['REQUIRED'], f.clean, '')
         self.assertFormErrors(['INVALID'], f.clean, 'abc.c')
+        self.assertFormErrors(
+            ['"http://djangoproject.com" has more than 17 characters.'],
+            f.clean,
+            'djangoproject.com'
+        )
 
     def test_booleanfield(self):
         e = {
@@ -196,17 +198,6 @@ class FormsErrorMessagesTestCase(TestCase, AssertFormErrorsMixin):
         self.assertFormErrors(['REQUIRED'], f.clean, '')
         self.assertFormErrors(['INVALID DATE', 'INVALID TIME'], f.clean, ['a', 'b'])
 
-    def test_ipaddressfield(self):
-        e = {
-            'required': 'REQUIRED',
-            'invalid': 'INVALID IP ADDRESS',
-        }
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("always")
-            f = IPAddressField(error_messages=e)
-        self.assertFormErrors(['REQUIRED'], f.clean, '')
-        self.assertFormErrors(['INVALID IP ADDRESS'], f.clean, '127.0.0')
-
     def test_generic_ipaddressfield(self):
         e = {
             'required': 'REQUIRED',
@@ -237,8 +228,14 @@ class FormsErrorMessagesTestCase(TestCase, AssertFormErrorsMixin):
 
         # This form should print errors the default way.
         form1 = TestForm({'first_name': 'John'})
-        self.assertHTMLEqual(str(form1['last_name'].errors), '<ul class="errorlist"><li>This field is required.</li></ul>')
-        self.assertHTMLEqual(str(form1.errors['__all__']), '<ul class="errorlist"><li>I like to be awkward.</li></ul>')
+        self.assertHTMLEqual(
+            str(form1['last_name'].errors),
+            '<ul class="errorlist"><li>This field is required.</li></ul>'
+        )
+        self.assertHTMLEqual(
+            str(form1.errors['__all__']),
+            '<ul class="errorlist nonfield"><li>I like to be awkward.</li></ul>'
+        )
 
         # This one should wrap error groups in the customized way.
         form2 = TestForm({'first_name': 'John'}, error_class=CustomErrorList)
