@@ -24,8 +24,8 @@ class FormsRegressionsTestCase(TestCase):
 
         self.assertHTMLEqual(
             TestForm(auto_id=False).as_p(),
-            '<p>F1: <input type="text" class="special" name="f1" maxlength="10" /></p>\n'
-            '<p>F2: <input type="text" class="special" name="f2" /></p>'
+            '<p>F1: <input type="text" class="special" name="f1" maxlength="10" required /></p>\n'
+            '<p>F2: <input type="text" class="special" name="f2" required /></p>'
         )
 
     def test_regression_3600(self):
@@ -39,7 +39,7 @@ class FormsRegressionsTestCase(TestCase):
         self.assertHTMLEqual(
             f.as_p(),
             '<p><label for="id_username">username:</label>'
-            '<input id="id_username" type="text" name="username" maxlength="10" /></p>'
+            '<input id="id_username" type="text" name="username" maxlength="10" required /></p>'
         )
 
         # Translations are done at rendering time, so multi-lingual apps can define forms)
@@ -47,13 +47,13 @@ class FormsRegressionsTestCase(TestCase):
             self.assertHTMLEqual(
                 f.as_p(),
                 '<p><label for="id_username">Benutzername:</label>'
-                '<input id="id_username" type="text" name="username" maxlength="10" /></p>'
+                '<input id="id_username" type="text" name="username" maxlength="10" required /></p>'
             )
         with translation.override('pl'):
             self.assertHTMLEqual(
                 f.as_p(),
                 '<p><label for="id_username">u\u017cytkownik:</label>'
-                '<input id="id_username" type="text" name="username" maxlength="10" /></p>'
+                '<input id="id_username" type="text" name="username" maxlength="10" required /></p>'
             )
 
     def test_regression_5216(self):
@@ -82,12 +82,12 @@ class FormsRegressionsTestCase(TestCase):
             '<p><label for="id_somechoice_0">\xc5\xf8\xdf:</label>'
             '<ul id="id_somechoice">\n'
             '<li><label for="id_somechoice_0">'
-            '<input type="radio" id="id_somechoice_0" value="\xc5" name="somechoice" /> '
+            '<input type="radio" id="id_somechoice_0" value="\xc5" name="somechoice" required /> '
             'En tied\xe4</label></li>\n'
             '<li><label for="id_somechoice_1">'
-            '<input type="radio" id="id_somechoice_1" value="\xf8" name="somechoice" /> '
+            '<input type="radio" id="id_somechoice_1" value="\xf8" name="somechoice" required /> '
             'Mies</label></li>\n<li><label for="id_somechoice_2">'
-            '<input type="radio" id="id_somechoice_2" value="\xdf" name="somechoice" /> '
+            '<input type="radio" id="id_somechoice_2" value="\xdf" name="somechoice" required /> '
             'Nainen</label></li>\n</ul></p>'
         )
 
@@ -101,12 +101,12 @@ class FormsRegressionsTestCase(TestCase):
                 '\u043d\u043e\u0435 \u043f\u043e\u043b\u0435.</li></ul>\n'
                 '<p><label for="id_somechoice_0">\xc5\xf8\xdf:</label>'
                 ' <ul id="id_somechoice">\n<li><label for="id_somechoice_0">'
-                '<input type="radio" id="id_somechoice_0" value="\xc5" name="somechoice" /> '
+                '<input type="radio" id="id_somechoice_0" value="\xc5" name="somechoice" required /> '
                 'En tied\xe4</label></li>\n'
                 '<li><label for="id_somechoice_1">'
-                '<input type="radio" id="id_somechoice_1" value="\xf8" name="somechoice" /> '
+                '<input type="radio" id="id_somechoice_1" value="\xf8" name="somechoice" required /> '
                 'Mies</label></li>\n<li><label for="id_somechoice_2">'
-                '<input type="radio" id="id_somechoice_2" value="\xdf" name="somechoice" /> '
+                '<input type="radio" id="id_somechoice_2" value="\xdf" name="somechoice" required /> '
                 'Nainen</label></li>\n</ul></p>'
             )
 
