@@ -5,14 +5,14 @@ import copy
 
 from django.core.exceptions import ValidationError
 from django.forms.utils import ErrorDict, ErrorList, flatatt
-from django.test import TestCase
+from django.test import SimpleTestCase
 from django.utils import six
 from django.utils.encoding import force_text, python_2_unicode_compatible
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy
 
 
-class FormsUtilsTestCase(TestCase):
+class FormsUtilsTestCase(SimpleTestCase):
     # Tests for forms/utils.py module.
 
     def test_flatatt(self):
@@ -22,9 +22,18 @@ class FormsUtilsTestCase(TestCase):
 
         self.assertEqual(flatatt({'id': "header"}), ' id="header"')
         self.assertEqual(flatatt({'class': "news", 'title': "Read this"}), ' class="news" title="Read this"')
-        self.assertEqual(flatatt({'class': "news", 'title': "Read this", 'required': "required"}), ' class="news" required="required" title="Read this"')
-        self.assertEqual(flatatt({'class': "news", 'title': "Read this", 'required': True}), ' class="news" title="Read this" required')
-        self.assertEqual(flatatt({'class': "news", 'title': "Read this", 'required': False}), ' class="news" title="Read this"')
+        self.assertEqual(
+            flatatt({'class': "news", 'title': "Read this", 'required': "required"}),
+            ' class="news" required="required" title="Read this"'
+        )
+        self.assertEqual(
+            flatatt({'class': "news", 'title': "Read this", 'required': True}),
+            ' class="news" title="Read this" required'
+        )
+        self.assertEqual(
+            flatatt({'class': "news", 'title': "Read this", 'required': False}),
+            ' class="news" title="Read this"'
+        )
         self.assertEqual(flatatt({}), '')
 
     def test_flatatt_no_side_effects(self):
@@ -50,27 +59,30 @@ class FormsUtilsTestCase(TestCase):
         ###################
 
         # Can take a string.
-        self.assertHTMLEqual(str(ErrorList(ValidationError("There was an error.").messages)),
-                         '<ul class="errorlist"><li>There was an error.</li></ul>')
-
+        self.assertHTMLEqual(
+            str(ErrorList(ValidationError("There was an error.").messages)),
+            '<ul class="errorlist"><li>There was an error.</li></ul>'
+        )
         # Can take a unicode string.
-        self.assertHTMLEqual(six.text_type(ErrorList(ValidationError("Not \u03C0.").messages)),
-                         '<ul class="errorlist"><li>Not π.</li></ul>')
-
+        self.assertHTMLEqual(
+            six.text_type(ErrorList(ValidationError("Not \u03C0.").messages)),
+            '<ul class="errorlist"><li>Not π.</li></ul>'
+        )
         # Can take a lazy string.
-        self.assertHTMLEqual(str(ErrorList(ValidationError(ugettext_lazy("Error.")).messages)),
-                         '<ul class="errorlist"><li>Error.</li></ul>')
-
+        self.assertHTMLEqual(
+            str(ErrorList(ValidationError(ugettext_lazy("Error.")).messages)),
+            '<ul class="errorlist"><li>Error.</li></ul>'
+        )
         # Can take a list.
-        self.assertHTMLEqual(str(ErrorList(ValidationError(["Error one.", "Error two."]).messages)),
-                         '<ul class="errorlist"><li>Error one.</li><li>Error two.</li></ul>')
-
+        self.assertHTMLEqual(
+            str(ErrorList(ValidationError(["Error one.", "Error two."]).messages)),
+            '<ul class="errorlist"><li>Error one.</li><li>Error two.</li></ul>'
+        )
         # Can take a dict.
         self.assertHTMLEqual(
             str(ErrorList(sorted(ValidationError({'error_1': "1. Error one.", 'error_2': "2. Error two."}).messages))),
             '<ul class="errorlist"><li>1. Error one.</li><li>2. Error two.</li></ul>'
         )
-
         # Can take a mixture in a list.
         self.assertHTMLEqual(
             str(ErrorList(sorted(ValidationError([
@@ -97,19 +109,33 @@ class FormsUtilsTestCase(TestCase):
                 return "A very bad error."
 
         # Can take a non-string.
-        self.assertHTMLEqual(str(ErrorList(ValidationError(VeryBadError()).messages)),
-                         '<ul class="errorlist"><li>A very bad error.</li></ul>')
+        self.assertHTMLEqual(
+            str(ErrorList(ValidationError(VeryBadError()).messages)),
+            '<ul class="errorlist"><li>A very bad error.</li></ul>'
+        )
 
         # Escapes non-safe input but not input marked safe.
         example = 'Example of link: <a href="http://www.example.com/">example</a>'
-        self.assertHTMLEqual(str(ErrorList([example])),
-                         '<ul class="errorlist"><li>Example of link: &lt;a href=&quot;http://www.example.com/&quot;&gt;example&lt;/a&gt;</li></ul>')
-        self.assertHTMLEqual(str(ErrorList([mark_safe(example)])),
-                         '<ul class="errorlist"><li>Example of link: <a href="http://www.example.com/">example</a></li></ul>')
-        self.assertHTMLEqual(str(ErrorDict({'name': example})),
-                         '<ul class="errorlist"><li>nameExample of link: &lt;a href=&quot;http://www.example.com/&quot;&gt;example&lt;/a&gt;</li></ul>')
-        self.assertHTMLEqual(str(ErrorDict({'name': mark_safe(example)})),
-                         '<ul class="errorlist"><li>nameExample of link: <a href="http://www.example.com/">example</a></li></ul>')
+        self.assertHTMLEqual(
+            str(ErrorList([example])),
+            '<ul class="errorlist"><li>Example of link: '
+            '&lt;a href=&quot;http://www.example.com/&quot;&gt;example&lt;/a&gt;</li></ul>'
+        )
+        self.assertHTMLEqual(
+            str(ErrorList([mark_safe(example)])),
+            '<ul class="errorlist"><li>Example of link: '
+            '<a href="http://www.example.com/">example</a></li></ul>'
+        )
+        self.assertHTMLEqual(
+            str(ErrorDict({'name': example})),
+            '<ul class="errorlist"><li>nameExample of link: '
+            '&lt;a href=&quot;http://www.example.com/&quot;&gt;example&lt;/a&gt;</li></ul>'
+        )
+        self.assertHTMLEqual(
+            str(ErrorDict({'name': mark_safe(example)})),
+            '<ul class="errorlist"><li>nameExample of link: '
+            '<a href="http://www.example.com/">example</a></li></ul>'
+        )
 
     def test_error_dict_copy(self):
         e = ErrorDict()
