@@ -384,8 +384,8 @@ class SelectDateWidgetTest(WidgetTest):
             """,
         )
 
-        self.assertRaisesMessage(ValueError, 'empty_label list/tuple must have 3 elements.',
-            SelectDateWidget, years=('2014',), empty_label=('not enough', 'values'))
+        with self.assertRaisesMessage(ValueError, 'empty_label list/tuple must have 3 elements.'):
+            SelectDateWidget(years=('2014',), empty_label=('not enough', 'values'))
 
     @override_settings(USE_L10N=True)
     @translation.override('nl')
@@ -477,3 +477,11 @@ class SelectDateWidgetTest(WidgetTest):
             w.value_from_datadict({'date_year': '1899', 'date_month': '8', 'date_day': '13'}, {}, 'date'),
             '13-08-1899',
         )
+
+    def test_value_omitted_from_data(self):
+        self.assertIs(self.widget.value_omitted_from_data({}, {}, 'field'), True)
+        self.assertIs(self.widget.value_omitted_from_data({'field_month': '12'}, {}, 'field'), False)
+        self.assertIs(self.widget.value_omitted_from_data({'field_year': '2000'}, {}, 'field'), False)
+        self.assertIs(self.widget.value_omitted_from_data({'field_day': '1'}, {}, 'field'), False)
+        data = {'field_day': '1', 'field_month': '12', 'field_year': '2000'}
+        self.assertIs(self.widget.value_omitted_from_data(data, {}, 'field'), False)
