@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
 import datetime
@@ -7,7 +8,7 @@ from django.test import TestCase, override_settings, skipUnlessDBFeature
 from django.test.utils import requires_tz_support
 from django.utils import timezone
 
-from .models import Book, BookSigning
+from .models import Artist, Author, Book, BookSigning, Page
 
 
 def _make_books(n, base_date):
@@ -19,9 +20,25 @@ def _make_books(n, base_date):
             pubdate=base_date - datetime.timedelta(days=i))
 
 
-class ArchiveIndexViewTests(TestCase):
-    fixtures = ['generic-views-test-data.json']
-    urls = 'generic_views.urls'
+class TestDataMixin(object):
+
+    @classmethod
+    def setUpTestData(cls):
+        cls.artist1 = Artist.objects.create(name='Rene Magritte')
+        cls.author1 = Author.objects.create(name='Roberto Bolaño', slug='roberto-bolano')
+        cls.author2 = Author.objects.create(name='Scott Rosenberg', slug='scott-rosenberg')
+        cls.book1 = Book.objects.create(name='2066', slug='2066', pages=800, pubdate=datetime.date(2008, 10, 1))
+        cls.book1.authors.add(cls.author1)
+        cls.book2 = Book.objects.create(
+            name='Dreaming in Code', slug='dreaming-in-code', pages=300, pubdate=datetime.date(2006, 5, 1)
+        )
+        cls.page1 = Page.objects.create(
+            content='I was once bitten by a moose.', template='generic_views/page_template.html'
+        )
+
+
+@override_settings(ROOT_URLCONF='generic_views.urls')
+class ArchiveIndexViewTests(TestDataMixin, TestCase):
 
     def test_archive_view(self):
         res = self.client.get('/dates/books/')
@@ -35,7 +52,7 @@ class ArchiveIndexViewTests(TestCase):
         self.assertEqual(res.status_code, 200)
         self.assertEqual(list(res.context['date_list']), list(Book.objects.dates('pubdate', 'year', 'DESC')))
         self.assertEqual(list(res.context['thingies']), list(Book.objects.all()))
-        self.assertFalse('latest' in res.context)
+        self.assertNotIn('latest', res.context)
         self.assertTemplateUsed(res, 'generic_views/book_archive.html')
 
     def test_empty_archive_view(self):
@@ -65,7 +82,8 @@ class ArchiveIndexViewTests(TestCase):
         self.assertTemplateUsed(res, 'generic_views/book_detail.html')
 
     def test_archive_view_invalid(self):
-        self.assertRaises(ImproperlyConfigured, self.client.get, '/dates/books/invalid/')
+        with self.assertRaises(ImproperlyConfigured):
+            self.client.get('/dates/books/invalid/')
 
     def test_archive_view_by_month(self):
         res = self.client.get('/dates/books/by_month/')
@@ -120,10 +138,25 @@ class ArchiveIndexViewTests(TestCase):
         self.assertEqual(res.status_code, 200)
         self.assertEqual(list(res.context['date_list']), list(reversed(sorted(res.context['date_list']))))
 
+    def test_archive_view_custom_sorting(self):
+        Book.objects.create(name="Zebras for Dummies", pages=600, pubdate=datetime.date(2007, 5, 1))
+        res = self.client.get('/dates/books/sortedbyname/')
+        self.assertEqual(res.status_code, 200)
+        self.assertEqual(list(res.context['date_list']), list(Book.objects.dates('pubdate', 'year', 'DESC')))
+        self.assertEqual(list(res.context['latest']), list(Book.objects.order_by('name').all()))
+        self.assertTemplateUsed(res, 'generic_views/book_archive.html')
 
-class YearArchiveViewTests(TestCase):
-    fixtures = ['generic-views-test-data.json']
-    urls = 'generic_views.urls'
+    def test_archive_view_custom_sorting_dec(self):
+        Book.objects.create(name="Zebras for Dummies", pages=600, pubdate=datetime.date(2007, 5, 1))
+        res = self.client.get('/dates/books/sortedbynamedec/')
+        self.assertEqual(res.status_code, 200)
+        self.assertEqual(list(res.context['date_list']), list(Book.objects.dates('pubdate', 'year', 'DESC')))
+        self.assertEqual(list(res.context['latest']), list(Book.objects.order_by('-name').all()))
+        self.assertTemplateUsed(res, 'generic_views/book_archive.html')
+
+
+@override_settings(ROOT_URLCONF='generic_views.urls')
+class YearArchiveViewTests(TestDataMixin, TestCase):
 
     def test_year_view(self):
         res = self.client.get('/dates/books/2008/')
@@ -133,7 +166,7 @@ class YearArchiveViewTests(TestCase):
         self.assertTemplateUsed(res, 'generic_views/book_archive_year.html')
 
         # Since allow_empty=False, next/prev years must be valid (#7164)
-        self.assertEqual(res.context['next_year'], None)
+        self.assertIsNone(res.context['next_year'])
         self.assertEqual(res.context['previous_year'], datetime.date(2006, 1, 1))
 
     def test_year_view_make_object_list(self):
@@ -178,6 +211,41 @@ class YearArchiveViewTests(TestCase):
         self.assertEqual(list(res.context['object_list']), list(Book.objects.filter(pubdate__year=2006)))
         self.assertTemplateUsed(res, 'generic_views/book_archive_year.html')
 
+    def test_year_view_custom_sort_order(self):
+        # Zebras comes after Dreaming by name, but before on '-pubdate' which is the default sorting
+        Book.objects.create(name="Zebras for Dummies", pages=600, pubdate=datetime.date(2006, 9, 1))
+        res = self.client.get('/dates/books/2006/sortedbyname/')
+        self.assertEqual(res.status_code, 200)
+        self.assertEqual(list(res.context['date_list']), [datetime.date(2006, 5, 1), datetime.date(2006, 9, 1)])
+        self.assertEqual(
+            list(res.context['book_list']),
+            list(Book.objects.filter(pubdate__year=2006).order_by('name'))
+        )
+        self.assertEqual(
+            list(res.context['object_list']),
+            list(Book.objects.filter(pubdate__year=2006).order_by('name'))
+        )
+        self.assertTemplateUsed(res, 'generic_views/book_archive_year.html')
+
+    def test_year_view_two_custom_sort_orders(self):
+        Book.objects.create(name="Zebras for Dummies", pages=300, pubdate=datetime.date(2006, 9, 1))
+        Book.objects.create(name="Hunting Hippos", pages=400, pubdate=datetime.date(2006, 3, 1))
+        res = self.client.get('/dates/books/2006/sortedbypageandnamedec/')
+        self.assertEqual(res.status_code, 200)
+        self.assertEqual(
+            list(res.context['date_list']),
+            [datetime.date(2006, 3, 1), datetime.date(2006, 5, 1), datetime.date(2006, 9, 1)]
+        )
+        self.assertEqual(
+            list(res.context['book_list']),
+            list(Book.objects.filter(pubdate__year=2006).order_by('pages', '-name'))
+        )
+        self.assertEqual(
+            list(res.context['object_list']),
+            list(Book.objects.filter(pubdate__year=2006).order_by('pages', '-name'))
+        )
+        self.assertTemplateUsed(res, 'generic_views/book_archive_year.html')
+
     def test_year_view_invalid_pattern(self):
         res = self.client.get('/dates/books/no_year/')
         self.assertEqual(res.status_code, 404)
@@ -206,9 +274,8 @@ class YearArchiveViewTests(TestCase):
         self.assertEqual(list(res.context['date_list']), list(sorted(res.context['date_list'])))
 
 
-class MonthArchiveViewTests(TestCase):
-    fixtures = ['generic-views-test-data.json']
-    urls = 'generic_views.urls'
+@override_settings(ROOT_URLCONF='generic_views.urls')
+class MonthArchiveViewTests(TestDataMixin, TestCase):
 
     def test_month_view(self):
         res = self.client.get('/dates/books/2008/oct/')
@@ -220,7 +287,7 @@ class MonthArchiveViewTests(TestCase):
         self.assertEqual(res.context['month'], datetime.date(2008, 10, 1))
 
         # Since allow_empty=False, next/prev months must be valid (#7164)
-        self.assertEqual(res.context['next_month'], None)
+        self.assertIsNone(res.context['next_month'])
         self.assertEqual(res.context['previous_month'], datetime.date(2006, 5, 1))
 
     def test_month_view_allow_empty(self):
@@ -243,7 +310,7 @@ class MonthArchiveViewTests(TestCase):
         url = datetime.date.today().strftime('/dates/books/%Y/%b/allow_empty/').lower()
         res = self.client.get(url)
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(res.context['next_month'], None)
+        self.assertIsNone(res.context['next_month'])
 
     def test_month_view_allow_future(self):
         future = (datetime.date.today() + datetime.timedelta(days=60)).replace(day=1)
@@ -263,7 +330,7 @@ class MonthArchiveViewTests(TestCase):
 
         # Since allow_future = True but not allow_empty, next/prev are not
         # allowed to be empty months (#7164)
-        self.assertEqual(res.context['next_month'], None)
+        self.assertIsNone(res.context['next_month'])
         self.assertEqual(res.context['previous_month'], datetime.date(2008, 10, 1))
 
         # allow_future, but not allow_empty, with a current month. So next
@@ -276,8 +343,14 @@ class MonthArchiveViewTests(TestCase):
     def test_month_view_paginated(self):
         res = self.client.get('/dates/books/2008/oct/paginated/')
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(list(res.context['book_list']), list(Book.objects.filter(pubdate__year=2008, pubdate__month=10)))
-        self.assertEqual(list(res.context['object_list']), list(Book.objects.filter(pubdate__year=2008, pubdate__month=10)))
+        self.assertEqual(
+            list(res.context['book_list']),
+            list(Book.objects.filter(pubdate__year=2008, pubdate__month=10))
+        )
+        self.assertEqual(
+            list(res.context['object_list']),
+            list(Book.objects.filter(pubdate__year=2008, pubdate__month=10))
+        )
         self.assertTemplateUsed(res, 'generic_views/book_archive_month.html')
 
     def test_custom_month_format(self):
@@ -333,9 +406,8 @@ class MonthArchiveViewTests(TestCase):
         self.assertEqual(list(res.context['date_list']), list(sorted(res.context['date_list'])))
 
 
-class WeekArchiveViewTests(TestCase):
-    fixtures = ['generic-views-test-data.json']
-    urls = 'generic_views.urls'
+@override_settings(ROOT_URLCONF='generic_views.urls')
+class WeekArchiveViewTests(TestDataMixin, TestCase):
 
     def test_week_view(self):
         res = self.client.get('/dates/books/2008/week/39/')
@@ -345,7 +417,7 @@ class WeekArchiveViewTests(TestCase):
         self.assertEqual(res.context['week'], datetime.date(2008, 9, 28))
 
         # Since allow_empty=False, next/prev weeks must be valid
-        self.assertEqual(res.context['next_week'], None)
+        self.assertIsNone(res.context['next_week'])
         self.assertEqual(res.context['previous_week'], datetime.date(2006, 4, 30))
 
     def test_week_view_allow_empty(self):
@@ -367,7 +439,7 @@ class WeekArchiveViewTests(TestCase):
         url = datetime.date.today().strftime('/dates/books/%Y/week/%U/allow_empty/').lower()
         res = self.client.get(url)
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(res.context['next_week'], None)
+        self.assertIsNone(res.context['next_week'])
 
     def test_week_view_allow_future(self):
         # January 7th always falls in week 1, given Python's definition of week numbers
@@ -385,7 +457,7 @@ class WeekArchiveViewTests(TestCase):
 
         # Since allow_future = True but not allow_empty, next/prev are not
         # allowed to be empty weeks
-        self.assertEqual(res.context['next_week'], None)
+        self.assertIsNone(res.context['next_week'])
         self.assertEqual(res.context['previous_week'], datetime.date(2008, 9, 28))
 
         # allow_future, but not allow_empty, with a current week. So next
@@ -400,8 +472,14 @@ class WeekArchiveViewTests(TestCase):
         week_end = week_start + datetime.timedelta(days=7)
         res = self.client.get('/dates/books/2008/week/39/')
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(list(res.context['book_list']), list(Book.objects.filter(pubdate__gte=week_start, pubdate__lt=week_end)))
-        self.assertEqual(list(res.context['object_list']), list(Book.objects.filter(pubdate__gte=week_start, pubdate__lt=week_end)))
+        self.assertEqual(
+            list(res.context['book_list']),
+            list(Book.objects.filter(pubdate__gte=week_start, pubdate__lt=week_end))
+        )
+        self.assertEqual(
+            list(res.context['object_list']),
+            list(Book.objects.filter(pubdate__gte=week_start, pubdate__lt=week_end))
+        )
         self.assertTemplateUsed(res, 'generic_views/book_archive_week.html')
 
     def test_week_view_invalid_pattern(self):
@@ -430,9 +508,8 @@ class WeekArchiveViewTests(TestCase):
         self.assertEqual(res.status_code, 200)
 
 
-class DayArchiveViewTests(TestCase):
-    fixtures = ['generic-views-test-data.json']
-    urls = 'generic_views.urls'
+@override_settings(ROOT_URLCONF='generic_views.urls')
+class DayArchiveViewTests(TestDataMixin, TestCase):
 
     def test_day_view(self):
         res = self.client.get('/dates/books/2008/oct/01/')
@@ -443,7 +520,7 @@ class DayArchiveViewTests(TestCase):
         self.assertEqual(res.context['day'], datetime.date(2008, 10, 1))
 
         # Since allow_empty=False, next/prev days must be valid.
-        self.assertEqual(res.context['next_day'], None)
+        self.assertIsNone(res.context['next_day'])
         self.assertEqual(res.context['previous_day'], datetime.date(2006, 5, 1))
 
     def test_day_view_allow_empty(self):
@@ -465,7 +542,7 @@ class DayArchiveViewTests(TestCase):
         url = datetime.date.today().strftime('/dates/books/%Y/%b/%d/allow_empty/').lower()
         res = self.client.get(url)
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(res.context['next_day'], None)
+        self.assertIsNone(res.context['next_day'])
 
     def test_day_view_allow_future(self):
         future = (datetime.date.today() + datetime.timedelta(days=60))
@@ -483,7 +560,7 @@ class DayArchiveViewTests(TestCase):
         self.assertEqual(res.context['day'], future)
 
         # allow_future but not allow_empty, next/prev must be valid
-        self.assertEqual(res.context['next_day'], None)
+        self.assertIsNone(res.context['next_day'])
         self.assertEqual(res.context['previous_day'], datetime.date(2008, 10, 1))
 
         # allow_future, but not allow_empty, with a current month.
@@ -502,13 +579,19 @@ class DayArchiveViewTests(TestCase):
     def test_day_view_paginated(self):
         res = self.client.get('/dates/books/2008/oct/1/')
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(list(res.context['book_list']), list(Book.objects.filter(pubdate__year=2008, pubdate__month=10, pubdate__day=1)))
-        self.assertEqual(list(res.context['object_list']), list(Book.objects.filter(pubdate__year=2008, pubdate__month=10, pubdate__day=1)))
+        self.assertEqual(
+            list(res.context['book_list']),
+            list(Book.objects.filter(pubdate__year=2008, pubdate__month=10, pubdate__day=1))
+        )
+        self.assertEqual(
+            list(res.context['object_list']),
+            list(Book.objects.filter(pubdate__year=2008, pubdate__month=10, pubdate__day=1))
+        )
         self.assertTemplateUsed(res, 'generic_views/book_archive_day.html')
 
     def test_next_prev_context(self):
         res = self.client.get('/dates/books/2008/oct/01/')
-        self.assertEqual(res.content, b"Archive for Oct. 1, 2008. Previous day is May 1, 2006")
+        self.assertEqual(res.content, b"Archive for Oct. 1, 2008. Previous day is May 1, 2006\n")
 
     def test_custom_month_format(self):
         res = self.client.get('/dates/books/2008/10/01/')
@@ -548,15 +631,14 @@ class DayArchiveViewTests(TestCase):
         self.assertEqual(res.status_code, 404)
 
 
-class DateDetailViewTests(TestCase):
-    fixtures = ['generic-views-test-data.json']
-    urls = 'generic_views.urls'
+@override_settings(ROOT_URLCONF='generic_views.urls')
+class DateDetailViewTests(TestDataMixin, TestCase):
 
     def test_date_detail_by_pk(self):
-        res = self.client.get('/dates/books/2008/oct/01/1/')
+        res = self.client.get('/dates/books/2008/oct/01/%s/' % self.book1.pk)
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(res.context['object'], Book.objects.get(pk=1))
-        self.assertEqual(res.context['book'], Book.objects.get(pk=1))
+        self.assertEqual(res.context['object'], self.book1)
+        self.assertEqual(res.context['book'], self.book1)
         self.assertTemplateUsed(res, 'generic_views/book_detail.html')
 
     def test_date_detail_by_slug(self):
@@ -565,9 +647,9 @@ class DateDetailViewTests(TestCase):
         self.assertEqual(res.context['book'], Book.objects.get(slug='dreaming-in-code'))
 
     def test_date_detail_custom_month_format(self):
-        res = self.client.get('/dates/books/2008/10/01/1/')
+        res = self.client.get('/dates/books/2008/10/01/%s/' % self.book1.pk)
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(res.context['book'], Book.objects.get(pk=1))
+        self.assertEqual(res.context['book'], self.book1)
 
     def test_date_detail_allow_future(self):
         future = (datetime.date.today() + datetime.timedelta(days=60))
@@ -583,7 +665,8 @@ class DateDetailViewTests(TestCase):
         self.assertTemplateUsed(res, 'generic_views/book_detail.html')
 
     def test_invalid_url(self):
-        self.assertRaises(AttributeError, self.client.get, "/dates/books/2008/oct/01/nopk/")
+        with self.assertRaises(AttributeError):
+            self.client.get("/dates/books/2008/oct/01/nopk/")
 
     def test_get_object_custom_queryset(self):
         """
@@ -592,16 +675,20 @@ class DateDetailViewTests(TestCase):
         Refs #16918.
         """
         res = self.client.get(
-            '/dates/books/get_object_custom_queryset/2006/may/01/2/')
+            '/dates/books/get_object_custom_queryset/2006/may/01/%s/' % self.book2.pk)
         self.assertEqual(res.status_code, 200)
-        self.assertEqual(res.context['object'], Book.objects.get(pk=2))
-        self.assertEqual(res.context['book'], Book.objects.get(pk=2))
+        self.assertEqual(res.context['object'], self.book2)
+        self.assertEqual(res.context['book'], self.book2)
         self.assertTemplateUsed(res, 'generic_views/book_detail.html')
 
         res = self.client.get(
-            '/dates/books/get_object_custom_queryset/2008/oct/01/1/')
+            '/dates/books/get_object_custom_queryset/2008/oct/01/9999999/')
         self.assertEqual(res.status_code, 404)
 
+    def test_get_object_custom_queryset_numqueries(self):
+        with self.assertNumQueries(1):
+            self.client.get('/dates/books/get_object_custom_queryset/2006/may/01/2/')
+
     def test_datetime_date_detail(self):
         bs = BookSigning.objects.create(event_date=datetime.datetime(2008, 4, 2, 12, 0))
         res = self.client.get('/dates/booksignings/2008/apr/2/%d/' % bs.pk)
