@@ -1,4 +1,3 @@
-from django.contrib.gis.gdal import HAS_GDAL
 from django.utils.encoding import python_2_unicode_compatible
 
 from ..admin import admin
@@ -17,6 +16,18 @@ class City(models.Model):
     def __str__(self):
         return self.name
 
+
+@python_2_unicode_compatible
+class CityMercator(models.Model):
+    name = models.CharField(max_length=30)
+    point = models.PointField(srid=3857)
+
+    class Meta:
+        required_db_features = ['gis_enabled']
+
+    def __str__(self):
+        return self.name
+
 site = admin.AdminSite(name='admin_gis')
-if HAS_GDAL:
-    site.register(City, admin.OSMGeoAdmin)
+site.register(City, admin.OSMGeoAdmin)
+site.register(CityMercator, admin.OSMGeoAdmin)
