@@ -3,30 +3,33 @@ from __future__ import unicode_literals
 import ctypes
 import json
 import random
-import unittest
 from binascii import a2b_hex, b2a_hex
 from io import BytesIO
 from unittest import skipUnless
 
+from django.contrib.gis import gdal
 from django.contrib.gis.gdal import HAS_GDAL
-from django.contrib.gis.geos import HAS_GEOS
+from django.contrib.gis.geos import (
+    HAS_GEOS, GeometryCollection, GEOSException, GEOSGeometry, LinearRing,
+    LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon,
+    fromfile, fromstr,
+)
+from django.contrib.gis.geos.base import GEOSBase
+from django.contrib.gis.geos.libgeos import geos_version_info
+from django.contrib.gis.shortcuts import numpy
+from django.template import Context
+from django.template.engine import Engine
+from django.test import SimpleTestCase, ignore_warnings, mock
 from django.utils import six
+from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.encoding import force_bytes
 from django.utils.six.moves import range
 
 from ..test_data import TestDataMixin
 
-if HAS_GEOS:
-    from django.contrib.gis.geos import (
-        GEOSException, GEOSIndexError, GEOSGeometry, GeometryCollection, Point,
-        MultiPoint, Polygon, MultiPolygon, LinearRing, LineString,
-        MultiLineString, fromfile, fromstr, geos_version_info,
-    )
-    from django.contrib.gis.geos.base import gdal, numpy, GEOSBase
-
 
 @skipUnless(HAS_GEOS, "Geos is required.")
-class GEOSTest(unittest.TestCase, TestDataMixin):
+class GEOSTest(SimpleTestCase, TestDataMixin):
 
     def test_base(self):
         "Tests out the GEOSBase class."
@@ -59,7 +62,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # a C method is given a NULL memory reference.
         for fg in (fg1, fg2):
             # Equivalent to `fg.ptr`
-            self.assertRaises(GEOSException, fg._get_ptr)
+            with self.assertRaises(GEOSException):
+                fg._get_ptr()
 
         # Anything that is either not None or the acceptable pointer type will
         # result in a TypeError when trying to assign it to the `ptr` property.
@@ -68,14 +72,16 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         bad_ptrs = (5, ctypes.c_char_p(b'foobar'))
         for bad_ptr in bad_ptrs:
             # Equivalent to `fg.ptr = bad_ptr`
-            self.assertRaises(TypeError, fg1._set_ptr, bad_ptr)
-            self.assertRaises(TypeError, fg2._set_ptr, bad_ptr)
+            with self.assertRaises(TypeError):
+                fg1._set_ptr(bad_ptr)
+            with self.assertRaises(TypeError):
+                fg2._set_ptr(bad_ptr)
 
     def test_wkt(self):
         "Testing WKT output."
         for g in self.geometries.wkt_out:
             geom = fromstr(g.wkt)
-            if geom.hasz and geos_version_info()['version'] >= '3.3.0':
+            if geom.hasz:
                 self.assertEqual(g.ewkt, geom.wkt)
 
     def test_hex(self):
@@ -106,7 +112,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # garbage into 3D coordinate if there is none.
         self.assertEqual(hexewkb_2d, pnt_2d.hexewkb)
         self.assertEqual(hexewkb_3d, pnt_3d.hexewkb)
-        self.assertEqual(True, GEOSGeometry(hexewkb_3d).hasz)
+        self.assertIs(GEOSGeometry(hexewkb_3d).hasz, True)
 
         # Same for EWKB.
         self.assertEqual(six.memoryview(a2b_hex(hexewkb_2d)), pnt_2d.ewkb)
@@ -131,15 +137,18 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 fromstr(err.wkt)
 
         # Bad WKB
-        self.assertRaises(GEOSException, GEOSGeometry, six.memoryview(b'0'))
+        with self.assertRaises(GEOSException):
+            GEOSGeometry(six.memoryview(b'0'))
 
         class NotAGeometry(object):
             pass
 
         # Some other object
-        self.assertRaises(TypeError, GEOSGeometry, NotAGeometry())
+        with self.assertRaises(TypeError):
+            GEOSGeometry(NotAGeometry())
         # None
-        self.assertRaises(TypeError, GEOSGeometry, None)
+        with self.assertRaises(TypeError):
+            GEOSGeometry(None)
 
     def test_wkb(self):
         "Testing WKB output."
@@ -226,6 +235,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             pnt = fromstr(p.wkt)
             self.assertEqual(pnt.geom_type, 'Point')
             self.assertEqual(pnt.geom_typeid, 0)
+            self.assertEqual(pnt.dims, 0)
             self.assertEqual(p.x, pnt.x)
             self.assertEqual(p.y, pnt.y)
             self.assertEqual(pnt, fromstr(p.wkt))
@@ -237,14 +247,14 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
 
             # Testing the third dimension, and getting the tuple arguments
             if hasattr(p, 'z'):
-                self.assertEqual(True, pnt.hasz)
+                self.assertIs(pnt.hasz, True)
                 self.assertEqual(p.z, pnt.z)
                 self.assertEqual(p.z, pnt.tuple[2], 9)
                 tup_args = (p.x, p.y, p.z)
                 set_tup1 = (2.71, 3.14, 5.23)
                 set_tup2 = (5.23, 2.71, 3.14)
             else:
-                self.assertEqual(False, pnt.hasz)
+                self.assertIs(pnt.hasz, False)
                 self.assertIsNone(pnt.z)
                 tup_args = (p.x, p.y)
                 set_tup1 = (2.71, 3.14)
@@ -279,18 +289,20 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             mpnt = fromstr(mp.wkt)
             self.assertEqual(mpnt.geom_type, 'MultiPoint')
             self.assertEqual(mpnt.geom_typeid, 4)
+            self.assertEqual(mpnt.dims, 0)
 
             self.assertAlmostEqual(mp.centroid[0], mpnt.centroid.tuple[0], 9)
             self.assertAlmostEqual(mp.centroid[1], mpnt.centroid.tuple[1], 9)
 
-            self.assertRaises(GEOSIndexError, mpnt.__getitem__, len(mpnt))
+            with self.assertRaises(IndexError):
+                mpnt.__getitem__(len(mpnt))
             self.assertEqual(mp.centroid, mpnt.centroid.tuple)
             self.assertEqual(mp.coords, tuple(m.tuple for m in mpnt))
             for p in mpnt:
                 self.assertEqual(p.geom_type, 'Point')
                 self.assertEqual(p.geom_typeid, 0)
-                self.assertEqual(p.empty, False)
-                self.assertEqual(p.valid, True)
+                self.assertIs(p.empty, False)
+                self.assertIs(p.valid, True)
 
     def test_linestring(self):
         "Testing LineString objects."
@@ -299,8 +311,9 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             ls = fromstr(l.wkt)
             self.assertEqual(ls.geom_type, 'LineString')
             self.assertEqual(ls.geom_typeid, 1)
-            self.assertEqual(ls.empty, False)
-            self.assertEqual(ls.ring, False)
+            self.assertEqual(ls.dims, 1)
+            self.assertIs(ls.empty, False)
+            self.assertIs(ls.ring, False)
             if hasattr(l, 'centroid'):
                 self.assertEqual(l.centroid, ls.centroid.tuple)
             if hasattr(l, 'tup'):
@@ -308,7 +321,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
 
             self.assertEqual(ls, fromstr(l.wkt))
             self.assertEqual(False, ls == prev)  # Use assertEqual to test __eq__
-            self.assertRaises(GEOSIndexError, ls.__getitem__, len(ls))
+            with self.assertRaises(IndexError):
+                ls.__getitem__(len(ls))
             prev = ls
 
             # Creating a LineString from a tuple, list, and numpy array
@@ -320,6 +334,20 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             if numpy:
                 self.assertEqual(ls, LineString(numpy.array(ls.tuple)))  # as numpy array
 
+        with self.assertRaisesMessage(TypeError, 'Each coordinate should be a sequence (list or tuple)'):
+            LineString((0, 0))
+
+        with self.assertRaisesMessage(ValueError, 'LineString requires at least 2 points, got 1.'):
+            LineString([(0, 0)])
+
+        if numpy:
+            with self.assertRaisesMessage(ValueError, 'LineString requires at least 2 points, got 1.'):
+                LineString(numpy.array([(0, 0)]))
+
+        with mock.patch('django.contrib.gis.geos.linestring.numpy', False):
+            with self.assertRaisesMessage(TypeError, 'Invalid initialization input for LineStrings.'):
+                LineString('wrong input')
+
     def test_multilinestring(self):
         "Testing MultiLineString objects."
         prev = fromstr('POINT(0 0)')
@@ -327,6 +355,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             ml = fromstr(l.wkt)
             self.assertEqual(ml.geom_type, 'MultiLineString')
             self.assertEqual(ml.geom_typeid, 5)
+            self.assertEqual(ml.dims, 1)
 
             self.assertAlmostEqual(l.centroid[0], ml.centroid.x, 9)
             self.assertAlmostEqual(l.centroid[1], ml.centroid.y, 9)
@@ -338,9 +367,10 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             for ls in ml:
                 self.assertEqual(ls.geom_type, 'LineString')
                 self.assertEqual(ls.geom_typeid, 1)
-                self.assertEqual(ls.empty, False)
+                self.assertIs(ls.empty, False)
 
-            self.assertRaises(GEOSIndexError, ml.__getitem__, len(ml))
+            with self.assertRaises(IndexError):
+                ml.__getitem__(len(ml))
             self.assertEqual(ml.wkt, MultiLineString(*tuple(s.clone() for s in ml)).wkt)
             self.assertEqual(ml, MultiLineString(*tuple(LineString(s.tuple) for s in ml)))
 
@@ -350,9 +380,10 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             lr = fromstr(rr.wkt)
             self.assertEqual(lr.geom_type, 'LinearRing')
             self.assertEqual(lr.geom_typeid, 2)
+            self.assertEqual(lr.dims, 1)
             self.assertEqual(rr.n_p, len(lr))
-            self.assertEqual(True, lr.valid)
-            self.assertEqual(False, lr.empty)
+            self.assertIs(lr.valid, True)
+            self.assertIs(lr.empty, False)
 
             # Creating a LinearRing from a tuple, list, and numpy array
             self.assertEqual(lr, LinearRing(lr.tuple))
@@ -361,6 +392,16 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             if numpy:
                 self.assertEqual(lr, LinearRing(numpy.array(lr.tuple)))
 
+        with self.assertRaisesMessage(ValueError, 'LinearRing requires at least 4 points, got 3.'):
+            LinearRing((0, 0), (1, 1), (0, 0))
+
+        with self.assertRaisesMessage(ValueError, 'LinearRing requires at least 4 points, got 1.'):
+            LinearRing([(0, 0)])
+
+        if numpy:
+            with self.assertRaisesMessage(ValueError, 'LinearRing requires at least 4 points, got 1.'):
+                LinearRing(numpy.array([(0, 0)]))
+
     def test_polygons_from_bbox(self):
         "Testing `from_bbox` class method."
         bbox = (-180, -90, 180, 90)
@@ -383,8 +424,9 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             poly = fromstr(p.wkt)
             self.assertEqual(poly.geom_type, 'Polygon')
             self.assertEqual(poly.geom_typeid, 3)
-            self.assertEqual(poly.empty, False)
-            self.assertEqual(poly.ring, False)
+            self.assertEqual(poly.dims, 2)
+            self.assertIs(poly.empty, False)
+            self.assertIs(poly.ring, False)
             self.assertEqual(p.n_i, poly.num_interior_rings)
             self.assertEqual(p.n_i + 1, len(poly))  # Testing __len__
             self.assertEqual(p.n_p, poly.num_points)
@@ -409,9 +451,12 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.assertEqual(p.ext_ring_cs, poly[0].tuple)  # Testing __getitem__
 
             # Testing __getitem__ and __setitem__ on invalid indices
-            self.assertRaises(GEOSIndexError, poly.__getitem__, len(poly))
-            self.assertRaises(GEOSIndexError, poly.__setitem__, len(poly), False)
-            self.assertRaises(GEOSIndexError, poly.__getitem__, -1 * len(poly) - 1)
+            with self.assertRaises(IndexError):
+                poly.__getitem__(len(poly))
+            with self.assertRaises(IndexError):
+                poly.__setitem__(len(poly), False)
+            with self.assertRaises(IndexError):
+                poly.__getitem__(-1 * len(poly) - 1)
 
             # Testing __iter__
             for r in poly:
@@ -419,8 +464,10 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                 self.assertEqual(r.geom_typeid, 2)
 
             # Testing polygon construction.
-            self.assertRaises(TypeError, Polygon, 0, [1, 2, 3])
-            self.assertRaises(TypeError, Polygon, 'foo')
+            with self.assertRaises(TypeError):
+                Polygon(0, [1, 2, 3])
+            with self.assertRaises(TypeError):
+                Polygon('foo')
 
             # Polygon(shell, (hole1, ... holeN))
             rings = tuple(r for r in poly)
@@ -434,6 +481,14 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(poly.wkt, Polygon(*tuple(r for r in poly)).wkt)
             self.assertEqual(poly.wkt, Polygon(*tuple(LinearRing(r.tuple) for r in poly)).wkt)
 
+    def test_polygons_templates(self):
+        # Accessing Polygon attributes in templates should work.
+        engine = Engine()
+        template = engine.from_string('{{ polygons.0.wkt }}')
+        polygons = [fromstr(p.wkt) for p in self.geometries.multipolygons[:2]]
+        content = template.render(Context({'polygons': polygons}))
+        self.assertIn('MULTIPOLYGON (((100', content)
+
     def test_polygon_comparison(self):
         p1 = Polygon(((0, 0), (0, 1), (1, 1), (1, 0), (0, 0)))
         p2 = Polygon(((0, 0), (0, 1), (1, 0), (0, 0)))
@@ -452,17 +507,19 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             mpoly = fromstr(mp.wkt)
             self.assertEqual(mpoly.geom_type, 'MultiPolygon')
             self.assertEqual(mpoly.geom_typeid, 6)
+            self.assertEqual(mpoly.dims, 2)
             self.assertEqual(mp.valid, mpoly.valid)
 
             if mp.valid:
                 self.assertEqual(mp.num_geom, mpoly.num_geom)
                 self.assertEqual(mp.n_p, mpoly.num_coords)
                 self.assertEqual(mp.num_geom, len(mpoly))
-                self.assertRaises(GEOSIndexError, mpoly.__getitem__, len(mpoly))
+                with self.assertRaises(IndexError):
+                    mpoly.__getitem__(len(mpoly))
                 for p in mpoly:
                     self.assertEqual(p.geom_type, 'Polygon')
                     self.assertEqual(p.geom_typeid, 3)
-                    self.assertEqual(p.valid, True)
+                    self.assertIs(p.valid, True)
                 self.assertEqual(mpoly.wkt, MultiPolygon(*tuple(poly.clone() for poly in mpoly)).wkt)
 
     def test_memory_hijinks(self):
@@ -521,7 +578,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
     def test_relate_pattern(self):
         "Testing relate() and relate_pattern()."
         g = fromstr('POINT (0 0)')
-        self.assertRaises(GEOSException, g.relate_pattern, 0, 'invalid pattern, yo')
+        with self.assertRaises(GEOSException):
+            g.relate_pattern(0, 'invalid pattern, yo')
         for rg in self.geometries.relate_geoms:
             a = fromstr(rg.wkt_a)
             b = fromstr(rg.wkt_b)
@@ -534,7 +592,7 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             a = fromstr(self.geometries.topology_geoms[i].wkt_a)
             b = fromstr(self.geometries.topology_geoms[i].wkt_b)
             i1 = fromstr(self.geometries.intersect_geoms[i].wkt)
-            self.assertEqual(True, a.intersects(b))
+            self.assertIs(a.intersects(b), True)
             i2 = a.intersection(b)
             self.assertEqual(i1, i2)
             self.assertEqual(i1, a & b)  # __and__ is intersection operator
@@ -553,6 +611,15 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             a |= b  # testing __ior__
             self.assertEqual(u1, a)
 
+    def test_unary_union(self):
+        "Testing unary_union."
+        for i in range(len(self.geometries.topology_geoms)):
+            a = fromstr(self.geometries.topology_geoms[i].wkt_a)
+            b = fromstr(self.geometries.topology_geoms[i].wkt_b)
+            u1 = fromstr(self.geometries.union_geoms[i].wkt)
+            u2 = GeometryCollection(a, b).unary_union
+            self.assertTrue(u1.equals(u2))
+
     def test_difference(self):
         "Testing difference()."
         for i in range(len(self.geometries.topology_geoms)):
@@ -588,7 +655,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             width = bg.width
 
             # Can't use a floating-point for the number of quadsegs.
-            self.assertRaises(ctypes.ArgumentError, g.buffer, width, float(quadsegs))
+            with self.assertRaises(ctypes.ArgumentError):
+                g.buffer(width, float(quadsegs))
 
             # Constructing our buffer
             buf = g.buffer(width, quadsegs)
@@ -605,6 +673,25 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
                     self.assertAlmostEqual(exp_ring[k][0], buf_ring[k][0], 9)
                     self.assertAlmostEqual(exp_ring[k][1], buf_ring[k][1], 9)
 
+    def test_covers(self):
+        poly = Polygon(((0, 0), (0, 10), (10, 10), (10, 0), (0, 0)))
+        self.assertTrue(poly.covers(Point(5, 5)))
+        self.assertFalse(poly.covers(Point(100, 100)))
+
+    def test_closed(self):
+        ls_closed = LineString((0, 0), (1, 1), (0, 0))
+        ls_not_closed = LineString((0, 0), (1, 1))
+        self.assertFalse(ls_not_closed.closed)
+        self.assertTrue(ls_closed.closed)
+
+        if geos_version_info()['version'] >= '3.5':
+            self.assertFalse(MultiLineString(ls_closed, ls_not_closed).closed)
+            self.assertTrue(MultiLineString(ls_closed, ls_closed).closed)
+
+        with mock.patch('django.contrib.gis.geos.collections.geos_version_info', lambda: {'version': '3.4.9'}):
+            with self.assertRaisesMessage(GEOSException, "MultiLineString.closed requires GEOS >= 3.5.0."):
+                MultiLineString().closed
+
     def test_srid(self):
         "Testing the SRID property and keyword."
         # Testing SRID keyword on Point
@@ -612,7 +699,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         self.assertEqual(4326, pnt.srid)
         pnt.srid = 3084
         self.assertEqual(3084, pnt.srid)
-        self.assertRaises(ctypes.ArgumentError, pnt.set_srid, '4326')
+        with self.assertRaises(ctypes.ArgumentError):
+            pnt.srid = '4326'
 
         # Testing SRID keyword on fromstr(), and on Polygon rings.
         poly = fromstr(self.geometries.polygons[1].wkt, srid=4269)
@@ -640,25 +728,30 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         p3 = fromstr(p1.hex, srid=-1)  # -1 is intended.
         self.assertEqual(-1, p3.srid)
 
+        # Testing that geometry SRID could be set to its own value
+        pnt_wo_srid = Point(1, 1)
+        pnt_wo_srid.srid = pnt_wo_srid.srid
+
     @skipUnless(HAS_GDAL, "GDAL is required.")
     def test_custom_srid(self):
-        """ Test with a srid unknown from GDAL """
-        pnt = Point(111200, 220900, srid=999999)
-        self.assertTrue(pnt.ewkt.startswith("SRID=999999;POINT (111200.0"))
-        self.assertIsInstance(pnt.ogr, gdal.OGRGeometry)
-        self.assertIsNone(pnt.srs)
+        """Test with a null srid and a srid unknown to GDAL."""
+        for srid in [None, 999999]:
+            pnt = Point(111200, 220900, srid=srid)
+            self.assertTrue(pnt.ewkt.startswith(("SRID=%s;" % srid if srid else '') + "POINT (111200"))
+            self.assertIsInstance(pnt.ogr, gdal.OGRGeometry)
+            self.assertIsNone(pnt.srs)
 
-        # Test conversion from custom to a known srid
-        c2w = gdal.CoordTransform(
-            gdal.SpatialReference(
-                '+proj=mill +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +R_A +ellps=WGS84 '
-                '+datum=WGS84 +units=m +no_defs'
-            ),
-            gdal.SpatialReference(4326))
-        new_pnt = pnt.transform(c2w, clone=True)
-        self.assertEqual(new_pnt.srid, 4326)
-        self.assertAlmostEqual(new_pnt.x, 1, 3)
-        self.assertAlmostEqual(new_pnt.y, 2, 3)
+            # Test conversion from custom to a known srid
+            c2w = gdal.CoordTransform(
+                gdal.SpatialReference(
+                    '+proj=mill +lat_0=0 +lon_0=0 +x_0=0 +y_0=0 +R_A +ellps=WGS84 '
+                    '+datum=WGS84 +units=m +no_defs'
+                ),
+                gdal.SpatialReference(4326))
+            new_pnt = pnt.transform(c2w, clone=True)
+            self.assertEqual(new_pnt.srid, 4326)
+            self.assertAlmostEqual(new_pnt.x, 1, 3)
+            self.assertAlmostEqual(new_pnt.y, 2, 3)
 
     def test_mutable_geometries(self):
         "Testing the mutability of Polygons and Geometry Collections."
@@ -667,7 +760,8 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             poly = fromstr(p.wkt)
 
             # Should only be able to use __setitem__ with LinearRing geometries.
-            self.assertRaises(TypeError, poly.__setitem__, 0, LineString((1, 1), (2, 2)))
+            with self.assertRaises(TypeError):
+                poly.__setitem__(0, LineString((1, 1), (2, 2)))
 
             # Constructing the new shell by adding 500 to every point in the old shell.
             shell_tup = poly.shell.tuple
@@ -726,19 +820,81 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         # self.assertEqual((3.14, 2.71), mpoly[0].shell[0])
         # del mpoly
 
+    def test_point_list_assignment(self):
+        p = Point(0, 0)
+
+        p[:] = (1, 2, 3)
+        self.assertEqual(p, Point(1, 2, 3))
+
+        p[:] = ()
+        self.assertEqual(p.wkt, Point())
+
+        p[:] = (1, 2)
+        self.assertEqual(p.wkt, Point(1, 2))
+
+        with self.assertRaises(ValueError):
+            p[:] = (1,)
+        with self.assertRaises(ValueError):
+            p[:] = (1, 2, 3, 4, 5)
+
+    def test_linestring_list_assignment(self):
+        ls = LineString((0, 0), (1, 1))
+
+        ls[:] = ()
+        self.assertEqual(ls, LineString())
+
+        ls[:] = ((0, 0), (1, 1), (2, 2))
+        self.assertEqual(ls, LineString((0, 0), (1, 1), (2, 2)))
+
+        with self.assertRaises(ValueError):
+            ls[:] = (1,)
+
+    def test_linearring_list_assignment(self):
+        ls = LinearRing((0, 0), (0, 1), (1, 1), (0, 0))
+
+        ls[:] = ()
+        self.assertEqual(ls, LinearRing())
+
+        ls[:] = ((0, 0), (0, 1), (1, 1), (1, 0), (0, 0))
+        self.assertEqual(ls, LinearRing((0, 0), (0, 1), (1, 1), (1, 0), (0, 0)))
+
+        with self.assertRaises(ValueError):
+            ls[:] = ((0, 0), (1, 1), (2, 2))
+
+    def test_polygon_list_assignment(self):
+        pol = Polygon()
+
+        pol[:] = (((0, 0), (0, 1), (1, 1), (1, 0), (0, 0)),)
+        self.assertEqual(pol, Polygon(((0, 0), (0, 1), (1, 1), (1, 0), (0, 0)),))
+
+        pol[:] = ()
+        self.assertEqual(pol, Polygon())
+
+    def test_geometry_collection_list_assignment(self):
+        p = Point()
+        gc = GeometryCollection()
+
+        gc[:] = [p]
+        self.assertEqual(gc, GeometryCollection(p))
+
+        gc[:] = ()
+        self.assertEqual(gc, GeometryCollection())
+
     def test_threed(self):
         "Testing three-dimensional geometries."
         # Testing a 3D Point
         pnt = Point(2, 3, 8)
         self.assertEqual((2., 3., 8.), pnt.coords)
-        self.assertRaises(TypeError, pnt.set_coords, (1., 2.))
+        with self.assertRaises(TypeError):
+            pnt.tuple = (1., 2.)
         pnt.coords = (1., 2., 3.)
         self.assertEqual((1., 2., 3.), pnt.coords)
 
         # Testing a 3D LineString
         ls = LineString((2., 3., 8.), (50., 250., -117.))
         self.assertEqual(((2., 3., 8.), (50., 250., -117.)), ls.tuple)
-        self.assertRaises(TypeError, ls.__setitem__, 0, (1., 2.))
+        with self.assertRaises(TypeError):
+            ls.__setitem__(0, (1., 2.))
         ls[0] = (1., 2., 3.)
         self.assertEqual((1., 2., 3.), ls[0])
 
@@ -780,17 +936,28 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
 
     def test_emptyCollections(self):
         "Testing empty geometries and collections."
-        gc1 = GeometryCollection([])
-        gc2 = fromstr('GEOMETRYCOLLECTION EMPTY')
-        pnt = fromstr('POINT EMPTY')
-        ls = fromstr('LINESTRING EMPTY')
-        poly = fromstr('POLYGON EMPTY')
-        mls = fromstr('MULTILINESTRING EMPTY')
-        mpoly1 = fromstr('MULTIPOLYGON EMPTY')
-        mpoly2 = MultiPolygon(())
+        geoms = [
+            GeometryCollection([]),
+            fromstr('GEOMETRYCOLLECTION EMPTY'),
+            GeometryCollection(),
+            fromstr('POINT EMPTY'),
+            Point(),
+            fromstr('LINESTRING EMPTY'),
+            LineString(),
+            fromstr('POLYGON EMPTY'),
+            Polygon(),
+            fromstr('MULTILINESTRING EMPTY'),
+            MultiLineString(),
+            fromstr('MULTIPOLYGON EMPTY'),
+            MultiPolygon(()),
+            MultiPolygon(),
+        ]
 
-        for g in [gc1, gc2, pnt, ls, poly, mls, mpoly1, mpoly2]:
-            self.assertEqual(True, g.empty)
+        if numpy:
+            geoms.append(LineString(numpy.array([])))
+
+        for g in geoms:
+            self.assertIs(g.empty, True)
 
             # Testing len() and num_geom.
             if isinstance(g, Polygon):
@@ -806,15 +973,31 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
 
             # Testing __getitem__ (doesn't work on Point or Polygon)
             if isinstance(g, Point):
-                self.assertRaises(GEOSIndexError, g.get_x)
+                with self.assertRaises(IndexError):
+                    g.x
             elif isinstance(g, Polygon):
                 lr = g.shell
                 self.assertEqual('LINEARRING EMPTY', lr.wkt)
                 self.assertEqual(0, len(lr))
-                self.assertEqual(True, lr.empty)
-                self.assertRaises(GEOSIndexError, lr.__getitem__, 0)
+                self.assertIs(lr.empty, True)
+                with self.assertRaises(IndexError):
+                    lr.__getitem__(0)
             else:
-                self.assertRaises(GEOSIndexError, g.__getitem__, 0)
+                with self.assertRaises(IndexError):
+                    g.__getitem__(0)
+
+    def test_collection_dims(self):
+        gc = GeometryCollection([])
+        self.assertEqual(gc.dims, -1)
+
+        gc = GeometryCollection(Point(0, 0))
+        self.assertEqual(gc.dims, 0)
+
+        gc = GeometryCollection(LineString((0, 0), (1, 1)), Point(0, 0))
+        self.assertEqual(gc.dims, 1)
+
+        gc = GeometryCollection(LineString((0, 0), (1, 1)), Polygon(((0, 0), (0, 1), (1, 1), (0, 0))), Point(0, 0))
+        self.assertEqual(gc.dims, 2)
 
     def test_collections_of_collections(self):
         "Testing GeometryCollection handling of other collections."
@@ -897,7 +1080,19 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         """ Testing `transform` method (SRID match) """
         # transform() should no-op if source & dest SRIDs match,
         # regardless of whether GDAL is available.
-        if gdal.HAS_GDAL:
+        g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
+        gt = g.tuple
+        g.transform(4326)
+        self.assertEqual(g.tuple, gt)
+        self.assertEqual(g.srid, 4326)
+
+        g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
+        g1 = g.transform(4326, clone=True)
+        self.assertEqual(g1.tuple, g.tuple)
+        self.assertEqual(g1.srid, 4326)
+        self.assertIsNot(g1, g, "Clone didn't happen")
+
+        with mock.patch('django.contrib.gis.gdal.HAS_GDAL', False):
             g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
             gt = g.tuple
             g.transform(4326)
@@ -910,53 +1105,36 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(g1.srid, 4326)
             self.assertIsNot(g1, g, "Clone didn't happen")
 
-        old_has_gdal = gdal.HAS_GDAL
-        try:
-            gdal.HAS_GDAL = False
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
-            gt = g.tuple
-            g.transform(4326)
-            self.assertEqual(g.tuple, gt)
-            self.assertEqual(g.srid, 4326)
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
-            g1 = g.transform(4326, clone=True)
-            self.assertEqual(g1.tuple, g.tuple)
-            self.assertEqual(g1.srid, 4326)
-            self.assertIsNot(g1, g, "Clone didn't happen")
-        finally:
-            gdal.HAS_GDAL = old_has_gdal
-
+    @skipUnless(HAS_GDAL, "GDAL is required.")
     def test_transform_nosrid(self):
         """ Testing `transform` method (no SRID or negative SRID) """
 
         g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
-        self.assertRaises(GEOSException, g.transform, 2774)
+        with self.assertRaises(GEOSException):
+            g.transform(2774)
 
         g = GEOSGeometry('POINT (-104.609 38.255)', srid=None)
-        self.assertRaises(GEOSException, g.transform, 2774, clone=True)
+        with self.assertRaises(GEOSException):
+            g.transform(2774, clone=True)
 
         g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
-        self.assertRaises(GEOSException, g.transform, 2774)
+        with self.assertRaises(GEOSException):
+            g.transform(2774)
 
         g = GEOSGeometry('POINT (-104.609 38.255)', srid=-1)
-        self.assertRaises(GEOSException, g.transform, 2774, clone=True)
+        with self.assertRaises(GEOSException):
+            g.transform(2774, clone=True)
 
-    @skipUnless(HAS_GDAL, "GDAL is required.")
+    @mock.patch('django.contrib.gis.gdal.HAS_GDAL', False)
     def test_transform_nogdal(self):
         """ Testing `transform` method (GDAL not available) """
-        old_has_gdal = gdal.HAS_GDAL
-        try:
-            gdal.HAS_GDAL = False
+        g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
+        with self.assertRaises(GEOSException):
+            g.transform(2774)
 
-            g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
-            self.assertRaises(GEOSException, g.transform, 2774)
-
-            g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
-            self.assertRaises(GEOSException, g.transform, 2774, clone=True)
-        finally:
-            gdal.HAS_GDAL = old_has_gdal
+        g = GEOSGeometry('POINT (-104.609 38.255)', 4326)
+        with self.assertRaises(GEOSException):
+            g.transform(2774, clone=True)
 
     def test_extent(self):
         "Testing `extent` method."
@@ -1004,22 +1182,20 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
 
         # A set of test points.
         pnts = [Point(5, 5), Point(7.5, 7.5), Point(2.5, 7.5)]
-        covers = [True, True, False]  # No `covers` op for regular GEOS geoms.
-        for pnt, c in zip(pnts, covers):
+        for pnt in pnts:
             # Results should be the same (but faster)
             self.assertEqual(mpoly.contains(pnt), prep.contains(pnt))
             self.assertEqual(mpoly.intersects(pnt), prep.intersects(pnt))
-            self.assertEqual(c, prep.covers(pnt))
+            self.assertEqual(mpoly.covers(pnt), prep.covers(pnt))
 
-        if geos_version_info()['version'] > '3.3.0':
-            self.assertTrue(prep.crosses(fromstr('LINESTRING(1 1, 15 15)')))
-            self.assertTrue(prep.disjoint(Point(-5, -5)))
-            poly = Polygon(((-1, -1), (1, 1), (1, 0), (-1, -1)))
-            self.assertTrue(prep.overlaps(poly))
-            poly = Polygon(((-5, 0), (-5, 5), (0, 5), (-5, 0)))
-            self.assertTrue(prep.touches(poly))
-            poly = Polygon(((-1, -1), (-1, 11), (11, 11), (11, -1), (-1, -1)))
-            self.assertTrue(prep.within(poly))
+        self.assertTrue(prep.crosses(fromstr('LINESTRING(1 1, 15 15)')))
+        self.assertTrue(prep.disjoint(Point(-5, -5)))
+        poly = Polygon(((-1, -1), (1, 1), (1, 0), (-1, -1)))
+        self.assertTrue(prep.overlaps(poly))
+        poly = Polygon(((-5, 0), (-5, 5), (0, 5), (-5, 0)))
+        self.assertTrue(prep.touches(poly))
+        poly = Polygon(((-1, -1), (-1, 11), (11, 11), (11, -1), (-1, -1)))
+        self.assertTrue(prep.within(poly))
 
         # Original geometry deletion should not crash the prepared one (#21662)
         del mpoly
@@ -1050,7 +1226,6 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         self.assertIsInstance(g.valid_reason, six.string_types)
         self.assertTrue(g.valid_reason.startswith("Too few points in geometry component"))
 
-    @skipUnless(HAS_GEOS, "Geos is required.")
     def test_linearref(self):
         "Testing linear referencing"
 
@@ -1071,6 +1246,71 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
         self.assertEqual(mls.interpolate(9), Point(0, 9))
         self.assertEqual(mls.interpolate(17), Point(10, 7))
 
+    def test_deconstructible(self):
+        """
+        Geometry classes should be deconstructible.
+        """
+        point = Point(4.337844, 50.827537, srid=4326)
+        path, args, kwargs = point.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.point.Point')
+        self.assertEqual(args, (4.337844, 50.827537))
+        self.assertEqual(kwargs, {'srid': 4326})
+
+        ls = LineString(((0, 0), (1, 1)))
+        path, args, kwargs = ls.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.linestring.LineString')
+        self.assertEqual(args, (((0, 0), (1, 1)),))
+        self.assertEqual(kwargs, {})
+
+        ls2 = LineString([Point(0, 0), Point(1, 1)], srid=4326)
+        path, args, kwargs = ls2.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.linestring.LineString')
+        self.assertEqual(args, ([Point(0, 0), Point(1, 1)],))
+        self.assertEqual(kwargs, {'srid': 4326})
+
+        ext_coords = ((0, 0), (0, 1), (1, 1), (1, 0), (0, 0))
+        int_coords = ((0.4, 0.4), (0.4, 0.6), (0.6, 0.6), (0.6, 0.4), (0.4, 0.4))
+        poly = Polygon(ext_coords, int_coords)
+        path, args, kwargs = poly.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.polygon.Polygon')
+        self.assertEqual(args, (ext_coords, int_coords))
+        self.assertEqual(kwargs, {})
+
+        lr = LinearRing((0, 0), (0, 1), (1, 1), (0, 0))
+        path, args, kwargs = lr.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.linestring.LinearRing')
+        self.assertEqual(args, ((0, 0), (0, 1), (1, 1), (0, 0)))
+        self.assertEqual(kwargs, {})
+
+        mp = MultiPoint(Point(0, 0), Point(1, 1))
+        path, args, kwargs = mp.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.collections.MultiPoint')
+        self.assertEqual(args, (Point(0, 0), Point(1, 1)))
+        self.assertEqual(kwargs, {})
+
+        ls1 = LineString((0, 0), (1, 1))
+        ls2 = LineString((2, 2), (3, 3))
+        mls = MultiLineString(ls1, ls2)
+        path, args, kwargs = mls.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.collections.MultiLineString')
+        self.assertEqual(args, (ls1, ls2))
+        self.assertEqual(kwargs, {})
+
+        p1 = Polygon(((0, 0), (0, 1), (1, 1), (0, 0)))
+        p2 = Polygon(((1, 1), (1, 2), (2, 2), (1, 1)))
+        mp = MultiPolygon(p1, p2)
+        path, args, kwargs = mp.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.collections.MultiPolygon')
+        self.assertEqual(args, (p1, p2, ))
+        self.assertEqual(kwargs, {})
+
+        poly = Polygon(((0, 0), (0, 1), (1, 1), (0, 0)))
+        gc = GeometryCollection(Point(0, 0), MultiPoint(Point(0, 0), Point(1, 1)), poly)
+        path, args, kwargs = gc.deconstruct()
+        self.assertEqual(path, 'django.contrib.gis.geos.collections.GeometryCollection')
+        self.assertEqual(args, (Point(0, 0), MultiPoint(Point(0, 0), Point(1, 1)), poly))
+        self.assertEqual(kwargs, {})
+
     def test_geos_version(self):
         """Testing the GEOS version regular expression."""
         from django.contrib.gis.geos.libgeos import version_regex
@@ -1083,3 +1323,35 @@ class GEOSTest(unittest.TestCase, TestDataMixin):
             self.assertTrue(m, msg="Unable to parse the version string '%s'" % v_init)
             self.assertEqual(m.group('version'), v_geos)
             self.assertEqual(m.group('capi_version'), v_capi)
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    def test_deprecated_srid_getters_setters(self):
+        p = Point(1, 2, srid=123)
+        self.assertEqual(p.get_srid(), p.srid)
+
+        p.set_srid(321)
+        self.assertEqual(p.srid, 321)
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    def test_deprecated_point_coordinate_getters_setters(self):
+        p = Point(1, 2, 3)
+        self.assertEqual((p.get_x(), p.get_y(), p.get_z()), (p.x, p.y, p.z))
+
+        p.set_x(3)
+        p.set_y(2)
+        p.set_z(1)
+        self.assertEqual((p.x, p.y, p.z), (3, 2, 1))
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    def test_deprecated_point_tuple_getters_setters(self):
+        p = Point(1, 2, 3)
+        self.assertEqual(p.get_coords(), (p.x, p.y, p.z))
+
+        p.set_coords((3, 2, 1))
+        self.assertEqual(p.get_coords(), (3, 2, 1))
+
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    def test_deprecated_cascaded_union(self):
+        for geom in self.geometries.multipolygons:
+            mpoly = GEOSGeometry(geom.wkt)
+            self.assertEqual(mpoly.cascaded_union, mpoly.unary_union)
