@@ -2,14 +2,15 @@ from __future__ import unicode_literals
 
 import os
 
+from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.core.urlresolvers import (
-    clear_url_caches, reverse, set_script_prefix,
-)
 from django.http import HttpResponsePermanentRedirect
 from django.middleware.locale import LocaleMiddleware
 from django.template import Context, Template
-from django.test import TestCase, override_settings
+from django.test import SimpleTestCase, override_settings
+from django.test.client import RequestFactory
+from django.test.utils import override_script_prefix
+from django.urls import clear_url_caches, reverse, translate_url
 from django.utils import translation
 from django.utils._os import upath
 
@@ -20,19 +21,19 @@ class PermanentRedirectLocaleMiddleWare(LocaleMiddleware):
 
 @override_settings(
     USE_I18N=True,
-    LOCALE_PATHS=(
+    LOCALE_PATHS=[
         os.path.join(os.path.dirname(upath(__file__)), 'locale'),
-    ),
+    ],
     LANGUAGE_CODE='en-us',
-    LANGUAGES=(
+    LANGUAGES=[
         ('nl', 'Dutch'),
         ('en', 'English'),
         ('pt-br', 'Brazilian Portuguese'),
-    ),
-    MIDDLEWARE_CLASSES=(
+    ],
+    MIDDLEWARE=[
         'django.middleware.locale.LocaleMiddleware',
         'django.middleware.common.CommonMiddleware',
-    ),
+    ],
     ROOT_URLCONF='i18n.patterns.urls.default',
     TEMPLATES=[{
         'BACKEND': 'django.template.backends.django.DjangoTemplates',
@@ -44,7 +45,7 @@ class PermanentRedirectLocaleMiddleWare(LocaleMiddleware):
         },
     }],
 )
-class URLTestCaseBase(TestCase):
+class URLTestCaseBase(SimpleTestCase):
     """
     TestCase base-class for the URL tests.
     """
@@ -75,10 +76,13 @@ class URLPrefixTests(URLTestCaseBase):
             self.assertEqual(reverse('prefixed'), '/en/prefixed/')
         with translation.override('nl'):
             self.assertEqual(reverse('prefixed'), '/nl/prefixed/')
+        with translation.override(None):
+            self.assertEqual(reverse('prefixed'), '/%s/prefixed/' % settings.LANGUAGE_CODE)
 
     @override_settings(ROOT_URLCONF='i18n.patterns.urls.wrong')
     def test_invalid_prefix_use(self):
-        self.assertRaises(ImproperlyConfigured, lambda: reverse('account:register'))
+        with self.assertRaises(ImproperlyConfigured):
+            reverse('account:register')
 
 
 @override_settings(ROOT_URLCONF='i18n.patterns.urls.disabled')
@@ -92,10 +96,22 @@ class URLDisabledTests(URLTestCaseBase):
             self.assertEqual(reverse('prefixed'), '/prefixed/')
 
 
+class RequestURLConfTests(SimpleTestCase):
+
+    @override_settings(ROOT_URLCONF='i18n.patterns.urls.path_unused')
+    def test_request_urlconf_considered(self):
+        request = RequestFactory().get('/nl/')
+        request.urlconf = 'i18n.patterns.urls.default'
+        middleware = LocaleMiddleware()
+        with translation.override('nl'):
+            middleware.process_request(request)
+        self.assertEqual(request.LANGUAGE_CODE, 'nl')
+
+
 @override_settings(ROOT_URLCONF='i18n.patterns.urls.path_unused')
 class PathUnusedTests(URLTestCaseBase):
     """
-    Check that if no i18n_patterns is used in root urlconfs, then no
+    Check that if no i18n_patterns is used in root URLconfs, then no
     language activation happens based on url prefix.
     """
 
@@ -135,6 +151,18 @@ class URLTranslationTests(URLTestCaseBase):
         with translation.override('pt-br'):
             self.assertEqual(reverse('users'), '/pt-br/usuarios/')
 
+    def test_translate_url_utility(self):
+        with translation.override('en'):
+            self.assertEqual(translate_url('/en/non-existent/', 'nl'), '/en/non-existent/')
+            self.assertEqual(translate_url('/en/users/', 'nl'), '/nl/gebruikers/')
+            # Namespaced URL
+            self.assertEqual(translate_url('/en/account/register/', 'nl'), '/nl/profiel/registeren/')
+            self.assertEqual(translation.get_language(), 'en')
+
+        with translation.override('nl'):
+            self.assertEqual(translate_url('/nl/gebruikers/', 'en'), '/en/users/')
+            self.assertEqual(translation.get_language(), 'nl')
+
 
 class URLNamespaceTests(URLTestCaseBase):
     """
@@ -195,10 +223,10 @@ class URLRedirectTests(URLTestCaseBase):
         self.assertEqual(response.status_code, 200)
 
     @override_settings(
-        MIDDLEWARE_CLASSES=(
+        MIDDLEWARE=[
             'i18n.patterns.tests.PermanentRedirectLocaleMiddleWare',
             'django.middleware.common.CommonMiddleware',
-        ),
+        ],
     )
     def test_custom_redirect_class(self):
         response = self.client.get('/account/register/', HTTP_ACCEPT_LANGUAGE='en')
@@ -236,8 +264,8 @@ class URLRedirectWithoutTrailingSlashTests(URLTestCaseBase):
 
     def test_en_redirect(self):
         response = self.client.get('/account/register', HTTP_ACCEPT_LANGUAGE='en', follow=True)
-        # target status code of 301 because of CommonMiddleware redirecting
-        self.assertIn(('http://testserver/en/account/register/', 301), response.redirect_chain)
+        # We only want one redirect, bypassing CommonMiddleware
+        self.assertListEqual(response.redirect_chain, [('/en/account/register/', 302)])
         self.assertRedirects(response, '/en/account/register/', 302)
 
         response = self.client.get('/prefixed.xml', HTTP_ACCEPT_LANGUAGE='en', follow=True)
@@ -302,19 +330,11 @@ class URLRedirectWithScriptAliasTests(URLTestCaseBase):
     """
     #21579 - LocaleMiddleware should respect the script prefix.
     """
-    def setUp(self):
-        super(URLRedirectWithScriptAliasTests, self).setUp()
-        self.script_prefix = '/script_prefix'
-        set_script_prefix(self.script_prefix)
-
-    def tearDown(self):
-        super(URLRedirectWithScriptAliasTests, self).tearDown()
-        # reset script prefix
-        set_script_prefix('')
-
     def test_language_prefix_with_script_prefix(self):
-        response = self.client.get('/prefixed/', HTTP_ACCEPT_LANGUAGE='en', SCRIPT_NAME=self.script_prefix)
-        self.assertRedirects(response, '%s/en/prefixed/' % self.script_prefix, target_status_code=404)
+        prefix = '/script_prefix'
+        with override_script_prefix(prefix):
+            response = self.client.get('/prefixed/', HTTP_ACCEPT_LANGUAGE='en', SCRIPT_NAME=prefix)
+            self.assertRedirects(response, '%s/en/prefixed/' % prefix, target_status_code=404)
 
 
 class URLTagTests(URLTestCaseBase):
