@@ -1,52 +1,30 @@
 from unittest import skipUnless
 
-from django.core.management.color import no_style
 from django.db import connection
-from django.test import TestCase, ignore_warnings
-from django.utils.deprecation import RemovedInDjango110Warning
+from django.db.models.deletion import CASCADE
+from django.db.models.fields.related import ForeignKey
+from django.test import TestCase, TransactionTestCase
 
 from .models import Article, ArticleTranslation, IndexTogetherSingleList
 
 
-@ignore_warnings(category=RemovedInDjango110Warning)
-class CreationIndexesTests(TestCase):
-    """
-    Test index handling by the to-be-deprecated connection.creation interface.
-    """
-    def test_index_together(self):
-        index_sql = connection.creation.sql_indexes_for_model(Article, no_style())
-        self.assertEqual(len(index_sql), 1)
-
-    def test_index_together_single_list(self):
-        # Test for using index_together with a single list (#22172)
-        index_sql = connection.creation.sql_indexes_for_model(IndexTogetherSingleList, no_style())
-        self.assertEqual(len(index_sql), 1)
-
-    @skipUnless(connection.vendor == 'postgresql',
-        "This is a postgresql-specific issue")
-    def test_postgresql_text_indexes(self):
-        """Test creation of PostgreSQL-specific text indexes (#12234)"""
-        from .models import IndexedArticle
-        index_sql = connection.creation.sql_indexes_for_model(IndexedArticle, no_style())
-        self.assertEqual(len(index_sql), 5)
-        self.assertIn('("headline" varchar_pattern_ops)', index_sql[1])
-        self.assertIn('("body" text_pattern_ops)', index_sql[3])
-        # unique=True and db_index=True should only create the varchar-specific
-        # index (#19441).
-        self.assertIn('("slug" varchar_pattern_ops)', index_sql[4])
-
-    @skipUnless(connection.vendor == 'postgresql',
-        "This is a postgresql-specific issue")
-    def test_postgresql_virtual_relation_indexes(self):
-        """Test indexes are not created for related objects"""
-        index_sql = connection.creation.sql_indexes_for_model(Article, no_style())
-        self.assertEqual(len(index_sql), 1)
-
-
 class SchemaIndexesTests(TestCase):
     """
     Test index handling by the db.backends.schema infrastructure.
     """
+
+    def test_index_name_hash(self):
+        """
+        Index names should be deterministic.
+        """
+        with connection.schema_editor() as editor:
+            index_name = editor._create_index_name(
+                model=Article,
+                column_names=("c1", "c2", "c3"),
+                suffix="123",
+            )
+        self.assertEqual(index_name, "indexes_article_c1_7ce4cc86123")
+
     def test_index_together(self):
         editor = connection.schema_editor()
         index_sql = editor._model_indexes_sql(Article)
@@ -64,8 +42,7 @@ class SchemaIndexesTests(TestCase):
         index_sql = connection.schema_editor()._model_indexes_sql(IndexTogetherSingleList)
         self.assertEqual(len(index_sql), 1)
 
-    @skipUnless(connection.vendor == 'postgresql',
-        "This is a postgresql-specific issue")
+    @skipUnless(connection.vendor == 'postgresql', "This is a postgresql-specific issue")
     def test_postgresql_text_indexes(self):
         """Test creation of PostgreSQL-specific text indexes (#12234)"""
         from .models import IndexedArticle
@@ -77,18 +54,21 @@ class SchemaIndexesTests(TestCase):
         # index (#19441).
         self.assertIn('("slug" varchar_pattern_ops)', index_sql[4])
 
-    @skipUnless(connection.vendor == 'postgresql',
-        "This is a postgresql-specific issue")
+    @skipUnless(connection.vendor == 'postgresql', "This is a postgresql-specific issue")
     def test_postgresql_virtual_relation_indexes(self):
         """Test indexes are not created for related objects"""
         index_sql = connection.schema_editor()._model_indexes_sql(Article)
         self.assertEqual(len(index_sql), 1)
 
-    @skipUnless(connection.vendor == 'mysql', "This is a mysql-specific issue")
+
+@skipUnless(connection.vendor == 'mysql', 'MySQL tests')
+class SchemaIndexesMySQLTests(TransactionTestCase):
+    available_apps = ['indexes']
+
     def test_no_index_for_foreignkey(self):
         """
         MySQL on InnoDB already creates indexes automatically for foreign keys.
-        (#14180).
+        (#14180). An index should be created if db_constraint=False (#26171).
         """
         storage = connection.introspection.get_storage_engine(
             connection.cursor(), ArticleTranslation._meta.db_table
@@ -96,4 +76,25 @@ class SchemaIndexesTests(TestCase):
         if storage != "InnoDB":
             self.skip("This test only applies to the InnoDB storage engine")
         index_sql = connection.schema_editor()._model_indexes_sql(ArticleTranslation)
-        self.assertEqual(index_sql, [])
+        self.assertEqual(index_sql, [
+            'CREATE INDEX `indexes_articletranslation_99fb53c2` '
+            'ON `indexes_articletranslation` (`article_no_constraint_id`)'
+        ])
+
+        # The index also shouldn't be created if the ForeignKey is added after
+        # the model was created.
+        field_created = False
+        try:
+            with connection.schema_editor() as editor:
+                new_field = ForeignKey(Article, CASCADE)
+                new_field.set_attributes_from_name('new_foreign_key')
+                editor.add_field(ArticleTranslation, new_field)
+                self.assertEqual(editor.deferred_sql, [
+                    'ALTER TABLE `indexes_articletranslation` '
+                    'ADD CONSTRAINT `indexes_articl_new_foreign_key_id_d27a9146_fk_indexes_article_id` '
+                    'FOREIGN KEY (`new_foreign_key_id`) REFERENCES `indexes_article` (`id`)'
+                ])
+        finally:
+            if field_created:
+                with connection.schema_editor() as editor:
+                    editor.remove_field(ArticleTranslation, new_field)
