@@ -6,7 +6,8 @@ from unittest import skipUnless
 
 from django.core.management import call_command
 from django.db import connection
-from django.test import TestCase, skipUnlessDBFeature
+from django.test import TestCase, mock, skipUnlessDBFeature
+from django.utils.encoding import force_text
 from django.utils.six import PY3, StringIO
 
 from .models import ColumnTypes
@@ -27,6 +28,17 @@ class InspectDBTestCase(TestCase):
         # inspected
         self.assertNotIn("class DjangoContentType(models.Model):", out.getvalue(), msg=error_message)
 
+    def test_table_option(self):
+        """
+        inspectdb can inspect a subset of tables by passing the table names as
+        arguments.
+        """
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_people', stdout=out)
+        output = out.getvalue()
+        self.assertIn('class InspectdbPeople(models.Model):', output)
+        self.assertNotIn("InspectdbPeopledata", output)
+
     def make_field_type_asserter(self):
         """Call inspectdb and return a function to validate a field type in its output"""
         out = StringIO()
@@ -61,11 +73,9 @@ class InspectDBTestCase(TestCase):
             assertFieldType('file_field', "models.CharField(max_length=100)")
             assertFieldType('file_path_field', "models.CharField(max_length=100)")
         if connection.features.can_introspect_ip_address_field:
-            assertFieldType('ip_address_field', "models.GenericIPAddressField()")
             assertFieldType('gen_ip_adress_field', "models.GenericIPAddressField()")
         elif (connection.features.can_introspect_max_length and
                 not connection.features.interprets_empty_strings_as_nulls):
-            assertFieldType('ip_address_field', "models.CharField(max_length=15)")
             assertFieldType('gen_ip_adress_field', "models.CharField(max_length=39)")
         if (connection.features.can_introspect_max_length and
                 not connection.features.interprets_empty_strings_as_nulls):
@@ -146,15 +156,25 @@ class InspectDBTestCase(TestCase):
         output = out.getvalue()
         error_message = "inspectdb generated an attribute name which is a python keyword"
         # Recursive foreign keys should be set to 'self'
-        self.assertIn("parent = models.ForeignKey('self')", output)
-        self.assertNotIn("from = models.ForeignKey(InspectdbPeople)", output, msg=error_message)
+        self.assertIn("parent = models.ForeignKey('self', models.DO_NOTHING)", output)
+        self.assertNotIn(
+            "from = models.ForeignKey(InspectdbPeople, models.DO_NOTHING)",
+            output,
+            msg=error_message,
+        )
         # As InspectdbPeople model is defined after InspectdbMessage, it should be quoted
-        self.assertIn("from_field = models.ForeignKey('InspectdbPeople', db_column='from_id')",
-                      output)
-        self.assertIn("people_pk = models.ForeignKey(InspectdbPeople, primary_key=True)",
-                      output)
-        self.assertIn("people_unique = models.ForeignKey(InspectdbPeople, unique=True)",
-                      output)
+        self.assertIn(
+            "from_field = models.ForeignKey('InspectdbPeople', models.DO_NOTHING, db_column='from_id')",
+            output,
+        )
+        self.assertIn(
+            "people_pk = models.ForeignKey(InspectdbPeople, models.DO_NOTHING, primary_key=True)",
+            output,
+        )
+        self.assertIn(
+            "people_unique = models.ForeignKey(InspectdbPeople, models.DO_NOTHING, unique=True)",
+            output,
+        )
 
     def test_digits_column_name_introspection(self):
         """Introspection of column names consist/start with digits (#16536/#17676)"""
@@ -260,3 +280,17 @@ class InspectDBTestCase(TestCase):
             self.assertIn("big_int_field = models.BigIntegerField()", output)
         finally:
             connection.introspection.data_types_reverse = orig_data_types_reverse
+
+    def test_introspection_errors(self):
+        """
+        Introspection errors should not crash the command, and the error should
+        be visible in the output.
+        """
+        out = StringIO()
+        with mock.patch('django.db.backends.base.introspection.BaseDatabaseIntrospection.table_names',
+                        return_value=['nonexistent']):
+            call_command('inspectdb', stdout=out)
+        output = force_text(out.getvalue())
+        self.assertIn("# Unable to inspect table 'nonexistent'", output)
+        # The error message depends on the backend
+        self.assertIn("# The error was:", output)
