@@ -6,7 +6,7 @@ from django.db import connection
 from django.db.utils import DatabaseError
 from django.test import TransactionTestCase, mock, skipUnlessDBFeature
 
-from .models import Article, Reporter
+from .models import Article, ArticleReporter, City, Reporter
 
 
 class IntrospectionTests(TransactionTestCase):
@@ -16,10 +16,8 @@ class IntrospectionTests(TransactionTestCase):
     def test_table_names(self):
         tl = connection.introspection.table_names()
         self.assertEqual(tl, sorted(tl))
-        self.assertIn(Reporter._meta.db_table, tl,
-                     "'%s' isn't in table_list()." % Reporter._meta.db_table)
-        self.assertIn(Article._meta.db_table, tl,
-                     "'%s' isn't in table_list()." % Article._meta.db_table)
+        self.assertIn(Reporter._meta.db_table, tl, "'%s' isn't in table_list()." % Reporter._meta.db_table)
+        self.assertIn(Article._meta.db_table, tl, "'%s' isn't in table_list()." % Article._meta.db_table)
 
     def test_django_table_names(self):
         with connection.cursor() as cursor:
@@ -48,10 +46,12 @@ class IntrospectionTests(TransactionTestCase):
                 else:
                     raise
 
-        self.assertIn('introspection_article_view',
-                      connection.introspection.table_names(include_views=True))
-        self.assertNotIn('introspection_article_view',
-                         connection.introspection.table_names())
+        self.assertIn('introspection_article_view', connection.introspection.table_names(include_views=True))
+        self.assertNotIn('introspection_article_view', connection.introspection.table_names())
+
+    def test_unmanaged_through_model(self):
+        tables = connection.introspection.django_table_names()
+        self.assertNotIn(ArticleReporter._meta.db_table, tables)
 
     def test_installed_models(self):
         tables = [Article._meta.db_table, Reporter._meta.db_table]
@@ -61,8 +61,7 @@ class IntrospectionTests(TransactionTestCase):
     def test_sequence_list(self):
         sequences = connection.introspection.sequence_list()
         expected = {'table': Reporter._meta.db_table, 'column': 'id'}
-        self.assertIn(expected, sequences,
-                     'Reporter sequence not found in sequence_list()')
+        self.assertIn(expected, sequences, 'Reporter sequence not found in sequence_list()')
 
     def test_get_table_description_names(self):
         with connection.cursor() as cursor:
@@ -103,6 +102,12 @@ class IntrospectionTests(TransactionTestCase):
             [False, nullable_by_backend, nullable_by_backend, nullable_by_backend, True, True, False]
         )
 
+    @skipUnlessDBFeature('can_introspect_autofield')
+    def test_bigautofield(self):
+        with connection.cursor() as cursor:
+            desc = connection.introspection.get_table_description(cursor, City._meta.db_table)
+        self.assertIn('BigAutoField', [datatype(r[1], r) for r in desc])
+
     # Regression test for #9991 - 'real' types in postgres
     @skipUnlessDBFeature('has_real_datatype')
     def test_postgresql_real_type(self):
@@ -138,9 +143,13 @@ class IntrospectionTests(TransactionTestCase):
     def test_get_relations_alt_format(self):
         """With SQLite, foreign keys can be added with different syntaxes."""
         with connection.cursor() as cursor:
-            cursor.fetchone = mock.Mock(return_value=[
-                "CREATE TABLE track(id, art_id INTEGER, FOREIGN KEY(art_id) REFERENCES %s(id));" % Article._meta.db_table
-            ])
+            cursor.fetchone = mock.Mock(
+                return_value=[
+                    "CREATE TABLE track(id, art_id INTEGER, FOREIGN KEY(art_id) REFERENCES {}(id));".format(
+                        Article._meta.db_table
+                    )
+                ]
+            )
             relations = connection.introspection.get_relations(cursor, 'mocked_table')
         self.assertEqual(relations, {'art_id': ('id', Article._meta.db_table)})
 
