@@ -3,19 +3,23 @@ from __future__ import unicode_literals
 
 import logging
 import warnings
+from contextlib import contextmanager
 
 from admin_scripts.tests import AdminScriptTestCase
 
+from django.conf import settings
 from django.core import mail
 from django.core.files.temp import NamedTemporaryFile
-from django.test import RequestFactory, TestCase, override_settings
-from django.test.utils import patch_logger
+from django.core.management import color
+from django.db import connection
+from django.test import RequestFactory, SimpleTestCase, override_settings
+from django.test.utils import LoggingCaptureMixin, patch_logger
+from django.utils import six
 from django.utils.deprecation import RemovedInNextVersionWarning
-from django.utils.encoding import force_text
 from django.utils.log import (
-    AdminEmailHandler, CallbackFilter, RequireDebugFalse, RequireDebugTrue,
+    DEFAULT_LOGGING, AdminEmailHandler, CallbackFilter, RequireDebugFalse,
+    RequireDebugTrue, ServerFormatter,
 )
-from django.utils.six import StringIO
 
 from .logconfig import MyEmailBackend
 
@@ -39,7 +43,7 @@ OLD_LOGGING = {
 }
 
 
-class LoggingFiltersTest(TestCase):
+class LoggingFiltersTest(SimpleTestCase):
     def test_require_debug_false_filter(self):
         """
         Test the RequireDebugFalse filter class.
@@ -47,10 +51,10 @@ class LoggingFiltersTest(TestCase):
         filter_ = RequireDebugFalse()
 
         with self.settings(DEBUG=True):
-            self.assertEqual(filter_.filter("record is not used"), False)
+            self.assertIs(filter_.filter("record is not used"), False)
 
         with self.settings(DEBUG=False):
-            self.assertEqual(filter_.filter("record is not used"), True)
+            self.assertIs(filter_.filter("record is not used"), True)
 
     def test_require_debug_true_filter(self):
         """
@@ -59,35 +63,91 @@ class LoggingFiltersTest(TestCase):
         filter_ = RequireDebugTrue()
 
         with self.settings(DEBUG=True):
-            self.assertEqual(filter_.filter("record is not used"), True)
+            self.assertIs(filter_.filter("record is not used"), True)
 
         with self.settings(DEBUG=False):
-            self.assertEqual(filter_.filter("record is not used"), False)
+            self.assertIs(filter_.filter("record is not used"), False)
 
 
-class DefaultLoggingTest(TestCase):
-    def setUp(self):
-        self.logger = logging.getLogger('django')
-        self.old_stream = self.logger.handlers[0].stream
+class SetupDefaultLoggingMixin(object):
 
-    def tearDown(self):
-        self.logger.handlers[0].stream = self.old_stream
+    @classmethod
+    def setUpClass(cls):
+        super(SetupDefaultLoggingMixin, cls).setUpClass()
+        cls._logging = settings.LOGGING
+        logging.config.dictConfig(DEFAULT_LOGGING)
+
+    @classmethod
+    def tearDownClass(cls):
+        super(SetupDefaultLoggingMixin, cls).tearDownClass()
+        logging.config.dictConfig(cls._logging)
+
+
+class DefaultLoggingTests(SetupDefaultLoggingMixin, LoggingCaptureMixin, SimpleTestCase):
 
     def test_django_logger(self):
         """
         The 'django' base logger only output anything when DEBUG=True.
         """
-        output = StringIO()
-        self.logger.handlers[0].stream = output
         self.logger.error("Hey, this is an error.")
-        self.assertEqual(output.getvalue(), '')
+        self.assertEqual(self.logger_output.getvalue(), '')
 
         with self.settings(DEBUG=True):
             self.logger.error("Hey, this is an error.")
-            self.assertEqual(output.getvalue(), 'Hey, this is an error.\n')
+            self.assertEqual(self.logger_output.getvalue(), 'Hey, this is an error.\n')
+
+    @override_settings(DEBUG=True)
+    def test_django_logger_warning(self):
+        self.logger.warning('warning')
+        self.assertEqual(self.logger_output.getvalue(), 'warning\n')
+
+    @override_settings(DEBUG=True)
+    def test_django_logger_info(self):
+        self.logger.info('info')
+        self.assertEqual(self.logger_output.getvalue(), 'info\n')
+
+    @override_settings(DEBUG=True)
+    def test_django_logger_debug(self):
+        self.logger.debug('debug')
+        self.assertEqual(self.logger_output.getvalue(), '')
 
 
-class WarningLoggerTests(TestCase):
+@override_settings(DEBUG=True, ROOT_URLCONF='logging_tests.urls')
+class HandlerLoggingTests(SetupDefaultLoggingMixin, LoggingCaptureMixin, SimpleTestCase):
+
+    def test_page_found_no_warning(self):
+        self.client.get('/innocent/')
+        self.assertEqual(self.logger_output.getvalue(), '')
+
+    def test_page_not_found_warning(self):
+        self.client.get('/does_not_exist/')
+        self.assertEqual(self.logger_output.getvalue(), 'Not Found: /does_not_exist/\n')
+
+
+@override_settings(
+    DEBUG=True,
+    USE_I18N=True,
+    LANGUAGES=[('en', 'English')],
+    MIDDLEWARE=[
+        'django.middleware.locale.LocaleMiddleware',
+        'django.middleware.common.CommonMiddleware',
+    ],
+    ROOT_URLCONF='logging_tests.urls_i18n',
+)
+class I18nLoggingTests(SetupDefaultLoggingMixin, LoggingCaptureMixin, SimpleTestCase):
+
+    def test_i18n_page_found_no_warning(self):
+        self.client.get('/exists/')
+        self.client.get('/en/exists/')
+        self.assertEqual(self.logger_output.getvalue(), '')
+
+    def test_i18n_page_not_found_warning(self):
+        self.client.get('/this_does_not/')
+        self.client.get('/en/nor_this/')
+        self.assertEqual(self.logger_output.getvalue(), 'Not Found: /this_does_not/\nNot Found: /en/nor_this/\n')
+
+
+class WarningLoggerTests(SimpleTestCase):
     """
     Tests that warnings output for RemovedInDjangoXXWarning (XX being the next
     Django version) is enabled and captured to the logging system
@@ -101,39 +161,10 @@ class WarningLoggerTests(TestCase):
         self._old_capture_state = bool(getattr(logging, '_warnings_showwarning', False))
         logging.captureWarnings(True)
 
-        # this convoluted setup is to avoid printing this deprecation to
-        # stderr during test running - as the test runner forces deprecations
-        # to be displayed at the global py.warnings level
-        self.logger = logging.getLogger('py.warnings')
-        self.outputs = []
-        self.old_streams = []
-        for handler in self.logger.handlers:
-            self.old_streams.append(handler.stream)
-            self.outputs.append(StringIO())
-            handler.stream = self.outputs[-1]
-
     def tearDown(self):
-        for i, handler in enumerate(self.logger.handlers):
-            self.logger.handlers[i].stream = self.old_streams[i]
-
         # Reset warnings state.
         logging.captureWarnings(self._old_capture_state)
 
-    @override_settings(DEBUG=True)
-    def test_warnings_capture(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('always')
-            warnings.warn('Foo Deprecated', RemovedInNextVersionWarning)
-            output = force_text(self.outputs[0].getvalue())
-            self.assertIn('Foo Deprecated', output)
-
-    def test_warnings_capture_debug_false(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('always')
-            warnings.warn('Foo Deprecated', RemovedInNextVersionWarning)
-            output = force_text(self.outputs[0].getvalue())
-            self.assertNotIn('Foo Deprecated', output)
-
     @override_settings(DEBUG=True)
     def test_error_filter_still_raises(self):
         with warnings.catch_warnings():
@@ -145,7 +176,7 @@ class WarningLoggerTests(TestCase):
                 warnings.warn('Foo Deprecated', RemovedInNextVersionWarning)
 
 
-class CallbackFilterTest(TestCase):
+class CallbackFilterTest(SimpleTestCase):
     def test_sense(self):
         f_false = CallbackFilter(lambda r: False)
         f_true = CallbackFilter(lambda r: True)
@@ -166,12 +197,11 @@ class CallbackFilterTest(TestCase):
         self.assertEqual(collector, ["a record"])
 
 
-class AdminEmailHandlerTest(TestCase):
-    logger = logging.getLogger('django.request')
+class AdminEmailHandlerTest(SimpleTestCase):
+    logger = logging.getLogger('django')
 
     def get_admin_email_handler(self, logger):
-        # Inspired from views/views.py: send_log()
-        # ensuring the AdminEmailHandler does not get filtered out
+        # Ensure that AdminEmailHandler does not get filtered out
         # even with DEBUG=True.
         admin_email_handler = [
             h for h in logger.handlers
@@ -184,7 +214,7 @@ class AdminEmailHandlerTest(TestCase):
         self.assertTrue(admin_email_handler.connection().fail_silently)
 
     @override_settings(
-        ADMINS=(('whatever admin', 'admin@example.com'),),
+        ADMINS=[('whatever admin', 'admin@example.com')],
         EMAIL_SUBJECT_PREFIX='-SuperAwesomeSubject-'
     )
     def test_accepts_args(self):
@@ -214,9 +244,9 @@ class AdminEmailHandlerTest(TestCase):
             admin_email_handler.filters = orig_filters
 
     @override_settings(
-        ADMINS=(('whatever admin', 'admin@example.com'),),
+        ADMINS=[('whatever admin', 'admin@example.com')],
         EMAIL_SUBJECT_PREFIX='-SuperAwesomeSubject-',
-        INTERNAL_IPS=('127.0.0.1',),
+        INTERNAL_IPS=['127.0.0.1'],
     )
     def test_accepts_args_and_request(self):
         """
@@ -234,7 +264,8 @@ class AdminEmailHandlerTest(TestCase):
             admin_email_handler.filters = []
             rf = RequestFactory()
             request = rf.get('/')
-            self.logger.error(message, token1, token2,
+            self.logger.error(
+                message, token1, token2,
                 extra={
                     'status_code': 403,
                     'request': request,
@@ -249,7 +280,7 @@ class AdminEmailHandlerTest(TestCase):
             admin_email_handler.filters = orig_filters
 
     @override_settings(
-        ADMINS=(('admin', 'admin@example.com'),),
+        ADMINS=[('admin', 'admin@example.com')],
         EMAIL_SUBJECT_PREFIX='',
         DEBUG=False,
     )
@@ -272,29 +303,7 @@ class AdminEmailHandlerTest(TestCase):
         self.assertEqual(mail.outbox[0].subject, expected_subject)
 
     @override_settings(
-        ADMINS=(('admin', 'admin@example.com'),),
-        EMAIL_SUBJECT_PREFIX='',
-        DEBUG=False,
-    )
-    def test_truncate_subject(self):
-        """
-        RFC 2822's hard limit is 998 characters per line.
-        So, minus "Subject: ", the actual subject must be no longer than 989
-        characters.
-        Refs #17281.
-        """
-        message = 'a' * 1000
-        expected_subject = 'ERROR: aa' + 'a' * 980
-
-        self.assertEqual(len(mail.outbox), 0)
-
-        self.logger.error(message)
-
-        self.assertEqual(len(mail.outbox), 1)
-        self.assertEqual(mail.outbox[0].subject, expected_subject)
-
-    @override_settings(
-        ADMINS=(('admin', 'admin@example.com'),),
+        ADMINS=[('admin', 'admin@example.com')],
         DEBUG=False,
     )
     def test_uses_custom_email_backend(self):
@@ -326,7 +335,7 @@ class AdminEmailHandlerTest(TestCase):
             admin_email_handler.email_backend = orig_email_backend
 
     @override_settings(
-        ADMINS=(('whatever admin', 'admin@example.com'),),
+        ADMINS=[('whatever admin', 'admin@example.com')],
     )
     def test_emit_non_ascii(self):
         """
@@ -343,10 +352,10 @@ class AdminEmailHandlerTest(TestCase):
         msg = mail.outbox[0]
         self.assertEqual(msg.to, ['admin@example.com'])
         self.assertEqual(msg.subject, "[Django] ERROR (EXTERNAL IP): message")
-        self.assertIn("path:%s" % url_path, msg.body)
+        self.assertIn("Report at %s" % url_path, msg.body)
 
     @override_settings(
-        MANAGERS=(('manager', 'manager@example.com'),),
+        MANAGERS=[('manager', 'manager@example.com')],
         DEBUG=False,
     )
     def test_customize_send_mail_method(self):
@@ -361,6 +370,25 @@ class AdminEmailHandlerTest(TestCase):
         self.assertEqual(len(mail.outbox), 1)
         self.assertEqual(mail.outbox[0].to, ['manager@example.com'])
 
+    @override_settings(ALLOWED_HOSTS='example.com')
+    def test_disallowed_host_doesnt_crash(self):
+        admin_email_handler = self.get_admin_email_handler(self.logger)
+        old_include_html = admin_email_handler.include_html
+
+        # Text email
+        admin_email_handler.include_html = False
+        try:
+            self.client.get('/', HTTP_HOST='evil.com')
+        finally:
+            admin_email_handler.include_html = old_include_html
+
+        # HTML email
+        admin_email_handler.include_html = True
+        try:
+            self.client.get('/', HTTP_HOST='evil.com')
+        finally:
+            admin_email_handler.include_html = old_include_html
+
 
 class SettingsConfigTest(AdminScriptTestCase):
     """
@@ -384,7 +412,7 @@ class SettingsConfigTest(AdminScriptTestCase):
 
     def test_circular_dependency(self):
         # validate is just an example command to trigger settings configuration
-        out, err = self.run_manage(['validate'])
+        out, err = self.run_manage(['check'])
         self.assertNoOutput(err)
         self.assertOutput(out, "System check identified no issues (0 silenced).")
 
@@ -394,7 +422,7 @@ def dictConfig(config):
 dictConfig.called = False
 
 
-class SetupConfigureLogging(TestCase):
+class SetupConfigureLogging(SimpleTestCase):
     """
     Test that calling django.setup() initializes the logging configuration.
     """
@@ -407,7 +435,7 @@ class SetupConfigureLogging(TestCase):
 
 
 @override_settings(DEBUG=True, ROOT_URLCONF='logging_tests.urls')
-class SecurityLoggerTest(TestCase):
+class SecurityLoggerTest(SimpleTestCase):
 
     def test_suspicious_operation_creates_log_message(self):
         with patch_logger('django.security.SuspiciousOperation', 'error') as calls:
@@ -422,13 +450,13 @@ class SecurityLoggerTest(TestCase):
             self.assertEqual(calls[0], 'dubious')
 
     @override_settings(
-        ADMINS=(('admin', 'admin@example.com'),),
+        ADMINS=[('admin', 'admin@example.com')],
         DEBUG=False,
     )
     def test_suspicious_email_admins(self):
         self.client.get('/suspicious/')
         self.assertEqual(len(mail.outbox), 1)
-        self.assertIn('path:/suspicious/,', mail.outbox[0].body)
+        self.assertIn('Report at /suspicious/', mail.outbox[0].body)
 
 
 class SettingsCustomLoggingTest(AdminScriptTestCase):
@@ -465,6 +493,70 @@ format=%(message)s
         self.remove_settings('settings.py')
 
     def test_custom_logging(self):
-        out, err = self.run_manage(['validate'])
+        out, err = self.run_manage(['check'])
         self.assertNoOutput(err)
         self.assertOutput(out, "System check identified no issues (0 silenced).")
+
+
+class SchemaLoggerTests(SimpleTestCase):
+
+    def test_extra_args(self):
+        editor = connection.schema_editor(collect_sql=True)
+        sql = "SELECT * FROM foo WHERE id in (%s, %s)"
+        params = [42, 1337]
+        with patch_logger('django.db.backends.schema', 'debug', log_kwargs=True) as logger:
+            editor.execute(sql, params)
+        self.assertEqual(
+            logger,
+            [(
+                'SELECT * FROM foo WHERE id in (%s, %s); (params [42, 1337])',
+                {'extra': {
+                    'sql': 'SELECT * FROM foo WHERE id in (%s, %s)',
+                    'params': [42, 1337],
+                }},
+            )]
+        )
+
+
+class LogFormattersTests(SimpleTestCase):
+
+    def test_server_formatter_styles(self):
+        color_style = color.make_style('')
+        formatter = ServerFormatter()
+        formatter.style = color_style
+        log_msg = 'log message'
+        status_code_styles = [
+            (200, 'HTTP_SUCCESS'),
+            (100, 'HTTP_INFO'),
+            (304, 'HTTP_NOT_MODIFIED'),
+            (300, 'HTTP_REDIRECT'),
+            (404, 'HTTP_NOT_FOUND'),
+            (400, 'HTTP_BAD_REQUEST'),
+            (500, 'HTTP_SERVER_ERROR'),
+        ]
+        for status_code, style in status_code_styles:
+            record = logging.makeLogRecord({'msg': log_msg, 'status_code': status_code})
+            self.assertEqual(formatter.format(record), getattr(color_style, style)(log_msg))
+        record = logging.makeLogRecord({'msg': log_msg})
+        self.assertEqual(formatter.format(record), log_msg)
+
+    def test_server_formatter_default_format(self):
+        server_time = '2016-09-25 10:20:30'
+        log_msg = 'log message'
+        logger = logging.getLogger('django.server')
+
+        @contextmanager
+        def patch_django_server_logger():
+            old_stream = logger.handlers[0].stream
+            new_stream = six.StringIO()
+            logger.handlers[0].stream = new_stream
+            yield new_stream
+            logger.handlers[0].stream = old_stream
+
+        with patch_django_server_logger() as logger_output:
+            logger.info(log_msg, extra={'server_time': server_time})
+            self.assertEqual('[%s] %s\n' % (server_time, log_msg), logger_output.getvalue())
+
+        with patch_django_server_logger() as logger_output:
+            logger.info(log_msg)
+            six.assertRegex(self, logger_output.getvalue(), r'^\[[-:,.\s\d]+\] %s' % log_msg)
