@@ -115,14 +115,15 @@ class Child5(AbstractBase3):
         return self.name
 
 
-# Will inherit managers from AbstractBase1, but not Child4.
 class Child6(Child4):
     value = models.IntegerField()
 
+    class Meta:
+        manager_inheritance_from_future = True
+
 
-# Will not inherit default manager from parent.
 class Child7(Parent):
-    pass
+    objects = models.Manager()
 
 
 # RelatedManagers
@@ -137,11 +138,11 @@ class RelatedModel(models.Model):
 
 @python_2_unicode_compatible
 class RelationModel(models.Model):
-    fk = models.ForeignKey(RelatedModel, related_name='test_fk')
+    fk = models.ForeignKey(RelatedModel, models.CASCADE, related_name='test_fk')
 
     m2m = models.ManyToManyField(RelatedModel, related_name='test_m2m')
 
-    gfk_ctype = models.ForeignKey(ContentType, null=True)
+    gfk_ctype = models.ForeignKey(ContentType, models.SET_NULL, null=True)
     gfk_id = models.IntegerField(null=True)
     gfk = GenericForeignKey(ct_field='gfk_ctype', fk_field='gfk_id')
 
