@@ -2,19 +2,27 @@
 from __future__ import unicode_literals
 
 import codecs
+import datetime
 import importlib
 import os
+import sys
 
 from django.apps import apps
 from django.core.management import CommandError, call_command
-from django.db import DatabaseError, connection, models
+from django.db import (
+    ConnectionHandler, DatabaseError, connection, connections, models,
+)
+from django.db.migrations.exceptions import (
+    InconsistentMigrationHistory, MigrationSchemaMissing,
+)
 from django.db.migrations.recorder import MigrationRecorder
 from django.test import ignore_warnings, mock, override_settings
 from django.utils import six
-from django.utils.deprecation import RemovedInDjango110Warning
+from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.encoding import force_text
 
 from .models import UnicodeModel, UnserializableModel
+from .routers import TestRouter
 from .test_base import MigrationTestBase
 
 
@@ -22,6 +30,7 @@ class MigrateTests(MigrationTestBase):
     """
     Tests running the migrate command.
     """
+    multi_db = True
 
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations"})
     def test_migrate(self):
@@ -51,6 +60,16 @@ class MigrateTests(MigrationTestBase):
         self.assertTableNotExists("migrations_tribble")
         self.assertTableNotExists("migrations_book")
 
+    @override_settings(INSTALLED_APPS=[
+        'django.contrib.auth',
+        'django.contrib.contenttypes',
+        'migrations.migrations_test_apps.migrated_app',
+    ])
+    def test_migrate_with_system_checks(self):
+        out = six.StringIO()
+        call_command('migrate', skip_checks=False, no_color=True, stdout=out)
+        self.assertIn('Apply all migrations: migrated_app', out.getvalue())
+
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations_initial_false"})
     def test_migrate_initial_false(self):
         """
@@ -75,25 +94,36 @@ class MigrateTests(MigrationTestBase):
         self.assertTableNotExists("migrations_tribble")
         self.assertTableNotExists("migrations_book")
 
-    @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations"})
+    @override_settings(
+        MIGRATION_MODULES={"migrations": "migrations.test_migrations"},
+        DATABASE_ROUTERS=['migrations.routers.TestRouter'],
+    )
     def test_migrate_fake_initial(self):
         """
-        #24184 - Tests that --fake-initial only works if all tables created in
-        the initial migration of an app exists
+        --fake-initial only works if all tables created in the initial
+        migration of an app exists. Database routers must be obeyed when doing
+        that check.
         """
         # Make sure no tables are created
-        self.assertTableNotExists("migrations_author")
-        self.assertTableNotExists("migrations_tribble")
+        for db in connections:
+            self.assertTableNotExists("migrations_author", using=db)
+            self.assertTableNotExists("migrations_tribble", using=db)
         # Run the migrations to 0001 only
         call_command("migrate", "migrations", "0001", verbosity=0)
+        call_command("migrate", "migrations", "0001", verbosity=0, database="other")
         # Make sure the right tables exist
         self.assertTableExists("migrations_author")
-        self.assertTableExists("migrations_tribble")
+        self.assertTableNotExists("migrations_tribble")
+        # Also check the "other" database
+        self.assertTableNotExists("migrations_author", using="other")
+        self.assertTableExists("migrations_tribble", using="other")
+
         # Fake a roll-back
         call_command("migrate", "migrations", "zero", fake=True, verbosity=0)
+        call_command("migrate", "migrations", "zero", fake=True, verbosity=0, database="other")
         # Make sure the tables still exist
         self.assertTableExists("migrations_author")
-        self.assertTableExists("migrations_tribble")
+        self.assertTableExists("migrations_tribble", using="other")
         # Try to run initial migration
         with self.assertRaises(DatabaseError):
             call_command("migrate", "migrations", "0001", verbosity=0)
@@ -101,18 +131,24 @@ class MigrateTests(MigrationTestBase):
         out = six.StringIO()
         with mock.patch('django.core.management.color.supports_color', lambda *args: False):
             call_command("migrate", "migrations", "0001", fake_initial=True, stdout=out, verbosity=1)
+            call_command("migrate", "migrations", "0001", fake_initial=True, verbosity=0, database="other")
         self.assertIn(
             "migrations.0001_initial... faked",
             out.getvalue().lower()
         )
         # Run migrations all the way
         call_command("migrate", verbosity=0)
+        call_command("migrate", verbosity=0, database="other")
         # Make sure the right tables exist
         self.assertTableExists("migrations_author")
         self.assertTableNotExists("migrations_tribble")
         self.assertTableExists("migrations_book")
+        self.assertTableNotExists("migrations_author", using="other")
+        self.assertTableNotExists("migrations_tribble", using="other")
+        self.assertTableNotExists("migrations_book", using="other")
         # Fake a roll-back
         call_command("migrate", "migrations", "zero", fake=True, verbosity=0)
+        call_command("migrate", "migrations", "zero", fake=True, verbosity=0, database="other")
         # Make sure the tables still exist
         self.assertTableExists("migrations_author")
         self.assertTableNotExists("migrations_tribble")
@@ -127,12 +163,15 @@ class MigrateTests(MigrationTestBase):
             call_command("migrate", "migrations", fake_initial=True, verbosity=0)
         # Fake a apply
         call_command("migrate", "migrations", fake=True, verbosity=0)
+        call_command("migrate", "migrations", fake=True, verbosity=0, database="other")
         # Unmigrate everything
         call_command("migrate", "migrations", "zero", verbosity=0)
+        call_command("migrate", "migrations", "zero", verbosity=0, database="other")
         # Make sure it's all gone
-        self.assertTableNotExists("migrations_author")
-        self.assertTableNotExists("migrations_tribble")
-        self.assertTableNotExists("migrations_book")
+        for db in connections:
+            self.assertTableNotExists("migrations_author", using=db)
+            self.assertTableNotExists("migrations_tribble", using=db)
+            self.assertTableNotExists("migrations_book", using=db)
 
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations_fake_split_initial"})
     def test_migrate_fake_split_initial(self):
@@ -160,37 +199,6 @@ class MigrateTests(MigrationTestBase):
         with self.assertRaisesMessage(CommandError, "Conflicting migrations detected"):
             call_command("migrate", "migrations")
 
-    @ignore_warnings(category=RemovedInDjango110Warning)
-    @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations"})
-    def test_migrate_list(self):
-        """
-        Tests --list output of migrate command
-        """
-        out = six.StringIO()
-        with mock.patch('django.core.management.color.supports_color', lambda *args: True):
-            call_command("migrate", list=True, stdout=out, verbosity=0, no_color=False)
-        self.assertEqual(
-            '\x1b[1mmigrations\n\x1b[0m'
-            ' [ ] 0001_initial\n'
-            ' [ ] 0002_second\n',
-            out.getvalue().lower()
-        )
-
-        call_command("migrate", "migrations", "0001", verbosity=0)
-
-        out = six.StringIO()
-        # Giving the explicit app_label tests for selective `show_migration_list` in the command
-        call_command("migrate", "migrations", list=True, stdout=out, verbosity=0, no_color=True)
-        self.assertEqual(
-            'migrations\n'
-            ' [x] 0001_initial\n'
-            ' [ ] 0002_second\n',
-            out.getvalue().lower()
-        )
-
-        # Cleanup by unmigrating everything
-        call_command("migrate", "migrations", "zero", verbosity=0)
-
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations"})
     def test_showmigrations_list(self):
         """
@@ -227,39 +235,38 @@ class MigrateTests(MigrationTestBase):
         """
         out = six.StringIO()
         call_command("showmigrations", format='plan', stdout=out)
-        self.assertIn(
+        self.assertEqual(
             "[ ]  migrations.0001_initial\n"
             "[ ]  migrations.0003_third\n"
-            "[ ]  migrations.0002_second",
+            "[ ]  migrations.0002_second\n",
             out.getvalue().lower()
         )
 
         out = six.StringIO()
         call_command("showmigrations", format='plan', stdout=out, verbosity=2)
-        self.assertIn(
+        self.assertEqual(
             "[ ]  migrations.0001_initial\n"
             "[ ]  migrations.0003_third ... (migrations.0001_initial)\n"
-            "[ ]  migrations.0002_second ... (migrations.0001_initial)",
+            "[ ]  migrations.0002_second ... (migrations.0001_initial, migrations.0003_third)\n",
             out.getvalue().lower()
         )
-
         call_command("migrate", "migrations", "0003", verbosity=0)
 
         out = six.StringIO()
         call_command("showmigrations", format='plan', stdout=out)
-        self.assertIn(
+        self.assertEqual(
             "[x]  migrations.0001_initial\n"
             "[x]  migrations.0003_third\n"
-            "[ ]  migrations.0002_second",
+            "[ ]  migrations.0002_second\n",
             out.getvalue().lower()
         )
 
         out = six.StringIO()
         call_command("showmigrations", format='plan', stdout=out, verbosity=2)
-        self.assertIn(
+        self.assertEqual(
             "[x]  migrations.0001_initial\n"
             "[x]  migrations.0003_third ... (migrations.0001_initial)\n"
-            "[ ]  migrations.0002_second ... (migrations.0001_initial)",
+            "[ ]  migrations.0002_second ... (migrations.0001_initial, migrations.0003_third)\n",
             out.getvalue().lower()
         )
 
@@ -347,16 +354,26 @@ class MigrateTests(MigrationTestBase):
         index_tx_end = output.find(connection.ops.end_transaction_sql().lower())
 
         self.assertGreater(index_tx_start, -1, "Transaction start not found")
-        self.assertGreater(index_op_desc_author, index_tx_start,
-            "Operation description (author) not found or found before transaction start")
-        self.assertGreater(index_create_table, index_op_desc_author,
-            "CREATE TABLE not found or found before operation description (author)")
-        self.assertGreater(index_op_desc_tribble, index_create_table,
-            "Operation description (tribble) not found or found before CREATE TABLE (author)")
-        self.assertGreater(index_op_desc_unique_together, index_op_desc_tribble,
-            "Operation description (unique_together) not found or found before operation description (tribble)")
-        self.assertGreater(index_tx_end, index_op_desc_unique_together,
-            "Transaction end not found or found before operation description (unique_together)")
+        self.assertGreater(
+            index_op_desc_author, index_tx_start,
+            "Operation description (author) not found or found before transaction start"
+        )
+        self.assertGreater(
+            index_create_table, index_op_desc_author,
+            "CREATE TABLE not found or found before operation description (author)"
+        )
+        self.assertGreater(
+            index_op_desc_tribble, index_create_table,
+            "Operation description (tribble) not found or found before CREATE TABLE (author)"
+        )
+        self.assertGreater(
+            index_op_desc_unique_together, index_op_desc_tribble,
+            "Operation description (unique_together) not found or found before operation description (tribble)"
+        )
+        self.assertGreater(
+            index_tx_end, index_op_desc_unique_together,
+            "Transaction end not found or found before operation description (unique_together)"
+        )
 
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations"})
     def test_sqlmigrate_backwards(self):
@@ -378,21 +395,44 @@ class MigrateTests(MigrationTestBase):
         index_tx_end = output.find(connection.ops.end_transaction_sql().lower())
 
         self.assertGreater(index_tx_start, -1, "Transaction start not found")
-        self.assertGreater(index_op_desc_unique_together, index_tx_start,
-            "Operation description (unique_together) not found or found before transaction start")
-        self.assertGreater(index_op_desc_tribble, index_op_desc_unique_together,
-            "Operation description (tribble) not found or found before operation description (unique_together)")
-        self.assertGreater(index_op_desc_author, index_op_desc_tribble,
-            "Operation description (author) not found or found before operation description (tribble)")
+        self.assertGreater(
+            index_op_desc_unique_together, index_tx_start,
+            "Operation description (unique_together) not found or found before transaction start"
+        )
+        self.assertGreater(
+            index_op_desc_tribble, index_op_desc_unique_together,
+            "Operation description (tribble) not found or found before operation description (unique_together)"
+        )
+        self.assertGreater(
+            index_op_desc_author, index_op_desc_tribble,
+            "Operation description (author) not found or found before operation description (tribble)"
+        )
 
-        self.assertGreater(index_drop_table, index_op_desc_author,
-            "DROP TABLE not found or found before operation description (author)")
-        self.assertGreater(index_tx_end, index_op_desc_unique_together,
-            "Transaction end not found or found before DROP TABLE")
+        self.assertGreater(
+            index_drop_table, index_op_desc_author,
+            "DROP TABLE not found or found before operation description (author)"
+        )
+        self.assertGreater(
+            index_tx_end, index_op_desc_unique_together,
+            "Transaction end not found or found before DROP TABLE"
+        )
 
         # Cleanup by unmigrating everything
         call_command("migrate", "migrations", "zero", verbosity=0)
 
+    @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations_non_atomic"})
+    def test_sqlmigrate_for_non_atomic_migration(self):
+        """
+        Transaction wrappers aren't shown for non-atomic migrations.
+        """
+        out = six.StringIO()
+        call_command("sqlmigrate", "migrations", "0001", stdout=out)
+        output = out.getvalue().lower()
+        queries = [q.strip() for q in output.splitlines()]
+        if connection.ops.start_transaction_sql():
+            self.assertNotIn(connection.ops.start_transaction_sql().lower(), queries)
+        self.assertNotIn(connection.ops.end_transaction_sql().lower(), queries)
+
     @override_settings(
         INSTALLED_APPS=[
             "migrations.migrations_test_apps.migrated_app",
@@ -414,6 +454,12 @@ class MigrateTests(MigrationTestBase):
         """
         call_command("migrate", "migrated_unapplied_app", stdout=six.StringIO())
 
+        # unmigrated_app.SillyModel has a foreign key to 'migrations.Tribble',
+        # but that model is only defined in a migration, so the global app
+        # registry never sees it and the reference is left dangling. Remove it
+        # to avoid problems in subsequent tests.
+        del apps._pending_operations[('migrations', 'tribble')]
+
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations_squashed"})
     def test_migrate_record_replaced(self):
         """
@@ -459,6 +505,20 @@ class MigrateTests(MigrationTestBase):
         )
         # No changes were actually applied so there is nothing to rollback
 
+    @override_settings(MIGRATION_MODULES={'migrations': 'migrations.test_migrations'})
+    def test_migrate_inconsistent_history(self):
+        """
+        Running migrate with some migrations applied before their dependencies
+        should not be allowed.
+        """
+        recorder = MigrationRecorder(connection)
+        recorder.record_applied("migrations", "0002_second")
+        msg = "Migration migrations.0002_second is applied before its dependency migrations.0001_initial"
+        with self.assertRaisesMessage(InconsistentMigrationHistory, msg):
+            call_command("migrate")
+        applied_migrations = recorder.applied_migrations()
+        self.assertNotIn(("migrations", "0001_initial"), applied_migrations)
+
 
 class MakeMigrationsTests(MigrationTestBase):
     """
@@ -525,12 +585,77 @@ class MakeMigrationsTests(MigrationTestBase):
             call_command('makemigrations', 'migrations', '--empty', '-n', 'a', '-v', '0')
             self.assertTrue(os.path.exists(os.path.join(migration_dir, '0002_a.py')))
 
+    def test_makemigrations_empty_connections(self):
+        empty_connections = ConnectionHandler({'default': {}})
+        with mock.patch('django.core.management.commands.makemigrations.connections', new=empty_connections):
+            # with no apps
+            out = six.StringIO()
+            call_command('makemigrations', stdout=out)
+            self.assertIn('No changes detected', out.getvalue())
+            # with an app
+            with self.temporary_migration_module() as migration_dir:
+                call_command('makemigrations', 'migrations', verbosity=0)
+                init_file = os.path.join(migration_dir, '__init__.py')
+                self.assertTrue(os.path.exists(init_file))
+
+    @override_settings(INSTALLED_APPS=['migrations', 'migrations2'])
+    def test_makemigrations_consistency_checks_respect_routers(self):
+        """
+        The history consistency checks in makemigrations respect
+        settings.DATABASE_ROUTERS.
+        """
+        def patched_ensure_schema(migration_recorder):
+            if migration_recorder.connection is connections['other']:
+                raise MigrationSchemaMissing('Patched')
+            else:
+                return mock.DEFAULT
+
+        self.assertTableNotExists('migrations_unicodemodel')
+        apps.register_model('migrations', UnicodeModel)
+        with mock.patch.object(
+                MigrationRecorder, 'ensure_schema',
+                autospec=True, side_effect=patched_ensure_schema) as ensure_schema:
+            with self.temporary_migration_module() as migration_dir:
+                call_command("makemigrations", "migrations", verbosity=0)
+                initial_file = os.path.join(migration_dir, "0001_initial.py")
+                self.assertTrue(os.path.exists(initial_file))
+                self.assertEqual(ensure_schema.call_count, 1)  # 'default' is checked
+
+                # Router says not to migrate 'other' so consistency shouldn't
+                # be checked.
+                with self.settings(DATABASE_ROUTERS=['migrations.routers.TestRouter']):
+                    call_command('makemigrations', 'migrations', verbosity=0)
+                self.assertEqual(ensure_schema.call_count, 2)  # 'default' again
+
+                # With a router that doesn't prohibit migrating 'other',
+                # consistency is checked.
+                with self.settings(DATABASE_ROUTERS=['migrations.routers.EmptyRouter']):
+                    with self.assertRaisesMessage(MigrationSchemaMissing, 'Patched'):
+                        call_command('makemigrations', 'migrations', verbosity=0)
+                self.assertEqual(ensure_schema.call_count, 4)  # 'default' and 'other'
+
+                # With a router that doesn't allow migrating on any database,
+                # no consistency checks are made.
+                with self.settings(DATABASE_ROUTERS=['migrations.routers.TestRouter']):
+                    with mock.patch.object(TestRouter, 'allow_migrate', return_value=False) as allow_migrate:
+                        call_command('makemigrations', 'migrations', verbosity=0)
+                allow_migrate.assert_any_call('other', 'migrations', model_name='UnicodeModel')
+                # allow_migrate() is called with the correct arguments.
+                self.assertGreater(len(allow_migrate.mock_calls), 0)
+                for mock_call in allow_migrate.mock_calls:
+                    _, call_args, call_kwargs = mock_call
+                    connection_alias, app_name = call_args
+                    self.assertIn(connection_alias, ['default', 'other'])
+                    # Raises an error if invalid app_name/model_name occurs.
+                    apps.get_app_config(app_name).get_model(call_kwargs['model_name'])
+                self.assertEqual(ensure_schema.call_count, 4)
+
     def test_failing_migration(self):
         # If a migration fails to serialize, it shouldn't generate an empty file. #21280
         apps.register_model('migrations', UnserializableModel)
 
         with self.temporary_migration_module() as migration_dir:
-            with six.assertRaisesRegex(self, ValueError, r'Cannot serialize'):
+            with self.assertRaisesMessage(ValueError, 'Cannot serialize'):
                 call_command("makemigrations", "migrations", verbosity=0)
 
             initial_file = os.path.join(migration_dir, "0001_initial.py")
@@ -550,10 +675,7 @@ class MakeMigrationsTests(MigrationTestBase):
         """
         out = six.StringIO()
         with self.temporary_migration_module(module="migrations.test_migrations"):
-            try:
-                call_command("makemigrations", merge=True, stdout=out)
-            except CommandError:
-                self.fail("Makemigrations errored in merge mode with no conflicts")
+            call_command("makemigrations", merge=True, stdout=out)
         self.assertIn("No conflicts detected to merge.", out.getvalue())
 
     def test_makemigrations_no_app_sys_exit(self):
@@ -577,10 +699,7 @@ class MakeMigrationsTests(MigrationTestBase):
         Makes sure that makemigrations properly constructs an empty migration.
         """
         with self.temporary_migration_module() as migration_dir:
-            try:
-                call_command("makemigrations", "migrations", empty=True, verbosity=0)
-            except CommandError:
-                self.fail("Makemigrations errored in creating empty migration for a proper app.")
+            call_command("makemigrations", "migrations", empty=True, verbosity=0)
 
             # Check for existing 0001_initial.py file in migration folder
             initial_file = os.path.join(migration_dir, "0001_initial.py")
@@ -662,13 +781,10 @@ class MakeMigrationsTests(MigrationTestBase):
         """
         # Monkeypatch interactive questioner to auto reject
         with mock.patch('django.db.migrations.questioner.input', mock.Mock(return_value='N')):
-            try:
-                with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
-                    call_command("makemigrations", "migrations", merge=True, interactive=True, verbosity=0)
-                    merge_file = os.path.join(migration_dir, '0003_merge.py')
-                    self.assertFalse(os.path.exists(merge_file))
-            except CommandError:
-                self.fail("Makemigrations failed while running interactive questioner")
+            with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
+                call_command("makemigrations", "migrations", name="merge", merge=True, interactive=True, verbosity=0)
+                merge_file = os.path.join(migration_dir, '0003_merge.py')
+                self.assertFalse(os.path.exists(merge_file))
 
     def test_makemigrations_interactive_accept(self):
         """
@@ -677,13 +793,21 @@ class MakeMigrationsTests(MigrationTestBase):
         # Monkeypatch interactive questioner to auto accept
         with mock.patch('django.db.migrations.questioner.input', mock.Mock(return_value='y')):
             out = six.StringIO()
-            try:
-                with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
-                    call_command("makemigrations", "migrations", merge=True, interactive=True, stdout=out)
-                    merge_file = os.path.join(migration_dir, '0003_merge.py')
-                    self.assertTrue(os.path.exists(merge_file))
-            except CommandError:
-                self.fail("Makemigrations failed while running interactive questioner")
+            with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
+                call_command("makemigrations", "migrations", name="merge", merge=True, interactive=True, stdout=out)
+                merge_file = os.path.join(migration_dir, '0003_merge.py')
+                self.assertTrue(os.path.exists(merge_file))
+            self.assertIn("Created new merge migration", force_text(out.getvalue()))
+
+    @mock.patch('django.db.migrations.utils.datetime')
+    def test_makemigrations_default_merge_name(self, mock_datetime):
+        mock_datetime.datetime.now.return_value = datetime.datetime(2016, 1, 2, 3, 4)
+        with mock.patch('django.db.migrations.questioner.input', mock.Mock(return_value='y')):
+            out = six.StringIO()
+            with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
+                call_command("makemigrations", "migrations", merge=True, interactive=True, stdout=out)
+                merge_file = os.path.join(migration_dir, '0003_merge_20160102_0304.py')
+                self.assertTrue(os.path.exists(merge_file))
             self.assertIn("Created new merge migration", force_text(out.getvalue()))
 
     def test_makemigrations_non_interactive_not_null_addition(self):
@@ -715,11 +839,8 @@ class MakeMigrationsTests(MigrationTestBase):
                 app_label = "migrations"
 
         out = six.StringIO()
-        try:
-            with self.temporary_migration_module(module="migrations.test_migrations"):
-                call_command("makemigrations", "migrations", interactive=False, stdout=out)
-        except CommandError:
-            self.fail("Makemigrations failed while running non-interactive questioner.")
+        with self.temporary_migration_module(module="migrations.test_migrations"):
+            call_command("makemigrations", "migrations", interactive=False, stdout=out)
         self.assertIn("Alter field slug on author", force_text(out.getvalue()))
 
     def test_makemigrations_non_interactive_no_model_rename(self):
@@ -733,11 +854,8 @@ class MakeMigrationsTests(MigrationTestBase):
                 app_label = "migrations"
 
         out = six.StringIO()
-        try:
-            with self.temporary_migration_module(module="migrations.test_migrations_no_default"):
-                call_command("makemigrations", "migrations", interactive=False, stdout=out)
-        except CommandError:
-            self.fail("Makemigrations failed while running non-interactive questioner")
+        with self.temporary_migration_module(module="migrations.test_migrations_no_default"):
+            call_command("makemigrations", "migrations", interactive=False, stdout=out)
         self.assertIn("Delete model SillyModel", force_text(out.getvalue()))
         self.assertIn("Create model RenamedModel", force_text(out.getvalue()))
 
@@ -752,11 +870,8 @@ class MakeMigrationsTests(MigrationTestBase):
                 app_label = "migrations"
 
         out = six.StringIO()
-        try:
-            with self.temporary_migration_module(module="migrations.test_migrations_no_default"):
-                call_command("makemigrations", "migrations", interactive=False, stdout=out)
-        except CommandError:
-            self.fail("Makemigrations failed while running non-interactive questioner")
+        with self.temporary_migration_module(module="migrations.test_migrations_no_default"):
+            call_command("makemigrations", "migrations", interactive=False, stdout=out)
         self.assertIn("Remove field silly_field from sillymodel", force_text(out.getvalue()))
         self.assertIn("Add field silly_rename to sillymodel", force_text(out.getvalue()))
 
@@ -766,7 +881,7 @@ class MakeMigrationsTests(MigrationTestBase):
         """
         out = six.StringIO()
         with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
-            call_command("makemigrations", "migrations", merge=True, interactive=False, stdout=out)
+            call_command("makemigrations", "migrations", name="merge", merge=True, interactive=False, stdout=out)
             merge_file = os.path.join(migration_dir, '0003_merge.py')
             self.assertTrue(os.path.exists(merge_file))
         output = force_text(out.getvalue())
@@ -782,7 +897,10 @@ class MakeMigrationsTests(MigrationTestBase):
         """
         out = six.StringIO()
         with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
-            call_command("makemigrations", "migrations", dry_run=True, merge=True, interactive=False, stdout=out)
+            call_command(
+                "makemigrations", "migrations", name="merge", dry_run=True,
+                merge=True, interactive=False, stdout=out,
+            )
             merge_file = os.path.join(migration_dir, '0003_merge.py')
             self.assertFalse(os.path.exists(merge_file))
         output = force_text(out.getvalue())
@@ -798,8 +916,10 @@ class MakeMigrationsTests(MigrationTestBase):
         """
         out = six.StringIO()
         with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
-            call_command("makemigrations", "migrations", dry_run=True, merge=True, interactive=False,
-                         stdout=out, verbosity=3)
+            call_command(
+                "makemigrations", "migrations", name="merge", dry_run=True,
+                merge=True, interactive=False, stdout=out, verbosity=3,
+            )
             merge_file = os.path.join(migration_dir, '0003_merge.py')
             self.assertFalse(os.path.exists(merge_file))
         output = force_text(out.getvalue())
@@ -900,14 +1020,11 @@ class MakeMigrationsTests(MigrationTestBase):
         # Monkeypatch interactive questioner to auto reject
         out = six.StringIO()
         with mock.patch('django.db.migrations.questioner.input', mock.Mock(return_value='N')):
-            try:
-                with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
-                    call_command("makemigrations", "migrations", merge=True, stdout=out)
-                    merge_file = os.path.join(migration_dir, '0003_merge.py')
-                    # This will fail if interactive is False by default
-                    self.assertFalse(os.path.exists(merge_file))
-            except CommandError:
-                self.fail("Makemigrations failed while running interactive questioner")
+            with self.temporary_migration_module(module="migrations.test_migrations_conflict") as migration_dir:
+                call_command("makemigrations", "migrations", name="merge", merge=True, stdout=out)
+                merge_file = os.path.join(migration_dir, '0003_merge.py')
+                # This will fail if interactive is False by default
+                self.assertFalse(os.path.exists(merge_file))
             self.assertNotIn("Created new merge migration", out.getvalue())
 
     @override_settings(
@@ -919,11 +1036,8 @@ class MakeMigrationsTests(MigrationTestBase):
         Makes sure that makemigrations does not raise a CommandError when an
         unspecified app has conflicting migrations.
         """
-        try:
-            with self.temporary_migration_module(module="migrations.test_migrations_no_changes"):
-                call_command("makemigrations", "migrations", merge=False, verbosity=0)
-        except CommandError:
-            self.fail("Makemigrations fails resolving conflicts in an unspecified app")
+        with self.temporary_migration_module(module="migrations.test_migrations_no_changes"):
+            call_command("makemigrations", "migrations", merge=False, verbosity=0)
 
     @override_settings(
         INSTALLED_APPS=[
@@ -937,14 +1051,11 @@ class MakeMigrationsTests(MigrationTestBase):
         # Monkeypatch interactive questioner to auto accept
         with mock.patch('django.db.migrations.questioner.input', mock.Mock(return_value='y')):
             out = six.StringIO()
-            try:
-                with self.temporary_migration_module(app_label="migrated_app") as migration_dir:
-                    call_command("makemigrations", "migrated_app", merge=True, interactive=True, stdout=out)
-                    merge_file = os.path.join(migration_dir, '0003_merge.py')
-                    self.assertFalse(os.path.exists(merge_file))
-                self.assertIn("No conflicts detected to merge.", out.getvalue())
-            except CommandError:
-                self.fail("Makemigrations fails resolving conflicts in an unspecified app")
+            with self.temporary_migration_module(app_label="migrated_app") as migration_dir:
+                call_command("makemigrations", "migrated_app", name="merge", merge=True, interactive=True, stdout=out)
+                merge_file = os.path.join(migration_dir, '0003_merge.py')
+                self.assertFalse(os.path.exists(merge_file))
+            self.assertIn("No conflicts detected to merge.", out.getvalue())
 
     @override_settings(
         INSTALLED_APPS=[
@@ -986,10 +1097,7 @@ class MakeMigrationsTests(MigrationTestBase):
         with self.temporary_migration_module() as migration_dir:
 
             def cmd(migration_count, migration_name, *args):
-                try:
-                    call_command("makemigrations", "migrations", "--verbosity", "0", "--name", migration_name, *args)
-                except CommandError:
-                    self.fail("Makemigrations errored in creating empty migration with custom name for a proper app.")
+                call_command("makemigrations", "migrations", "--verbosity", "0", "--name", migration_name, *args)
                 migration_file = os.path.join(migration_dir, "%s_%s.py" % (migration_count, migration_name))
                 # Check for existing migration file in migration folder
                 self.assertTrue(os.path.exists(migration_file))
@@ -1015,6 +1123,7 @@ class MakeMigrationsTests(MigrationTestBase):
             self.assertIn("dependencies=[\n('migrations','0001_%s'),\n]" % migration_name_0001, content)
             self.assertIn("operations=[\n]", content)
 
+    @ignore_warnings(category=RemovedInDjango20Warning)
     def test_makemigrations_exit(self):
         """
         makemigrations --exit should exit with sys.exit(1) when there are no
@@ -1027,6 +1136,66 @@ class MakeMigrationsTests(MigrationTestBase):
             with self.assertRaises(SystemExit):
                 call_command("makemigrations", "--exit", "migrations", verbosity=0)
 
+    def test_makemigrations_check(self):
+        """
+        makemigrations --check should exit with a non-zero status when
+        there are changes to an app requiring migrations.
+        """
+        with self.temporary_migration_module():
+            with self.assertRaises(SystemExit):
+                call_command("makemigrations", "--check", "migrations", verbosity=0)
+
+        with self.temporary_migration_module(module="migrations.test_migrations_no_changes"):
+            call_command("makemigrations", "--check", "migrations", verbosity=0)
+
+    def test_makemigrations_migration_path_output(self):
+        """
+        makemigrations should print the relative paths to the migrations unless
+        they are outside of the current tree, in which case the absolute path
+        should be shown.
+        """
+        out = six.StringIO()
+        apps.register_model('migrations', UnicodeModel)
+        with self.temporary_migration_module() as migration_dir:
+            call_command("makemigrations", "migrations", stdout=out)
+            self.assertIn(os.path.join(migration_dir, '0001_initial.py'), out.getvalue())
+
+    def test_makemigrations_inconsistent_history(self):
+        """
+        makemigrations should raise InconsistentMigrationHistory exception if
+        there are some migrations applied before their dependencies.
+        """
+        recorder = MigrationRecorder(connection)
+        recorder.record_applied('migrations', '0002_second')
+        msg = "Migration migrations.0002_second is applied before its dependency migrations.0001_initial"
+        with self.temporary_migration_module(module="migrations.test_migrations"):
+            with self.assertRaisesMessage(InconsistentMigrationHistory, msg):
+                call_command("makemigrations")
+
+    @mock.patch('django.db.migrations.questioner.input', return_value='1')
+    @mock.patch('django.db.migrations.questioner.sys.stdin', mock.MagicMock(encoding=sys.getdefaultencoding()))
+    def test_makemigrations_auto_now_add_interactive(self, *args):
+        """
+        makemigrations prompts the user when adding auto_now_add to an existing
+        model.
+        """
+        class Entry(models.Model):
+            title = models.CharField(max_length=255)
+            creation_date = models.DateTimeField(auto_now_add=True)
+
+            class Meta:
+                app_label = 'migrations'
+
+        # Monkeypatch interactive questioner to auto accept
+        with mock.patch('django.db.migrations.questioner.sys.stdout', new_callable=six.StringIO) as prompt_stdout:
+            out = six.StringIO()
+            with self.temporary_migration_module(module='migrations.test_auto_now_add'):
+                call_command('makemigrations', 'migrations', interactive=True, stdout=out)
+            output = force_text(out.getvalue())
+            prompt_output = force_text(prompt_stdout.getvalue())
+            self.assertIn("You can accept the default 'timezone.now' by pressing 'Enter'", prompt_output)
+            self.assertIn("Add field creation_date to entry", output)
+
 
 class SquashMigrationsTests(MigrationTestBase):
     """
@@ -1059,7 +1228,7 @@ class SquashMigrationsTests(MigrationTestBase):
         out = six.StringIO()
         with self.temporary_migration_module(module="migrations.test_migrations"):
             call_command("squashmigrations", "migrations", "0002", interactive=False, verbosity=1, stdout=out)
-        self.assertIn("Optimized from 7 operations to 3 operations.", force_text(out.getvalue()))
+        self.assertIn("Optimized from 8 operations to 3 operations.", force_text(out.getvalue()))
 
     def test_ticket_23799_squashmigrations_no_optimize(self):
         """
