@@ -1,21 +1,24 @@
 from django.apps.registry import Apps
 from django.contrib.contenttypes.fields import GenericForeignKey
 from django.db import models
+from django.db.migrations.exceptions import InvalidBasesError
 from django.db.migrations.operations import (
     AddField, AlterField, DeleteModel, RemoveField,
 )
 from django.db.migrations.state import (
-    InvalidBasesError, ModelState, ProjectState, get_related_models_recursive,
+    ModelState, ProjectState, get_related_models_recursive,
 )
-from django.test import SimpleTestCase, TestCase, override_settings
+from django.test import SimpleTestCase, override_settings
+from django.test.utils import isolate_apps
 from django.utils import six
 
 from .models import (
     FoodManager, FoodQuerySet, ModelWithCustomBase, NoMigrationFoodManager,
+    UnicodeModel,
 )
 
 
-class StateTests(TestCase):
+class StateTests(SimpleTestCase):
     """
     Tests state construction, rendering and modification by operations.
     """
@@ -54,7 +57,7 @@ class StateTests(TestCase):
 
         class Book(models.Model):
             title = models.CharField(max_length=1000)
-            author = models.ForeignKey(Author)
+            author = models.ForeignKey(Author, models.CASCADE)
             contributors = models.ManyToManyField(Author)
 
             class Meta:
@@ -118,16 +121,19 @@ class StateTests(TestCase):
         self.assertEqual(author_state.name, "Author")
         self.assertEqual([x for x, y in author_state.fields], ["id", "name", "bio", "age"])
         self.assertEqual(author_state.fields[1][1].max_length, 255)
-        self.assertEqual(author_state.fields[2][1].null, False)
-        self.assertEqual(author_state.fields[3][1].null, True)
-        self.assertEqual(author_state.options, {"unique_together": {("name", "bio")}, "index_together": {("bio", "age")}})
+        self.assertIs(author_state.fields[2][1].null, False)
+        self.assertIs(author_state.fields[3][1].null, True)
+        self.assertEqual(
+            author_state.options,
+            {"unique_together": {("name", "bio")}, "index_together": {("bio", "age")}}
+        )
         self.assertEqual(author_state.bases, (models.Model, ))
 
         self.assertEqual(book_state.app_label, "migrations")
         self.assertEqual(book_state.name, "Book")
         self.assertEqual([x for x, y in book_state.fields], ["id", "title", "author", "contributors"])
         self.assertEqual(book_state.fields[1][1].max_length, 1000)
-        self.assertEqual(book_state.fields[2][1].null, False)
+        self.assertIs(book_state.fields[2][1].null, False)
         self.assertEqual(book_state.fields[3][1].__class__.__name__, "ManyToManyField")
         self.assertEqual(book_state.options, {"verbose_name": "tome", "db_table": "test_tome"})
         self.assertEqual(book_state.bases, (models.Model, ))
@@ -165,6 +171,145 @@ class StateTests(TestCase):
         self.assertEqual([mgr.args for name, mgr in food_order_manager_state.managers],
                          [('a', 'b', 1, 2), ('x', 'y', 3, 4)])
 
+    def test_custom_default_manager_added_to_the_model_state(self):
+        """
+        When the default manager of the model is a custom manager,
+        it needs to be added to the model state.
+        """
+        new_apps = Apps(['migrations'])
+        custom_manager = models.Manager()
+
+        class Author(models.Model):
+            objects = models.TextField()
+            authors = custom_manager
+
+            class Meta:
+                app_label = 'migrations'
+                apps = new_apps
+
+        project_state = ProjectState.from_apps(new_apps)
+        author_state = project_state.models['migrations', 'author']
+        self.assertEqual(author_state.managers, [('authors', custom_manager)])
+
+    def test_custom_default_manager_named_objects_with_false_migration_flag(self):
+        """
+        When a manager is added with a name of 'objects' but it does not
+        have `use_in_migrations = True`, no migration should be added to the
+        model state (#26643).
+        """
+        new_apps = Apps(['migrations'])
+
+        class Author(models.Model):
+            objects = models.Manager()
+
+            class Meta:
+                app_label = 'migrations'
+                apps = new_apps
+
+        project_state = ProjectState.from_apps(new_apps)
+        author_state = project_state.models['migrations', 'author']
+        self.assertEqual(author_state.managers, [])
+
+    def test_no_duplicate_managers(self):
+        """
+        When a manager is added with `use_in_migrations = True` and a parent
+        model had a manager with the same name and `use_in_migrations = True`,
+        the parent's manager shouldn't appear in the model state (#26881).
+        """
+        new_apps = Apps(['migrations'])
+
+        class PersonManager(models.Manager):
+            use_in_migrations = True
+
+        class Person(models.Model):
+            objects = PersonManager()
+
+            class Meta:
+                abstract = True
+
+        class BossManager(PersonManager):
+            use_in_migrations = True
+
+        class Boss(Person):
+            objects = BossManager()
+
+            class Meta:
+                app_label = 'migrations'
+                apps = new_apps
+
+        project_state = ProjectState.from_apps(new_apps)
+        boss_state = project_state.models['migrations', 'boss']
+        self.assertEqual(boss_state.managers, [('objects', Boss.objects)])
+
+    def test_custom_default_manager(self):
+        new_apps = Apps(['migrations'])
+
+        class Author(models.Model):
+            manager1 = models.Manager()
+            manager2 = models.Manager()
+
+            class Meta:
+                app_label = 'migrations'
+                apps = new_apps
+                default_manager_name = 'manager2'
+
+        project_state = ProjectState.from_apps(new_apps)
+        author_state = project_state.models['migrations', 'author']
+        self.assertEqual(author_state.options['default_manager_name'], 'manager2')
+        self.assertEqual(author_state.managers, [('manager2', Author.manager1)])
+
+    def test_custom_base_manager(self):
+        new_apps = Apps(['migrations'])
+
+        class Author(models.Model):
+            manager1 = models.Manager()
+            manager2 = models.Manager()
+
+            class Meta:
+                app_label = 'migrations'
+                apps = new_apps
+                base_manager_name = 'manager2'
+
+        class Author2(models.Model):
+            manager1 = models.Manager()
+            manager2 = models.Manager()
+
+            class Meta:
+                app_label = 'migrations'
+                apps = new_apps
+                base_manager_name = 'manager1'
+
+        project_state = ProjectState.from_apps(new_apps)
+
+        author_state = project_state.models['migrations', 'author']
+        self.assertEqual(author_state.options['base_manager_name'], 'manager2')
+        self.assertEqual(author_state.managers, [
+            ('manager1', Author.manager1),
+            ('manager2', Author.manager2),
+        ])
+
+        author2_state = project_state.models['migrations', 'author2']
+        self.assertEqual(author2_state.options['base_manager_name'], 'manager1')
+        self.assertEqual(author2_state.managers, [
+            ('manager1', Author2.manager1),
+        ])
+
+    def test_apps_bulk_update(self):
+        """
+        StateApps.bulk_update() should update apps.ready to False and reset
+        the value afterwards.
+        """
+        project_state = ProjectState()
+        apps = project_state.apps
+        with apps.bulk_update():
+            self.assertFalse(apps.ready)
+        self.assertTrue(apps.ready)
+        with self.assertRaises(ValueError):
+            with apps.bulk_update():
+                self.assertFalse(apps.ready)
+                raise ValueError()
+        self.assertTrue(apps.ready)
+
     def test_render(self):
         """
         Tests rendering a ProjectState into an Apps.
@@ -184,11 +329,12 @@ class StateTests(TestCase):
             name="SubTag",
             fields=[
                 ('tag_ptr', models.OneToOneField(
+                    'migrations.Tag',
+                    models.CASCADE,
                     auto_created=True,
                     primary_key=True,
                     to_field='id',
                     serialize=False,
-                    to='migrations.Tag',
                 )),
                 ("awesome", models.BooleanField()),
             ],
@@ -214,18 +360,16 @@ class StateTests(TestCase):
 
         new_apps = project_state.apps
         self.assertEqual(new_apps.get_model("migrations", "Tag")._meta.get_field("name").max_length, 100)
-        self.assertEqual(new_apps.get_model("migrations", "Tag")._meta.get_field("hidden").null, False)
+        self.assertIs(new_apps.get_model("migrations", "Tag")._meta.get_field("hidden").null, False)
 
         self.assertEqual(len(new_apps.get_model("migrations", "SubTag")._meta.local_fields), 2)
 
         Food = new_apps.get_model("migrations", "Food")
-        managers = sorted(Food._meta.managers)
-        self.assertEqual([mgr.name for _, mgr, _ in managers],
+        self.assertEqual([mgr.name for mgr in Food._meta.managers],
                          ['default', 'food_mgr1', 'food_mgr2'])
-        self.assertTrue(all(isinstance(mgr.name, six.text_type) for _, mgr, _ in managers))
-        self.assertEqual([mgr.__class__ for _, mgr, _ in managers],
+        self.assertTrue(all(isinstance(mgr.name, six.text_type) for mgr in Food._meta.managers))
+        self.assertEqual([mgr.__class__ for mgr in Food._meta.managers],
                          [models.Manager, FoodManager, FoodManager])
-        self.assertIs(managers[0][1], Food._default_manager)
 
     def test_render_model_inheritance(self):
         class Book(models.Model):
@@ -411,7 +555,11 @@ class StateTests(TestCase):
         self.assertIs(model_a_old._meta.get_field('b').related_model, model_b_old)
         self.assertIs(model_b_old._meta.get_field('a_ptr').related_model, model_a_old)
 
-        operation = AddField('c', 'to_a', models.OneToOneField('something.A', related_name='from_c'))
+        operation = AddField('c', 'to_a', models.OneToOneField(
+            'something.A',
+            models.CASCADE,
+            related_name='from_c',
+        ))
         operation.state_forwards('something', project_state)
         model_a_new = project_state.apps.get_model('something', 'A')
         model_b_new = project_state.apps.get_model('something', 'B')
@@ -443,7 +591,7 @@ class StateTests(TestCase):
                 apps = new_apps
 
         class B(models.Model):
-            to_a = models.ForeignKey(A)
+            to_a = models.ForeignKey(A, models.CASCADE)
 
             class Meta:
                 app_label = "something"
@@ -519,16 +667,16 @@ class StateTests(TestCase):
         self.assertEqual(field_to_a_old.m2m_field_name(), "from_a")
         self.assertEqual(field_to_a_old.m2m_reverse_field_name(), "to_a")
         self.assertIs(field_to_a_old.related_model, model_a_old)
-        self.assertIs(field_to_a_old.rel.through._meta.get_field('to_a').related_model, model_a_old)
-        self.assertIs(field_to_a_old.rel.through._meta.get_field('from_a').related_model, model_a_old)
+        self.assertIs(field_to_a_old.remote_field.through._meta.get_field('to_a').related_model, model_a_old)
+        self.assertIs(field_to_a_old.remote_field.through._meta.get_field('from_a').related_model, model_a_old)
 
         # Tests that the new model's _meta is still consistent
         field_to_a_new = model_a_new._meta.get_field("to_a")
         self.assertEqual(field_to_a_new.m2m_field_name(), "from_a")
         self.assertEqual(field_to_a_new.m2m_reverse_field_name(), "to_a")
         self.assertIs(field_to_a_new.related_model, model_a_new)
-        self.assertIs(field_to_a_new.rel.through._meta.get_field('to_a').related_model, model_a_new)
-        self.assertIs(field_to_a_new.rel.through._meta.get_field('from_a').related_model, model_a_new)
+        self.assertIs(field_to_a_new.remote_field.through._meta.get_field('to_a').related_model, model_a_new)
+        self.assertIs(field_to_a_new.remote_field.through._meta.get_field('from_a').related_model, model_a_new)
 
     def test_equality(self):
         """
@@ -552,8 +700,8 @@ class StateTests(TestCase):
         other_state = project_state.clone()
         self.assertEqual(project_state, project_state)
         self.assertEqual(project_state, other_state)
-        self.assertEqual(project_state != project_state, False)
-        self.assertEqual(project_state != other_state, False)
+        self.assertIs(project_state != project_state, False)
+        self.assertIs(project_state != other_state, False)
         self.assertNotEqual(project_state.apps, other_state.apps)
 
         # Make a very small change (max_len 99) and see if that affects it
@@ -570,7 +718,7 @@ class StateTests(TestCase):
             None,
         ))
         self.assertNotEqual(project_state, other_state)
-        self.assertEqual(project_state == other_state, False)
+        self.assertIs(project_state == other_state, False)
 
     def test_dangling_references_throw_error(self):
         new_apps = Apps()
@@ -582,8 +730,16 @@ class StateTests(TestCase):
                 app_label = "migrations"
                 apps = new_apps
 
+        class Publisher(models.Model):
+            name = models.TextField()
+
+            class Meta:
+                app_label = "migrations"
+                apps = new_apps
+
         class Book(models.Model):
-            author = models.ForeignKey(Author)
+            author = models.ForeignKey(Author, models.CASCADE)
+            publisher = models.ForeignKey(Publisher, models.CASCADE)
 
             class Meta:
                 app_label = "migrations"
@@ -599,20 +755,48 @@ class StateTests(TestCase):
         # Make a valid ProjectState and render it
         project_state = ProjectState()
         project_state.add_model(ModelState.from_model(Author))
+        project_state.add_model(ModelState.from_model(Publisher))
         project_state.add_model(ModelState.from_model(Book))
         project_state.add_model(ModelState.from_model(Magazine))
-        self.assertEqual(len(project_state.apps.get_models()), 3)
+        self.assertEqual(len(project_state.apps.get_models()), 4)
 
         # now make an invalid one with a ForeignKey
         project_state = ProjectState()
         project_state.add_model(ModelState.from_model(Book))
-        with self.assertRaises(ValueError):
+        msg = (
+            "The field migrations.Book.author was declared with a lazy reference "
+            "to 'migrations.author', but app 'migrations' doesn't provide model 'author'.\n"
+            "The field migrations.Book.publisher was declared with a lazy reference "
+            "to 'migrations.publisher', but app 'migrations' doesn't provide model 'publisher'."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
             project_state.apps
 
-        # and another with ManyToManyField
+        # And another with ManyToManyField.
         project_state = ProjectState()
         project_state.add_model(ModelState.from_model(Magazine))
-        with self.assertRaises(ValueError):
+        msg = (
+            "The field migrations.Magazine.authors was declared with a lazy reference "
+            "to 'migrations.author\', but app 'migrations' doesn't provide model 'author'.\n"
+            "The field migrations.Magazine_authors.author was declared with a lazy reference "
+            "to \'migrations.author\', but app 'migrations' doesn't provide model 'author'."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            project_state.apps
+
+        # And now with multiple models and multiple fields.
+        project_state.add_model(ModelState.from_model(Book))
+        msg = (
+            "The field migrations.Book.author was declared with a lazy reference "
+            "to 'migrations.author', but app 'migrations' doesn't provide model 'author'.\n"
+            "The field migrations.Book.publisher was declared with a lazy reference "
+            "to 'migrations.publisher', but app 'migrations' doesn't provide model 'publisher'.\n"
+            "The field migrations.Magazine.authors was declared with a lazy reference "
+            "to 'migrations.author', but app 'migrations' doesn't provide model 'author'.\n"
+            "The field migrations.Magazine_authors.author was declared with a lazy reference "
+            "to 'migrations.author', but app 'migrations' doesn't provide model 'author'."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
             project_state.apps
 
     def test_real_apps(self):
@@ -623,7 +807,7 @@ class StateTests(TestCase):
         new_apps = Apps()
 
         class TestModel(models.Model):
-            ct = models.ForeignKey("contenttypes.ContentType")
+            ct = models.ForeignKey("contenttypes.ContentType", models.CASCADE)
 
             class Meta:
                 app_label = "migrations"
@@ -659,7 +843,7 @@ class StateTests(TestCase):
                 apps = new_apps
 
         class Book(models.Model):
-            author = models.ForeignKey(Author)
+            author = models.ForeignKey(Author, models.CASCADE)
 
             class Meta:
                 app_label = "migrations"
@@ -712,8 +896,28 @@ class StateTests(TestCase):
         self.assertIsNot(old_model.food_mgr.model, new_model.food_mgr.model)
         self.assertIsNot(old_model.food_qs.model, new_model.food_qs.model)
 
+    def test_choices_iterator(self):
+        """
+        #24483 - ProjectState.from_apps should not destructively consume
+        Field.choices iterators.
+        """
+        new_apps = Apps(["migrations"])
+        choices = [('a', 'A'), ('b', 'B')]
 
-class ModelStateTests(TestCase):
+        class Author(models.Model):
+            name = models.CharField(max_length=255)
+            choice = models.CharField(max_length=255, choices=iter(choices))
+
+            class Meta:
+                app_label = "migrations"
+                apps = new_apps
+
+        ProjectState.from_apps(new_apps)
+        choices_field = Author._meta.get_field('choice')
+        self.assertEqual(list(choices_field.choices), choices)
+
+
+class ModelStateTests(SimpleTestCase):
     def test_custom_model_base(self):
         state = ModelState.from_model(ModelWithCustomBase)
         self.assertEqual(state.bases, (models.Model,))
@@ -721,8 +925,26 @@ class ModelStateTests(TestCase):
     def test_bound_field_sanity_check(self):
         field = models.CharField(max_length=1)
         field.model = models.Model
-        with self.assertRaisesMessage(ValueError,
-                'ModelState.fields cannot be bound to a model - "field" is.'):
+        with self.assertRaisesMessage(ValueError, 'ModelState.fields cannot be bound to a model - "field" is.'):
+            ModelState('app', 'Model', [('field', field)])
+
+    def test_sanity_check_to(self):
+        field = models.ForeignKey(UnicodeModel, models.CASCADE)
+        with self.assertRaisesMessage(
+            ValueError,
+            'ModelState.fields cannot refer to a model class - "field.to" does. '
+            'Use a string reference instead.'
+        ):
+            ModelState('app', 'Model', [('field', field)])
+
+    def test_sanity_check_through(self):
+        field = models.ManyToManyField('UnicodeModel')
+        field.remote_field.through = UnicodeModel
+        with self.assertRaisesMessage(
+            ValueError,
+            'ModelState.fields cannot refer to a model class - "field.through" does. '
+            'Use a string reference instead.'
+        ):
             ModelState('app', 'Model', [('field', field)])
 
     def test_fields_immutability(self):
@@ -767,8 +989,8 @@ class ModelStateTests(TestCase):
         self.assertEqual(author_state.name, 'Author')
         self.assertEqual([x for x, y in author_state.fields], ['id', 'name', 'bio', 'age'])
         self.assertEqual(author_state.fields[1][1].max_length, 255)
-        self.assertEqual(author_state.fields[2][1].null, False)
-        self.assertEqual(author_state.fields[3][1].null, True)
+        self.assertIs(author_state.fields[2][1].null, False)
+        self.assertIs(author_state.fields[3][1].null, True)
         self.assertEqual(author_state.options, {'swappable': 'TEST_SWAPPABLE_MODEL'})
         self.assertEqual(author_state.bases, (models.Model, ))
         self.assertEqual(author_state.managers, [])
@@ -797,6 +1019,19 @@ class ModelStateTests(TestCase):
         self.assertEqual([name for name, mgr in food_state.managers], ['food_mgr'])
         self.assertEqual(food_state.managers[0][1].args, ('a', 'b', 1, 2))
 
+    @isolate_apps('migrations', 'django.contrib.contenttypes')
+    def test_order_with_respect_to_private_field(self):
+        class PrivateFieldModel(models.Model):
+            content_type = models.ForeignKey('contenttypes.ContentType', models.CASCADE)
+            object_id = models.PositiveIntegerField()
+            private = GenericForeignKey()
+
+            class Meta:
+                order_with_respect_to = 'private'
+
+        state = ModelState.from_model(PrivateFieldModel)
+        self.assertNotIn('order_with_respect_to', state.options)
+
 
 class RelatedModelsTests(SimpleTestCase):
 
@@ -838,35 +1073,45 @@ class RelatedModelsTests(SimpleTestCase):
         self.assertRelated(B, [])
 
     def test_direct_fk(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('B')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('B', models.CASCADE)])
         B = self.create_model("B")
         self.assertRelated(A, [B])
         self.assertRelated(B, [A])
 
     def test_direct_hidden_fk(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('B', related_name='+')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('B', models.CASCADE, related_name='+')])
         B = self.create_model("B")
         self.assertRelated(A, [B])
         self.assertRelated(B, [A])
 
+    def test_fk_through_proxy(self):
+        A = self.create_model("A")
+        B = self.create_model("B", bases=(A,), proxy=True)
+        C = self.create_model("C", bases=(B,), proxy=True)
+        D = self.create_model("D", foreign_keys=[models.ForeignKey('C', models.CASCADE)])
+        self.assertRelated(A, [B, C, D])
+        self.assertRelated(B, [A, C, D])
+        self.assertRelated(C, [A, B, D])
+        self.assertRelated(D, [A, B, C])
+
     def test_nested_fk(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('B')])
-        B = self.create_model("B", foreign_keys=[models.ForeignKey('C')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('B', models.CASCADE)])
+        B = self.create_model("B", foreign_keys=[models.ForeignKey('C', models.CASCADE)])
         C = self.create_model("C")
         self.assertRelated(A, [B, C])
         self.assertRelated(B, [A, C])
         self.assertRelated(C, [A, B])
 
     def test_two_sided(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('B')])
-        B = self.create_model("B", foreign_keys=[models.ForeignKey('A')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('B', models.CASCADE)])
+        B = self.create_model("B", foreign_keys=[models.ForeignKey('A', models.CASCADE)])
         self.assertRelated(A, [B])
         self.assertRelated(B, [A])
 
     def test_circle(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('B')])
-        B = self.create_model("B", foreign_keys=[models.ForeignKey('C')])
-        C = self.create_model("C", foreign_keys=[models.ForeignKey('A')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('B', models.CASCADE)])
+        B = self.create_model("B", foreign_keys=[models.ForeignKey('C', models.CASCADE)])
+        C = self.create_model("C", foreign_keys=[models.ForeignKey('A', models.CASCADE)])
         self.assertRelated(A, [B, C])
         self.assertRelated(B, [A, C])
         self.assertRelated(C, [A, B])
@@ -912,7 +1157,7 @@ class RelatedModelsTests(SimpleTestCase):
         self.assertRelated(Z, [Y])
 
     def test_base_to_base_fk(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('Y')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('Y', models.CASCADE)])
         B = self.create_model("B", bases=(A,))
         Y = self.create_model("Y")
         Z = self.create_model("Z", bases=(Y,))
@@ -922,7 +1167,7 @@ class RelatedModelsTests(SimpleTestCase):
         self.assertRelated(Z, [A, B, Y])
 
     def test_base_to_subclass_fk(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('Z')])
+        A = self.create_model("A", foreign_keys=[models.ForeignKey('Z', models.CASCADE)])
         B = self.create_model("B", bases=(A,))
         Y = self.create_model("Y")
         Z = self.create_model("Z", bases=(Y,))
@@ -934,23 +1179,29 @@ class RelatedModelsTests(SimpleTestCase):
     def test_direct_m2m(self):
         A = self.create_model("A", foreign_keys=[models.ManyToManyField('B')])
         B = self.create_model("B")
-        self.assertRelated(A, [A.a_1.through, B])
-        self.assertRelated(B, [A, A.a_1.through])
+        self.assertRelated(A, [A.a_1.rel.through, B])
+        self.assertRelated(B, [A, A.a_1.rel.through])
 
     def test_direct_m2m_self(self):
         A = self.create_model("A", foreign_keys=[models.ManyToManyField('A')])
-        self.assertRelated(A, [A.a_1.through])
+        self.assertRelated(A, [A.a_1.rel.through])
 
     def test_intermediate_m2m_self(self):
         A = self.create_model("A", foreign_keys=[models.ManyToManyField('A', through='T')])
-        T = self.create_model("T", foreign_keys=[models.ForeignKey('A'), models.ForeignKey('A')])
+        T = self.create_model("T", foreign_keys=[
+            models.ForeignKey('A', models.CASCADE),
+            models.ForeignKey('A', models.CASCADE),
+        ])
         self.assertRelated(A, [T])
         self.assertRelated(T, [A])
 
     def test_intermediate_m2m(self):
         A = self.create_model("A", foreign_keys=[models.ManyToManyField('B', through='T')])
         B = self.create_model("B")
-        T = self.create_model("T", foreign_keys=[models.ForeignKey('A'), models.ForeignKey('B')])
+        T = self.create_model("T", foreign_keys=[
+            models.ForeignKey('A', models.CASCADE),
+            models.ForeignKey('B', models.CASCADE),
+        ])
         self.assertRelated(A, [B, T])
         self.assertRelated(B, [A, T])
         self.assertRelated(T, [A, B])
@@ -960,7 +1211,9 @@ class RelatedModelsTests(SimpleTestCase):
         B = self.create_model("B")
         Z = self.create_model("Z")
         T = self.create_model("T", foreign_keys=[
-            models.ForeignKey('A'), models.ForeignKey('B'), models.ForeignKey('Z'),
+            models.ForeignKey('A', models.CASCADE),
+            models.ForeignKey('B', models.CASCADE),
+            models.ForeignKey('Z', models.CASCADE),
         ])
         self.assertRelated(A, [B, T, Z])
         self.assertRelated(B, [A, T, Z])
@@ -971,15 +1224,23 @@ class RelatedModelsTests(SimpleTestCase):
         A = self.create_model("A", foreign_keys=[models.ManyToManyField('B', through='T')])
         B = self.create_model("B")
         S = self.create_model("S")
-        T = self.create_model("T", foreign_keys=[models.ForeignKey('A'), models.ForeignKey('B')], bases=(S,))
+        T = self.create_model("T", foreign_keys=[
+            models.ForeignKey('A', models.CASCADE),
+            models.ForeignKey('B', models.CASCADE),
+        ], bases=(S,))
         self.assertRelated(A, [B, S, T])
         self.assertRelated(B, [A, S, T])
         self.assertRelated(S, [A, B, T])
         self.assertRelated(T, [A, B, S])
 
     def test_generic_fk(self):
-        A = self.create_model("A", foreign_keys=[models.ForeignKey('B'), GenericForeignKey()])
-        B = self.create_model("B", foreign_keys=[models.ForeignKey('C')])
+        A = self.create_model("A", foreign_keys=[
+            models.ForeignKey('B', models.CASCADE),
+            GenericForeignKey(),
+        ])
+        B = self.create_model("B", foreign_keys=[
+            models.ForeignKey('C', models.CASCADE),
+        ])
         self.assertRelated(A, [B])
         self.assertRelated(B, [A])
 
