@@ -26,12 +26,12 @@ class Foo(models.Model):
 
 
 def get_foo():
-    return Foo.objects.get(id=1)
+    return Foo.objects.get(id=1).pk
 
 
 class Bar(models.Model):
     b = models.CharField(max_length=10)
-    a = models.ForeignKey(Foo, default=get_foo, related_name=b'bars')
+    a = models.ForeignKey(Foo, models.CASCADE, default=get_foo, related_name=b'bars')
 
 
 class Whiz(models.Model):
@@ -86,6 +86,10 @@ class BigS(models.Model):
     s = models.SlugField(max_length=255)
 
 
+class UnicodeSlugField(models.Model):
+    s = models.SlugField(max_length=255, allow_unicode=True)
+
+
 class SmallIntegerModel(models.Model):
     value = models.SmallIntegerField()
 
@@ -117,7 +121,7 @@ class NullBooleanModel(models.Model):
 
 
 class BooleanModel(models.Model):
-    bfield = models.BooleanField(default=None)
+    bfield = models.BooleanField()
     string = models.CharField(max_length=10, default='abc')
 
 
@@ -141,13 +145,13 @@ class PrimaryKeyCharModel(models.Model):
 
 class FksToBooleans(models.Model):
     """Model with FKs to models with {Null,}BooleanField's, #15040"""
-    bf = models.ForeignKey(BooleanModel)
-    nbf = models.ForeignKey(NullBooleanModel)
+    bf = models.ForeignKey(BooleanModel, models.CASCADE)
+    nbf = models.ForeignKey(NullBooleanModel, models.CASCADE)
 
 
 class FkToChar(models.Model):
     """Model with FK to a model with a CharField primary key, #19299"""
-    out = models.ForeignKey(PrimaryKeyCharModel)
+    out = models.ForeignKey(PrimaryKeyCharModel, models.CASCADE)
 
 
 class RenamedField(models.Model):
@@ -170,18 +174,17 @@ class VerboseNameField(models.Model):
     # Don't want to depend on Pillow in this test
     # field_image = models.ImageField("verbose field")
     field12 = models.IntegerField("verbose field12")
-    field13 = models.IPAddressField("verbose field13")
-    field14 = models.GenericIPAddressField("verbose field14", protocol="ipv4")
-    field15 = models.NullBooleanField("verbose field15")
-    field16 = models.PositiveIntegerField("verbose field16")
-    field17 = models.PositiveSmallIntegerField("verbose field17")
-    field18 = models.SlugField("verbose field18")
-    field19 = models.SmallIntegerField("verbose field19")
-    field20 = models.TextField("verbose field20")
-    field21 = models.TimeField("verbose field21")
-    field22 = models.URLField("verbose field22")
-    field23 = models.UUIDField("verbose field23")
-    field24 = models.DurationField("verbose field24")
+    field13 = models.GenericIPAddressField("verbose field13", protocol="ipv4")
+    field14 = models.NullBooleanField("verbose field14")
+    field15 = models.PositiveIntegerField("verbose field15")
+    field16 = models.PositiveSmallIntegerField("verbose field16")
+    field17 = models.SlugField("verbose field17")
+    field18 = models.SmallIntegerField("verbose field18")
+    field19 = models.TextField("verbose field19")
+    field20 = models.TimeField("verbose field20")
+    field21 = models.URLField("verbose field21")
+    field22 = models.UUIDField("verbose field22")
+    field23 = models.DurationField("verbose field23")
 
 
 class GenericIPAddress(models.Model):
@@ -327,7 +330,6 @@ class AllFieldsModel(models.Model):
     file_path = models.FilePathField()
     floatf = models.FloatField()
     integer = models.IntegerField()
-    ip_address = models.IPAddressField()
     generic_ip = models.GenericIPAddressField()
     null_boolean = models.NullBooleanField()
     positive_integer = models.PositiveIntegerField()
@@ -341,19 +343,21 @@ class AllFieldsModel(models.Model):
 
     fo = ForeignObject(
         'self',
+        on_delete=models.CASCADE,
         from_fields=['abstract_non_concrete_id'],
         to_fields=['id'],
         related_name='reverse'
     )
     fk = ForeignKey(
         'self',
+        models.CASCADE,
         related_name='reverse2'
     )
     m2m = ManyToManyField('self')
-    oto = OneToOneField('self')
+    oto = OneToOneField('self', models.CASCADE)
 
     object_id = models.PositiveIntegerField()
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, models.CASCADE)
     gfk = GenericForeignKey()
     gr = GenericRelation(DataModel)
 
@@ -374,7 +378,7 @@ class PrimaryKeyUUIDModel(models.Model):
 
 
 class RelatedToUUIDModel(models.Model):
-    uuid_fk = models.ForeignKey('PrimaryKeyUUIDModel')
+    uuid_fk = models.ForeignKey('PrimaryKeyUUIDModel', models.CASCADE)
 
 
 class UUIDChild(PrimaryKeyUUIDModel):
