@@ -17,8 +17,10 @@ except ImproperlyConfigured:
     Image = None
 
 if Image:
-    from .models import (Person, PersonWithHeight, PersonWithHeightAndWidth,
-        PersonDimensionsFirst, PersonTwoImages, TestImageFieldFile)
+    from .models import (
+        Person, PersonWithHeight, PersonWithHeightAndWidth,
+        PersonDimensionsFirst, PersonTwoImages, TestImageFieldFile,
+    )
     from .models import temp_storage_dir
 else:
     # Pillow not available, create dummy classes (tests will be skipped anyway)
@@ -52,10 +54,10 @@ class ImageFieldTestMixin(SerializeMixin):
         os.mkdir(temp_storage_dir)
 
         file_path1 = os.path.join(os.path.dirname(upath(__file__)), "4x8.png")
-        self.file1 = self.File(open(file_path1, 'rb'))
+        self.file1 = self.File(open(file_path1, 'rb'), name='4x8.png')
 
         file_path2 = os.path.join(os.path.dirname(upath(__file__)), "8x4.png")
-        self.file2 = self.File(open(file_path2, 'rb'))
+        self.file2 = self.File(open(file_path2, 'rb'), name='8x4.png')
 
     def tearDown(self):
         """
@@ -83,8 +85,10 @@ class ImageFieldTestMixin(SerializeMixin):
         field = getattr(instance, field_name)
         # Check height/width attributes of field.
         if width is None and height is None:
-            self.assertRaises(ValueError, getattr, field, 'width')
-            self.assertRaises(ValueError, getattr, field, 'height')
+            with self.assertRaises(ValueError):
+                getattr(field, 'width')
+            with self.assertRaises(ValueError):
+                getattr(field, 'height')
         else:
             self.assertEqual(field.width, width)
             self.assertEqual(field.height, height)
@@ -116,18 +120,18 @@ class ImageFieldTests(ImageFieldTestMixin, TestCase):
         p1.mugshot.save("mug", self.file1)
         p2 = self.PersonModel(name="Bob")
         p2.mugshot.save("mug", self.file2)
-        self.assertEqual(p1.mugshot == p2.mugshot, False)
-        self.assertEqual(p1.mugshot != p2.mugshot, True)
+        self.assertIs(p1.mugshot == p2.mugshot, False)
+        self.assertIs(p1.mugshot != p2.mugshot, True)
 
         # Test again with an instance fetched from the db.
         p1_db = self.PersonModel.objects.get(name="Joe")
-        self.assertEqual(p1_db.mugshot == p2.mugshot, False)
-        self.assertEqual(p1_db.mugshot != p2.mugshot, True)
+        self.assertIs(p1_db.mugshot == p2.mugshot, False)
+        self.assertIs(p1_db.mugshot != p2.mugshot, True)
 
         # Instance from db should match the local instance.
-        self.assertEqual(p1_db.mugshot == p1.mugshot, True)
+        self.assertIs(p1_db.mugshot == p1.mugshot, True)
         self.assertEqual(hash(p1_db.mugshot), hash(p1.mugshot))
-        self.assertEqual(p1_db.mugshot != p1.mugshot, False)
+        self.assertIs(p1_db.mugshot != p1.mugshot, False)
 
     def test_instantiate_missing(self):
         """
@@ -161,11 +165,11 @@ class ImageFieldTests(ImageFieldTestMixin, TestCase):
         # Get a "clean" model instance
         p = self.PersonModel.objects.get(name="Joan")
         # It won't have an opened file.
-        self.assertEqual(p.mugshot.closed, True)
+        self.assertIs(p.mugshot.closed, True)
 
         # After asking for the size, the file should still be closed.
         p.mugshot.size
-        self.assertEqual(p.mugshot.closed, True)
+        self.assertIs(p.mugshot.closed, True)
 
     def test_pickle(self):
         """
@@ -184,6 +188,13 @@ class ImageFieldTests(ImageFieldTestMixin, TestCase):
         loaded_p = pickle.loads(dump)
         self.assertEqual(p.mugshot, loaded_p.mugshot)
 
+    def test_defer(self):
+        self.PersonModel.objects.create(name='Joe', mugshot=self.file1)
+        with self.assertNumQueries(1):
+            qs = list(self.PersonModel.objects.defer('mugshot'))
+        with self.assertNumQueries(0):
+            self.assertEqual(qs[0].name, 'Joe')
+
 
 @skipIf(Image is None, "Pillow is required to test ImageField")
 class ImageFieldTwoDimensionsTests(ImageFieldTestMixin, TestCase):
@@ -209,7 +220,7 @@ class ImageFieldTwoDimensionsTests(ImageFieldTestMixin, TestCase):
         # attr_class, a  TestImageFieldFile, with name == None, which will
         # cause it to evaluate as False.
         self.assertIsInstance(p.mugshot, TestImageFieldFile)
-        self.assertEqual(bool(p.mugshot), False)
+        self.assertFalse(p.mugshot)
 
         # Test setting a fresh created model instance.
         p = self.PersonModel(name='Joe')
@@ -231,7 +242,7 @@ class ImageFieldTwoDimensionsTests(ImageFieldTestMixin, TestCase):
         """
         p = self.PersonModel()
         self.assertIsInstance(p.mugshot, TestImageFieldFile)
-        self.assertEqual(bool(p.mugshot), False)
+        self.assertFalse(p.mugshot)
 
     def test_assignment_to_None(self):
         """
@@ -280,23 +291,23 @@ class ImageFieldTwoDimensionsTests(ImageFieldTestMixin, TestCase):
         # Bug 11084: Dimensions should not get recalculated if file is
         # coming from the database.  We test this by checking if the file
         # was opened.
-        self.assertEqual(p.mugshot.was_opened, False)
+        self.assertIs(p.mugshot.was_opened, False)
         self.check_dimensions(p, 4, 8)
         # After checking dimensions on the image field, the file will have
         # opened.
-        self.assertEqual(p.mugshot.was_opened, True)
+        self.assertIs(p.mugshot.was_opened, True)
         # Dimensions should now be cached, and if we reset was_opened and
         # check dimensions again, the file should not have opened.
         p.mugshot.was_opened = False
         self.check_dimensions(p, 4, 8)
-        self.assertEqual(p.mugshot.was_opened, False)
+        self.assertIs(p.mugshot.was_opened, False)
 
         # If we assign a new image to the instance, the dimensions should
         # update.
         p.mugshot = self.file2
         self.check_dimensions(p, 8, 4)
         # Dimensions were recalculated, and hence file should have opened.
-        self.assertEqual(p.mugshot.was_opened, True)
+        self.assertIs(p.mugshot.was_opened, True)
 
 
 @skipIf(Image is None, "Pillow is required to test ImageField")
@@ -415,22 +426,22 @@ class TwoImageFieldTests(ImageFieldTestMixin, TestCase):
         # Bug 11084: Dimensions should not get recalculated if file is
         # coming from the database.  We test this by checking if the file
         # was opened.
-        self.assertEqual(p.mugshot.was_opened, False)
-        self.assertEqual(p.headshot.was_opened, False)
+        self.assertIs(p.mugshot.was_opened, False)
+        self.assertIs(p.headshot.was_opened, False)
         self.check_dimensions(p, 4, 8, 'mugshot')
         self.check_dimensions(p, 8, 4, 'headshot')
         # After checking dimensions on the image fields, the files will
         # have been opened.
-        self.assertEqual(p.mugshot.was_opened, True)
-        self.assertEqual(p.headshot.was_opened, True)
+        self.assertIs(p.mugshot.was_opened, True)
+        self.assertIs(p.headshot.was_opened, True)
         # Dimensions should now be cached, and if we reset was_opened and
         # check dimensions again, the file should not have opened.
         p.mugshot.was_opened = False
         p.headshot.was_opened = False
         self.check_dimensions(p, 4, 8, 'mugshot')
         self.check_dimensions(p, 8, 4, 'headshot')
-        self.assertEqual(p.mugshot.was_opened, False)
-        self.assertEqual(p.headshot.was_opened, False)
+        self.assertIs(p.mugshot.was_opened, False)
+        self.assertIs(p.headshot.was_opened, False)
 
         # If we assign a new image to the instance, the dimensions should
         # update.
@@ -439,5 +450,5 @@ class TwoImageFieldTests(ImageFieldTestMixin, TestCase):
         self.check_dimensions(p, 8, 4, 'mugshot')
         self.check_dimensions(p, 4, 8, 'headshot')
         # Dimensions were recalculated, and hence file should have opened.
-        self.assertEqual(p.mugshot.was_opened, True)
-        self.assertEqual(p.headshot.was_opened, True)
+        self.assertIs(p.mugshot.was_opened, True)
+        self.assertIs(p.headshot.was_opened, True)
