@@ -2,12 +2,11 @@ from __future__ import unicode_literals
 
 from operator import attrgetter
 
-from django.apps.registry import Apps
 from django.core.exceptions import FieldError, ValidationError
 from django.core.management import call_command
 from django.db import connection, models
 from django.test import TestCase, TransactionTestCase
-from django.test.utils import CaptureQueriesContext
+from django.test.utils import CaptureQueriesContext, isolate_apps
 from django.utils import six
 
 from .models import (
@@ -20,7 +19,7 @@ from .models import (
 class ModelInheritanceTests(TestCase):
     def test_abstract(self):
         # The Student and Worker models both have 'name' and 'age' fields on
-        # them and inherit the __unicode__() method, just as with normal Python
+        # them and inherit the __str__() method, just as with normal Python
         # subclassing. This is useful if you want to factor out common
         # information for programming purposes, but still completely
         # independent separate models at the database level.
@@ -49,14 +48,16 @@ class ModelInheritanceTests(TestCase):
 
         # However, the CommonInfo class cannot be used as a normal model (it
         # doesn't exist as a model).
-        self.assertRaises(AttributeError, lambda: CommonInfo.objects.all())
+        with self.assertRaises(AttributeError):
+            CommonInfo.objects.all()
 
     def test_reverse_relation_for_different_hierarchy_tree(self):
         # Even though p.supplier for a Place 'p' (a parent of a Supplier), a
         # Restaurant object cannot access that reverse relation, since it's not
         # part of the Place-Supplier Hierarchy.
         self.assertQuerysetEqual(Place.objects.filter(supplier__name="foo"), [])
-        self.assertRaises(FieldError, Restaurant.objects.filter, supplier__name="foo")
+        with self.assertRaises(FieldError):
+            Restaurant.objects.filter(supplier__name="foo")
 
     def test_model_with_distinct_accessors(self):
         # The Post model has distinct accessors for the Comment and Link models.
@@ -69,9 +70,17 @@ class ModelInheritanceTests(TestCase):
 
         # The Post model doesn't have an attribute called
         # 'attached_%(class)s_set'.
-        self.assertRaises(
-            AttributeError, getattr, post, "attached_%(class)s_set"
-        )
+        with self.assertRaises(AttributeError):
+            getattr(post, "attached_%(class)s_set")
+
+    def test_model_with_distinct_related_query_name(self):
+        self.assertQuerysetEqual(Post.objects.filter(attached_model_inheritance_comments__is_spam=True), [])
+
+        # The Post model doesn't have a related query accessor based on
+        # related_name (attached_comment_set).
+        msg = "Cannot resolve keyword 'attached_comment_set' into field."
+        with self.assertRaisesMessage(FieldError, msg):
+            Post.objects.filter(attached_comment_set__is_spam=True)
 
     def test_meta_fields_and_ordering(self):
         # Make sure Restaurant and ItalianRestaurant have the right fields in
@@ -131,23 +140,19 @@ class ModelInheritanceTests(TestCase):
         m = MixinModel()
         self.assertEqual(m.other_attr, 1)
 
+    @isolate_apps('model_inheritance')
     def test_abstract_parent_link(self):
-        test_apps = Apps(['model_inheritance'])
-
         class A(models.Model):
-            class Meta:
-                apps = test_apps
+            pass
 
         class B(A):
             a = models.OneToOneField('A', parent_link=True, on_delete=models.CASCADE)
 
             class Meta:
-                apps = test_apps
                 abstract = True
 
         class C(B):
-            class Meta:
-                apps = test_apps
+            pass
 
         self.assertIs(C._meta.parents[A], C._meta.get_field('a'))
 
@@ -232,18 +237,14 @@ class ModelInheritanceDataTests(TestCase):
     def test_parent_child_one_to_one_link_on_nonrelated_objects(self):
         # This won't work because the Demon Dogs restaurant is not an Italian
         # restaurant.
-        self.assertRaises(
-            ItalianRestaurant.DoesNotExist,
-            lambda: Place.objects.get(name="Demon Dogs").restaurant.italianrestaurant
-        )
+        with self.assertRaises(ItalianRestaurant.DoesNotExist):
+            Place.objects.get(name="Demon Dogs").restaurant.italianrestaurant
 
     def test_inherited_does_not_exist_exception(self):
         # An ItalianRestaurant which does not exist is also a Place which does
         # not exist.
-        self.assertRaises(
-            Place.DoesNotExist,
-            ItalianRestaurant.objects.get, name="The Noodle Void"
-        )
+        with self.assertRaises(Place.DoesNotExist):
+            ItalianRestaurant.objects.get(name="The Noodle Void")
 
     def test_inherited_multiple_objects_returned_exception(self):
         # MultipleObjectsReturned is also inherited.
@@ -253,16 +254,15 @@ class ModelInheritanceDataTests(TestCase):
     def test_related_objects_for_inherited_models(self):
         # Related objects work just as they normally do.
         s1 = Supplier.objects.create(name="Joe's Chickens", address="123 Sesame St")
-        s1.customers = [self.restaurant, self.italian_restaurant]
+        s1.customers .set([self.restaurant, self.italian_restaurant])
         s2 = Supplier.objects.create(name="Luigi's Pasta", address="456 Sesame St")
-        s2.customers = [self.italian_restaurant]
+        s2.customers.set([self.italian_restaurant])
 
         # This won't work because the Place we select is not a Restaurant (it's
         # a Supplier).
         p = Place.objects.get(name="Joe's Chickens")
-        self.assertRaises(
-            Restaurant.DoesNotExist, lambda: p.restaurant
-        )
+        with self.assertRaises(Restaurant.DoesNotExist):
+            p.restaurant
 
         self.assertEqual(p.supplier, s1)
         self.assertQuerysetEqual(
@@ -336,10 +336,8 @@ class ModelInheritanceDataTests(TestCase):
         #23370 - Should be able to defer child fields when using
         select_related() from parent to child.
         """
-        qs = (Restaurant.objects
-            .select_related("italianrestaurant")
-            .defer("italianrestaurant__serves_gnocchi")
-            .order_by("rating"))
+        qs = (Restaurant.objects.select_related("italianrestaurant")
+              .defer("italianrestaurant__serves_gnocchi").order_by("rating"))
 
         # Test that the field was actually deferred
         with self.assertNumQueries(2):
