@@ -13,10 +13,9 @@ from django.test import TestCase
 from .models import (
     ArticleWithAuthor, BachelorParty, BirthdayParty, BusStation, Child,
     DerivedM, InternalCertificationAudit, ItalianRestaurant, M2MChild,
-    MessyBachelorParty, ParkingLot, ParkingLot2, ParkingLot3, ParkingLot4A,
-    ParkingLot4B, Person, Place, Profile, QualityControl, Restaurant,
-    SelfRefChild, SelfRefParent, Senator, Supplier, TrainStation, User,
-    Wholesaler,
+    MessyBachelorParty, ParkingLot, ParkingLot3, ParkingLot4A, ParkingLot4B,
+    Person, Place, Profile, QualityControl, Restaurant, SelfRefChild,
+    SelfRefParent, Senator, Supplier, TrainStation, User, Wholesaler,
 )
 
 
@@ -171,14 +170,10 @@ class ModelInheritanceTest(TestCase):
         # the ItalianRestaurant.
         Restaurant.objects.all().delete()
 
-        self.assertRaises(
-            Place.DoesNotExist,
-            Place.objects.get,
-            pk=ident)
-        self.assertRaises(
-            ItalianRestaurant.DoesNotExist,
-            ItalianRestaurant.objects.get,
-            pk=ident)
+        with self.assertRaises(Place.DoesNotExist):
+            Place.objects.get(pk=ident)
+        with self.assertRaises(ItalianRestaurant.DoesNotExist):
+            ItalianRestaurant.objects.get(pk=ident)
 
     def test_issue_6755(self):
         """
@@ -241,14 +236,12 @@ class ModelInheritanceTest(TestCase):
 
         self.assertEqual(c1.get_next_by_pub_date(), c2)
         self.assertEqual(c2.get_next_by_pub_date(), c3)
-        self.assertRaises(
-            ArticleWithAuthor.DoesNotExist,
-            c3.get_next_by_pub_date)
+        with self.assertRaises(ArticleWithAuthor.DoesNotExist):
+            c3.get_next_by_pub_date()
         self.assertEqual(c3.get_previous_by_pub_date(), c2)
         self.assertEqual(c2.get_previous_by_pub_date(), c1)
-        self.assertRaises(
-            ArticleWithAuthor.DoesNotExist,
-            c1.get_previous_by_pub_date)
+        with self.assertRaises(ArticleWithAuthor.DoesNotExist):
+            c1.get_previous_by_pub_date()
 
     def test_inherited_fields(self):
         """
@@ -299,20 +292,11 @@ class ModelInheritanceTest(TestCase):
 
     def test_use_explicit_o2o_to_parent_as_pk(self):
         """
-        Regression tests for #10406
-        If there's a one-to-one link between a child model and the parent and
-        no explicit pk declared, we can use the one-to-one link as the pk on
-        the child.
+        The connector from child to parent need not be the pk on the child.
         """
-        self.assertEqual(ParkingLot2._meta.pk.name, "parent")
-
-        # However, the connector from child to parent need not be the pk on
-        # the child at all.
         self.assertEqual(ParkingLot3._meta.pk.name, "primary_key")
         # the child->parent link
-        self.assertEqual(
-            ParkingLot3._meta.get_ancestor_link(Place).name,
-            "parent")
+        self.assertEqual(ParkingLot3._meta.get_ancestor_link(Place).name, "parent")
 
     def test_use_explicit_o2o_to_parent_from_abstract_model(self):
         self.assertEqual(ParkingLot4A._meta.pk.name, "parent")
@@ -350,10 +334,10 @@ class ModelInheritanceTest(TestCase):
 
         birthday = BirthdayParty.objects.create(
             name='Birthday party for Alice')
-        birthday.attendees = [p1, p3]
+        birthday.attendees.set([p1, p3])
 
         bachelor = BachelorParty.objects.create(name='Bachelor party for Bob')
-        bachelor.attendees = [p2, p4]
+        bachelor.attendees.set([p2, p4])
 
         parties = list(p1.birthdayparty_set.all())
         self.assertEqual(parties, [birthday])
@@ -371,7 +355,7 @@ class ModelInheritanceTest(TestCase):
         # ... but it does inherit the m2m from its parent
         messy = MessyBachelorParty.objects.create(
             name='Bachelor party for Dave')
-        messy.attendees = [p4]
+        messy.attendees.set([p4])
         messy_parent = messy.bachelorparty_ptr
 
         parties = list(p4.bachelorparty_set.all())
@@ -461,7 +445,10 @@ class ModelInheritanceTest(TestCase):
             name='John Doe', title='X', state='Y'
         )
 
-        Senator.objects.get(pk=senator.pk)
+        senator = Senator.objects.get(pk=senator.pk)
+        self.assertEqual(senator.name, 'John Doe')
+        self.assertEqual(senator.title, 'X')
+        self.assertEqual(senator.state, 'Y')
 
     def test_inheritance_resolve_columns(self):
         Restaurant.objects.create(name='Bobs Cafe', address="Somewhere",
@@ -490,3 +477,22 @@ class ModelInheritanceTest(TestCase):
 
         jane = Supplier.objects.order_by("name").select_related("restaurant")[0]
         self.assertEqual(jane.restaurant.name, "Craft")
+
+    def test_related_filtering_query_efficiency_ticket_15844(self):
+        r = Restaurant.objects.create(
+            name="Guido's House of Pasta",
+            address='944 W. Fullerton',
+            serves_hot_dogs=True,
+            serves_pizza=False,
+        )
+        s = Supplier.objects.create(restaurant=r)
+        with self.assertNumQueries(1):
+            self.assertQuerysetEqual(
+                Supplier.objects.filter(restaurant=r),
+                [s], lambda x: x,
+            )
+        with self.assertNumQueries(1):
+            self.assertQuerysetEqual(
+                r.supplier_set.all(),
+                [s], lambda x: x,
+            )
