@@ -18,9 +18,8 @@ class Band(models.Model):
 
 
 class Concert(models.Model):
-    main_band = models.ForeignKey(Band, related_name='main_concerts')
-    opening_band = models.ForeignKey(Band, related_name='opening_concerts',
-        blank=True)
+    main_band = models.ForeignKey(Band, models.CASCADE, related_name='main_concerts')
+    opening_band = models.ForeignKey(Band, models.CASCADE, related_name='opening_concerts', blank=True)
     day = models.CharField(max_length=3, choices=((1, 'Fri'), (2, 'Sat')))
     transport = models.CharField(max_length=100, choices=(
         (1, 'Plane'),
@@ -36,12 +35,13 @@ class ValidationTestModel(models.Model):
     state = models.CharField(max_length=2, choices=(("CO", "Colorado"), ("WA", "Washington")))
     is_active = models.BooleanField(default=False)
     pub_date = models.DateTimeField()
-    band = models.ForeignKey(Band)
-    no = models.IntegerField(verbose_name="Number", blank=True, null=True)  # This field is intentionally 2 characters long. See #16080.
+    band = models.ForeignKey(Band, models.CASCADE)
+    # This field is intentionally 2 characters long (#16080).
+    no = models.IntegerField(verbose_name="Number", blank=True, null=True)
 
     def decade_published_in(self):
         return self.pub_date.strftime('%Y')[:3] + "0's"
 
 
 class ValidationTestInlineModel(models.Model):
-    parent = models.ForeignKey(ValidationTestModel)
+    parent = models.ForeignKey(ValidationTestModel, models.CASCADE)
