@@ -1,5 +1,5 @@
 """
-6. Specifying ordering
+Specifying ordering
 
 Specify default ordering for a model using the ``ordering`` attribute, which
 should be a list or tuple of field names. This tells Django how to order
@@ -24,7 +24,8 @@ class Author(models.Model):
 
 @python_2_unicode_compatible
 class Article(models.Model):
-    author = models.ForeignKey(Author, null=True)
+    author = models.ForeignKey(Author, models.SET_NULL, null=True)
+    second_author = models.ForeignKey(Author, models.SET_NULL, null=True)
     headline = models.CharField(max_length=100)
     pub_date = models.DateTimeField()
 
@@ -33,3 +34,16 @@ class Article(models.Model):
 
     def __str__(self):
         return self.headline
+
+
+class OrderedByAuthorArticle(Article):
+    class Meta:
+        proxy = True
+        ordering = ('author', 'second_author')
+
+
+class Reference(models.Model):
+    article = models.ForeignKey(OrderedByAuthorArticle, models.CASCADE)
+
+    class Meta:
+        ordering = ('article',)
