@@ -2,8 +2,11 @@ from __future__ import unicode_literals
 
 import datetime
 import pickle
+import unittest
 
+from django.db import models
 from django.test import TestCase
+from django.utils import six
 from django.utils.version import get_version
 
 from .models import Container, Event, Group, Happening, M2MModel
@@ -26,21 +29,16 @@ class PickleabilityTestCase(TestCase):
     def test_datetime_callable_default_filter(self):
         self.assert_pickles(Happening.objects.filter(when=datetime.datetime.now()))
 
-    def test_lambda_as_default(self):
+    def test_string_as_default(self):
         self.assert_pickles(Happening.objects.filter(name="test"))
 
     def test_standalone_method_as_default(self):
         self.assert_pickles(Happening.objects.filter(number1=1))
 
+    @unittest.skipIf(six.PY2, "Field doesn't exist on Python 2.")
     def test_staticmethod_as_default(self):
         self.assert_pickles(Happening.objects.filter(number2=1))
 
-    def test_classmethod_as_default(self):
-        self.assert_pickles(Happening.objects.filter(number3=1))
-
-    def test_membermethod_as_default(self):
-        self.assert_pickles(Happening.objects.filter(number4=1))
-
     def test_filter_reverse_fk(self):
         self.assert_pickles(Group.objects.filter(event=1))
 
@@ -59,7 +57,7 @@ class PickleabilityTestCase(TestCase):
 
     def test_model_pickle(self):
         """
-        Test that a model not defined on module level is pickleable.
+        Test that a model not defined on module level is picklable.
         """
         original = Container.SomeModel(pk=1)
         dumped = pickle.dumps(original)
@@ -80,7 +78,7 @@ class PickleabilityTestCase(TestCase):
         m1 = M2MModel.objects.create()
         g1 = Group.objects.create(name='foof')
         m1.groups.add(g1)
-        m2m_through = M2MModel._meta.get_field('groups').rel.through
+        m2m_through = M2MModel._meta.get_field('groups').remote_field.through
         original = m2m_through.objects.get()
         dumped = pickle.dumps(original)
         reloaded = pickle.loads(dumped)
@@ -100,8 +98,7 @@ class PickleabilityTestCase(TestCase):
     def test_specialized_queryset(self):
         self.assert_pickles(Happening.objects.values('name'))
         self.assert_pickles(Happening.objects.values('name').dates('when', 'year'))
-
-        # ValuesQuerySet with related field (#14515)
+        # With related field (#14515)
         self.assert_pickles(
             Event.objects.select_related('group').order_by('title').values_list('title', 'group__name')
         )
@@ -132,6 +129,11 @@ class PickleabilityTestCase(TestCase):
         m2ms = pickle.loads(pickle.dumps(m2ms))
         self.assertQuerysetEqual(m2ms, [m2m], lambda x: x)
 
+    def test_annotation_with_callable_default(self):
+        # Happening.when has a callable default of datetime.datetime.now.
+        qs = Happening.objects.annotate(latest_time=models.Max('when'))
+        self.assert_pickles(qs)
+
     def test_missing_django_version_unpickling(self):
         """
         #21430 -- Verifies a warning is raised for querysets that are
