@@ -1,28 +1,29 @@
 #!/usr/bin/env python
-import logging
+import argparse
+import atexit
+import copy
 import os
 import shutil
 import subprocess
 import sys
 import tempfile
 import warnings
-from argparse import ArgumentParser
 
 import django
 from django.apps import apps
 from django.conf import settings
-from django.db import connection
+from django.db import connection, connections
 from django.test import TestCase, TransactionTestCase
+from django.test.runner import default_test_processes
+from django.test.selenium import SeleniumTestCaseBase
 from django.test.utils import get_runner
 from django.utils import six
 from django.utils._os import upath
-from django.utils.deprecation import (
-    RemovedInDjango19Warning, RemovedInDjango110Warning,
-)
+from django.utils.deprecation import RemovedInDjango20Warning
+from django.utils.log import DEFAULT_LOGGING
 
 # Make deprecation warnings errors to ensure no usage of deprecated features.
-warnings.simplefilter("error", RemovedInDjango19Warning)
-warnings.simplefilter("error", RemovedInDjango110Warning)
+warnings.simplefilter("error", RemovedInDjango20Warning)
 # Make runtime warning errors to ensure no usage of error prone patterns.
 warnings.simplefilter("error", RuntimeWarning)
 # Ignore known warnings in test dependencies.
@@ -38,12 +39,17 @@ TMPDIR = tempfile.mkdtemp(prefix='django_')
 # so that children processes inherit it.
 tempfile.tempdir = os.environ['TMPDIR'] = TMPDIR
 
+# Removing the temporary TMPDIR. Ensure we pass in unicode so that it will
+# successfully remove temp trees containing non-ASCII filenames on Windows.
+# (We're assuming the temp dir name itself only contains ASCII characters.)
+atexit.register(shutil.rmtree, six.text_type(TMPDIR))
+
+
 SUBDIRS_TO_SKIP = [
     'data',
     'import_error_package',
     'test_discovery_sample',
     'test_discovery_sample2',
-    'test_runner_deprecation_app',
 ]
 
 ALWAYS_INSTALLED_APPS = [
@@ -56,18 +62,17 @@ ALWAYS_INSTALLED_APPS = [
     'django.contrib.staticfiles',
 ]
 
-ALWAYS_MIDDLEWARE_CLASSES = (
+ALWAYS_MIDDLEWARE = [
     'django.contrib.sessions.middleware.SessionMiddleware',
     'django.middleware.common.CommonMiddleware',
     'django.middleware.csrf.CsrfViewMiddleware',
     'django.contrib.auth.middleware.AuthenticationMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware',
-)
+]
 
 # Need to add the associated contrib app to INSTALLED_APPS in some cases to
 # avoid "RuntimeError: Model class X doesn't declare an explicit app_label
-# and either isn't in an application in INSTALLED_APPS or else was imported
-# before its application was loaded."
+# and isn't in an application in INSTALLED_APPS."
 CONTRIB_TESTS_TO_APPS = {
     'flatpages_tests': 'django.contrib.flatpages',
     'redirects_tests': 'django.contrib.redirects',
@@ -78,23 +83,17 @@ def get_test_modules():
     modules = []
     discovery_paths = [
         (None, RUNTESTS_DIR),
+        # GIS tests are in nested apps
+        ('gis_tests', os.path.join(RUNTESTS_DIR, 'gis_tests')),
     ]
-    # GIS tests are in nested apps
-    if connection.features.gis_enabled:
-        discovery_paths.append(
-            ('gis_tests', os.path.join(RUNTESTS_DIR, 'gis_tests'))
-        )
 
     for modpath, dirpath in discovery_paths:
         for f in os.listdir(dirpath):
             if ('.' in f or
-                    f.startswith('sql') or
                     os.path.basename(f) in SUBDIRS_TO_SKIP or
                     os.path.isfile(f) or
                     not os.path.exists(os.path.join(dirpath, f, '__init__.py'))):
                 continue
-            if connection.vendor != 'postgresql' and f == 'postgres_tests':
-                continue
             modules.append((modpath, f))
     return modules
 
@@ -103,8 +102,13 @@ def get_installed():
     return [app_config.name for app_config in apps.get_app_configs()]
 
 
-def setup(verbosity, test_labels):
-    print("Testing against Django installed in '%s'" % os.path.dirname(django.__file__))
+def setup(verbosity, test_labels, parallel):
+    if verbosity >= 1:
+        msg = "Testing against Django installed in '%s'" % os.path.dirname(django.__file__)
+        max_parallel = default_test_processes() if parallel == 0 else parallel
+        if max_parallel > 1:
+            msg += " with up to %d processes" % max_parallel
+        print(msg)
 
     # Force declaring available_apps in TransactionTestCase for faster tests.
     def no_available_apps(self):
@@ -120,7 +124,7 @@ def setup(verbosity, test_labels):
         'LANGUAGE_CODE': settings.LANGUAGE_CODE,
         'STATIC_URL': settings.STATIC_URL,
         'STATIC_ROOT': settings.STATIC_ROOT,
-        'MIDDLEWARE_CLASSES': settings.MIDDLEWARE_CLASSES,
+        'MIDDLEWARE': settings.MIDDLEWARE,
     }
 
     # Redirect some settings for the duration of these tests.
@@ -143,28 +147,24 @@ def setup(verbosity, test_labels):
     }]
     settings.LANGUAGE_CODE = 'en'
     settings.SITE_ID = 1
-    settings.MIDDLEWARE_CLASSES = ALWAYS_MIDDLEWARE_CLASSES
-    # Ensure the middleware classes are seen as overridden otherwise we get a compatibility warning.
-    settings._explicit_settings.add('MIDDLEWARE_CLASSES')
+    settings.MIDDLEWARE = ALWAYS_MIDDLEWARE
     settings.MIGRATION_MODULES = {
-        # these 'tests.migrations' modules don't actually exist, but this lets
-        # us skip creating migrations for the test models.
-        'auth': 'django.contrib.auth.tests.migrations',
-        'contenttypes': 'contenttypes_tests.migrations',
+        # This lets us skip creating migrations for the test models as many of
+        # them depend on one of the following contrib applications.
+        'auth': None,
+        'contenttypes': None,
+        'sessions': None,
     }
-
-    if verbosity > 0:
-        # Ensure any warnings captured to logging are piped through a verbose
-        # logging handler.  If any -W options were passed explicitly on command
-        # line, warnings are not captured, and this has no effect.
-        logger = logging.getLogger('py.warnings')
-        handler = logging.StreamHandler()
-        logger.addHandler(handler)
+    log_config = copy.deepcopy(DEFAULT_LOGGING)
+    # Filter out non-error logging so we don't have to capture it in lots of
+    # tests.
+    log_config['loggers']['django']['level'] = 'ERROR'
+    settings.LOGGING = log_config
 
     warnings.filterwarnings(
         'ignore',
-        'django.contrib.webdesign will be removed in Django 1.10.',
-        RemovedInDjango110Warning
+        'The GeoManager class is deprecated.',
+        RemovedInDjango20Warning
     )
 
     # Load all the ALWAYS_INSTALLED_APPS.
@@ -218,28 +218,40 @@ def setup(verbosity, test_labels):
 
 
 def teardown(state):
-    try:
-        # Removing the temporary TMPDIR. Ensure we pass in unicode
-        # so that it will successfully remove temp trees containing
-        # non-ASCII filenames on Windows. (We're assuming the temp dir
-        # name itself does not contain non-ASCII characters.)
-        shutil.rmtree(six.text_type(TMPDIR))
-    except OSError:
-        print('Failed to remove temp directory: %s' % TMPDIR)
-
     # Restore the old settings.
     for key, value in state.items():
         setattr(settings, key, value)
 
 
-def django_tests(verbosity, interactive, failfast, keepdb, reverse, test_labels, debug_sql):
-    state = setup(verbosity, test_labels)
-    extra_tests = []
+def actual_test_processes(parallel):
+    if parallel == 0:
+        # This doesn't work before django.setup() on some databases.
+        if all(conn.features.can_clone_databases for conn in connections.all()):
+            return default_test_processes()
+        else:
+            return 1
+    else:
+        return parallel
 
-    if test_labels and 'postgres_tests' in test_labels and connection.vendor != 'postgresql':
-        if verbosity >= 2:
-            print("Removed postgres_tests from tests as we're not running with PostgreSQL.")
-        test_labels.remove('postgres_tests')
+
+class ActionSelenium(argparse.Action):
+    """
+    Validate the comma-separated list of requested browsers.
+    """
+    def __call__(self, parser, namespace, values, option_string=None):
+        browsers = values.split(',')
+        for browser in browsers:
+            try:
+                SeleniumTestCaseBase.import_webdriver(browser)
+            except ImportError:
+                raise argparse.ArgumentError(self, "Selenium browser specification '%s' is not valid." % browser)
+        setattr(namespace, self.dest, browsers)
+
+
+def django_tests(verbosity, interactive, failfast, keepdb, reverse,
+                 test_labels, debug_sql, parallel, tags, exclude_tags):
+    state = setup(verbosity, test_labels, parallel)
+    extra_tests = []
 
     # Run the test suite, including the extra validation tests.
     if not hasattr(settings, 'TEST_RUNNER'):
@@ -253,29 +265,37 @@ def django_tests(verbosity, interactive, failfast, keepdb, reverse, test_labels,
         keepdb=keepdb,
         reverse=reverse,
         debug_sql=debug_sql,
+        parallel=actual_test_processes(parallel),
+        tags=tags,
+        exclude_tags=exclude_tags,
+    )
+    failures = test_runner.run_tests(
+        test_labels or get_installed(),
+        extra_tests=extra_tests,
     )
-    # Catch warnings thrown in test DB setup -- remove in Django 1.9
-    with warnings.catch_warnings():
-        warnings.filterwarnings(
-            'ignore',
-            "Custom SQL location '<app_label>/models/sql' is deprecated, "
-            "use '<app_label>/sql' instead.",
-            RemovedInDjango19Warning
-        )
-        warnings.filterwarnings(
-            'ignore',
-            'initial_data fixtures are deprecated. Use data migrations instead.',
-            RemovedInDjango19Warning
-        )
-        failures = test_runner.run_tests(
-            test_labels or get_installed(), extra_tests=extra_tests)
-
     teardown(state)
     return failures
 
 
-def bisect_tests(bisection_label, options, test_labels):
-    state = setup(options.verbosity, test_labels)
+def get_subprocess_args(options):
+    subprocess_args = [
+        sys.executable, upath(__file__), '--settings=%s' % options.settings
+    ]
+    if options.failfast:
+        subprocess_args.append('--failfast')
+    if options.verbosity:
+        subprocess_args.append('--verbosity=%s' % options.verbosity)
+    if not options.interactive:
+        subprocess_args.append('--noinput')
+    if options.tags:
+        subprocess_args.append('--tag=%s' % options.tags)
+    if options.exclude_tags:
+        subprocess_args.append('--exclude_tag=%s' % options.exclude_tags)
+    return subprocess_args
+
+
+def bisect_tests(bisection_label, options, test_labels, parallel):
+    state = setup(options.verbosity, test_labels, parallel)
 
     test_labels = test_labels or get_installed()
 
@@ -289,14 +309,7 @@ def bisect_tests(bisection_label, options, test_labels):
         except ValueError:
             pass
 
-    subprocess_args = [
-        sys.executable, upath(__file__), '--settings=%s' % options.settings]
-    if options.failfast:
-        subprocess_args.append('--failfast')
-    if options.verbosity:
-        subprocess_args.append('--verbosity=%s' % options.verbosity)
-    if not options.interactive:
-        subprocess_args.append('--noinput')
+    subprocess_args = get_subprocess_args(options)
 
     iteration = 1
     while len(test_labels) > 1:
@@ -332,8 +345,8 @@ def bisect_tests(bisection_label, options, test_labels):
     teardown(state)
 
 
-def paired_tests(paired_test, options, test_labels):
-    state = setup(options.verbosity, test_labels)
+def paired_tests(paired_test, options, test_labels, parallel):
+    state = setup(options.verbosity, test_labels, parallel)
 
     test_labels = test_labels or get_installed()
 
@@ -347,14 +360,7 @@ def paired_tests(paired_test, options, test_labels):
         except ValueError:
             pass
 
-    subprocess_args = [
-        sys.executable, upath(__file__), '--settings=%s' % options.settings]
-    if options.failfast:
-        subprocess_args.append('--failfast')
-    if options.verbosity:
-        subprocess_args.append('--verbosity=%s' % options.verbosity)
-    if not options.interactive:
-        subprocess_args.append('--noinput')
+    subprocess_args = get_subprocess_args(options)
 
     for i, label in enumerate(test_labels):
         print('***** %d of %d: Check test pairing with %s' % (
@@ -369,47 +375,76 @@ def paired_tests(paired_test, options, test_labels):
 
 
 if __name__ == "__main__":
-    parser = ArgumentParser(description="Run the Django test suite.")
-    parser.add_argument('modules', nargs='*', metavar='module',
+    parser = argparse.ArgumentParser(description="Run the Django test suite.")
+    parser.add_argument(
+        'modules', nargs='*', metavar='module',
         help='Optional path(s) to test modules; e.g. "i18n" or '
-             '"i18n.tests.TranslationTests.test_lazy_objects".')
+             '"i18n.tests.TranslationTests.test_lazy_objects".',
+    )
     parser.add_argument(
         '-v', '--verbosity', default=1, type=int, choices=[0, 1, 2, 3],
-        help='Verbosity level; 0=minimal output, 1=normal output, 2=all output')
+        help='Verbosity level; 0=minimal output, 1=normal output, 2=all output',
+    )
     parser.add_argument(
         '--noinput', action='store_false', dest='interactive', default=True,
-        help='Tells Django to NOT prompt the user for input of any kind.')
+        help='Tells Django to NOT prompt the user for input of any kind.',
+    )
     parser.add_argument(
         '--failfast', action='store_true', dest='failfast', default=False,
-        help='Tells Django to stop running the test suite after first failed '
-             'test.')
+        help='Tells Django to stop running the test suite after first failed test.',
+    )
     parser.add_argument(
         '-k', '--keepdb', action='store_true', dest='keepdb', default=False,
-        help='Tells Django to preserve the test database between runs.')
+        help='Tells Django to preserve the test database between runs.',
+    )
     parser.add_argument(
         '--settings',
         help='Python path to settings module, e.g. "myproject.settings". If '
              'this isn\'t provided, either the DJANGO_SETTINGS_MODULE '
-             'environment variable or "test_sqlite" will be used.')
-    parser.add_argument('--bisect',
+             'environment variable or "test_sqlite" will be used.',
+    )
+    parser.add_argument(
+        '--bisect',
         help='Bisect the test suite to discover a test that causes a test '
-             'failure when combined with the named test.')
-    parser.add_argument('--pair',
-        help='Run the test suite in pairs with the named test to find problem '
-             'pairs.')
-    parser.add_argument('--reverse', action='store_true', default=False,
+             'failure when combined with the named test.',
+    )
+    parser.add_argument(
+        '--pair',
+        help='Run the test suite in pairs with the named test to find problem pairs.',
+    )
+    parser.add_argument(
+        '--reverse', action='store_true', default=False,
         help='Sort test suites and test cases in opposite order to debug '
-             'test side effects not apparent with normal execution lineup.')
-    parser.add_argument('--liveserver',
+             'test side effects not apparent with normal execution lineup.',
+    )
+    parser.add_argument(
+        '--liveserver',
         help='Overrides the default address where the live server (used with '
              'LiveServerTestCase) is expected to run from. The default value '
-             'is localhost:8081.')
+             'is localhost:8081-8179.',
+    )
     parser.add_argument(
-        '--selenium', action='store_true', dest='selenium', default=False,
-        help='Run the Selenium tests as well (if Selenium is installed)')
+        '--selenium', dest='selenium', action=ActionSelenium, metavar='BROWSERS',
+        help='A comma-separated list of browsers to run the Selenium tests against.',
+    )
     parser.add_argument(
         '--debug-sql', action='store_true', dest='debug_sql', default=False,
-        help='Turn on the SQL query logger within tests')
+        help='Turn on the SQL query logger within tests.',
+    )
+    parser.add_argument(
+        '--parallel', dest='parallel', nargs='?', default=0, type=int,
+        const=default_test_processes(), metavar='N',
+        help='Run tests using up to N parallel processes.',
+    )
+    parser.add_argument(
+        '--tag', dest='tags', action='append',
+        help='Run only tests with the specified tags. Can be used multiple times.',
+    )
+    parser.add_argument(
+        '--exclude-tag', dest='exclude_tags', action='append',
+        help='Do not run tests with the specified tag. Can be used multiple times.',
+    )
+
     options = parser.parse_args()
 
     # mock is a required dependency
@@ -436,16 +471,22 @@ if __name__ == "__main__":
         os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = options.liveserver
 
     if options.selenium:
-        os.environ['DJANGO_SELENIUM_TESTS'] = '1'
+        if not options.tags:
+            options.tags = ['selenium']
+        elif 'selenium' not in options.tags:
+            options.tags.append('selenium')
+        SeleniumTestCaseBase.browsers = options.selenium
 
     if options.bisect:
-        bisect_tests(options.bisect, options, options.modules)
+        bisect_tests(options.bisect, options, options.modules, options.parallel)
     elif options.pair:
-        paired_tests(options.pair, options, options.modules)
+        paired_tests(options.pair, options, options.modules, options.parallel)
     else:
-        failures = django_tests(options.verbosity, options.interactive,
-                                options.failfast, options.keepdb,
-                                options.reverse, options.modules,
-                                options.debug_sql)
+        failures = django_tests(
+            options.verbosity, options.interactive, options.failfast,
+            options.keepdb, options.reverse, options.modules,
+            options.debug_sql, options.parallel, options.tags,
+            options.exclude_tags,
+        )
         if failures:
             sys.exit(bool(failures))
