@@ -34,7 +34,7 @@ class AuthorWithEvenLongerName(models.Model):
 
 
 class Book(models.Model):
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
     # tags = models.ManyToManyField("Tag", related_name="books")
@@ -44,7 +44,7 @@ class Book(models.Model):
 
 
 class BookWeak(models.Model):
-    author = models.ForeignKey(Author, db_constraint=False)
+    author = models.ForeignKey(Author, models.CASCADE, db_constraint=False)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
 
@@ -53,14 +53,17 @@ class BookWeak(models.Model):
 
 
 class BookWithLongName(models.Model):
-    author_foreign_key_with_really_long_field_name = models.ForeignKey(AuthorWithEvenLongerName)
+    author_foreign_key_with_really_long_field_name = models.ForeignKey(
+        AuthorWithEvenLongerName,
+        models.CASCADE,
+    )
 
     class Meta:
         apps = new_apps
 
 
 class BookWithO2O(models.Model):
-    author = models.OneToOneField(Author)
+    author = models.OneToOneField(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
 
@@ -70,7 +73,7 @@ class BookWithO2O(models.Model):
 
 
 class BookWithSlug(models.Model):
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
     slug = models.CharField(max_length=20, unique=True)
@@ -89,6 +92,14 @@ class BookWithoutAuthor(models.Model):
         db_table = "schema_book"
 
 
+class BookForeignObj(models.Model):
+    title = models.CharField(max_length=100, db_index=True)
+    author_id = models.IntegerField()
+
+    class Meta:
+        apps = new_apps
+
+
 class IntegerPK(models.Model):
     i = models.IntegerField(primary_key=True)
     j = models.IntegerField(unique=True)
@@ -166,3 +177,8 @@ class UniqueTest(models.Model):
     class Meta:
         apps = new_apps
         unique_together = ["year", "slug"]
+
+
+class Node(models.Model):
+    node_id = models.AutoField(primary_key=True)
+    parent = models.ForeignKey('self', models.CASCADE, null=True, blank=True)
