@@ -1,9 +1,11 @@
 from __future__ import unicode_literals
 
+from django.apps.registry import Apps
 from django.db import models
 from django.db.models import signals
 from django.dispatch import receiver
-from django.test import TestCase
+from django.test import TestCase, mock
+from django.test.utils import isolate_apps
 from django.utils import six
 
 from .models import Author, Book, Car, Person
@@ -222,9 +224,9 @@ class SignalTests(BaseSignalTest):
             data[:] = []
 
             # Assigning and removing to/from m2m shouldn't generate an m2m signal.
-            b1.authors = [a1]
+            b1.authors.set([a1])
             self.assertEqual(data, [])
-            b1.authors = []
+            b1.authors.set([])
             self.assertEqual(data, [])
         finally:
             signals.pre_save.disconnect(pre_save_handler)
@@ -256,6 +258,19 @@ class SignalTests(BaseSignalTest):
         self.assertTrue(b._run)
         self.assertEqual(signals.post_save.receivers, [])
 
+    @mock.patch('weakref.ref')
+    def test_lazy_model_signal(self, ref):
+        def callback(sender, args, **kwargs):
+            pass
+        signals.pre_init.connect(callback)
+        signals.pre_init.disconnect(callback)
+        self.assertTrue(ref.called)
+        ref.reset_mock()
+
+        signals.pre_init.connect(callback, weak=False)
+        signals.pre_init.disconnect(callback)
+        ref.assert_not_called()
+
 
 class LazyModelRefTest(BaseSignalTest):
     def setUp(self):
@@ -266,9 +281,8 @@ class LazyModelRefTest(BaseSignalTest):
         self.received.append(kwargs)
 
     def test_invalid_sender_model_name(self):
-        with self.assertRaisesMessage(ValueError,
-                    "Specified sender must either be a model or a "
-                    "model name of the 'app_label.ModelName' form."):
+        msg = "Invalid model reference 'invalid'. String model references must be of the form 'app_label.ModelName'."
+        with self.assertRaisesMessage(ValueError, msg):
             signals.post_init.connect(self.receiver, sender='invalid')
 
     def test_already_loaded_model(self):
@@ -285,9 +299,10 @@ class LazyModelRefTest(BaseSignalTest):
         finally:
             signals.post_init.disconnect(self.receiver, sender=Book)
 
-    def test_not_loaded_model(self):
+    @isolate_apps('signals', kwarg_name='apps')
+    def test_not_loaded_model(self, apps):
         signals.post_init.connect(
-            self.receiver, sender='signals.Created', weak=False
+            self.receiver, sender='signals.Created', weak=False, apps=apps
         )
 
         try:
@@ -300,3 +315,30 @@ class LazyModelRefTest(BaseSignalTest):
             }])
         finally:
             signals.post_init.disconnect(self.receiver, sender=Created)
+
+    @isolate_apps('signals', kwarg_name='apps')
+    def test_disconnect(self, apps):
+        received = []
+
+        def receiver(**kwargs):
+            received.append(kwargs)
+
+        signals.post_init.connect(receiver, sender='signals.Created', apps=apps)
+        signals.post_init.disconnect(receiver, sender='signals.Created', apps=apps)
+
+        class Created(models.Model):
+            pass
+
+        Created()
+        self.assertEqual(received, [])
+
+    def test_register_model_class_senders_immediately(self):
+        """
+        Model signals registered with model classes as senders don't use the
+        Apps.lazy_model_operation() mechanism.
+        """
+        # Book isn't registered with apps2, so it will linger in
+        # apps2._pending_operations if ModelSignal does the wrong thing.
+        apps2 = Apps()
+        signals.post_init.connect(self.receiver, sender=Book, apps=apps2)
+        self.assertEqual(list(apps2._pending_operations), [])
