@@ -1,7 +1,6 @@
 from __future__ import unicode_literals
 
 import datetime
-import warnings
 
 from django.core import signing
 from django.test import SimpleTestCase
@@ -23,8 +22,7 @@ class TestSigner(SimpleTestCase):
         ):
             self.assertEqual(
                 signer.signature(s),
-                signing.base64_hmac(signer.salt + 'signer', s,
-                    'predictable-secret').decode()
+                signing.base64_hmac(signer.salt + 'signer', s, 'predictable-secret').decode()
             )
             self.assertNotEqual(signer.signature(s), signer2.signature(s))
 
@@ -33,8 +31,7 @@ class TestSigner(SimpleTestCase):
         signer = signing.Signer('predictable-secret', salt='extra-salt')
         self.assertEqual(
             signer.signature('hello'),
-            signing.base64_hmac('extra-salt' + 'signer',
-                                'hello', 'predictable-secret').decode()
+            signing.base64_hmac('extra-salt' + 'signer', 'hello', 'predictable-secret').decode()
         )
         self.assertNotEqual(
             signing.Signer('predictable-secret', salt='one').signature('hello'),
@@ -71,8 +68,8 @@ class TestSigner(SimpleTestCase):
         )
         self.assertEqual(value, signer.unsign(signed_value))
         for transform in transforms:
-            self.assertRaises(
-                signing.BadSignature, signer.unsign, transform(signed_value))
+            with self.assertRaises(signing.BadSignature):
+                signer.unsign(transform(signed_value))
 
     def test_dumps_loads(self):
         "dumps and loads be reversible for any JSON serializable object"
@@ -104,8 +101,8 @@ class TestSigner(SimpleTestCase):
         encoded = signing.dumps(value)
         self.assertEqual(value, signing.loads(encoded))
         for transform in transforms:
-            self.assertRaises(
-                signing.BadSignature, signing.loads, transform(encoded))
+            with self.assertRaises(signing.BadSignature):
+                signing.loads(transform(encoded))
 
     def test_works_with_non_ascii_keys(self):
         binary_key = b'\xe7'  # Set some binary (non-ASCII key)
@@ -121,14 +118,11 @@ class TestSigner(SimpleTestCase):
 
     def test_invalid_sep(self):
         """should warn on invalid separator"""
+        msg = 'Unsafe Signer separator: %r (cannot be empty or consist of only A-z0-9-_=)'
         separators = ['', '-', 'abc']
         for sep in separators:
-            with warnings.catch_warnings(record=True) as recorded:
-                warnings.simplefilter('always')
+            with self.assertRaisesMessage(ValueError, msg % sep):
                 signing.Signer(sep=sep)
-                self.assertEqual(len(recorded), 1)
-                msg = str(recorded[0].message)
-                self.assertTrue(msg.startswith('Unsafe Signer separator'))
 
 
 class TestTimestampSigner(SimpleTestCase):
@@ -138,12 +132,12 @@ class TestTimestampSigner(SimpleTestCase):
         with freeze_time(123456789):
             signer = signing.TimestampSigner('predictable-key')
             ts = signer.sign(value)
-            self.assertNotEqual(ts,
-                signing.Signer('predictable-key').sign(value))
+            self.assertNotEqual(ts, signing.Signer('predictable-key').sign(value))
             self.assertEqual(signer.unsign(ts), value)
 
         with freeze_time(123456800):
             self.assertEqual(signer.unsign(ts, max_age=12), value)
             # max_age parameter can also accept a datetime.timedelta object
             self.assertEqual(signer.unsign(ts, max_age=datetime.timedelta(seconds=11)), value)
-            self.assertRaises(signing.SignatureExpired, signer.unsign, ts, max_age=10)
+            with self.assertRaises(signing.SignatureExpired):
+                signer.unsign(ts, max_age=10)
