@@ -1,9 +1,5 @@
-import warnings
-
 from django.template import TemplateSyntaxError
-from django.test import SimpleTestCase, ignore_warnings
-from django.test.utils import reset_warning_registry
-from django.utils.deprecation import RemovedInDjango110Warning
+from django.test import SimpleTestCase
 
 from ..utils import TestObj, setup
 
@@ -112,6 +108,32 @@ class IfTagTests(SimpleTestCase):
         output = self.engine.render_to_string('if-tag-eq05')
         self.assertEqual(output, 'no')
 
+    # Inequality
+    @setup({'if-tag-noteq01': '{% if foo != bar %}yes{% else %}no{% endif %}'})
+    def test_if_tag_noteq01(self):
+        output = self.engine.render_to_string('if-tag-noteq01')
+        self.assertEqual(output, 'no')
+
+    @setup({'if-tag-noteq02': '{% if foo != bar %}yes{% else %}no{% endif %}'})
+    def test_if_tag_noteq02(self):
+        output = self.engine.render_to_string('if-tag-noteq02', {'foo': 1})
+        self.assertEqual(output, 'yes')
+
+    @setup({'if-tag-noteq03': '{% if foo != bar %}yes{% else %}no{% endif %}'})
+    def test_if_tag_noteq03(self):
+        output = self.engine.render_to_string('if-tag-noteq03', {'foo': 1, 'bar': 1})
+        self.assertEqual(output, 'no')
+
+    @setup({'if-tag-noteq04': '{% if foo != bar %}yes{% else %}no{% endif %}'})
+    def test_if_tag_noteq04(self):
+        output = self.engine.render_to_string('if-tag-noteq04', {'foo': 1, 'bar': 2})
+        self.assertEqual(output, 'yes')
+
+    @setup({'if-tag-noteq05': '{% if foo != "" %}yes{% else %}no{% endif %}'})
+    def test_if_tag_noteq05(self):
+        output = self.engine.render_to_string('if-tag-noteq05')
+        self.assertEqual(output, 'yes')
+
     # Comparison
     @setup({'if-tag-gt-01': '{% if 2 > 1 %}yes{% else %}no{% endif %}'})
     def test_if_tag_gt_01(self):
@@ -526,42 +548,50 @@ class IfTagTests(SimpleTestCase):
         output = self.engine.render_to_string('if-tag-badarg04')
         self.assertEqual(output, 'no')
 
-    @setup({'if-tag-eq-deprecated': '{% if foo = bar %}yes{% else %}no{% endif %}'},
-        test_once=True)
-    def test_if_tag_eq_deprecated(self):
-        reset_warning_registry()
-        with warnings.catch_warnings(record=True) as warns:
-            warnings.simplefilter('always')
-            output = self.engine.render_to_string('if-tag-eq-deprecated')
-            self.assertEqual(output, 'yes')
-        self.assertEqual(len(warns), 1)
-        self.assertEqual(
-            str(warns[0].message),
-            "Operator '=' is deprecated and will be removed in Django 1.10. "
-            "Use '==' instead."
-        )
+    @setup({'if-tag-single-eq': '{% if foo = bar %}yes{% else %}no{% endif %}'})
+    def test_if_tag_single_eq(self):
+        # A single equals sign is a syntax error.
+        with self.assertRaises(TemplateSyntaxError):
+            self.engine.render_to_string('if-tag-single-eq', {'foo': 1})
 
-
-@ignore_warnings(category=RemovedInDjango110Warning)
-class TestEqualitySingleEqualsSign(SimpleTestCase):
-    # The following tests should be changed to template.TemplateSyntaxError
-    # (or simply removed) when the deprecation path ends in Django 1.10.
-    @setup({'if-tag-eq01': '{% if foo = bar %}yes{% else %}no{% endif %}'})
-    def test_if_tag_eq01(self):
-        output = self.engine.render_to_string('if-tag-eq01', {'foo': 1})
-        self.assertEqual(output, 'no')
-
-    @setup({'if-tag-eq02': '{% if foo = bar %}yes{% else %}no{% endif %}'})
-    def test_if_tag_eq02(self):
-        output = self.engine.render_to_string('if-tag-eq02', {'foo': 1, 'bar': 1})
+    @setup({'template': '{% if foo is True %}yes{% else %}no{% endif %}'})
+    def test_if_is_match(self):
+        output = self.engine.render_to_string('template', {'foo': True})
         self.assertEqual(output, 'yes')
 
-    @setup({'if-tag-eq03': '{% if foo = bar %}yes{% else %}no{% endif %}'})
-    def test_if_tag_eq03(self):
-        output = self.engine.render_to_string('if-tag-eq03', {'foo': 1, 'bar': 2})
+    @setup({'template': '{% if foo is True %}yes{% else %}no{% endif %}'})
+    def test_if_is_no_match(self):
+        output = self.engine.render_to_string('template', {'foo': 1})
         self.assertEqual(output, 'no')
 
-    @setup({'if-tag-eq04': '{% if foo == \'\' %}yes{% else %}no{% endif %}'})
-    def test_if_tag_eq04(self):
-        output = self.engine.render_to_string('if-tag-eq04')
+    @setup({'template': '{% if foo is bar %}yes{% else %}no{% endif %}'})
+    def test_if_is_variable_missing(self):
+        output = self.engine.render_to_string('template', {'foo': 1})
+        self.assertEqual(output, 'no')
+
+    @setup({'template': '{% if foo is bar %}yes{% else %}no{% endif %}'})
+    def test_if_is_both_variables_missing(self):
+        output = self.engine.render_to_string('template', {})
+        self.assertEqual(output, 'yes')
+
+    @setup({'template': '{% if foo is not None %}yes{% else %}no{% endif %}'})
+    def test_if_is_not_match(self):
+        # For this to act as a regression test, it's important not to use
+        # foo=True because True is (not None)
+        output = self.engine.render_to_string('template', {'foo': False})
+        self.assertEqual(output, 'yes')
+
+    @setup({'template': '{% if foo is not None %}yes{% else %}no{% endif %}'})
+    def test_if_is_not_no_match(self):
+        output = self.engine.render_to_string('template', {'foo': None})
+        self.assertEqual(output, 'no')
+
+    @setup({'template': '{% if foo is not bar %}yes{% else %}no{% endif %}'})
+    def test_if_is_not_variable_missing(self):
+        output = self.engine.render_to_string('template', {'foo': False})
+        self.assertEqual(output, 'yes')
+
+    @setup({'template': '{% if foo is not bar %}yes{% else %}no{% endif %}'})
+    def test_if_is_not_both_variables_missing(self):
+        output = self.engine.render_to_string('template', {})
         self.assertEqual(output, 'no')
