@@ -13,8 +13,7 @@ from django.template.engine import Engine
 from django.test import SimpleTestCase, ignore_warnings, override_settings
 from django.utils import six
 from django.utils.deprecation import RemovedInDjango20Warning
-from django.utils.encoding import force_text
-from django.utils.functional import lazy
+from django.utils.functional import lazystr
 
 from .utils import TEMPLATE_DIR
 
@@ -113,7 +112,7 @@ class CachedLoaderTests(SimpleTestCase):
         """
         loader = self.engine.template_loaders[0]
 
-        self.assertFalse('missing.html' in loader.template_cache)
+        self.assertNotIn('missing.html', loader.template_cache)
 
         with self.assertRaises(TemplateDoesNotExist):
             loader.load_template("missing.html")
@@ -160,7 +159,6 @@ class CachedLoaderTests(SimpleTestCase):
         #26603 -- A template name specified as a lazy string should be forced
         to text before computing its cache key.
         """
-        lazystr = lazy(force_text, six.text_type)
         self.assertEqual(self.engine.template_loaders[0].cache_key(lazystr('template.html'), []), 'template.html')
 
 
