@@ -25,7 +25,7 @@ class DiscoverRunnerTest(TestCase):
             ["test_discovery_sample.tests_sample"],
         ).countTestCases()
 
-        self.assertEqual(count, 4)
+        self.assertEqual(count, 6)
 
     def test_dotted_test_class_vanilla_unittest(self):
         count = DiscoverRunner().build_suite(
@@ -61,7 +61,7 @@ class DiscoverRunnerTest(TestCase):
                 ["test_discovery_sample/"],
             ).countTestCases()
 
-        self.assertEqual(count, 5)
+        self.assertEqual(count, 7)
 
     def test_empty_label(self):
         """
@@ -121,9 +121,11 @@ class DiscoverRunnerTest(TestCase):
         """
         Tests shouldn't be discovered twice when discovering on overlapping paths.
         """
-        single = DiscoverRunner().build_suite(["gis_tests"]).countTestCases()
-        dups = DiscoverRunner().build_suite(
-            ["gis_tests", "gis_tests.geo3d"]).countTestCases()
+        base_app = 'gis_tests'
+        sub_app = 'gis_tests.geo3d'
+        with self.modify_settings(INSTALLED_APPS={'append': sub_app}):
+            single = DiscoverRunner().build_suite([base_app]).countTestCases()
+            dups = DiscoverRunner().build_suite([base_app, sub_app]).countTestCases()
         self.assertEqual(single, dups)
 
     def test_reverse(self):
@@ -163,3 +165,19 @@ class DiscoverRunnerTest(TestCase):
 
     def test_overridable_test_loader(self):
         self.assertEqual(DiscoverRunner().test_loader, defaultTestLoader)
+
+    def test_tags(self):
+        runner = DiscoverRunner(tags=['core'])
+        self.assertEqual(runner.build_suite(['test_discovery_sample.tests_sample']).countTestCases(), 1)
+        runner = DiscoverRunner(tags=['fast'])
+        self.assertEqual(runner.build_suite(['test_discovery_sample.tests_sample']).countTestCases(), 2)
+        runner = DiscoverRunner(tags=['slow'])
+        self.assertEqual(runner.build_suite(['test_discovery_sample.tests_sample']).countTestCases(), 2)
+
+    def test_exclude_tags(self):
+        runner = DiscoverRunner(tags=['fast'], exclude_tags=['core'])
+        self.assertEqual(runner.build_suite(['test_discovery_sample.tests_sample']).countTestCases(), 1)
+        runner = DiscoverRunner(tags=['fast'], exclude_tags=['slow'])
+        self.assertEqual(runner.build_suite(['test_discovery_sample.tests_sample']).countTestCases(), 0)
+        runner = DiscoverRunner(exclude_tags=['slow'])
+        self.assertEqual(runner.build_suite(['test_discovery_sample.tests_sample']).countTestCases(), 4)
