@@ -6,10 +6,10 @@ from .models import Event, Timestamp
 class EventAdmin(admin.ModelAdmin):
     list_display = ('dt',)
 
-admin.site.register(Event, EventAdmin)
-
 
 class TimestampAdmin(admin.ModelAdmin):
     readonly_fields = ('created', 'updated')
 
-admin.site.register(Timestamp, TimestampAdmin)
+site = admin.AdminSite(name='admin_tz')
+site.register(Event, EventAdmin)
+site.register(Timestamp, TimestampAdmin)
