@@ -12,13 +12,11 @@ from django.db import (
 from django.test import (
     TransactionTestCase, skipIfDBFeature, skipUnlessDBFeature,
 )
-from django.utils import six
 
 from .models import Reporter
 
 
-@skipUnless(connection.features.uses_savepoints,
-        "'atomic' requires transactions and savepoints.")
+@skipUnless(connection.features.uses_savepoints, "'atomic' requires transactions and savepoints.")
 class AtomicTests(TransactionTestCase):
     """
     Tests for the atomic decorator and context manager.
@@ -45,7 +43,7 @@ class AtomicTests(TransactionTestCase):
         def make_reporter():
             Reporter.objects.create(first_name="Haddock")
             raise Exception("Oops, that's his last name")
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             make_reporter()
         self.assertQuerysetEqual(Reporter.objects.all(), [])
 
@@ -61,7 +59,7 @@ class AtomicTests(TransactionTestCase):
         def make_reporter():
             Reporter.objects.create(first_name="Haddock")
             raise Exception("Oops, that's his last name")
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             make_reporter()
         self.assertQuerysetEqual(Reporter.objects.all(), [])
 
@@ -71,7 +69,7 @@ class AtomicTests(TransactionTestCase):
         self.assertQuerysetEqual(Reporter.objects.all(), ['<Reporter: Tintin>'])
 
     def test_rollback(self):
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with transaction.atomic():
                 Reporter.objects.create(first_name="Haddock")
                 raise Exception("Oops, that's his last name")
@@ -82,20 +80,22 @@ class AtomicTests(TransactionTestCase):
             Reporter.objects.create(first_name="Tintin")
             with transaction.atomic():
                 Reporter.objects.create(first_name="Archibald", last_name="Haddock")
-        self.assertQuerysetEqual(Reporter.objects.all(),
-                ['<Reporter: Archibald Haddock>', '<Reporter: Tintin>'])
+        self.assertQuerysetEqual(
+            Reporter.objects.all(),
+            ['<Reporter: Archibald Haddock>', '<Reporter: Tintin>']
+        )
 
     def test_nested_commit_rollback(self):
         with transaction.atomic():
             Reporter.objects.create(first_name="Tintin")
-            with six.assertRaisesRegex(self, Exception, "Oops"):
+            with self.assertRaisesMessage(Exception, "Oops"):
                 with transaction.atomic():
                     Reporter.objects.create(first_name="Haddock")
                     raise Exception("Oops, that's his last name")
         self.assertQuerysetEqual(Reporter.objects.all(), ['<Reporter: Tintin>'])
 
     def test_nested_rollback_commit(self):
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with transaction.atomic():
                 Reporter.objects.create(last_name="Tintin")
                 with transaction.atomic():
@@ -104,10 +104,10 @@ class AtomicTests(TransactionTestCase):
         self.assertQuerysetEqual(Reporter.objects.all(), [])
 
     def test_nested_rollback_rollback(self):
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with transaction.atomic():
                 Reporter.objects.create(last_name="Tintin")
-                with six.assertRaisesRegex(self, Exception, "Oops"):
+                with self.assertRaisesMessage(Exception, "Oops"):
                     with transaction.atomic():
                         Reporter.objects.create(first_name="Haddock")
                     raise Exception("Oops, that's his last name")
@@ -119,13 +119,15 @@ class AtomicTests(TransactionTestCase):
             Reporter.objects.create(first_name="Tintin")
             with transaction.atomic(savepoint=False):
                 Reporter.objects.create(first_name="Archibald", last_name="Haddock")
-        self.assertQuerysetEqual(Reporter.objects.all(),
-                ['<Reporter: Archibald Haddock>', '<Reporter: Tintin>'])
+        self.assertQuerysetEqual(
+            Reporter.objects.all(),
+            ['<Reporter: Archibald Haddock>', '<Reporter: Tintin>']
+        )
 
     def test_merged_commit_rollback(self):
         with transaction.atomic():
             Reporter.objects.create(first_name="Tintin")
-            with six.assertRaisesRegex(self, Exception, "Oops"):
+            with self.assertRaisesMessage(Exception, "Oops"):
                 with transaction.atomic(savepoint=False):
                     Reporter.objects.create(first_name="Haddock")
                     raise Exception("Oops, that's his last name")
@@ -133,7 +135,7 @@ class AtomicTests(TransactionTestCase):
         self.assertQuerysetEqual(Reporter.objects.all(), [])
 
     def test_merged_rollback_commit(self):
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with transaction.atomic():
                 Reporter.objects.create(last_name="Tintin")
                 with transaction.atomic(savepoint=False):
@@ -142,10 +144,10 @@ class AtomicTests(TransactionTestCase):
         self.assertQuerysetEqual(Reporter.objects.all(), [])
 
     def test_merged_rollback_rollback(self):
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with transaction.atomic():
                 Reporter.objects.create(last_name="Tintin")
-                with six.assertRaisesRegex(self, Exception, "Oops"):
+                with self.assertRaisesMessage(Exception, "Oops"):
                     with transaction.atomic(savepoint=False):
                         Reporter.objects.create(first_name="Haddock")
                     raise Exception("Oops, that's his last name")
@@ -158,14 +160,13 @@ class AtomicTests(TransactionTestCase):
             Reporter.objects.create(first_name="Tintin")
             with atomic:
                 Reporter.objects.create(first_name="Archibald", last_name="Haddock")
-        self.assertQuerysetEqual(Reporter.objects.all(),
-                ['<Reporter: Archibald Haddock>', '<Reporter: Tintin>'])
+        self.assertQuerysetEqual(Reporter.objects.all(), ['<Reporter: Archibald Haddock>', '<Reporter: Tintin>'])
 
     def test_reuse_commit_rollback(self):
         atomic = transaction.atomic()
         with atomic:
             Reporter.objects.create(first_name="Tintin")
-            with six.assertRaisesRegex(self, Exception, "Oops"):
+            with self.assertRaisesMessage(Exception, "Oops"):
                 with atomic:
                     Reporter.objects.create(first_name="Haddock")
                     raise Exception("Oops, that's his last name")
@@ -173,7 +174,7 @@ class AtomicTests(TransactionTestCase):
 
     def test_reuse_rollback_commit(self):
         atomic = transaction.atomic()
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with atomic:
                 Reporter.objects.create(last_name="Tintin")
                 with atomic:
@@ -183,10 +184,10 @@ class AtomicTests(TransactionTestCase):
 
     def test_reuse_rollback_rollback(self):
         atomic = transaction.atomic()
-        with six.assertRaisesRegex(self, Exception, "Oops"):
+        with self.assertRaisesMessage(Exception, "Oops"):
             with atomic:
                 Reporter.objects.create(last_name="Tintin")
-                with six.assertRaisesRegex(self, Exception, "Oops"):
+                with self.assertRaisesMessage(Exception, "Oops"):
                     with atomic:
                         Reporter.objects.create(first_name="Haddock")
                     raise Exception("Oops, that's his last name")
@@ -229,8 +230,10 @@ class AtomicInsideTransactionTests(AtomicTests):
         self.atomic.__exit__(*sys.exc_info())
 
 
-@skipIf(connection.features.autocommits_when_autocommit_is_off,
-        "This test requires a non-autocommit mode that doesn't autocommit.")
+@skipIf(
+    connection.features.autocommits_when_autocommit_is_off,
+    "This test requires a non-autocommit mode that doesn't autocommit."
+)
 class AtomicWithoutAutocommitTests(AtomicTests):
     """All basic tests for atomic should also pass when autocommit is turned off."""
 
@@ -244,8 +247,7 @@ class AtomicWithoutAutocommitTests(AtomicTests):
         transaction.set_autocommit(True)
 
 
-@skipUnless(connection.features.uses_savepoints,
-        "'atomic' requires transactions and savepoints.")
+@skipUnless(connection.features.uses_savepoints, "'atomic' requires transactions and savepoints.")
 class AtomicMergeTests(TransactionTestCase):
     """Test merging transactions with savepoint=False."""
 
@@ -256,7 +258,7 @@ class AtomicMergeTests(TransactionTestCase):
             Reporter.objects.create(first_name="Tintin")
             with transaction.atomic(savepoint=False):
                 Reporter.objects.create(first_name="Archibald", last_name="Haddock")
-                with six.assertRaisesRegex(self, Exception, "Oops"):
+                with self.assertRaisesMessage(Exception, "Oops"):
                     with transaction.atomic(savepoint=False):
                         Reporter.objects.create(first_name="Calculus")
                         raise Exception("Oops, that's his last name")
@@ -280,7 +282,7 @@ class AtomicMergeTests(TransactionTestCase):
             Reporter.objects.create(first_name="Tintin")
             with transaction.atomic():
                 Reporter.objects.create(first_name="Archibald", last_name="Haddock")
-                with six.assertRaisesRegex(self, Exception, "Oops"):
+                with self.assertRaisesMessage(Exception, "Oops"):
                     with transaction.atomic(savepoint=False):
                         Reporter.objects.create(first_name="Calculus")
                         raise Exception("Oops, that's his last name")
@@ -295,8 +297,7 @@ class AtomicMergeTests(TransactionTestCase):
         self.assertQuerysetEqual(Reporter.objects.all(), ['<Reporter: Tintin>'])
 
 
-@skipUnless(connection.features.uses_savepoints,
-        "'atomic' requires transactions and savepoints.")
+@skipUnless(connection.features.uses_savepoints, "'atomic' requires transactions and savepoints.")
 class AtomicErrorsTests(TransactionTestCase):
 
     available_apps = ['transactions']
@@ -383,7 +384,7 @@ class AtomicMySQLTests(TransactionTestCase):
         other_thread.start()
         other_thread_ready.wait()
 
-        with six.assertRaisesRegex(self, OperationalError, 'Deadlock found'):
+        with self.assertRaisesMessage(OperationalError, 'Deadlock found'):
             # Double atomic to enter a transaction and create a savepoint.
             with transaction.atomic():
                 with transaction.atomic():
@@ -419,7 +420,7 @@ class AtomicMiscTests(TransactionTestCase):
             with transaction.atomic():
 
                 # Swallow the intentional error raised in the sub-transaction.
-                with six.assertRaisesRegex(self, Exception, "Oops"):
+                with self.assertRaisesMessage(Exception, "Oops"):
 
                     # Start a sub-transaction with a savepoint.
                     with transaction.atomic():
