@@ -1,9 +1,9 @@
 from __future__ import unicode_literals
 
-from django.db.models.signals import pre_save, post_save
+from django.db.models.signals import post_save, pre_save
 from django.test import TestCase
 
-from .models import Person, Employee, ProxyEmployee, Profile, Account
+from .models import Account, Employee, Person, Profile, ProxyEmployee
 
 
 class UpdateOnlyFieldsTests(TestCase):
@@ -80,20 +80,17 @@ class UpdateOnlyFieldsTests(TestCase):
 
     def test_update_fields_inheritance_defer(self):
         profile_boss = Profile.objects.create(name='Boss', salary=3000)
-        e1 = Employee.objects.create(name='Sara', gender='F',
-            employee_num=1, profile=profile_boss)
+        e1 = Employee.objects.create(name='Sara', gender='F', employee_num=1, profile=profile_boss)
         e1 = Employee.objects.only('name').get(pk=e1.pk)
         e1.name = 'Linda'
         with self.assertNumQueries(1):
             e1.save()
-        self.assertEqual(Employee.objects.get(pk=e1.pk).name,
-                         'Linda')
+        self.assertEqual(Employee.objects.get(pk=e1.pk).name, 'Linda')
 
     def test_update_fields_fk_defer(self):
         profile_boss = Profile.objects.create(name='Boss', salary=3000)
         profile_receptionist = Profile.objects.create(name='Receptionist', salary=1000)
-        e1 = Employee.objects.create(name='Sara', gender='F',
-            employee_num=1, profile=profile_boss)
+        e1 = Employee.objects.create(name='Sara', gender='F', employee_num=1, profile=profile_boss)
         e1 = Employee.objects.only('profile').get(pk=e1.pk)
         e1.profile = profile_receptionist
         with self.assertNumQueries(1):
@@ -106,8 +103,7 @@ class UpdateOnlyFieldsTests(TestCase):
 
     def test_select_related_only_interaction(self):
         profile_boss = Profile.objects.create(name='Boss', salary=3000)
-        e1 = Employee.objects.create(name='Sara', gender='F',
-            employee_num=1, profile=profile_boss)
+        e1 = Employee.objects.create(name='Sara', gender='F', employee_num=1, profile=profile_boss)
         e1 = Employee.objects.only('profile__salary').select_related('profile').get(pk=e1.pk)
         profile_boss.name = 'Clerk'
         profile_boss.salary = 1000
@@ -122,13 +118,10 @@ class UpdateOnlyFieldsTests(TestCase):
 
     def test_update_fields_m2m(self):
         profile_boss = Profile.objects.create(name='Boss', salary=3000)
-        e1 = Employee.objects.create(name='Sara', gender='F',
-            employee_num=1, profile=profile_boss)
-
+        e1 = Employee.objects.create(name='Sara', gender='F', employee_num=1, profile=profile_boss)
         a1 = Account.objects.create(num=1)
         a2 = Account.objects.create(num=2)
-
-        e1.accounts = [a1, a2]
+        e1.accounts.set([a1, a2])
 
         with self.assertRaises(ValueError):
             e1.save(update_fields=['accounts'])
@@ -136,9 +129,7 @@ class UpdateOnlyFieldsTests(TestCase):
     def test_update_fields_inheritance(self):
         profile_boss = Profile.objects.create(name='Boss', salary=3000)
         profile_receptionist = Profile.objects.create(name='Receptionist', salary=1000)
-
-        e1 = Employee.objects.create(name='Sara', gender='F',
-            employee_num=1, profile=profile_boss)
+        e1 = Employee.objects.create(name='Sara', gender='F', employee_num=1, profile=profile_boss)
 
         e1.name = 'Ian'
         e1.gender = 'M'
@@ -168,9 +159,7 @@ class UpdateOnlyFieldsTests(TestCase):
     def test_update_fields_inheritance_with_proxy_model(self):
         profile_boss = Profile.objects.create(name='Boss', salary=3000)
         profile_receptionist = Profile.objects.create(name='Receptionist', salary=1000)
-
-        e1 = ProxyEmployee.objects.create(name='Sara', gender='F',
-            employee_num=1, profile=profile_boss)
+        e1 = ProxyEmployee.objects.create(name='Sara', gender='F', employee_num=1, profile=profile_boss)
 
         e1.name = 'Ian'
         e1.gender = 'M'
@@ -204,10 +193,10 @@ class UpdateOnlyFieldsTests(TestCase):
         p.save(update_fields=['name'])
         self.assertEqual(len(pre_save_data), 1)
         self.assertEqual(len(pre_save_data[0]), 1)
-        self.assertTrue('name' in pre_save_data[0])
+        self.assertIn('name', pre_save_data[0])
         self.assertEqual(len(post_save_data), 1)
         self.assertEqual(len(post_save_data[0]), 1)
-        self.assertTrue('name' in post_save_data[0])
+        self.assertIn('name', post_save_data[0])
 
         pre_save.disconnect(pre_save_receiver)
         post_save.disconnect(post_save_receiver)
