@@ -1,9 +1,15 @@
-from django.test import Client, TestCase, override_settings
+from django.template import TemplateDoesNotExist
+from django.test import (
+    Client, RequestFactory, SimpleTestCase, override_settings,
+)
+from django.test.utils import ignore_warnings
+from django.utils.deprecation import RemovedInDjango20Warning
 from django.utils.translation import override
+from django.views.csrf import CSRF_FAILURE_TEMPLATE_NAME, csrf_failure
 
 
-@override_settings(ROOT_URLCONF="view_tests.urls")
-class CsrfViewTests(TestCase):
+@override_settings(ROOT_URLCONF='view_tests.urls')
+class CsrfViewTests(SimpleTestCase):
 
     def setUp(self):
         super(CsrfViewTests, self).setUp()
@@ -11,11 +17,11 @@ class CsrfViewTests(TestCase):
 
     @override_settings(
         USE_I18N=True,
-        MIDDLEWARE_CLASSES=(
+        MIDDLEWARE=[
             'django.middleware.locale.LocaleMiddleware',
             'django.middleware.common.CommonMiddleware',
             'django.middleware.csrf.CsrfViewMiddleware',
-        ),
+        ],
     )
     def test_translation(self):
         """
@@ -34,6 +40,33 @@ class CsrfViewTests(TestCase):
                                 "CSRF-verificatie mislukt. Verzoek afgebroken.",
                                 status_code=403)
 
+    @ignore_warnings(category=RemovedInDjango20Warning)
+    @override_settings(
+        USE_I18N=True,
+        MIDDLEWARE=None,
+        MIDDLEWARE_CLASSES=[
+            'django.middleware.locale.LocaleMiddleware',
+            'django.middleware.common.CommonMiddleware',
+            'django.middleware.csrf.CsrfViewMiddleware',
+        ],
+    )
+    def test_translation_middleware_classes(self):
+        """
+        Test that an invalid request is rejected with a localized error message.
+        """
+        response = self.client.post('/')
+        self.assertContains(response, "Forbidden", status_code=403)
+        self.assertContains(response,
+                            "CSRF verification failed. Request aborted.",
+                            status_code=403)
+
+        with self.settings(LANGUAGE_CODE='nl'), override('en-us'):
+            response = self.client.post('/')
+            self.assertContains(response, "Verboden", status_code=403)
+            self.assertContains(response,
+                                "CSRF-verificatie mislukt. Verzoek afgebroken.",
+                                status_code=403)
+
     @override_settings(
         SECURE_PROXY_SSL_HEADER=('HTTP_X_FORWARDED_PROTO', 'https')
     )
@@ -63,14 +96,36 @@ class CsrfViewTests(TestCase):
                             "by third parties.",
                             status_code=403)
 
-    # In Django 1.10, this can be changed to TEMPLATES=[] because the code path
-    # that reads the TEMPLATE_* settings in that case will have been removed.
-    @override_settings(TEMPLATES=[{
-        'BACKEND': 'django.template.backends.dummy.TemplateStrings',
-    }])
+    @override_settings(TEMPLATES=[])
     def test_no_django_template_engine(self):
         """
         The CSRF view doesn't depend on the TEMPLATES configuration (#24388).
         """
         response = self.client.post('/')
         self.assertContains(response, "Forbidden", status_code=403)
+
+    @override_settings(TEMPLATES=[{
+        'BACKEND': 'django.template.backends.django.DjangoTemplates',
+        'OPTIONS': {
+            'loaders': [
+                ('django.template.loaders.locmem.Loader', {
+                    CSRF_FAILURE_TEMPLATE_NAME: 'Test template for CSRF failure'
+                }),
+            ],
+        },
+    }])
+    def test_custom_template(self):
+        """
+        A custom CSRF_FAILURE_TEMPLATE_NAME is used.
+        """
+        response = self.client.post('/')
+        self.assertContains(response, "Test template for CSRF failure", status_code=403)
+
+    def test_custom_template_does_not_exist(self):
+        """
+        An exception is raised if a nonexistent template is supplied.
+        """
+        factory = RequestFactory()
+        request = factory.post('/')
+        with self.assertRaises(TemplateDoesNotExist):
+            csrf_failure(request, template_name="nonexistent.html")
