@@ -1,27 +1,16 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.test import TestCase, override_settings
+from django.test import SimpleTestCase, override_settings
 
 
 @override_settings(ROOT_URLCONF='view_tests.generic_urls')
-class URLHandling(TestCase):
+class URLHandling(SimpleTestCase):
     """
     Tests for URL handling in views and responses.
     """
     redirect_target = "/%E4%B8%AD%E6%96%87/target/"
 
-    def test_combining_redirect(self):
-        """
-        Tests that redirecting to an IRI, requiring encoding before we use it
-        in an HTTP response, is handled correctly. In this case the arg to
-        HttpRedirect is ASCII but the current request path contains non-ASCII
-        characters so this test ensures the creation of the full path with a
-        base non-ASCII part is handled correctly.
-        """
-        response = self.client.get('/中文/')
-        self.assertRedirects(response, self.redirect_target)
-
     def test_nonascii_redirect(self):
         """
         Tests that a non-ASCII argument to HttpRedirect is handled properly.
