@@ -1,11 +1,11 @@
 from __future__ import unicode_literals
 
 import mimetypes
-from os import path
 import unittest
+from os import path
 
 from django.conf.urls.static import static
-from django.http import HttpResponseNotModified
+from django.http import FileResponse, HttpResponseNotModified
 from django.test import SimpleTestCase, override_settings
 from django.utils.http import http_date
 from django.views.static import was_modified_since
@@ -14,11 +14,10 @@ from .. import urls
 from ..urls import media_dir
 
 
-@override_settings(DEBUG=True)
+@override_settings(DEBUG=True, ROOT_URLCONF='view_tests.urls')
 class StaticTests(SimpleTestCase):
     """Tests django views in django/views/static.py"""
 
-    urls = 'view_tests.urls'
     prefix = 'site_media'
 
     def test_serve(self):
@@ -33,6 +32,16 @@ class StaticTests(SimpleTestCase):
             self.assertEqual(len(response_content), int(response['Content-Length']))
             self.assertEqual(mimetypes.guess_type(file_path)[1], response.get('Content-Encoding', None))
 
+    def test_chunked(self):
+        "The static view should stream files in chunks to avoid large memory usage"
+        response = self.client.get('/%s/%s' % (self.prefix, 'long-line.txt'))
+        first_chunk = next(response.streaming_content)
+        self.assertEqual(len(first_chunk), FileResponse.block_size)
+        second_chunk = next(response.streaming_content)
+        response.close()
+        # strip() to prevent OS line endings from causing differences
+        self.assertEqual(len(second_chunk.strip()), 1449)
+
     def test_unknown_mime_type(self):
         response = self.client.get('/%s/file.unknown' % self.prefix)
         self.assertEqual('application/octet-stream', response['Content-Type'])
@@ -47,8 +56,10 @@ class StaticTests(SimpleTestCase):
 
     def test_is_modified_since(self):
         file_name = 'file.txt'
-        response = self.client.get('/%s/%s' % (self.prefix, file_name),
-            HTTP_IF_MODIFIED_SINCE='Thu, 1 Jan 1970 00:00:00 GMT')
+        response = self.client.get(
+            '/%s/%s' % (self.prefix, file_name),
+            HTTP_IF_MODIFIED_SINCE='Thu, 1 Jan 1970 00:00:00 GMT'
+        )
         response_content = b''.join(response)
         with open(path.join(media_dir, file_name), 'rb') as fp:
             self.assertEqual(fp.read(), response_content)
@@ -97,6 +108,10 @@ class StaticTests(SimpleTestCase):
         response = self.client.get('/%s/non_existing_resource' % self.prefix)
         self.assertEqual(404, response.status_code)
 
+    def test_index(self):
+        response = self.client.get('/%s/' % self.prefix)
+        self.assertContains(response, 'Index of ./')
+
 
 class StaticHelperTest(StaticTests):
     """
